@@ -26,7 +26,7 @@ package de.lmu.ifi.dbs.elki.algorithm;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.BitSet;
-import java.util.Hashtable;
+import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 
@@ -36,16 +36,16 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.result.AprioriResult;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.OneMustBeSetGlobalConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.OnlyOneIsAllowedToBeSetGlobalConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
@@ -72,14 +72,14 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(APRIORI.class);
+  private static final Logging LOG = Logging.getLogger(APRIORI.class);
 
   /**
    * Optional parameter to specify the threshold for minimum frequency, must be
    * a double greater than or equal to 0 and less than or equal to 1.
    * Alternatively to parameter {@link #MINSUPP_ID}).
    */
-  public static final OptionID MINFREQ_ID = OptionID.getOrCreateOptionID("apriori.minfreq", "Threshold for minimum frequency as percentage value " + "(alternatively to parameter apriori.minsupp).");
+  public static final OptionID MINFREQ_ID = new OptionID("apriori.minfreq", "Threshold for minimum frequency as percentage value " + "(alternatively to parameter apriori.minsupp).");
 
   /**
    * Parameter to specify the threshold for minimum support as minimally
@@ -88,7 +88,7 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
    * {@link #MINSUPP_ID} is slightly preferable over setting {@link #MINFREQ_ID}
    * in terms of efficiency.
    */
-  public static final OptionID MINSUPP_ID = OptionID.getOrCreateOptionID("apriori.minsupp", "Threshold for minimum support as minimally required number of transactions " + "(alternatively to parameter apriori.minfreq" + " - setting apriori.minsupp is slightly preferable over setting " + "apriori.minfreq in terms of efficiency).");
+  public static final OptionID MINSUPP_ID = new OptionID("apriori.minsupp", "Threshold for minimum support as minimally required number of transactions " + "(alternatively to parameter apriori.minfreq" + " - setting apriori.minsupp is slightly preferable over setting " + "apriori.minfreq in terms of efficiency).");
 
   /**
    * Holds the value of {@link #MINFREQ_ID}.
@@ -128,13 +128,13 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
    * @return the AprioriResult learned by this APRIORI
    */
   public AprioriResult run(Database database, Relation<BitVector> relation) {
-    Map<BitSet, Integer> support = new Hashtable<BitSet, Integer>();
+    Map<BitSet, Integer> support = new HashMap<BitSet, Integer>();
     List<BitSet> solution = new ArrayList<BitSet>();
     final int size = relation.size();
     if(size > 0) {
       int dim;
       try {
-        dim = DatabaseUtil.dimensionality(relation);
+        dim = RelationUtil.dimensionality(relation);
       }
       catch(UnsupportedOperationException e) {
         dim = 0;
@@ -145,9 +145,9 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
         candidates[i].set(i);
       }
       while(candidates.length > 0) {
-        StringBuffer msg = new StringBuffer();
+        StringBuilder msg = new StringBuilder();
         BitSet[] frequentItemsets = frequentItemsets(support, candidates, relation);
-        if(logger.isVerbose()) {
+        if(LOG.isVerbose()) {
           msg.append("\ncandidates").append(Arrays.asList(candidates));
           msg.append("\nfrequentItemsets").append(Arrays.asList(frequentItemsets));
         }
@@ -156,9 +156,9 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
         }
         BitSet[] joined = join(frequentItemsets);
         candidates = prune(support, joined, size);
-        if(logger.isVerbose()) {
+        if(LOG.isVerbose()) {
           msg.append("\ncandidates after pruning").append(Arrays.asList(candidates));
-          logger.verbose(msg.toString());
+          LOG.verbose(msg.toString());
         }
       }
     }
@@ -300,7 +300,7 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
   
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -312,32 +312,35 @@ public class APRIORI extends AbstractAlgorithm<AprioriResult> {
    */
   public static class Parameterizer extends AbstractParameterizer {
     /**
-     * Parameter for minFreq
+     * Parameter for minFreq.
      */
     protected Double minfreq = null;
 
     /**
-     * Parameter for minSupp
+     * Parameter for minSupp.
      */
     protected Integer minsupp = null;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      DoubleParameter minfreqP = new DoubleParameter(MINFREQ_ID, true);
-      minfreqP.addConstraint(new IntervalConstraint(0, IntervalConstraint.IntervalBoundary.CLOSE, 1, IntervalConstraint.IntervalBoundary.CLOSE));
+      DoubleParameter minfreqP = new DoubleParameter(MINFREQ_ID);
+      minfreqP.setOptional(true);
+      minfreqP.addConstraint(new GreaterEqualConstraint(0));
+      minfreqP.addConstraint(new LessEqualConstraint(1));
       if(config.grab(minfreqP)) {
         minfreq = minfreqP.getValue();
       }
 
-      IntParameter minsuppP = new IntParameter(MINSUPP_ID, true);
+      IntParameter minsuppP = new IntParameter(MINSUPP_ID);
+      minsuppP.setOptional(true);
       minsuppP.addConstraint(new GreaterEqualConstraint(0));
       if(config.grab(minsuppP)) {
         minsupp = minsuppP.getValue();
       }
 
       // global parameter constraints
-      ArrayList<Parameter<?, ?>> globalConstraints = new ArrayList<Parameter<?, ?>>();
+      ArrayList<Parameter<?>> globalConstraints = new ArrayList<Parameter<?>>();
       globalConstraints.add(minfreqP);
       globalConstraints.add(minsuppP);
       config.checkConstraint(new OnlyOneIsAllowedToBeSetGlobalConstraint(globalConstraints));
