@@ -110,10 +110,6 @@ public abstract class AbstractAlgorithm<R extends Result> implements Algorithm {
 
     if(runmethod1 != null) {
       try {
-        StringBuffer buf = new StringBuffer();
-        for(Class<?> cls : signature1) {
-          buf.append(cls.toString()).append(",");
-        }
         return (R) runmethod1.invoke(this, relations1);
       }
       catch(IllegalArgumentException e) {
@@ -134,10 +130,6 @@ public abstract class AbstractAlgorithm<R extends Result> implements Algorithm {
     }
     else if(runmethod2 != null) {
       try {
-        StringBuffer buf = new StringBuffer();
-        for(Class<?> cls : signature1) {
-          buf.append(cls.toString()).append(",");
-        }
         return (R) runmethod2.invoke(this, relations2);
       }
       catch(IllegalArgumentException e) {
@@ -174,10 +166,10 @@ public abstract class AbstractAlgorithm<R extends Result> implements Algorithm {
    * 
    * @return the static logger
    */
-  abstract protected Logging getLogger();
+  protected abstract Logging getLogger();
 
   /**
-   * Make a default distance function configuration option
+   * Make a default distance function configuration option.
    * 
    * @param <F> Distance function type
    * @param defaultDistanceFunction Default value
