@@ -45,7 +45,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * @param <D> the type of Distance used by this Algorithm
  * @param <R> the type of result to retrieve from this Algorithm
  */
-public abstract class AbstractPrimitiveDistanceBasedAlgorithm<O, D extends Distance<D>, R extends Result> extends AbstractAlgorithm<R> {
+public abstract class AbstractPrimitiveDistanceBasedAlgorithm<O, D extends Distance<?>, R extends Result> extends AbstractAlgorithm<R> {
   /**
    * Holds the instance of the distance function specified by
    * {@link AbstractDistanceBasedAlgorithm#DISTANCE_FUNCTION_ID}.
@@ -79,6 +79,9 @@ public abstract class AbstractPrimitiveDistanceBasedAlgorithm<O, D extends Dista
    * @apiviz.exclude
    */
   public abstract static class Parameterizer<O, D extends Distance<D>> extends AbstractParameterizer {
+    /**
+     * Distance function to use.
+     */
     protected PrimitiveDistanceFunction<O, D> distanceFunction;
     
     @Override
