@@ -34,18 +34,18 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.PrimitiveDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.LinearEquationSystem;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredResult;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredRunner;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
@@ -63,6 +63,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * attributes of a given dataset based on a linear correlation PCA.
  * </p>
  * 
+ * <p>
  * Reference: <br>
  * E. Achtert, C. Böhm, H.-P. Kriegel, P. Kröger, A. Zimek: Deriving
  * Quantitative Dependencies for Correlation Clusters. <br>
@@ -76,32 +77,32 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 @Title("Dependency Derivator: Deriving numerical inter-dependencies on data")
 @Description("Derives an equality-system describing dependencies between attributes in a correlation-cluster")
 @Reference(authors = "E. Achtert, C. Böhm, H.-P. Kriegel, P. Kröger, A. Zimek", title = "Deriving Quantitative Dependencies for Correlation Clusters", booktitle = "Proc. 12th Int. Conf. on Knowledge Discovery and Data Mining (KDD '06), Philadelphia, PA 2006.", url = "http://dx.doi.org/10.1145/1150402.1150408")
-public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distance<D>> extends AbstractPrimitiveDistanceBasedAlgorithm<V, D, CorrelationAnalysisSolution<V>> {
+public class DependencyDerivator<V extends NumberVector<?>, D extends Distance<D>> extends AbstractPrimitiveDistanceBasedAlgorithm<V, D, CorrelationAnalysisSolution<V>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(DependencyDerivator.class);
+  private static final Logging LOG = Logging.getLogger(DependencyDerivator.class);
 
   /**
    * Flag to use random sample (use knn query around centroid, if flag is not
    * set).
    */
-  public static final OptionID DEPENDENCY_DERIVATOR_RANDOM_SAMPLE = OptionID.getOrCreateOptionID("derivator.randomSample", "Flag to use random sample (use knn query around centroid, if flag is not set).");
+  public static final OptionID DEPENDENCY_DERIVATOR_RANDOM_SAMPLE = new OptionID("derivator.randomSample", "Flag to use random sample (use knn query around centroid, if flag is not set).");
 
   /**
    * Parameter to specify the threshold for output accuracy fraction digits,
    * must be an integer equal to or greater than 0.
    */
-  public static final OptionID OUTPUT_ACCURACY_ID = OptionID.getOrCreateOptionID("derivator.accuracy", "Threshold for output accuracy fraction digits.");
+  public static final OptionID OUTPUT_ACCURACY_ID = new OptionID("derivator.accuracy", "Threshold for output accuracy fraction digits.");
 
   /**
    * Optional parameter to specify the threshold for the size of the random
    * sample to use, must be an integer greater than 0.
-   * <p/>
+   * <p>
    * Default value: the size of the complete dataset
    * </p>
    */
-  public static final OptionID SAMPLE_SIZE_ID = OptionID.getOrCreateOptionID("derivator.sampleSize", "Threshold for the size of the random sample to use. " + "Default value is size of the complete dataset.");
+  public static final OptionID SAMPLE_SIZE_ID = new OptionID("derivator.sampleSize", "Threshold for the size of the random sample to use. " + "Default value is size of the complete dataset.");
 
   /**
    * Holds the value of {@link #SAMPLE_SIZE_ID}.
@@ -116,7 +117,7 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
   /**
    * Number format for output of solution.
    */
-  public final NumberFormat NF;
+  private final NumberFormat nf;
 
   /**
    * Flag for random sampling vs. kNN
@@ -134,7 +135,7 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
    */
   public DependencyDerivator(PrimitiveDistanceFunction<V, D> distanceFunction, NumberFormat nf, PCAFilteredRunner<V> pca, int sampleSize, boolean randomsample) {
     super(distanceFunction);
-    this.NF = nf;
+    this.nf = nf;
     this.pca = pca;
     this.sampleSize = sampleSize;
     this.randomsample = randomsample;
@@ -150,26 +151,27 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
    *         DependencyDerivator
    */
   public CorrelationAnalysisSolution<V> run(Database database, Relation<V> relation) {
-    if(logger.isVerbose()) {
-      logger.verbose("retrieving database objects...");
+    if(LOG.isVerbose()) {
+      LOG.verbose("retrieving database objects...");
     }
-    V centroidDV = DatabaseUtil.centroid(relation);
+    Centroid centroid = Centroid.make(relation);
+    V centroidDV = centroid.toVector(relation);
     DBIDs ids;
     if(this.sampleSize > 0) {
       if(randomsample) {
-        ids = DBIDUtil.randomSample(relation.getDBIDs(), this.sampleSize, 1l);
+        ids = DBIDUtil.randomSample(relation.getDBIDs(), this.sampleSize, 1L);
       }
       else {
         DistanceQuery<V, D> distanceQuery = database.getDistanceQuery(relation, getDistanceFunction());
         KNNResult<D> queryResults = database.getKNNQuery(distanceQuery, this.sampleSize).getKNNForObject(centroidDV, this.sampleSize);
-        ids = DBIDUtil.newHashSet(queryResults.asDBIDs());
+        ids = DBIDUtil.newHashSet(queryResults);
       }
     }
     else {
       ids = relation.getDBIDs();
     }
 
-    return generateModel(relation, ids, centroidDV);
+    return generateModel(relation, ids, centroid);
   }
 
   /**
@@ -181,8 +183,7 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
    * @return a matrix of equations describing the dependencies
    */
   public CorrelationAnalysisSolution<V> generateModel(Relation<V> db, DBIDs ids) {
-    V centroidDV = DatabaseUtil.centroid(db, ids);
-    return generateModel(db, ids, centroidDV);
+    return generateModel(db, ids, Centroid.make(db, ids));
   }
 
   /**
@@ -190,13 +191,13 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
    * 
    * @param db the database
    * @param ids the set of ids
-   * @param centroidDV the centroid
+   * @param centroid the centroid
    * @return a matrix of equations describing the dependencies
    */
-  public CorrelationAnalysisSolution<V> generateModel(Relation<V> db, DBIDs ids, V centroidDV) {
+  public CorrelationAnalysisSolution<V> generateModel(Relation<V> db, DBIDs ids, Vector centroid) {
     CorrelationAnalysisSolution<V> sol;
-    if(logger.isDebuggingFine()) {
-      logger.debugFine("PCA...");
+    if(LOG.isDebuggingFine()) {
+      LOG.debugFine("PCA...");
     }
 
     PCAFilteredResult pcares = pca.processIds(ids, db);
@@ -206,7 +207,6 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
     // Matrix strongEigenvectors =
     // pca.getEigenvectors().times(pca.selectionMatrixOfStrongEigenvectors());
     Matrix strongEigenvectors = pcares.getStrongEigenvectors();
-    Vector centroid = centroidDV.getColumnVector();
 
     // TODO: what if we don't have any weak eigenvectors?
     if(weakEigenvectors.getColumnDimensionality() == 0) {
@@ -214,50 +214,49 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
     }
     else {
       Matrix transposedWeakEigenvectors = weakEigenvectors.transpose();
-      if(logger.isDebugging()) {
+      if(LOG.isDebugging()) {
         StringBuilder log = new StringBuilder();
         log.append("Strong Eigenvectors:\n");
-        log.append(FormatUtil.format(pcares.getEigenvectors().times(pcares.selectionMatrixOfStrongEigenvectors()), NF)).append('\n');
+        log.append(FormatUtil.format(pcares.getEigenvectors().times(pcares.selectionMatrixOfStrongEigenvectors()), nf)).append('\n');
         log.append("Transposed weak Eigenvectors:\n");
-        log.append(FormatUtil.format(transposedWeakEigenvectors, NF)).append('\n');
+        log.append(FormatUtil.format(transposedWeakEigenvectors, nf)).append('\n');
         log.append("Eigenvalues:\n");
         log.append(FormatUtil.format(pcares.getEigenvalues(), " , ", 2));
-        logger.debugFine(log.toString());
+        LOG.debugFine(log.toString());
       }
-      Vector B = transposedWeakEigenvectors.times(centroid);
-      if(logger.isDebugging()) {
+      Vector b = transposedWeakEigenvectors.times(centroid);
+      if(LOG.isDebugging()) {
         StringBuilder log = new StringBuilder();
         log.append("Centroid:\n").append(centroid).append('\n');
         log.append("tEV * Centroid\n");
-        log.append(B);
-        logger.debugFine(log.toString());
+        log.append(b);
+        LOG.debugFine(log.toString());
       }
 
       // +1 == + B.getColumnDimensionality()
       Matrix gaussJordan = new Matrix(transposedWeakEigenvectors.getRowDimensionality(), transposedWeakEigenvectors.getColumnDimensionality() + 1);
       gaussJordan.setMatrix(0, transposedWeakEigenvectors.getRowDimensionality() - 1, 0, transposedWeakEigenvectors.getColumnDimensionality() - 1, transposedWeakEigenvectors);
-      gaussJordan.setCol(transposedWeakEigenvectors.getColumnDimensionality(), B);
+      gaussJordan.setCol(transposedWeakEigenvectors.getColumnDimensionality(), b);
 
-      if(logger.isDebuggingFiner()) {
-        logger.debugFiner("Gauss-Jordan-Elimination of " + FormatUtil.format(gaussJordan, NF));
+      if(LOG.isDebuggingFiner()) {
+        LOG.debugFiner("Gauss-Jordan-Elimination of " + FormatUtil.format(gaussJordan, nf));
       }
 
       double[][] a = new double[transposedWeakEigenvectors.getRowDimensionality()][transposedWeakEigenvectors.getColumnDimensionality()];
       double[][] we = transposedWeakEigenvectors.getArrayRef();
-      double[] b = B.getArrayRef();
       System.arraycopy(we, 0, a, 0, transposedWeakEigenvectors.getRowDimensionality());
 
-      LinearEquationSystem lq = new LinearEquationSystem(a, b);
+      LinearEquationSystem lq = new LinearEquationSystem(a, b.getArrayRef());
       lq.solveByTotalPivotSearch();
 
       sol = new CorrelationAnalysisSolution<V>(lq, db, strongEigenvectors, pcares.getWeakEigenvectors(), pcares.similarityMatrix(), centroid);
 
-      if(logger.isDebuggingFine()) {
+      if(LOG.isDebuggingFine()) {
         StringBuilder log = new StringBuilder();
         log.append("Solution:\n");
         log.append("Standard deviation ").append(sol.getStandardDeviation());
-        log.append(lq.equationsToString(NF.getMaximumFractionDigits()));
-        logger.debugFine(log.toString());
+        log.append(lq.equationsToString(nf.getMaximumFractionDigits()));
+        LOG.debugFine(log.toString());
       }
     }
     return sol;
@@ -270,7 +269,7 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -280,55 +279,59 @@ public class DependencyDerivator<V extends NumberVector<V, ?>, D extends Distanc
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>, D extends Distance<D>> extends AbstractPrimitiveDistanceBasedAlgorithm.Parameterizer<V, D> {
+  public static class Parameterizer<V extends NumberVector<?>, D extends Distance<D>> extends AbstractPrimitiveDistanceBasedAlgorithm.Parameterizer<V, D> {
+    /**
+     * Output accuracy.
+     */
     protected int outputAccuracy = 0;
 
+    /**
+     * Sample size.
+     */
     protected int sampleSize = 0;
 
+    /**
+     * Flag to enable random sampling
+     */
     protected boolean randomSample = false;
 
+    /**
+     * Class to compute PCA with
+     */
     protected PCAFilteredRunner<V> pca = null;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      configAccuracy(config);
-      configSampleSize(config);
-      configRandomSampleFlag(config);
-      Class<PCAFilteredRunner<V>> cls = ClassGenericsUtil.uglyCastIntoSubclass(PCAFilteredRunner.class);
-      pca = config.tryInstantiate(cls);
-    }
-
-    public void configRandomSampleFlag(Parameterization config) {
-      Flag randomSampleF = new Flag(DEPENDENCY_DERIVATOR_RANDOM_SAMPLE);
-      if(config.grab(randomSampleF)) {
-        randomSample = randomSampleF.getValue();
-      }
-    }
-
-    public void configSampleSize(Parameterization config) {
-      IntParameter sampleSizeP = new IntParameter(SAMPLE_SIZE_ID, true);
-      sampleSizeP.addConstraint(new GreaterConstraint(0));
-      if(config.grab(sampleSizeP)) {
-        sampleSize = sampleSizeP.getValue();
-      }
-    }
-
-    public void configAccuracy(Parameterization config) {
+      
       IntParameter outputAccuracyP = new IntParameter(OUTPUT_ACCURACY_ID, 4);
       outputAccuracyP.addConstraint(new GreaterEqualConstraint(0));
       if(config.grab(outputAccuracyP)) {
         outputAccuracy = outputAccuracyP.getValue();
       }
+      
+      IntParameter sampleSizeP = new IntParameter(SAMPLE_SIZE_ID);
+      sampleSizeP.setOptional(true);
+      sampleSizeP.addConstraint(new GreaterConstraint(0));
+      if(config.grab(sampleSizeP)) {
+        sampleSize = sampleSizeP.getValue();
+      }
+      
+      Flag randomSampleF = new Flag(DEPENDENCY_DERIVATOR_RANDOM_SAMPLE);
+      if(config.grab(randomSampleF)) {
+        randomSample = randomSampleF.getValue();
+      }
+      Class<PCAFilteredRunner<V>> cls = ClassGenericsUtil.uglyCastIntoSubclass(PCAFilteredRunner.class);
+      pca = config.tryInstantiate(cls);
     }
 
     @Override
     protected DependencyDerivator<V, D> makeInstance() {
-      NumberFormat NF = NumberFormat.getInstance(Locale.US);
-      NF.setMaximumFractionDigits(outputAccuracy);
-      NF.setMinimumFractionDigits(outputAccuracy);
+      NumberFormat nf = NumberFormat.getInstance(Locale.US);
+      nf.setMaximumFractionDigits(outputAccuracy);
+      nf.setMinimumFractionDigits(outputAccuracy);
 
-      return new DependencyDerivator<V, D>(distanceFunction, NF, pca, sampleSize, randomSample);
+      return new DependencyDerivator<V, D>(distanceFunction, nf, pca, sampleSize, randomSample);
     }
   }
 }
\ No newline at end of file
