@@ -53,15 +53,14 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
  */
 @Title("Dummy Algorithm")
 @Description("The algorithm executes an Euclidean 10NN query on all data points, and can be used in unit testing")
-public class DummyAlgorithm<O extends NumberVector<?, ?>> extends AbstractAlgorithm<Result> {
+public class DummyAlgorithm<O extends NumberVector<?>> extends AbstractAlgorithm<Result> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(DummyAlgorithm.class);
+  private static final Logging LOG = Logging.getLogger(DummyAlgorithm.class);
 
   /**
-   * Constructor, adhering to
-   * {@link de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable}
+   * Constructor.
    */
   public DummyAlgorithm() {
     super();
@@ -96,6 +95,6 @@ public class DummyAlgorithm<O extends NumberVector<?, ?>> extends AbstractAlgori
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 }
\ No newline at end of file
