@@ -37,14 +37,16 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DoubleDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.DistanceUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.SpatialPrimitiveDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.SpatialPrimitiveDoubleDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNHeap;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNHeap;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.index.tree.LeafEntry;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialEntry;
 import de.lmu.ifi.dbs.elki.index.tree.spatial.SpatialIndexTree;
@@ -55,8 +57,6 @@ import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
 import de.lmu.ifi.dbs.elki.result.ResultUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNHeap;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNList;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
@@ -84,20 +84,20 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  */
 @Title("K-Nearest Neighbor Join")
 @Description("Algorithm to find the k-nearest neighbors of each object in a spatial database")
-public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N extends SpatialNode<N, E>, E extends SpatialEntry> extends AbstractDistanceBasedAlgorithm<V, D, DataStore<KNNList<D>>> {
+public class KNNJoin<V extends NumberVector<?>, D extends Distance<D>, N extends SpatialNode<N, E>, E extends SpatialEntry> extends AbstractDistanceBasedAlgorithm<V, D, DataStore<KNNResult<D>>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(KNNJoin.class);
+  private static final Logging LOG = Logging.getLogger(KNNJoin.class);
 
   /**
    * Parameter that specifies the k-nearest neighbors to be assigned, must be an
    * integer greater than 0. Default value: 1.
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("knnjoin.k", "Specifies the k-nearest neighbors to be assigned.");
+  public static final OptionID K_ID = new OptionID("knnjoin.k", "Specifies the k-nearest neighbors to be assigned.");
 
   /**
-   * The k parameter
+   * The k parameter.
    */
   int k;
 
@@ -120,12 +120,12 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
    * @return result
    */
   @SuppressWarnings("unchecked")
-  public WritableDataStore<KNNList<D>> run(Database database, Relation<V> relation) {
-    if(!(getDistanceFunction() instanceof SpatialPrimitiveDistanceFunction)) {
+  public WritableDataStore<KNNResult<D>> run(Database database, Relation<V> relation) {
+    if (!(getDistanceFunction() instanceof SpatialPrimitiveDistanceFunction)) {
       throw new IllegalStateException("Distance Function must be an instance of " + SpatialPrimitiveDistanceFunction.class.getName());
     }
     Collection<SpatialIndexTree<N, E>> indexes = ResultUtil.filterResults(database, SpatialIndexTree.class);
-    if(indexes.size() != 1) {
+    if (indexes.size() != 1) {
       throw new AbortException("KNNJoin found " + indexes.size() + " spatial indexes, expected exactly one.");
     }
     // FIXME: Ensure were looking at the right relation!
@@ -133,9 +133,6 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
     SpatialPrimitiveDistanceFunction<V, D> distFunction = (SpatialPrimitiveDistanceFunction<V, D>) getDistanceFunction();
     DBIDs ids = relation.getDBIDs();
 
-    // Optimize for double?
-    final boolean doubleOptimize = (getDistanceFunction() instanceof SpatialPrimitiveDoubleDistanceFunction);
-
     // data pages
     List<E> ps_candidates = new ArrayList<E>(index.getLeaves());
     // knn heaps
@@ -143,50 +140,49 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
     Heap<Task> pq = new Heap<Task>(ps_candidates.size() * ps_candidates.size() / 10);
 
     // Initialize with the page self-pairing
-    for(int i = 0; i < ps_candidates.size(); i++) {
+    for (int i = 0; i < ps_candidates.size(); i++) {
       E pr_entry = ps_candidates.get(i);
       N pr = index.getNode(pr_entry);
-      heaps.add(initHeaps(distFunction, doubleOptimize, pr));
+      heaps.add(initHeaps(distFunction, pr));
     }
 
     // Build priority queue
-    final int sqsize = ps_candidates.size() * (ps_candidates.size() - 1) / 2;
-    if(logger.isDebuggingFine()) {
-      logger.debugFine("Number of leaves: " + ps_candidates.size() + " so " + sqsize + " MBR computations.");
+    final int sqsize = ps_candidates.size() * (ps_candidates.size() - 1) >> 1;
+    if (LOG.isDebuggingFine()) {
+      LOG.debugFine("Number of leaves: " + ps_candidates.size() + " so " + sqsize + " MBR computations.");
     }
-    FiniteProgress mprogress = logger.isVerbose() ? new FiniteProgress("Comparing leaf MBRs", sqsize, logger) : null;
-    for(int i = 0; i < ps_candidates.size(); i++) {
+    FiniteProgress mprogress = LOG.isVerbose() ? new FiniteProgress("Comparing leaf MBRs", sqsize, LOG) : null;
+    for (int i = 0; i < ps_candidates.size(); i++) {
       E pr_entry = ps_candidates.get(i);
       List<KNNHeap<D>> pr_heaps = heaps.get(i);
       D pr_knn_distance = computeStopDistance(pr_heaps);
 
-      for(int j = i + 1; j < ps_candidates.size(); j++) {
+      for (int j = i + 1; j < ps_candidates.size(); j++) {
         E ps_entry = ps_candidates.get(j);
         List<KNNHeap<D>> ps_heaps = heaps.get(j);
         D ps_knn_distance = computeStopDistance(ps_heaps);
         D minDist = distFunction.minDist(pr_entry, ps_entry);
         // Resolve immediately:
-        if(minDist.isNullDistance()) {
+        if (minDist.isNullDistance()) {
           N pr = index.getNode(ps_candidates.get(i));
           N ps = index.getNode(ps_candidates.get(j));
-          processDataPagesOptimize(distFunction, doubleOptimize, pr_heaps, ps_heaps, pr, ps);
-        }
-        else if(minDist.compareTo(pr_knn_distance) <= 0 || minDist.compareTo(ps_knn_distance) <= 0) {
+          processDataPagesOptimize(distFunction, pr_heaps, ps_heaps, pr, ps);
+        } else if (minDist.compareTo(pr_knn_distance) <= 0 || minDist.compareTo(ps_knn_distance) <= 0) {
           pq.add(new Task(minDist, i, j));
         }
-        if(mprogress != null) {
-          mprogress.incrementProcessed(logger);
+        if (mprogress != null) {
+          mprogress.incrementProcessed(LOG);
         }
       }
     }
-    if(mprogress != null) {
-      mprogress.ensureCompleted(logger);
+    if (mprogress != null) {
+      mprogress.ensureCompleted(LOG);
     }
 
     // Process the queue
-    FiniteProgress qprogress = logger.isVerbose() ? new FiniteProgress("Processing queue", pq.size(), logger) : null;
-    IndefiniteProgress fprogress = logger.isVerbose() ? new IndefiniteProgress("Full comparisons", logger) : null;
-    while(!pq.isEmpty()) {
+    FiniteProgress qprogress = LOG.isVerbose() ? new FiniteProgress("Processing queue", pq.size(), LOG) : null;
+    IndefiniteProgress fprogress = LOG.isVerbose() ? new IndefiniteProgress("Full comparisons", LOG) : null;
+    while (!pq.isEmpty()) {
       Task task = pq.poll();
       List<KNNHeap<D>> pr_heaps = heaps.get(task.i);
       List<KNNHeap<D>> ps_heaps = heaps.get(task.j);
@@ -194,47 +190,45 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
       D ps_knn_distance = computeStopDistance(ps_heaps);
       boolean dor = task.mindist.compareTo(pr_knn_distance) <= 0;
       boolean dos = task.mindist.compareTo(ps_knn_distance) <= 0;
-      if(dor || dos) {
+      if (dor || dos) {
         N pr = index.getNode(ps_candidates.get(task.i));
         N ps = index.getNode(ps_candidates.get(task.j));
-        if(dor && dos) {
-          processDataPagesOptimize(distFunction, doubleOptimize, pr_heaps, ps_heaps, pr, ps);
-        }
-        else {
-          if(dor) {
-            processDataPagesOptimize(distFunction, doubleOptimize, pr_heaps, null, pr, ps);
-          }
-          else /* dos */{
-            processDataPagesOptimize(distFunction, doubleOptimize, ps_heaps, null, ps, pr);
+        if (dor && dos) {
+          processDataPagesOptimize(distFunction, pr_heaps, ps_heaps, pr, ps);
+        } else {
+          if (dor) {
+            processDataPagesOptimize(distFunction, pr_heaps, null, pr, ps);
+          } else /* dos */{
+            processDataPagesOptimize(distFunction, ps_heaps, null, ps, pr);
           }
         }
-        if(fprogress != null) {
-          fprogress.incrementProcessed(logger);
+        if (fprogress != null) {
+          fprogress.incrementProcessed(LOG);
         }
       }
-      if(qprogress != null) {
-        qprogress.incrementProcessed(logger);
+      if (qprogress != null) {
+        qprogress.incrementProcessed(LOG);
       }
     }
-    if(qprogress != null) {
-      qprogress.ensureCompleted(logger);
+    if (qprogress != null) {
+      qprogress.ensureCompleted(LOG);
     }
-    if(fprogress != null) {
-      fprogress.setCompleted(logger);
+    if (fprogress != null) {
+      fprogress.setCompleted(LOG);
     }
 
-    WritableDataStore<KNNList<D>> knnLists = DataStoreUtil.makeStorage(ids, DataStoreFactory.HINT_STATIC, KNNList.class);
+    WritableDataStore<KNNResult<D>> knnLists = DataStoreUtil.makeStorage(ids, DataStoreFactory.HINT_STATIC, KNNResult.class);
     // FiniteProgress progress = logger.isVerbose() ? new
     // FiniteProgress(this.getClass().getName(), relation.size(), logger) :
     // null;
-    FiniteProgress pageprog = logger.isVerbose() ? new FiniteProgress("Number of processed data pages", ps_candidates.size(), logger) : null;
+    FiniteProgress pageprog = LOG.isVerbose() ? new FiniteProgress("Number of processed data pages", ps_candidates.size(), LOG) : null;
     // int processed = 0;
-    for(int i = 0; i < ps_candidates.size(); i++) {
+    for (int i = 0; i < ps_candidates.size(); i++) {
       N pr = index.getNode(ps_candidates.get(i));
       List<KNNHeap<D>> pr_heaps = heaps.get(i);
 
       // Finalize lists
-      for(int j = 0; j < pr.getNumEntries(); j++) {
+      for (int j = 0; j < pr.getNumEntries(); j++) {
         knnLists.put(((LeafEntry) pr.getEntry(j)).getDBID(), pr_heaps.get(j).toKNNList());
       }
       // Forget heaps and pq
@@ -244,29 +238,35 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
       // if(progress != null) {
       // progress.setProcessed(processed, logger);
       // }
-      if(pageprog != null) {
-        pageprog.incrementProcessed(logger);
+      if (pageprog != null) {
+        pageprog.incrementProcessed(LOG);
       }
     }
     // if(progress != null) {
     // progress.ensureCompleted(logger);
     // }
-    if(pageprog != null) {
-      pageprog.ensureCompleted(logger);
+    if (pageprog != null) {
+      pageprog.ensureCompleted(LOG);
     }
     return knnLists;
   }
 
-  private List<KNNHeap<D>> initHeaps(SpatialPrimitiveDistanceFunction<V, D> distFunction, final boolean doubleOptimize, N pr) {
-    List<KNNHeap<D>> pr_heaps;
+  /**
+   * Initialize the heaps.
+   * 
+   * @param distFunction Distance function
+   * @param pr Node to initialize for
+   * @return List of heaps
+   */
+  private List<KNNHeap<D>> initHeaps(SpatialPrimitiveDistanceFunction<V, D> distFunction, N pr) {
+    List<KNNHeap<D>> pr_heaps = new ArrayList<KNNHeap<D>>(pr.getNumEntries());
     // Create for each data object a knn heap
-    pr_heaps = new ArrayList<KNNHeap<D>>(pr.getNumEntries());
-    for(int j = 0; j < pr.getNumEntries(); j++) {
-      pr_heaps.add(new KNNHeap<D>(k, distFunction.getDistanceFactory().infiniteDistance()));
+    for (int j = 0; j < pr.getNumEntries(); j++) {
+      pr_heaps.add(KNNUtil.newHeap(distFunction, k));
     }
     // Self-join first, as this is expected to improve most and cannot be
     // pruned.
-    processDataPagesOptimize(distFunction, doubleOptimize, pr_heaps, null, pr, pr);
+    processDataPagesOptimize(distFunction, pr_heaps, null, pr, pr);
     return pr_heaps;
   }
 
@@ -275,27 +275,26 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
    * neighbors of pr in ps.
    * 
    * @param distFunction the distance to use
-   * @param doubleOptimize Flag whether to optimize for doubles.
    * @param pr the first data page
    * @param ps the second data page
    * @param pr_heaps the knn lists for each data object in pr
    * @param ps_heaps the knn lists for each data object in ps (if ps != pr)
    */
-  private void processDataPagesOptimize(SpatialPrimitiveDistanceFunction<V, D> distFunction, final boolean doubleOptimize, List<KNNHeap<D>> pr_heaps, List<KNNHeap<D>> ps_heaps, N pr, N ps) {
-    if(doubleOptimize) {
+  @SuppressWarnings("unchecked")
+  private void processDataPagesOptimize(SpatialPrimitiveDistanceFunction<V, D> distFunction, List<? extends KNNHeap<D>> pr_heaps, List<? extends KNNHeap<D>> ps_heaps, N pr, N ps) {
+    if (DistanceUtil.isDoubleDistanceFunction(distFunction)) {
       List<?> khp = (List<?>) pr_heaps;
       List<?> khs = (List<?>) ps_heaps;
-      processDataPagesDouble((SpatialPrimitiveDoubleDistanceFunction<? super V>) distFunction, pr, ps, (List<KNNHeap<DoubleDistance>>) khp, (List<KNNHeap<DoubleDistance>>) khs);
-    }
-    else {
-      for(int j = 0; j < ps.getNumEntries(); j++) {
+      processDataPagesDouble((SpatialPrimitiveDoubleDistanceFunction<? super V>) distFunction, pr, ps, (List<DoubleDistanceKNNHeap>) khp, (List<DoubleDistanceKNNHeap>) khs);
+    } else {
+      for (int j = 0; j < ps.getNumEntries(); j++) {
         final SpatialPointLeafEntry s_e = (SpatialPointLeafEntry) ps.getEntry(j);
         DBID s_id = s_e.getDBID();
-        for(int i = 0; i < pr.getNumEntries(); i++) {
+        for (int i = 0; i < pr.getNumEntries(); i++) {
           final SpatialPointLeafEntry r_e = (SpatialPointLeafEntry) pr.getEntry(i);
           D distance = distFunction.minDist(s_e, r_e);
           pr_heaps.get(i).add(distance, s_id);
-          if(pr != ps && ps_heaps != null) {
+          if (pr != ps && ps_heaps != null) {
             ps_heaps.get(j).add(distance, r_e.getDBID());
           }
         }
@@ -313,40 +312,42 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
    * @param pr_heaps the knn lists for each data object
    * @param ps_heaps the knn lists for each data object in ps
    */
-  private void processDataPagesDouble(SpatialPrimitiveDoubleDistanceFunction<? super V> df, N pr, N ps, List<KNNHeap<DoubleDistance>> pr_heaps, List<KNNHeap<DoubleDistance>> ps_heaps) {
+  private void processDataPagesDouble(SpatialPrimitiveDoubleDistanceFunction<? super V> df, N pr, N ps, List<DoubleDistanceKNNHeap> pr_heaps, List<DoubleDistanceKNNHeap> ps_heaps) {
     // Compare pairwise
-    for(int j = 0; j < ps.getNumEntries(); j++) {
+    for (int j = 0; j < ps.getNumEntries(); j++) {
       final SpatialPointLeafEntry s_e = (SpatialPointLeafEntry) ps.getEntry(j);
       DBID s_id = s_e.getDBID();
-      for(int i = 0; i < pr.getNumEntries(); i++) {
+      for (int i = 0; i < pr.getNumEntries(); i++) {
         final SpatialPointLeafEntry r_e = (SpatialPointLeafEntry) pr.getEntry(i);
         double distance = df.doubleMinDist(s_e, r_e);
-        pr_heaps.get(i).add(new DoubleDistanceResultPair(distance, s_id));
-        if(pr != ps && ps_heaps != null) {
-          ps_heaps.get(j).add(new DoubleDistanceResultPair(distance, r_e.getDBID()));
+        pr_heaps.get(i).add(distance, s_id);
+        if (pr != ps && ps_heaps != null) {
+          ps_heaps.get(j).add(distance, r_e.getDBID());
         }
       }
     }
   }
 
   /**
-   * Compute the maximum stop distance
+   * Compute the maximum stop distance.
    * 
-   * @param heaps
+   * @param heaps Heaps list
    * @return the k-nearest neighbor distance of pr in ps
    */
   private D computeStopDistance(List<KNNHeap<D>> heaps) {
     // Update pruning distance
     D pr_knn_distance = null;
-    for(KNNHeap<D> knnList : heaps) {
+    for (KNNHeap<D> knnList : heaps) {
       // set kNN distance of r
-      if(pr_knn_distance == null) {
+      if (pr_knn_distance == null) {
         pr_knn_distance = knnList.getKNNDistance();
-      }
-      else {
+      } else {
         pr_knn_distance = DistanceUtil.max(knnList.getKNNDistance(), pr_knn_distance);
       }
     }
+    if (pr_knn_distance == null) {
+      return getDistanceFunction().getDistanceFactory().infiniteDistance();
+    }
     return pr_knn_distance;
   }
 
@@ -357,29 +358,38 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
-   * Task in the processing queue
+   * Task in the processing queue.
    * 
    * @author Erich Schubert
    * 
    * @apiviz.exclude
    */
   private class Task implements Comparable<Task> {
+    /**
+     * Minimum distance.
+     */
     final D mindist;
 
+    /**
+     * First offset.
+     */
     final int i;
 
+    /**
+     * Second offset.
+     */
     final int j;
 
     /**
      * Constructor.
      * 
-     * @param mindist
-     * @param i
-     * @param j
+     * @param mindist Minimum distance
+     * @param i First offset
+     * @param j Second offset
      */
     public Task(D mindist, int i, int j) {
       super();
@@ -401,7 +411,10 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>, D extends Distance<D>, N extends SpatialNode<N, E>, E extends SpatialEntry> extends AbstractPrimitiveDistanceBasedAlgorithm.Parameterizer<V, D> {
+  public static class Parameterizer<V extends NumberVector<?>, D extends Distance<D>, N extends SpatialNode<N, E>, E extends SpatialEntry> extends AbstractPrimitiveDistanceBasedAlgorithm.Parameterizer<V, D> {
+    /**
+     * K parameter.
+     */
     protected int k;
 
     @Override
@@ -409,7 +422,7 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
       super.makeOptions(config);
       IntParameter kP = new IntParameter(K_ID, 1);
       kP.addConstraint(new GreaterConstraint(0));
-      if(config.grab(kP)) {
+      if (config.grab(kP)) {
         k = kP.getValue();
       }
     }
@@ -419,4 +432,4 @@ public class KNNJoin<V extends NumberVector<V, ?>, D extends Distance<D>, N exte
       return new KNNJoin<V, D, N, E>(distanceFunction, k);
     }
   }
-}
\ No newline at end of file
+}
