@@ -48,49 +48,19 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * @param <R> the result we return
  * @param <V> the type of FeatureVector handled by this Algorithm
  */
-public abstract class AbstractProjectedClustering<R extends Clustering<?>, V extends NumberVector<V, ?>> extends AbstractAlgorithm<R> implements ClusteringAlgorithm<R> {
+public abstract class AbstractProjectedClustering<R extends Clustering<?>, V extends NumberVector<?>> extends AbstractAlgorithm<R> implements ClusteringAlgorithm<R> {
   /**
-   * Parameter to specify the number of clusters to find, must be an integer
-   * greater than 0.
-   * <p>
-   * Key: {@code -projectedclustering.k}
-   * </p>
-   */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("projectedclustering.k", "The number of clusters to find.");
-
-  /**
-   * Parameter to specify the multiplier for the initial number of seeds, must
-   * be an integer greater than 0.
-   * <p>
-   * Default value: {@code 30}
-   * </p>
-   * <p>
-   * Key: {@code -projectedclustering.k_i}
-   * </p>
-   */
-  public static final OptionID K_I_ID = OptionID.getOrCreateOptionID("projectedclustering.k_i", "The multiplier for the initial number of seeds.");
-
-  /**
-   * Parameter to specify the dimensionality of the clusters to find, must be an
-   * integer greater than 0.
-   * <p>
-   * Key: {@code -projectedclustering.l}
-   * </p>
-   */
-  public static final OptionID L_ID = OptionID.getOrCreateOptionID("projectedclustering.l", "The dimensionality of the clusters to find.");
-
-  /**
-   * Holds the value of {@link #K_ID}.
+   * Holds the value of {@link Parameterizer#K_ID}.
    */
   protected int k;
 
   /**
-   * Holds the value of {@link #K_I_ID}.
+   * Holds the value of {@link Parameterizer#K_I_ID}.
    */
   protected int k_i;
 
   /**
-   * Holds the value of {@link #L_ID}.
+   * Holds the value of {@link Parameterizer#L_ID}.
    */
   protected int l;
 
@@ -138,7 +108,37 @@ public abstract class AbstractProjectedClustering<R extends Clustering<?>, V ext
    * 
    * @apiviz.exclude
    */
-  public static abstract class Parameterizer extends AbstractParameterizer {
+  public abstract static class Parameterizer extends AbstractParameterizer {
+    /**
+     * Parameter to specify the number of clusters to find, must be an integer
+     * greater than 0.
+     * <p>
+     * Key: {@code -projectedclustering.k}
+     * </p>
+     */
+    public static final OptionID K_ID = new OptionID("projectedclustering.k", "The number of clusters to find.");
+
+    /**
+     * Parameter to specify the multiplier for the initial number of seeds, must
+     * be an integer greater than 0.
+     * <p>
+     * Default value: {@code 30}
+     * </p>
+     * <p>
+     * Key: {@code -projectedclustering.k_i}
+     * </p>
+     */
+    public static final OptionID K_I_ID = new OptionID("projectedclustering.k_i", "The multiplier for the initial number of seeds.");
+
+    /**
+     * Parameter to specify the dimensionality of the clusters to find, must be
+     * an integer greater than 0.
+     * <p>
+     * Key: {@code -projectedclustering.l}
+     * </p>
+     */
+    public static final OptionID L_ID = new OptionID("projectedclustering.l", "The dimensionality of the clusters to find.");
+
     protected int k;
 
     protected int k_i;
@@ -151,8 +151,9 @@ public abstract class AbstractProjectedClustering<R extends Clustering<?>, V ext
      * @param config Parameterization
      */
     protected void configK(Parameterization config) {
-      IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(0));
-      if(config.grab(kP)) {
+      IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(kP)) {
         k = kP.getValue();
       }
     }
@@ -163,8 +164,9 @@ public abstract class AbstractProjectedClustering<R extends Clustering<?>, V ext
      * @param config Parameterization
      */
     protected void configKI(Parameterization config) {
-      IntParameter k_iP = new IntParameter(K_I_ID, new GreaterConstraint(0), 30);
-      if(config.grab(k_iP)) {
+      IntParameter k_iP = new IntParameter(K_I_ID, 30);
+      k_iP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(k_iP)) {
         k_i = k_iP.getValue();
       }
     }
@@ -175,10 +177,11 @@ public abstract class AbstractProjectedClustering<R extends Clustering<?>, V ext
      * @param config Parameterization
      */
     protected void configL(Parameterization config) {
-      IntParameter lP = new IntParameter(L_ID, new GreaterConstraint(0));
-      if(config.grab(lP)) {
+      IntParameter lP = new IntParameter(L_ID);
+      lP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(lP)) {
         l = lP.getValue();
       }
     }
   }
-}
\ No newline at end of file
+}
