@@ -34,14 +34,16 @@ import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.HashSetModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
@@ -77,13 +79,13 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(DBSCAN.class);
+  private static final Logging LOG = Logging.getLogger(DBSCAN.class);
 
   /**
    * Parameter to specify the maximum radius of the neighborhood to be
    * considered, must be suitable to the distance function specified.
    */
-  public static final OptionID EPSILON_ID = OptionID.getOrCreateOptionID("dbscan.epsilon", "The maximum radius of the neighborhood to be considered.");
+  public static final OptionID EPSILON_ID = new OptionID("dbscan.epsilon", "The maximum radius of the neighborhood to be considered.");
 
   /**
    * Holds the value of {@link #EPSILON_ID}.
@@ -94,7 +96,7 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
    * Parameter to specify the threshold for minimum number of points in the
    * epsilon-neighborhood of a point, must be an integer greater than 0.
    */
-  public static final OptionID MINPTS_ID = OptionID.getOrCreateOptionID("dbscan.minpts", "Threshold for minimum number of points in the epsilon-neighborhood of a point.");
+  public static final OptionID MINPTS_ID = new OptionID("dbscan.minpts", "Threshold for minimum number of points in the epsilon-neighborhood of a point.");
 
   /**
    * Holds the value of {@link #MINPTS_ID}.
@@ -136,40 +138,36 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
     RangeQuery<O, D> rangeQuery = QueryUtil.getRangeQuery(relation, getDistanceFunction());
     final int size = relation.size();
 
-    FiniteProgress objprog = logger.isVerbose() ? new FiniteProgress("Processing objects", size, logger) : null;
-    IndefiniteProgress clusprog = logger.isVerbose() ? new IndefiniteProgress("Number of clusters", logger) : null;
+    FiniteProgress objprog = LOG.isVerbose() ? new FiniteProgress("Processing objects", size, LOG) : null;
+    IndefiniteProgress clusprog = LOG.isVerbose() ? new IndefiniteProgress("Number of clusters", LOG) : null;
     resultList = new ArrayList<ModifiableDBIDs>();
     noise = DBIDUtil.newHashSet();
     processedIDs = DBIDUtil.newHashSet(size);
-    if(size >= minpts) {
+    if(size < minpts) {
+      // The can't be any clusters
+      noise.addDBIDs(relation.getDBIDs());
+      objprog.setProcessed(noise.size(), LOG);
+    }
+    else {
       for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         if(!processedIDs.contains(iditer)) {
-          expandCluster(relation, rangeQuery, iditer.getDBID(), objprog, clusprog);
+          expandCluster(relation, rangeQuery, iditer, objprog, clusprog);
         }
         if(objprog != null && clusprog != null) {
-          objprog.setProcessed(processedIDs.size(), logger);
-          clusprog.setProcessed(resultList.size(), logger);
+          objprog.setProcessed(processedIDs.size(), LOG);
+          clusprog.setProcessed(resultList.size(), LOG);
         }
         if(processedIDs.size() == size) {
           break;
         }
       }
     }
-    else {
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        noise.add(iditer);
-        if(objprog != null && clusprog != null) {
-          objprog.setProcessed(noise.size(), logger);
-          clusprog.setProcessed(resultList.size(), logger);
-        }
-      }
-    }
     // Finish progress logging
     if(objprog != null) {
-      objprog.ensureCompleted(logger);
+      objprog.ensureCompleted(LOG);
     }
     if(clusprog != null) {
-      clusprog.setCompleted(logger);
+      clusprog.setCompleted(LOG);
     }
 
     Clustering<Model> result = new Clustering<Model>("DBSCAN Clustering", "dbscan-clustering");
@@ -194,40 +192,43 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
    * @param startObjectID potential seed of a new potential cluster
    * @param objprog the progress object for logging the current status
    */
-  protected void expandCluster(Relation<O> relation, RangeQuery<O, D> rangeQuery, DBID startObjectID, FiniteProgress objprog, IndefiniteProgress clusprog) {
-    List<DistanceResultPair<D>> seeds = rangeQuery.getRangeForDBID(startObjectID, epsilon);
+  protected void expandCluster(Relation<O> relation, RangeQuery<O, D> rangeQuery, DBIDRef startObjectID, FiniteProgress objprog, IndefiniteProgress clusprog) {
+    DistanceDBIDResult<D> neighbors = rangeQuery.getRangeForDBID(startObjectID, epsilon);
 
     // startObject is no core-object
-    if(seeds.size() < minpts) {
+    if(neighbors.size() < minpts) {
       noise.add(startObjectID);
       processedIDs.add(startObjectID);
       if(objprog != null && clusprog != null) {
-        objprog.setProcessed(processedIDs.size(), logger);
-        clusprog.setProcessed(resultList.size(), logger);
+        objprog.setProcessed(processedIDs.size(), LOG);
+        clusprog.setProcessed(resultList.size(), LOG);
       }
       return;
     }
 
     // try to expand the cluster
+    HashSetModifiableDBIDs seeds = DBIDUtil.newHashSet();
     ModifiableDBIDs currentCluster = DBIDUtil.newArray();
-    for(DistanceResultPair<D> seed : seeds) {
+    for(DBIDIter seed = neighbors.iter(); seed.valid(); seed.advance()) {
       if(!processedIDs.contains(seed)) {
         currentCluster.add(seed);
         processedIDs.add(seed);
+        seeds.add(seed);
       }
       else if(noise.contains(seed)) {
         currentCluster.add(seed);
         noise.remove(seed);
       }
     }
-    seeds.remove(0);
+    seeds.remove(startObjectID);
 
     while(seeds.size() > 0) {
-      DistanceResultPair<D> o = seeds.remove(0);
-      List<DistanceResultPair<D>> neighborhood = rangeQuery.getRangeForDBID(o, epsilon);
+      DBIDMIter o = seeds.iter();
+      DistanceDBIDResult<D> neighborhood = rangeQuery.getRangeForDBID(o, epsilon);
+      o.remove();
 
       if(neighborhood.size() >= minpts) {
-        for(DistanceResultPair<D> neighbor : neighborhood) {
+        for(DBIDIter neighbor = neighborhood.iter(); neighbor.valid(); neighbor.advance()) {
           boolean inNoise = noise.contains(neighbor);
           boolean unclassified = !processedIDs.contains(neighbor);
           if(inNoise || unclassified) {
@@ -248,9 +249,9 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
       }
 
       if(objprog != null && clusprog != null) {
-        objprog.setProcessed(processedIDs.size(), logger);
+        objprog.setProcessed(processedIDs.size(), LOG);
         int numClusters = currentCluster.size() > minpts ? resultList.size() + 1 : resultList.size();
-        clusprog.setProcessed(numClusters, logger);
+        clusprog.setProcessed(numClusters, LOG);
       }
     }
     if(currentCluster.size() >= minpts) {
@@ -270,7 +271,7 @@ public class DBSCAN<O, D extends Distance<D>> extends AbstractDistanceBasedAlgor
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
