@@ -27,6 +27,7 @@ import java.util.ArrayList;
 import java.util.List;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans.KMeans;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans.KMeansInitialization;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.kmeans.RandomlyGeneratedInitialMeans;
 import de.lmu.ifi.dbs.elki.data.Cluster;
@@ -44,12 +45,12 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.MathUtil;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
@@ -86,11 +87,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 @Title("EM-Clustering: Clustering by Expectation Maximization")
 @Description("Provides k Gaussian mixtures maximizing the probability of the given data")
 @Reference(authors = "A. P. Dempster, N. M. Laird, D. B. Rubin", title = "Maximum Likelihood from Incomplete Data via the EM algorithm", booktitle = "Journal of the Royal Statistical Society, Series B, 39(1), 1977, pp. 1-31", url = "http://www.jstor.org/stable/2984875")
-public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<EMModel<V>>> implements ClusteringAlgorithm<Clustering<EMModel<V>>> {
+public class EM<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<EMModel<V>>> implements ClusteringAlgorithm<Clustering<EMModel<V>>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(EM.class);
+  private static final Logging LOG = Logging.getLogger(EM.class);
 
   /**
    * Small value to increment diagonally of a matrix in order to avoid
@@ -102,7 +103,7 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    * Parameter to specify the number of clusters to find, must be an integer
    * greater than 0.
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("em.k", "The number of clusters to find.");
+  public static final OptionID K_ID = new OptionID("em.k", "The number of clusters to find.");
 
   /**
    * Holds the value of {@link #K_ID}.
@@ -113,12 +114,12 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    * Parameter to specify the termination criterion for maximization of E(M):
    * E(M) - E(M') < em.delta, must be a double equal to or greater than 0.
    */
-  public static final OptionID DELTA_ID = OptionID.getOrCreateOptionID("em.delta", "The termination criterion for maximization of E(M): " + "E(M) - E(M') < em.delta");
+  public static final OptionID DELTA_ID = new OptionID("em.delta", "The termination criterion for maximization of E(M): " + "E(M) - E(M') < em.delta");
 
   /**
    * Parameter to specify the initialization method
    */
-  public static final OptionID INIT_ID = OptionID.getOrCreateOptionID("kmeans.initialization", "Method to choose the initial means.");
+  public static final OptionID INIT_ID = new OptionID("kmeans.initialization", "Method to choose the initial means.");
 
   private static final double MIN_LOGLIKELIHOOD = -100000;
 
@@ -137,18 +138,25 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    */
   private KMeansInitialization<V> initializer;
 
+  /**
+   * Maximum number of iterations to allow
+   */
+  private int maxiter;
+
   /**
    * Constructor.
    * 
    * @param k k parameter
    * @param delta delta parameter
    * @param initializer Class to choose the initial means
+   * @param maxiter Maximum number of iterations
    */
-  public EM(int k, double delta, KMeansInitialization<V> initializer) {
+  public EM(int k, double delta, KMeansInitialization<V> initializer, int maxiter) {
     super();
     this.k = k;
     this.delta = delta;
     this.initializer = initializer;
+    this.maxiter = maxiter;
   }
 
   /**
@@ -164,15 +172,15 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    * @return Result
    */
   public Clustering<EMModel<V>> run(Database database, Relation<V> relation) {
-    if(relation.size() == 0) {
+    if (relation.size() == 0) {
       throw new IllegalArgumentException("database empty: must contain elements");
     }
     // initial models
-    if(logger.isVerbose()) {
-      logger.verbose("initializing " + k + " models");
+    if (LOG.isVerbose()) {
+      LOG.verbose("initializing " + k + " models");
     }
     List<Vector> means = new ArrayList<Vector>();
-    for(NumberVector<?, ?> nv : initializer.chooseInitialMeans(relation, k, EuclideanDistanceFunction.STATIC)) {
+    for (NumberVector<?> nv : initializer.chooseInitialMeans(relation, k, EuclideanDistanceFunction.STATIC)) {
       means.add(nv.getColumnVector());
     }
     List<Matrix> covarianceMatrices = new ArrayList<Matrix>(k);
@@ -182,113 +190,117 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
     probClusterIGivenX = DataStoreUtil.makeStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_SORTED, double[].class);
 
     final int dimensionality = means.get(0).getDimensionality();
-    for(int i = 0; i < k; i++) {
+    for (int i = 0; i < k; i++) {
       Matrix m = Matrix.identity(dimensionality, dimensionality);
       covarianceMatrices.add(m);
       normDistrFactor[i] = 1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * m.det());
       invCovMatr.add(m.inverse());
       clusterWeights[i] = 1.0 / k;
-      if(logger.isDebuggingFinest()) {
-        StringBuffer msg = new StringBuffer();
+      if (LOG.isDebuggingFinest()) {
+        StringBuilder msg = new StringBuilder();
         msg.append(" model ").append(i).append(":\n");
-        msg.append(" mean:    ").append(means.get(i)).append("\n");
-        msg.append(" m:\n").append(FormatUtil.format(m, "        ")).append("\n");
-        msg.append(" m.det(): ").append(m.det()).append("\n");
-        msg.append(" cluster weight: ").append(clusterWeights[i]).append("\n");
-        msg.append(" normDistFact:   ").append(normDistrFactor[i]).append("\n");
-        logger.debugFine(msg.toString());
+        msg.append(" mean:    ").append(means.get(i)).append('\n');
+        msg.append(" m:\n").append(FormatUtil.format(m, "        ")).append('\n');
+        msg.append(" m.det(): ").append(m.det()).append('\n');
+        msg.append(" cluster weight: ").append(clusterWeights[i]).append('\n');
+        msg.append(" normDistFact:   ").append(normDistrFactor[i]).append('\n');
+        LOG.debugFine(msg.toString());
       }
     }
     double emNew = assignProbabilitiesToInstances(relation, normDistrFactor, means, invCovMatr, clusterWeights, probClusterIGivenX);
 
     // iteration unless no change
-    if(logger.isVerbose()) {
-      logger.verbose("iterating EM");
+    if (LOG.isVerbose()) {
+      LOG.verbose("iterating EM");
+    }
+    if (LOG.isVerbose()) {
+      LOG.verbose("iteration " + 0 + " - expectation value: " + emNew);
     }
 
     double em;
-    int it = 0;
-    do {
-      it++;
-      if(logger.isVerbose()) {
-        logger.verbose("iteration " + it + " - expectation value: " + emNew);
-      }
+    for (int it = 1; it <= maxiter || maxiter < 0; it++) {
       em = emNew;
 
       // recompute models
       List<Vector> meanSums = new ArrayList<Vector>(k);
       double[] sumOfClusterProbabilities = new double[k];
 
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         clusterWeights[i] = 0.0;
         meanSums.add(new Vector(dimensionality));
         covarianceMatrices.set(i, Matrix.zeroMatrix(dimensionality));
       }
 
       // weights and means
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         double[] clusterProbabilities = probClusterIGivenX.get(iditer);
 
-        for(int i = 0; i < k; i++) {
+        for (int i = 0; i < k; i++) {
           sumOfClusterProbabilities[i] += clusterProbabilities[i];
           Vector summand = relation.get(iditer).getColumnVector().timesEquals(clusterProbabilities[i]);
           meanSums.get(i).plusEquals(summand);
         }
       }
       final int n = relation.size();
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         clusterWeights[i] = sumOfClusterProbabilities[i] / n;
         Vector newMean = meanSums.get(i).timesEquals(1 / sumOfClusterProbabilities[i]);
         means.set(i, newMean);
       }
       // covariance matrices
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         double[] clusterProbabilities = probClusterIGivenX.get(iditer);
         Vector instance = relation.get(iditer).getColumnVector();
-        for(int i = 0; i < k; i++) {
+        for (int i = 0; i < k; i++) {
           Vector difference = instance.minus(means.get(i));
           covarianceMatrices.get(i).plusEquals(difference.timesTranspose(difference).timesEquals(clusterProbabilities[i]));
         }
       }
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         covarianceMatrices.set(i, covarianceMatrices.get(i).times(1 / sumOfClusterProbabilities[i]).cheatToAvoidSingularity(SINGULARITY_CHEAT));
       }
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         normDistrFactor[i] = 1.0 / Math.sqrt(Math.pow(MathUtil.TWOPI, dimensionality) * covarianceMatrices.get(i).det());
         invCovMatr.set(i, covarianceMatrices.get(i).inverse());
       }
       // reassign probabilities
       emNew = assignProbabilitiesToInstances(relation, normDistrFactor, means, invCovMatr, clusterWeights, probClusterIGivenX);
-    }
-    while(Math.abs(em - emNew) > delta);
 
-    if(logger.isVerbose()) {
-      logger.verbose("assigning clusters");
+      if (LOG.isVerbose()) {
+        LOG.verbose("iteration " + it + " - expectation value: " + emNew);
+      }
+      if (Math.abs(em - emNew) <= delta) {
+        break;
+      }
+    }
+
+    if (LOG.isVerbose()) {
+      LOG.verbose("assigning clusters");
     }
 
     // fill result with clusters and models
     List<ModifiableDBIDs> hardClusters = new ArrayList<ModifiableDBIDs>(k);
-    for(int i = 0; i < k; i++) {
+    for (int i = 0; i < k; i++) {
       hardClusters.add(DBIDUtil.newHashSet());
     }
 
     // provide a hard clustering
-    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+    for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       double[] clusterProbabilities = probClusterIGivenX.get(iditer);
       int maxIndex = 0;
       double currentMax = 0.0;
-      for(int i = 0; i < k; i++) {
-        if(clusterProbabilities[i] > currentMax) {
+      for (int i = 0; i < k; i++) {
+        if (clusterProbabilities[i] > currentMax) {
           maxIndex = i;
           currentMax = clusterProbabilities[i];
         }
       }
       hardClusters.get(maxIndex).add(iditer);
     }
-    final V factory = DatabaseUtil.assumeVectorField(relation).getFactory();
+    final NumberVector.Factory<V, ?> factory = RelationUtil.getNumberVectorFactory(relation);
     Clustering<EMModel<V>> result = new Clustering<EMModel<V>>("EM Clustering", "em-clustering");
     // provide models within the result
-    for(int i = 0; i < k; i++) {
+    for (int i = 0; i < k; i++) {
       // TODO: re-do labeling.
       // SimpleClassLabel label = new SimpleClassLabel();
       // label.init(result.canonicalClusterLabel(i));
@@ -316,37 +328,36 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
   protected double assignProbabilitiesToInstances(Relation<V> database, double[] normDistrFactor, List<Vector> means, List<Matrix> invCovMatr, double[] clusterWeights, WritableDataStore<double[]> probClusterIGivenX) {
     double emSum = 0.0;
 
-    for(DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
+    for (DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
       Vector x = database.get(iditer).getColumnVector();
       double[] probabilities = new double[k];
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         Vector difference = x.minus(means.get(i));
         double rowTimesCovTimesCol = difference.transposeTimesTimes(invCovMatr.get(i), difference);
         double power = rowTimesCovTimesCol / 2.0;
         double prob = normDistrFactor[i] * Math.exp(-power);
-        if(logger.isDebuggingFinest()) {
-          logger.debugFinest(" difference vector= ( " + difference.toString() + " )\n" + " difference:\n" + FormatUtil.format(difference, "    ") + "\n" + " rowTimesCovTimesCol:\n" + rowTimesCovTimesCol + "\n" + " power= " + power + "\n" + " prob=" + prob + "\n" + " inv cov matrix: \n" + FormatUtil.format(invCovMatr.get(i), "     "));
+        if (LOG.isDebuggingFinest()) {
+          LOG.debugFinest(" difference vector= ( " + difference.toString() + " )\n" + " difference:\n" + FormatUtil.format(difference, "    ") + "\n" + " rowTimesCovTimesCol:\n" + rowTimesCovTimesCol + "\n" + " power= " + power + "\n" + " prob=" + prob + "\n" + " inv cov matrix: \n" + FormatUtil.format(invCovMatr.get(i), "     "));
         }
         probabilities[i] = prob;
       }
       double priorProbability = 0.0;
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         priorProbability += probabilities[i] * clusterWeights[i];
       }
       double logP = Math.max(Math.log(priorProbability), MIN_LOGLIKELIHOOD);
-      if(!Double.isNaN(logP)) {
+      if (!Double.isNaN(logP)) {
         emSum += logP;
       }
 
       double[] clusterProbabilities = new double[k];
-      for(int i = 0; i < k; i++) {
+      for (int i = 0; i < k; i++) {
         assert (priorProbability >= 0.0);
         assert (clusterWeights[i] >= 0.0);
         // do not divide by zero!
-        if(priorProbability == 0.0) {
+        if (priorProbability == 0.0) {
           clusterProbabilities[i] = 0.0;
-        }
-        else {
+        } else {
           clusterProbabilities[i] = probabilities[i] / priorProbability * clusterWeights[i];
         }
       }
@@ -373,7 +384,7 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -383,35 +394,46 @@ public class EM<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clusteri
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     protected int k;
 
     protected double delta;
 
     protected KMeansInitialization<V> initializer;
 
+    protected int maxiter = -1;
+
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(0));
-      if(config.grab(kP)) {
+      IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(kP)) {
         k = kP.getValue();
       }
 
       ObjectParameter<KMeansInitialization<V>> initialP = new ObjectParameter<KMeansInitialization<V>>(INIT_ID, KMeansInitialization.class, RandomlyGeneratedInitialMeans.class);
-      if(config.grab(initialP)) {
+      if (config.grab(initialP)) {
         initializer = initialP.instantiateClass(config);
       }
 
-      DoubleParameter deltaP = new DoubleParameter(DELTA_ID, new GreaterEqualConstraint(0.0), 0.0);
-      if(config.grab(deltaP)) {
+      DoubleParameter deltaP = new DoubleParameter(DELTA_ID, 0.0);
+      deltaP.addConstraint(new GreaterEqualConstraint(0.0));
+      if (config.grab(deltaP)) {
         delta = deltaP.getValue();
       }
+
+      IntParameter maxiterP = new IntParameter(KMeans.MAXITER_ID);
+      maxiterP.addConstraint(new GreaterEqualConstraint(0));
+      maxiterP.setOptional(true);
+      if (config.grab(maxiterP)) {
+        maxiter = maxiterP.getValue();
+      }
     }
 
     @Override
     protected EM<V> makeInstance() {
-      return new EM<V>(k, delta, initializer);
+      return new EM<V>(k, delta, initializer, maxiter);
     }
   }
-}
\ No newline at end of file
+}
