@@ -50,7 +50,8 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.HierarchyHashmapLi
 import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.ModifiableHierarchy;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ClassParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
@@ -60,7 +61,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
  * 
  * @author Erich Schubert
  * 
- * @apiviz.uses OPTICSTypeAlgorithm oneway
+ * @apiviz.composedOf OPTICSTypeAlgorithm oneway
  * @apiviz.uses ClusterOrderResult oneway
  * @apiviz.has SteepAreaResult
  * 
@@ -70,17 +71,17 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(OPTICSXi.class);
+  private static final Logging LOG = Logging.getLogger(OPTICSXi.class);
 
   /**
    * Parameter to specify the actual OPTICS algorithm to use.
    */
-  public static final OptionID XIALG_ID = OptionID.getOrCreateOptionID("opticsxi.algorithm", "The actual OPTICS-type algorithm to use.");
+  public static final OptionID XIALG_ID = new OptionID("opticsxi.algorithm", "The actual OPTICS-type algorithm to use.");
 
   /**
    * Parameter to specify the steepness threshold.
    */
-  public static final OptionID XI_ID = OptionID.getOrCreateOptionID("opticsxi.xi", "Threshold for the steepness requirement.");
+  public static final OptionID XI_ID = new OptionID("opticsxi.xi", "Threshold for the steepness requirement.");
 
   /**
    * The actual algorithm we use.
@@ -109,12 +110,12 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
     ClusterOrderResult<N> opticsresult = optics.run(database);
 
     if(!NumberDistance.class.isInstance(optics.getDistanceFactory())) {
-      logger.verbose("Xi cluster extraction only supported for number distances!");
+      LOG.verbose("Xi cluster extraction only supported for number distances!");
       return null;
     }
 
-    if(logger.isVerbose()) {
-      logger.verbose("Extracting clusters with Xi: " + xi);
+    if(LOG.isVerbose()) {
+      LOG.verbose("Extracting clusters with Xi: " + xi);
     }
     return extractClusters(opticsresult, relation, 1.0 - xi, optics.getMinPts());
   }
@@ -135,7 +136,7 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
     // TODO: make it configurable to keep this list; this is mostly useful for
     // visualization
     List<SteepArea> salist = new ArrayList<SteepArea>();
-    List<SteepDownArea> sdaset = new java.util.Vector<SteepDownArea>();
+    List<SteepDownArea> sdaset = new ArrayList<SteepDownArea>();
     ModifiableHierarchy<Cluster<OPTICSModel>> hier = new HierarchyHashmapList<Cluster<OPTICSModel>>();
     HashSet<Cluster<OPTICSModel>> curclusters = new HashSet<Cluster<OPTICSModel>>();
     HashSetModifiableDBIDs unclaimedids = DBIDUtil.newHashSet(relation.getDBIDs());
@@ -175,8 +176,8 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
           }
           mib = clusterOrder.get(endsteep).getReachability().doubleValue();
           final SteepDownArea sda = new SteepDownArea(startsteep, endsteep, startval, 0);
-          if(logger.isDebuggingFinest()) {
-            logger.debugFinest("Xi " + sda.toString());
+          if(LOG.isDebuggingFinest()) {
+            LOG.debugFinest("Xi " + sda.toString());
           }
           sdaset.add(sda);
           if(salist != null) {
@@ -220,8 +221,8 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
               }
             }
             sua = new SteepUpArea(startsteep, endsteep, esuccr);
-            if(logger.isDebuggingFinest()) {
-              logger.debugFinest("Xi " + sua.toString());
+            if(LOG.isDebuggingFinest()) {
+              LOG.debugFinest("Xi " + sua.toString());
             }
             if(salist != null) {
               salist.add(sua);
@@ -280,8 +281,8 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
                 dbids.add(dbid);
               }
             }
-            if(logger.isDebuggingFine()) {
-              logger.debugFine("Found cluster with " + dbids.size() + " new objects, length " + (cstart - cend + 1));
+            if(LOG.isDebuggingFine()) {
+              LOG.debugFine("Found cluster with " + dbids.size() + " new objects, length " + (cstart - cend + 1));
             }
             OPTICSModel model = new OPTICSModel(cstart, cend);
             Cluster<OPTICSModel> cluster = new Cluster<OPTICSModel>("Cluster_" + cstart + "_" + cend, dbids, model, hier);
@@ -362,7 +363,7 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -656,9 +657,10 @@ public class OPTICSXi<N extends NumberDistance<N, ?>> extends AbstractAlgorithm<
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       DoubleParameter xiP = new DoubleParameter(XI_ID);
-      xiP.addConstraint(new IntervalConstraint(0.0, IntervalConstraint.IntervalBoundary.CLOSE, 1.0, IntervalConstraint.IntervalBoundary.OPEN));
+      xiP.addConstraint(new GreaterEqualConstraint(0.0));
+      xiP.addConstraint(new LessConstraint(1.0));
       if(config.grab(xiP)) {
-        xi = xiP.getValue();
+        xi = xiP.doubleValue();
       }
 
       ClassParameter<OPTICSTypeAlgorithm<D>> opticsP = new ClassParameter<OPTICSTypeAlgorithm<D>>(XIALG_ID, OPTICSTypeAlgorithm.class, OPTICS.class);
