@@ -23,19 +23,20 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.List;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractAlgorithm;
 import de.lmu.ifi.dbs.elki.algorithm.DependencyDerivator;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.ClusteringAlgorithm;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.correlation.cash.CASHInterval;
 import de.lmu.ifi.dbs.elki.algorithm.clustering.correlation.cash.CASHIntervalSplit;
+import de.lmu.ifi.dbs.elki.algorithm.clustering.correlation.cash.ParameterizationFunction;
 import de.lmu.ifi.dbs.elki.data.Cluster;
 import de.lmu.ifi.dbs.elki.data.Clustering;
 import de.lmu.ifi.dbs.elki.data.DoubleVector;
 import de.lmu.ifi.dbs.elki.data.HyperBoundingBox;
-import de.lmu.ifi.dbs.elki.data.ParameterizationFunction;
+import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.model.ClusterModel;
 import de.lmu.ifi.dbs.elki.data.model.CorrelationAnalysisSolution;
 import de.lmu.ifi.dbs.elki.data.model.LinearEquationModel;
@@ -65,16 +66,13 @@ import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.FirstNEigenPairFilter;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredRunner;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.Heap;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.IntegerPriorityObject;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
-import de.lmu.ifi.dbs.elki.utilities.exceptions.UnableToComplyException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.ParameterException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
@@ -86,10 +84,6 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * Provides the CASH algorithm, an subspace clustering algorithm based on the
  * Hough transform.
  * 
- * <b>Note:</b> CASH requires explicitly setting the input vector type to
- * {@link ParameterizationFunction}:
- * (in the MiniGui, set option: parser.vector-type ParameterizationFunction).
- * 
  * <p>
  * Reference: E. Achtert, C. Böhm, J. David, P. Kröger, A. Zimek: Robust
  * clustering in arbitrarily oriented subspaces. <br>
@@ -101,16 +95,18 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * @apiviz.has CASHInterval
  * @apiviz.uses ParameterizationFunction
  * @apiviz.has LinearEquationModel
+ * 
+ * @param <V> Vector type
  */
 // todo elke hierarchy (later)
 @Title("CASH: Robust clustering in arbitrarily oriented subspaces")
 @Description("Subspace clustering algorithm based on the Hough transform.")
 @Reference(authors = "E. Achtert, C. Böhm, J. David, P. Kröger, A. Zimek", title = "Robust clustering in arbitraily oriented subspaces", booktitle = "Proc. 8th SIAM Int. Conf. on Data Mining (SDM'08), Atlanta, GA, 2008", url = "http://www.siam.org/proceedings/datamining/2008/dm08_69_AchtertBoehmDavidKroegerZimek.pdf")
-public class CASH extends AbstractAlgorithm<Clustering<Model>> implements ClusteringAlgorithm<Clustering<Model>> {
+public class CASH<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<Model>> implements ClusteringAlgorithm<Clustering<Model>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(CASH.class);
+  private static final Logging LOG = Logging.getLogger(CASH.class);
 
   /**
    * Parameter to specify the threshold for minimum number of points in a
@@ -119,7 +115,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Key: {@code -cash.minpts}
    * </p>
    */
-  public static final OptionID MINPTS_ID = OptionID.getOrCreateOptionID("cash.minpts", "Threshold for minimum number of points in a cluster.");
+  public static final OptionID MINPTS_ID = new OptionID("cash.minpts", "Threshold for minimum number of points in a cluster.");
 
   /**
    * Parameter to specify the maximum level for splitting the hypercube, must be
@@ -128,7 +124,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Key: {@code -cash.maxlevel}
    * </p>
    */
-  public static final OptionID MAXLEVEL_ID = OptionID.getOrCreateOptionID("cash.maxlevel", "The maximum level for splitting the hypercube.");
+  public static final OptionID MAXLEVEL_ID = new OptionID("cash.maxlevel", "The maximum level for splitting the hypercube.");
 
   /**
    * Parameter to specify the minimum dimensionality of the subspaces to be
@@ -140,7 +136,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Key: {@code -cash.mindim}
    * </p>
    */
-  public static final OptionID MINDIM_ID = OptionID.getOrCreateOptionID("cash.mindim", "The minimum dimensionality of the subspaces to be found.");
+  public static final OptionID MINDIM_ID = new OptionID("cash.mindim", "The minimum dimensionality of the subspaces to be found.");
 
   /**
    * Parameter to specify the maximum jitter for distance values, must be a
@@ -149,7 +145,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Key: {@code -cash.jitter}
    * </p>
    */
-  public static final OptionID JITTER_ID = OptionID.getOrCreateOptionID("cash.jitter", "The maximum jitter for distance values.");
+  public static final OptionID JITTER_ID = new OptionID("cash.jitter", "The maximum jitter for distance values.");
 
   /**
    * Flag to indicate that an adjustment of the applied heuristic for choosing
@@ -158,7 +154,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Key: {@code -cash.adjust}
    * </p>
    */
-  public static final OptionID ADJUST_ID = OptionID.getOrCreateOptionID("cash.adjust", "Flag to indicate that an adjustment of the applied heuristic for choosing an interval " + "is performed after an interval is selected.");
+  public static final OptionID ADJUST_ID = new OptionID("cash.adjust", "Flag to indicate that an adjustment of the applied heuristic for choosing an interval " + "is performed after an interval is selected.");
 
   /**
    * Holds the value of {@link #MINPTS_ID}.
@@ -196,7 +192,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
   private ModifiableDBIDs processedIDs;
 
   /**
-   * The entire database
+   * The entire relation.
    */
   private Relation<ParameterizationFunction> fulldatabase;
 
@@ -222,52 +218,61 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * Run CASH on the relation.
    * 
    * @param database Database
-   * @param relation Relation
+   * @param vrel Relation
    * @return Clustering result
    */
-  public Clustering<Model> run(Database database, Relation<ParameterizationFunction> relation) {
-    this.fulldatabase = relation;
-    if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer();
-      msg.append("DB size: ").append(relation.size());
+  public Clustering<Model> run(Database database, Relation<V> vrel) {
+    this.fulldatabase = preprocess(database, vrel);
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder();
+      msg.append("DB size: ").append(fulldatabase.size());
       msg.append("\nmin Dim: ").append(minDim);
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
-    try {
-      processedIDs = DBIDUtil.newHashSet(relation.size());
-      noiseDim = DatabaseUtil.dimensionality(relation);
+    processedIDs = DBIDUtil.newHashSet(fulldatabase.size());
+    noiseDim = dimensionality(fulldatabase);
 
-      FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("CASH Clustering", relation.size(), logger) : null;
-      Clustering<Model> result = doRun(relation, progress);
-      if(progress != null) {
-        progress.ensureCompleted(logger);
-      }
+    FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("CASH Clustering", fulldatabase.size(), LOG) : null;
+    Clustering<Model> result = doRun(fulldatabase, progress);
+    if (progress != null) {
+      progress.ensureCompleted(LOG);
+    }
 
-      if(logger.isVerbose()) {
-        StringBuffer msg = new StringBuffer();
-        for(Cluster<Model> c : result.getAllClusters()) {
-          if(c.getModel() instanceof LinearEquationModel) {
-            LinearEquationModel s = (LinearEquationModel) c.getModel();
-            msg.append("\n Cluster: Dim: " + s.getLes().subspacedim() + " size: " + c.size());
-          }
-          else {
-            msg.append("\n Cluster: " + c.getModel().getClass().getName() + " size: " + c.size());
-          }
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder();
+      for (Cluster<Model> c : result.getAllClusters()) {
+        if (c.getModel() instanceof LinearEquationModel) {
+          LinearEquationModel s = (LinearEquationModel) c.getModel();
+          msg.append("\n Cluster: Dim: " + s.getLes().subspacedim() + " size: " + c.size());
+        } else {
+          msg.append("\n Cluster: " + c.getModel().getClass().getName() + " size: " + c.size());
         }
-        logger.verbose(msg.toString());
       }
-      return result;
+      LOG.verbose(msg.toString());
     }
-    catch(UnableToComplyException e) {
-      throw new IllegalStateException(e);
-    }
-    catch(ParameterException e) {
-      throw new IllegalStateException(e);
-    }
-    catch(NonNumericFeaturesException e) {
-      throw new IllegalStateException(e);
+    return result;
+  }
+
+  /**
+   * Preprocess the dataset, precomputing the parameterization functions.
+   * 
+   * @param db Database
+   * @param vrel Vector relation
+   * @return Preprocessed relation
+   */
+  private Relation<ParameterizationFunction> preprocess(Database db, Relation<V> vrel) {
+    DBIDs ids = vrel.getDBIDs();
+    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<ParameterizationFunction>(ParameterizationFunction.class);
+    MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<ParameterizationFunction>(db, type, ids);
+
+    // Project
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      ParameterizationFunction f = new ParameterizationFunction(vrel.get(iter));
+      prep.set(iter, f);
     }
+
+    return prep;
   }
 
   /**
@@ -277,69 +282,62 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * @param relation the Relation to run the CASH algorithm on
    * @param progress the progress object for verbose messages
    * @return a mapping of subspace dimensionalities to clusters
-   * @throws UnableToComplyException if an error according to the database
-   *         occurs
-   * @throws ParameterException if the parameter setting is wrong
-   * @throws NonNumericFeaturesException if non numeric feature vectors are used
    */
-  private Clustering<Model> doRun(Relation<ParameterizationFunction> relation, FiniteProgress progress) throws UnableToComplyException, ParameterException, NonNumericFeaturesException {
+  private Clustering<Model> doRun(Relation<ParameterizationFunction> relation, FiniteProgress progress) {
     Clustering<Model> res = new Clustering<Model>("CASH clustering", "cash-clustering");
 
-    final int dim = DatabaseUtil.dimensionality(relation);
+    final int dim = dimensionality(relation);
 
     // init heap
     Heap<IntegerPriorityObject<CASHInterval>> heap = new Heap<IntegerPriorityObject<CASHInterval>>();
     ModifiableDBIDs noiseIDs = DBIDUtil.newHashSet(relation.getDBIDs());
     initHeap(heap, relation, dim, noiseIDs);
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
+    if (LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
       msg.append("\nXXXX dim ").append(dim);
       msg.append("\nXXXX database.size ").append(relation.size());
       msg.append("\nXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX");
-      logger.debugFine(msg.toString());
-    }
-    else if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer();
+      LOG.debugFine(msg.toString());
+    } else if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("XXXX dim ").append(dim).append(" database.size ").append(relation.size());
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // get the ''best'' d-dimensional intervals at max level
-    while(!heap.isEmpty()) {
+    while (!heap.isEmpty()) {
       CASHInterval interval = determineNextIntervalAtMaxLevel(heap);
-      if(logger.isDebugging()) {
-        logger.debugFine("next interval in dim " + dim + ": " + interval);
-      }
-      else if(logger.isVerbose()) {
-        logger.verbose("next interval in dim " + dim + ": " + interval);
+      if (LOG.isDebugging()) {
+        LOG.debugFine("next interval in dim " + dim + ": " + interval);
+      } else if (LOG.isVerbose()) {
+        LOG.verbose("next interval in dim " + dim + ": " + interval);
       }
 
       // only noise left
-      if(interval == null) {
+      if (interval == null) {
         break;
       }
 
       // do a dim-1 dimensional run
       ModifiableDBIDs clusterIDs = DBIDUtil.newHashSet();
-      if(dim > minDim + 1) {
+      if (dim > minDim + 1) {
         ModifiableDBIDs ids;
         Matrix basis_dim_minus_1;
-        if(adjust) {
+        if (adjust) {
           ids = DBIDUtil.newHashSet();
           basis_dim_minus_1 = runDerivator(relation, dim, interval, ids);
-        }
-        else {
+        } else {
           ids = interval.getIDs();
           basis_dim_minus_1 = determineBasis(SpatialUtil.centroid(interval));
         }
 
-        if(ids.size() != 0) {
+        if (ids.size() != 0) {
           MaterializedRelation<ParameterizationFunction> db = buildDB(dim, basis_dim_minus_1, ids, relation);
           // add result of dim-1 to this result
           Clustering<Model> res_dim_minus_1 = doRun(db, progress);
-          for(Cluster<Model> cluster : res_dim_minus_1.getAllClusters()) {
+          for (Cluster<Model> cluster : res_dim_minus_1.getAllClusters()) {
             res.addCluster(cluster);
             noiseIDs.removeDBIDs(cluster.getIDs());
             clusterIDs.addDBIDs(cluster.getIDs());
@@ -358,28 +356,31 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
       }
 
       // Rebuild heap
-      List<IntegerPriorityObject<CASHInterval>> heapVector = heap.toSortedArrayList();
-      for(IntegerPriorityObject<CASHInterval> pair : heapVector) {
+      ArrayList<IntegerPriorityObject<CASHInterval>> heapVector = new ArrayList<IntegerPriorityObject<CASHInterval>>(heap.size());
+      for (IntegerPriorityObject<CASHInterval> obj : heap) {
+        heapVector.add(obj);
+      }
+      heap.clear();
+      for (IntegerPriorityObject<CASHInterval> pair : heapVector) {
         CASHInterval currentInterval = pair.getObject();
         currentInterval.removeIDs(clusterIDs);
-        if(currentInterval.getIDs().size() >= minPts) {
+        if (currentInterval.getIDs().size() >= minPts) {
           heap.add(new IntegerPriorityObject<CASHInterval>(currentInterval.priority(), currentInterval));
         }
       }
 
-      if(progress != null) {
-        progress.setProcessed(processedIDs.size(), logger);
+      if (progress != null) {
+        progress.setProcessed(processedIDs.size(), LOG);
       }
     }
 
     // put noise to clusters
-    if(!noiseIDs.isEmpty()) {
-      if(dim == noiseDim) {
+    if (!noiseIDs.isEmpty()) {
+      if (dim == noiseDim) {
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, ClusterModel.CLUSTER);
         res.addCluster(c);
         processedIDs.addDBIDs(noiseIDs);
-      }
-      else if(noiseIDs.size() >= minPts) {
+      } else if (noiseIDs.size() >= minPts) {
         LinearEquationSystem les = runDerivator(fulldatabase, dim - 1, noiseIDs);
         Cluster<Model> c = new Cluster<Model>(noiseIDs, true, new LinearEquationModel(les));
         res.addCluster(c);
@@ -387,28 +388,37 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
       }
     }
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
+    if (LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("noise fuer dim ").append(dim).append(": ").append(noiseIDs.size());
 
-      for(Cluster<Model> c : res.getAllClusters()) {
-        if(c.getModel() instanceof LinearEquationModel) {
+      for (Cluster<Model> c : res.getAllClusters()) {
+        if (c.getModel() instanceof LinearEquationModel) {
           LinearEquationModel s = (LinearEquationModel) c.getModel();
           msg.append("\n Cluster: Dim: " + s.getLes().subspacedim() + " size: " + c.size());
-        }
-        else {
+        } else {
           msg.append("\n Cluster: " + c.getModel().getClass().getName() + " size: " + c.size());
         }
       }
-      logger.debugFine(msg.toString());
+      LOG.debugFine(msg.toString());
     }
 
-    if(progress != null) {
-      progress.setProcessed(processedIDs.size(), logger);
+    if (progress != null) {
+      progress.setProcessed(processedIDs.size(), LOG);
     }
     return res;
   }
 
+  /**
+   * Get the dimensionality of a vector field.
+   * 
+   * @param relation Relation
+   * @return Dimensionality
+   */
+  private static int dimensionality(Relation<ParameterizationFunction> relation) {
+    return relation.get(relation.iterDBIDs()).getDimensionality();
+  }
+
   /**
    * Initializes the heap with the root intervals.
    * 
@@ -431,21 +441,20 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     double[] d_mins = new double[numDIntervals];
     double[] d_maxs = new double[numDIntervals];
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
+    if (LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("d_min ").append(d_min);
       msg.append("\nd_max ").append(d_max);
       msg.append("\nnumDIntervals ").append(numDIntervals);
       msg.append("\ndIntervalSize ").append(dIntervalSize);
-      logger.debugFine(msg.toString());
-    }
-    else if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer();
+      LOG.debugFine(msg.toString());
+    } else if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("d_min ").append(d_min);
       msg.append("\nd_max ").append(d_max);
       msg.append("\nnumDIntervals ").append(numDIntervals);
       msg.append("\ndIntervalSize ").append(dIntervalSize);
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // alpha intervals
@@ -453,33 +462,31 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     double[] alphaMax = new double[dim - 1];
     Arrays.fill(alphaMax, Math.PI);
 
-    for(int i = 0; i < numDIntervals; i++) {
-      if(i == 0) {
+    for (int i = 0; i < numDIntervals; i++) {
+      if (i == 0) {
         d_mins[i] = d_min;
-      }
-      else {
+      } else {
         d_mins[i] = d_maxs[i - 1];
       }
 
-      if(i < numDIntervals - 1) {
+      if (i < numDIntervals - 1) {
         d_maxs[i] = d_mins[i] + dIntervalSize;
-      }
-      else {
+      } else {
         d_maxs[i] = d_max - d_mins[i];
       }
 
       HyperBoundingBox alphaInterval = new HyperBoundingBox(alphaMin, alphaMax);
       ModifiableDBIDs intervalIDs = split.determineIDs(ids, alphaInterval, d_mins[i], d_maxs[i]);
-      if(intervalIDs != null && intervalIDs.size() >= minPts) {
-        CASHInterval rootInterval = new CASHInterval(alphaMin, alphaMax, split, intervalIDs, 0, 0, d_mins[i], d_maxs[i]);
+      if (intervalIDs != null && intervalIDs.size() >= minPts) {
+        CASHInterval rootInterval = new CASHInterval(alphaMin, alphaMax, split, intervalIDs, -1, 0, d_mins[i], d_maxs[i]);
         heap.add(new IntegerPriorityObject<CASHInterval>(rootInterval.priority(), rootInterval));
       }
     }
 
-    if(logger.isDebuggingFiner()) {
-      StringBuffer msg = new StringBuffer();
+    if (LOG.isDebuggingFiner()) {
+      StringBuilder msg = new StringBuilder();
       msg.append("heap.size ").append(heap.size());
-      logger.debugFiner(msg.toString());
+      LOG.debugFiner(msg.toString());
     }
   }
 
@@ -493,23 +500,21 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * @param relation the database storing the parameterization functions
    * @return a dim-1 dimensional database where the objects are projected into
    *         the specified subspace
-   * @throws UnableToComplyException if an error according to the database
-   *         occurs
    */
-  private MaterializedRelation<ParameterizationFunction> buildDB(int dim, Matrix basis, DBIDs ids, Relation<ParameterizationFunction> relation) throws UnableToComplyException {
+  private MaterializedRelation<ParameterizationFunction> buildDB(int dim, Matrix basis, DBIDs ids, Relation<ParameterizationFunction> relation) {
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    VectorFieldTypeInformation<ParameterizationFunction> type = VectorFieldTypeInformation.get(ParameterizationFunction.class, basis.getColumnDimensionality());
+    SimpleTypeInformation<ParameterizationFunction> type = new SimpleTypeInformation<ParameterizationFunction>(ParameterizationFunction.class);
     MaterializedRelation<ParameterizationFunction> prep = new MaterializedRelation<ParameterizationFunction>(proxy, type, ids);
     proxy.addRelation(prep);
-    
+
     // Project
-    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
       ParameterizationFunction f = project(basis, relation.get(iter));
       prep.set(iter, f);
     }
 
-    if(logger.isDebugging()) {
-      logger.debugFine("db fuer dim " + (dim - 1) + ": " + ids.size());
+    if (LOG.isDebugging()) {
+      LOG.debugFine("db fuer dim " + (dim - 1) + ": " + ids.size());
     }
 
     return prep;
@@ -527,7 +532,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     // Matrix m = new Matrix(new
     // double[][]{f.getPointCoordinates()}).times(basis);
     Matrix m = f.getColumnVector().transposeTimes(basis);
-    ParameterizationFunction f_t = new ParameterizationFunction(m.getColumnPackedCopy());
+    ParameterizationFunction f_t = new ParameterizationFunction(new DoubleVector(m.getColumnPackedCopy()));
     return f_t;
   }
 
@@ -540,7 +545,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    */
   private Matrix determineBasis(double[] alpha) {
     double[] nn = new double[alpha.length + 1];
-    for(int i = 0; i < nn.length; i++) {
+    for (int i = 0; i < nn.length; i++) {
       double alpha_i = i == alpha.length ? 0 : alpha[i];
       nn[i] = sinusProduct(0, i, alpha) * StrictMath.cos(alpha_i);
     }
@@ -560,7 +565,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    */
   private double sinusProduct(int start, int end, double[] alpha) {
     double result = 1;
-    for(int j = start; j < end; j++) {
+    for (int j = start; j < end; j++) {
       result *= StrictMath.sin(alpha[j]);
     }
     return result;
@@ -576,8 +581,8 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
   private CASHInterval determineNextIntervalAtMaxLevel(Heap<IntegerPriorityObject<CASHInterval>> heap) {
     CASHInterval next = doDetermineNextIntervalAtMaxLevel(heap);
     // noise path was chosen
-    while(next == null) {
-      if(heap.isEmpty()) {
+    while (next == null) {
+      if (heap.isEmpty()) {
         return null;
       }
       next = doDetermineNextIntervalAtMaxLevel(heap);
@@ -596,48 +601,45 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
   private CASHInterval doDetermineNextIntervalAtMaxLevel(Heap<IntegerPriorityObject<CASHInterval>> heap) {
     CASHInterval interval = heap.poll().getObject();
     int dim = interval.getDimensionality();
-    while(true) {
+    while (true) {
       // max level is reached
-      if(interval.getLevel() >= maxLevel && interval.getMaxSplitDimension() == dim) {
+      if (interval.getLevel() >= maxLevel && interval.getMaxSplitDimension() == (dim - 1)) {
         return interval;
       }
 
-      if(heap.size() % 10000 == 0 && logger.isVerbose()) {
-        logger.verbose("heap size " + heap.size());
+      if (heap.size() % 10000 == 0 && LOG.isVerbose()) {
+        LOG.verbose("heap size " + heap.size());
       }
 
-      if(heap.size() >= 40000) {
-        logger.warning("Heap size > 40.000!!!");
+      if (heap.size() >= 40000) {
+        LOG.warning("Heap size > 40.000!!!");
         heap.clear();
         return null;
       }
 
-      if(logger.isDebuggingFiner()) {
-        logger.debugFiner("split " + interval.toString() + " " + interval.getLevel() + "-" + interval.getMaxSplitDimension());
+      if (LOG.isDebuggingFiner()) {
+        LOG.debugFiner("split " + interval.toString() + " " + interval.getLevel() + "-" + interval.getMaxSplitDimension());
       }
       interval.split();
 
       // noise
-      if(!interval.hasChildren()) {
+      if (!interval.hasChildren()) {
         return null;
       }
 
       CASHInterval bestInterval;
-      if(interval.getLeftChild() != null && interval.getRightChild() != null) {
+      if (interval.getLeftChild() != null && interval.getRightChild() != null) {
         int comp = interval.getLeftChild().compareTo(interval.getRightChild());
-        if(comp < 0) {
+        if (comp < 0) {
           bestInterval = interval.getRightChild();
           heap.add(new IntegerPriorityObject<CASHInterval>(interval.getLeftChild().priority(), interval.getLeftChild()));
-        }
-        else {
+        } else {
           bestInterval = interval.getLeftChild();
           heap.add(new IntegerPriorityObject<CASHInterval>(interval.getRightChild().priority(), interval.getRightChild()));
         }
-      }
-      else if(interval.getLeftChild() == null) {
+      } else if (interval.getLeftChild() == null) {
         bestInterval = interval.getRightChild();
-      }
-      else {
+      } else {
         bestInterval = interval.getLeftChild();
       }
 
@@ -662,7 +664,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
 
     double d_min = Double.POSITIVE_INFINITY;
     double d_max = Double.NEGATIVE_INFINITY;
-    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+    for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       ParameterizationFunction f = relation.get(iditer);
       HyperBoundingBox minMax = f.determineAlphaMinMax(box);
       double f_min = f.function(SpatialUtil.getMin(minMax));
@@ -684,11 +686,8 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * @param dim the dimensionality of the database
    * @param ids an empty set to assign the ids
    * @return a basis of the found subspace
-   * @throws UnableToComplyException if an error according to the database
-   *         occurs
-   * @throws ParameterException if the parameter setting is wrong
    */
-  private Matrix runDerivator(Relation<ParameterizationFunction> relation, int dim, CASHInterval interval, ModifiableDBIDs ids) throws UnableToComplyException, ParameterException {
+  private Matrix runDerivator(Relation<ParameterizationFunction> relation, int dim, CASHInterval interval, ModifiableDBIDs ids) {
     // build database for derivator
     Database derivatorDB = buildDerivatorDB(relation, interval);
 
@@ -705,14 +704,14 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     Matrix weightMatrix = model.getSimilarityMatrix();
     DoubleVector centroid = new DoubleVector(model.getCentroid());
     DistanceQuery<DoubleVector, DoubleDistance> df = QueryUtil.getDistanceQuery(derivatorDB, new WeightedDistanceFunction(weightMatrix));
-    DoubleDistance eps = df.getDistanceFactory().parseString("0.25");
+    DoubleDistance eps = new DoubleDistance(0.25);
 
     ids.addDBIDs(interval.getIDs());
     // Search for nearby vectors in original database
-    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+    for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       DoubleVector v = new DoubleVector(relation.get(iditer).getColumnVector().getArrayRef());
       DoubleDistance d = df.distance(v, centroid);
-      if(d.compareTo(eps) < 0) {
+      if (d.compareTo(eps) < 0) {
         ids.add(iditer);
       }
     }
@@ -729,25 +728,23 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * @param interval the interval to build the database from
    * @return a database for the derivator consisting of the ids in the specified
    *         interval
-   * @throws UnableToComplyException if an error according to the database
-   *         occurs
    */
-  private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, CASHInterval interval) throws UnableToComplyException {
+  private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, CASHInterval interval) {
     DBIDs ids = interval.getIDs();
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    int dim = DatabaseUtil.dimensionality(relation);
-    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dim, new DoubleVector(new double[dim]));
+    int dim = dimensionality(relation);
+    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dim);
     MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
-    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
       DoubleVector v = new DoubleVector(relation.get(iter).getColumnVector().getArrayRef());
       prep.set(iter, v);
     }
 
-    if(logger.isDebugging()) {
-      logger.debugFine("db fuer derivator : " + prep.size());
+    if (LOG.isDebugging()) {
+      LOG.debugFine("db fuer derivator : " + prep.size());
     }
 
     return proxy;
@@ -778,11 +775,7 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
       CorrelationAnalysisSolution<DoubleVector> model = derivator.run(derivatorDB);
       LinearEquationSystem les = model.getNormalizedLinearEquationSystem(null);
       return les;
-    }
-    catch(UnableToComplyException e) {
-      throw new IllegalStateException("Initialization of the database for the derivator failed: " + e);
-    }
-    catch(NonNumericFeaturesException e) {
+    } catch (NonNumericFeaturesException e) {
       throw new IllegalStateException("Error during normalization" + e);
     }
   }
@@ -795,18 +788,16 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
    * @param ids the ids to build the database from
    * @return a database for the derivator consisting of the ids in the specified
    *         interval
-   * @throws UnableToComplyException if initialization of the database is not
-   *         possible
    */
-  private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, DBIDs ids) throws UnableToComplyException {
+  private Database buildDerivatorDB(Relation<ParameterizationFunction> relation, DBIDs ids) {
     ProxyDatabase proxy = new ProxyDatabase(ids);
-    int dim = DatabaseUtil.dimensionality(relation);
-    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, dim, new DoubleVector(new double[dim]));
+    int dim = dimensionality(relation);
+    SimpleTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dim);
     MaterializedRelation<DoubleVector> prep = new MaterializedRelation<DoubleVector>(proxy, type, ids);
     proxy.addRelation(prep);
 
     // Project
-    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
       DoubleVector v = new DoubleVector(relation.get(iter).getColumnVector().getArrayRef());
       prep.set(iter, v);
     }
@@ -816,12 +807,12 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
 
   @Override
   public TypeInformation[] getInputTypeRestriction() {
-    return TypeUtil.array(VectorFieldTypeInformation.get(ParameterizationFunction.class));
+    return TypeUtil.array(TypeUtil.NUMBER_VECTOR_FIELD);
   }
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -845,31 +836,35 @@ public class CASH extends AbstractAlgorithm<Clustering<Model>> implements Cluste
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      IntParameter minptsP = new IntParameter(MINPTS_ID, new GreaterConstraint(0));
-      if(config.grab(minptsP)) {
+      IntParameter minptsP = new IntParameter(MINPTS_ID);
+      minptsP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(minptsP)) {
         minpts = minptsP.getValue();
       }
-      IntParameter maxlevelP = new IntParameter(MAXLEVEL_ID, new GreaterConstraint(0));
-      if(config.grab(maxlevelP)) {
+      IntParameter maxlevelP = new IntParameter(MAXLEVEL_ID);
+      maxlevelP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(maxlevelP)) {
         maxlevel = maxlevelP.getValue();
       }
-      IntParameter mindimP = new IntParameter(MINDIM_ID, new GreaterConstraint(0), 1);
-      if(config.grab(mindimP)) {
+      IntParameter mindimP = new IntParameter(MINDIM_ID, 1);
+      mindimP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(mindimP)) {
         mindim = mindimP.getValue();
       }
-      DoubleParameter jitterP = new DoubleParameter(JITTER_ID, new GreaterConstraint(0));
-      if(config.grab(jitterP)) {
+      DoubleParameter jitterP = new DoubleParameter(JITTER_ID);
+      jitterP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(jitterP)) {
         jitter = jitterP.getValue();
       }
       Flag adjustF = new Flag(ADJUST_ID);
-      if(config.grab(adjustF)) {
+      if (config.grab(adjustF)) {
         adjust = adjustF.getValue();
       }
     }
 
     @Override
-    protected CASH makeInstance() {
-      return new CASH(minpts, maxlevel, mindim, jitter, adjust);
+    protected CASH<NumberVector<?>> makeInstance() {
+      return new CASH<NumberVector<?>>(minpts, maxlevel, mindim, jitter, adjust);
     }
   }
-}
\ No newline at end of file
+}
