@@ -40,13 +40,13 @@ import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.ProxyDatabase;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.FilteredLocalPCABasedDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.IndexBasedDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.LocallyWeightedDistanceFunction;
@@ -56,7 +56,6 @@ import de.lmu.ifi.dbs.elki.index.preprocessed.LocalProjectionIndex;
 import de.lmu.ifi.dbs.elki.index.preprocessed.LocalProjectionIndex.Factory;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -92,11 +91,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("COPAC: COrrelation PArtition Clustering")
 @Description("Partitions a database according to the correlation dimension of its objects and performs " + "a clustering algorithm over the partitions.")
 @Reference(authors = "E. Achtert, C. Böhm, H.-P. Kriegel, P. Kröger P., A. Zimek", title = "Robust, Complete, and Efficient Correlation Clustering", booktitle = "Proc. 7th SIAM International Conference on Data Mining (SDM'07), Minneapolis, MN, 2007", url = "http://www.siam.org/proceedings/datamining/2007/dm07_037achtert.pdf")
-public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends AbstractAlgorithm<Clustering<Model>> implements ClusteringAlgorithm<Clustering<Model>> {
+public class COPAC<V extends NumberVector<?>, D extends Distance<D>> extends AbstractAlgorithm<Clustering<Model>> implements ClusteringAlgorithm<Clustering<Model>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(COPAC.class);
+  private static final Logging LOG = Logging.getLogger(COPAC.class);
 
   /**
    * Parameter to specify the local PCA preprocessor to derive partition
@@ -106,7 +105,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
    * Key: {@code -copac.preprocessor}
    * </p>
    */
-  public static final OptionID PREPROCESSOR_ID = OptionID.getOrCreateOptionID("copac.preprocessor", "Local PCA Preprocessor to derive partition criterion.");
+  public static final OptionID PREPROCESSOR_ID = new OptionID("copac.preprocessor", "Local PCA Preprocessor to derive partition criterion.");
 
   /**
    * Parameter to specify the distance function to use inside the partitions
@@ -120,7 +119,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
    * Key: {@code -copac.partitionDistance}
    * </p>
    */
-  public static final OptionID PARTITION_DISTANCE_ID = OptionID.getOrCreateOptionID("copac.partitionDistance", "Distance to use for the inner algorithms.");
+  public static final OptionID PARTITION_DISTANCE_ID = new OptionID("copac.partitionDistance", "Distance to use for the inner algorithms.");
 
   /**
    * Parameter to specify the clustering algorithm to apply to each partition,
@@ -130,7 +129,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
    * Key: {@code -copac.partitionAlgorithm}
    * </p>
    */
-  public static final OptionID PARTITION_ALGORITHM_ID = OptionID.getOrCreateOptionID("copac.partitionAlgorithm", "Clustering algorithm to apply to each partition.");
+  public static final OptionID PARTITION_ALGORITHM_ID = new OptionID("copac.partitionAlgorithm", "Clustering algorithm to apply to each partition.");
 
   /**
    * Holds the instance of the preprocessed distance function
@@ -178,8 +177,8 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
    */
   @SuppressWarnings("unchecked")
   public Clustering<Model> run(Relation<V> relation) {
-    if(logger.isVerbose()) {
-      logger.verbose("Running COPAC on db size = " + relation.size() + " with dimensionality = " + DatabaseUtil.dimensionality(relation));
+    if(LOG.isVerbose()) {
+      LOG.verbose("Running COPAC on db size = " + relation.size() + " with dimensionality = " + RelationUtil.dimensionality(relation));
     }
 
     partitionDistanceQuery = (FilteredLocalPCABasedDistanceFunction.Instance<V, LocalProjectionIndex<V, ?>, D>) partitionDistanceFunction.instantiate(relation);
@@ -187,7 +186,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
 
     // partitioning
     Map<Integer, ModifiableDBIDs> partitionMap = new HashMap<Integer, ModifiableDBIDs>();
-    FiniteProgress partitionProgress = logger.isVerbose() ? new FiniteProgress("Partitioning", relation.size(), logger) : null;
+    FiniteProgress partitionProgress = LOG.isVerbose() ? new FiniteProgress("Partitioning", relation.size(), LOG) : null;
     int processed = 1;
 
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
@@ -199,17 +198,17 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
 
       partitionMap.get(corrdim).add(iditer);
       if(partitionProgress != null) {
-        partitionProgress.setProcessed(processed++, logger);
+        partitionProgress.setProcessed(processed++, LOG);
       }
     }
 
     if(partitionProgress != null) {
-      partitionProgress.ensureCompleted(logger);
+      partitionProgress.ensureCompleted(LOG);
     }
-    if(logger.isVerbose()) {
+    if(LOG.isVerbose()) {
       for(Integer corrDim : partitionMap.keySet()) {
         ModifiableDBIDs list = partitionMap.get(corrDim);
-        logger.verbose("Partition [corrDim = " + corrDim + "]: " + list.size() + " objects.");
+        LOG.verbose("Partition [corrDim = " + corrDim + "]: " + list.size() + " objects.");
       }
     }
 
@@ -236,7 +235,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
     // TODO: use an extra finite progress for the partitions?
     for(Entry<Integer, DBIDs> pair : partitionMap.entrySet()) {
       // noise partition
-      if(pair.getKey() == DatabaseUtil.dimensionality(relation)) {
+      if(pair.getKey() == RelationUtil.dimensionality(relation)) {
         // Make a Noise cluster
         result.addCluster(new Cluster<Model>(pair.getValue(), true, ClusterModel.CLUSTER));
       }
@@ -245,8 +244,8 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
         ProxyDatabase proxy = new ProxyDatabase(partids, relation);
         
         ClusteringAlgorithm<Clustering<Model>> partitionAlgorithm = getPartitionAlgorithm(query);
-        if(logger.isVerbose()) {
-          logger.verbose("Running " + partitionAlgorithm.getClass().getName() + " on partition [corrDim = " + pair.getKey() + "]...");
+        if(LOG.isVerbose()) {
+          LOG.verbose("Running " + partitionAlgorithm.getClass().getName() + " on partition [corrDim = " + pair.getKey() + "]...");
         }
         Clustering<Model> p = partitionAlgorithm.run(proxy);
         // Re-Wrap resulting Clusters as DimensionModel clusters.
@@ -295,7 +294,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -305,7 +304,7 @@ public class COPAC<V extends NumberVector<V, ?>, D extends Distance<D>> extends
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>, D extends Distance<D>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>, D extends Distance<D>> extends AbstractParameterizer {
     protected LocalProjectionIndex.Factory<V, ?> indexI = null;
 
     protected FilteredLocalPCABasedDistanceFunction<V, ?, D> pdistI = null;
