@@ -44,6 +44,7 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.ProxyDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.correlation.ERiCDistanceFunction;
@@ -52,11 +53,11 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.IntegerDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.FirstNEigenPairFilter;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredResult;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredRunner;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -78,11 +79,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
  * 
  * @author Elke Achtert
  * 
- * @apiviz.uses COPAC
- * @apiviz.uses DBSCAN
- * @apiviz.uses ERiCDistanceFunction
- * @apiviz.uses FirstNEigenPairFilter
- * @apiviz.uses PCAFilteredRunner
+ * @apiviz.composedOf COPAC
+ * @apiviz.composedOf DBSCAN
+ * @apiviz.composedOf ERiCDistanceFunction
+ * @apiviz.composedOf FirstNEigenPairFilter
+ * @apiviz.composedOf PCAFilteredRunner
  * @apiviz.has CorrelationModel
  * 
  * @param <V> the type of NumberVector handled by this Algorithm
@@ -91,11 +92,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
 @Title("ERiC: Exploring Relationships among Correlation Clusters")
 @Description("Performs the DBSCAN algorithm on the data using a special distance function taking into account correlations among attributes and builds " + "a hierarchy that allows multiple inheritance from the correlation clustering result.")
 @Reference(authors = "E. Achtert, C. Böhm, H.-P. Kriegel, P. Kröger, and A. Zimek", title = "On Exploring Complex Relationships of Correlation Clusters", booktitle = "Proc. 19th International Conference on Scientific and Statistical Database Management (SSDBM 2007), Banff, Canada, 2007", url = "http://dx.doi.org/10.1109/SSDBM.2007.21")
-public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<CorrelationModel<V>>> implements ClusteringAlgorithm<Clustering<CorrelationModel<V>>> {
+public class ERiC<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<CorrelationModel<V>>> implements ClusteringAlgorithm<Clustering<CorrelationModel<V>>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ERiC.class);
+  private static final Logging LOG = Logging.getLogger(ERiC.class);
 
   /**
    * The COPAC clustering algorithm.
@@ -119,13 +120,13 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @return Clustering result
    */
   public Clustering<CorrelationModel<V>> run(Relation<V> relation) {
-    final int dimensionality = DatabaseUtil.dimensionality(relation);
+    final int dimensionality = RelationUtil.dimensionality(relation);
 
-    StepProgress stepprog = logger.isVerbose() ? new StepProgress(3) : null;
+    StepProgress stepprog = LOG.isVerbose() ? new StepProgress(3) : null;
 
     // run COPAC
     if(stepprog != null) {
-      stepprog.beginStep(1, "Preprocessing local correlation dimensionalities and partitioning data", logger);
+      stepprog.beginStep(1, "Preprocessing local correlation dimensionalities and partitioning data", LOG);
     }
     Clustering<Model> copacResult = copacAlgorithm.run(relation);
 
@@ -133,11 +134,11 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
 
     // extract correlation clusters
     if(stepprog != null) {
-      stepprog.beginStep(2, "Extract correlation clusters", logger);
+      stepprog.beginStep(2, "Extract correlation clusters", LOG);
     }
     SortedMap<Integer, List<Cluster<CorrelationModel<V>>>> clusterMap = extractCorrelationClusters(copacResult, relation, dimensionality);
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer("Step 2: Extract correlation clusters...");
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder("Step 2: Extract correlation clusters...");
       for(Integer corrDim : clusterMap.keySet()) {
         List<Cluster<CorrelationModel<V>>> correlationClusters = clusterMap.get(corrDim);
         msg.append("\n\ncorrDim ").append(corrDim);
@@ -149,23 +150,23 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
           // "  ids " + cluster.getIDs().size());
         }
       }
-      logger.debugFine(msg.toString());
+      LOG.debugFine(msg.toString());
     }
-    if(logger.isVerbose()) {
+    if(LOG.isVerbose()) {
       int clusters = 0;
       for(List<Cluster<CorrelationModel<V>>> correlationClusters : clusterMap.values()) {
         clusters += correlationClusters.size();
       }
-      logger.verbose(clusters + " clusters extracted.");
+      LOG.verbose(clusters + " clusters extracted.");
     }
 
     // build hierarchy
     if(stepprog != null) {
-      stepprog.beginStep(3, "Building hierarchy", logger);
+      stepprog.beginStep(3, "Building hierarchy", LOG);
     }
     buildHierarchy(clusterMap, query);
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer("Step 3: Build hierarchy");
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder("Step 3: Build hierarchy");
       for(Integer corrDim : clusterMap.keySet()) {
         List<Cluster<CorrelationModel<V>>> correlationClusters = clusterMap.get(corrDim);
         for(Cluster<CorrelationModel<V>> cluster : correlationClusters) {
@@ -179,10 +180,10 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
           }
         }
       }
-      logger.debugFine(msg.toString());
+      LOG.debugFine(msg.toString());
     }
     if(stepprog != null) {
-      stepprog.setCompleted(logger);
+      stepprog.setCompleted(LOG);
     }
 
     Clustering<CorrelationModel<V>> result = new Clustering<CorrelationModel<V>>("ERiC clustering", "eric-clustering");
@@ -221,7 +222,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
         Class<PCAFilteredRunner<V>> cls = ClassGenericsUtil.uglyCastIntoSubclass(PCAFilteredRunner.class);
         PCAFilteredRunner<V> pca = parameters.tryInstantiate(cls);
         for(ParameterException e : parameters.getErrors()) {
-          logger.warning("Error in internal parameterization: " + e.getMessage());
+          LOG.warning("Error in internal parameterization: " + e.getMessage());
         }
 
         // get cluster list for this dimension.
@@ -233,7 +234,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
 
         PCAFilteredResult pcares = pca.processIds(group, database);
 
-        V centroid = DatabaseUtil.centroid(database, group);
+        V centroid = Centroid.make(database, group).toVector(database);
         Cluster<CorrelationModel<V>> correlationCluster = new Cluster<CorrelationModel<V>>("[" + correlationDimension + "_" + correlationClusters.size() + "]", group, new CorrelationModel<V>(pcares, centroid), new ArrayList<Cluster<CorrelationModel<V>>>(), new ArrayList<Cluster<CorrelationModel<V>>>());
         correlationClusters.add(correlationCluster);
       }
@@ -264,11 +265,11 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
       Class<PCAFilteredRunner<V>> cls = ClassGenericsUtil.uglyCastIntoSubclass(PCAFilteredRunner.class);
       PCAFilteredRunner<V> pca = parameters.tryInstantiate(cls);
       for(ParameterException e : parameters.getErrors()) {
-        logger.warning("Error in internal parameterization: " + e.getMessage());
+        LOG.warning("Error in internal parameterization: " + e.getMessage());
       }
       PCAFilteredResult pcares = pca.processIds(noise.getIDs(), database);
 
-      V centroid = DatabaseUtil.centroid(database, noise.getIDs());
+      V centroid = Centroid.make(database, noise.getIDs()).toVector(database);
       Cluster<CorrelationModel<V>> correlationCluster = new Cluster<CorrelationModel<V>>("[noise]", noise.getIDs(), new CorrelationModel<V>(pcares, centroid), new ArrayList<Cluster<CorrelationModel<V>>>(), new ArrayList<Cluster<CorrelationModel<V>>>());
       correlationClusters.add(correlationCluster);
     }
@@ -292,7 +293,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
   }
 
   private void buildHierarchy(SortedMap<Integer, List<Cluster<CorrelationModel<V>>>> clusterMap, DistanceQuery<V, IntegerDistance> query) {
-    StringBuffer msg = new StringBuffer();
+    StringBuilder msg = new StringBuilder();
 
     DBSCAN<V, DoubleDistance> dbscan = ClassGenericsUtil.castWithGenericsOrNull(DBSCAN.class, copacAlgorithm.getPartitionAlgorithm(query));
     if(dbscan == null) {
@@ -310,7 +311,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
     for(Integer childCorrDim : clusterMap.keySet()) {
       List<Cluster<CorrelationModel<V>>> children = clusterMap.get(childCorrDim);
       SortedMap<Integer, List<Cluster<CorrelationModel<V>>>> parentMap = clusterMap.tailMap(childCorrDim + 1);
-      if(logger.isDebugging()) {
+      if(LOG.isDebugging()) {
         msg.append("\ncorrdim ").append(childCorrDim);
         msg.append("\nparents ").append(parentMap.keySet());
       }
@@ -323,8 +324,8 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
             if(subspaceDim_parent == lambda_max && child.getParents().isEmpty()) {
               parent.getChildren().add(child);
               child.getParents().add(parent);
-              if(logger.isDebugging()) {
-                msg.append("\n").append(parent).append(" is parent of ").append(child);
+              if(LOG.isDebugging()) {
+                msg.append('\n').append(parent).append(" is parent of ").append(child);
               }
             }
             else {
@@ -332,8 +333,8 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
               if(!dist.bitValue() && (child.getParents().isEmpty() || !isParent(distanceFunction, parent, child.getParents()))) {
                 parent.getChildren().add(child);
                 child.getParents().add(parent);
-                if(logger.isDebugging()) {
-                  msg.append("\n").append(parent).append(" is parent of ").append(child);
+                if(LOG.isDebugging()) {
+                  msg.append('\n').append(parent).append(" is parent of ").append(child);
                 }
               }
             }
@@ -341,8 +342,8 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
         }
       }
     }
-    if(logger.isDebugging()) {
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      LOG.debugFine(msg.toString());
     }
 
   }
@@ -360,7 +361,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    */
   private boolean isParent(ERiCDistanceFunction distanceFunction, Cluster<CorrelationModel<V>> parent, List<Cluster<CorrelationModel<V>>> children) {
 
-    StringBuffer msg = new StringBuffer();
+    StringBuilder msg = new StringBuilder();
 
     for(Cluster<CorrelationModel<V>> child : children) {
       if(parent.getModel().getPCAResult().getCorrelationDimension() == child.getModel().getPCAResult().getCorrelationDimension()) {
@@ -368,19 +369,19 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
       }
 
       BitDistance dist = distanceFunction.distance(parent.getModel().getCentroid(), child.getModel().getCentroid(), parent.getModel().getPCAResult(), child.getModel().getPCAResult());
-      if(logger.isDebugging()) {
+      if(LOG.isDebugging()) {
         msg.append("\ndist(").append(child).append(" - ").append(parent).append(") = ").append(dist);
       }
       if(!dist.bitValue()) {
-        if(logger.isDebugging()) {
-          logger.debugFine(msg.toString());
+        if(LOG.isDebugging()) {
+          LOG.debugFine(msg.toString());
         }
         return true;
       }
     }
 
-    if(logger.isDebugging()) {
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      LOG.debugFine(msg.toString());
     }
     return false;
   }
@@ -392,7 +393,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
   
   /**
@@ -402,7 +403,7 @@ public class ERiC<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     /**
      * The COPAC instance to use
      */
