@@ -1,26 +1,27 @@
 package de.lmu.ifi.dbs.elki.algorithm.clustering.correlation;
+
 /*
-This file is part of ELKI:
-Environment for Developing KDD-Applications Supported by Index-Structures
+ This file is part of ELKI:
+ Environment for Developing KDD-Applications Supported by Index-Structures
 
-Copyright (C) 2012
-Ludwig-Maximilians-Universität München
-Lehr- und Forschungseinheit für Datenbanksysteme
-ELKI Development Team
+ Copyright (C) 2012
+ Ludwig-Maximilians-Universität München
+ Lehr- und Forschungseinheit für Datenbanksysteme
+ ELKI Development Team
 
-This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as published by
-the Free Software Foundation, either version 3 of the License, or
-(at your option) any later version.
+ This program is free software: you can redistribute it and/or modify
+ it under the terms of the GNU Affero General Public License as published by
+ the Free Software Foundation, either version 3 of the License, or
+ (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU Affero General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
-along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ You should have received a copy of the GNU Affero General Public License
+ along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 import de.lmu.ifi.dbs.elki.algorithm.clustering.OPTICS;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
@@ -37,7 +38,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ChainedParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.ListParameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
@@ -64,11 +65,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 @Title("Mining Hierarchies of Correlation Clusters")
 @Description("Algorithm for detecting hierarchies of correlation clusters.")
 @Reference(authors = "E. Achtert, C. Böhm, P. Kröger, A. Zimek", title = "Mining Hierarchies of Correlation Clusterse", booktitle = "Proc. Int. Conf. on Scientific and Statistical Database Management (SSDBM'06), Vienna, Austria, 2006", url = "http://dx.doi.org/10.1109/SSDBM.2006.35")
-public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelationDistance> {
+public class HiCO<V extends NumberVector<?>> extends OPTICS<V, PCACorrelationDistance> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(HiCO.class);
+  private static final Logging LOG = Logging.getLogger(HiCO.class);
 
   /**
    * Parameter to specify the smoothing factor, must be an integer greater than
@@ -79,7 +80,7 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
    * Key: {@code -hico.mu}
    * </p>
    */
-  public static final OptionID MU_ID = OptionID.getOrCreateOptionID("hico.mu", "Specifies the smoothing factor. The mu-nearest neighbor is used to compute the correlation reachability of an object.");
+  public static final OptionID MU_ID = new OptionID("hico.mu", "Specifies the smoothing factor. The mu-nearest neighbor is used to compute the correlation reachability of an object.");
 
   /**
    * Optional parameter to specify the number of nearest neighbors considered in
@@ -92,7 +93,7 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
    * Default value: {@link #MU_ID}
    * </p>
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("hico.k", "Optional parameter to specify the number of nearest neighbors considered in the PCA. If this parameter is not set, k is set to the value of parameter mu.");
+  public static final OptionID K_ID = new OptionID("hico.k", "Optional parameter to specify the number of nearest neighbors considered in the PCA. If this parameter is not set, k is set to the value of parameter mu.");
 
   /**
    * Parameter to specify the threshold of a distance between a vector q and a
@@ -105,7 +106,7 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
    * Key: {@code -hico.delta}
    * </p>
    */
-  public static final OptionID DELTA_ID = OptionID.getOrCreateOptionID("hico.delta", "Threshold of a distance between a vector q and a given space that indicates that " + "q adds a new dimension to the space.");
+  public static final OptionID DELTA_ID = new OptionID("hico.delta", "Threshold of a distance between a vector q and a given space that indicates that " + "q adds a new dimension to the space.");
 
   /**
    * The threshold for 'strong' eigenvectors: the 'strong' eigenvectors explain
@@ -117,7 +118,7 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
    * Key: {@code -hico.alpha}
    * </p>
    */
-  public static final OptionID ALPHA_ID = OptionID.getOrCreateOptionID("hico.alpha", "The threshold for 'strong' eigenvectors: the 'strong' eigenvectors explain a portion of at least alpha of the total variance.");
+  public static final OptionID ALPHA_ID = new OptionID("hico.alpha", "The threshold for 'strong' eigenvectors: the 'strong' eigenvectors explain a portion of at least alpha of the total variance.");
 
   /**
    * The default value for {@link #DELTA_ID}.
@@ -131,7 +132,7 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
 
   /**
    * Constructor.
-   *
+   * 
    * @param distanceFunction Distance function
    * @param mu Mu parameter
    */
@@ -141,31 +142,34 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
    * Parameterization class.
-   *
+   * 
    * @author Erich Schubert
-   *
+   * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     int mu = -1;
-    
+
     PCABasedCorrelationDistanceFunction distance;
-    
+
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
 
-      IntParameter muP = new IntParameter(MU_ID, new GreaterConstraint(0));
+      IntParameter muP = new IntParameter(MU_ID);
+      muP.addConstraint(new GreaterConstraint(0));
       if (config.grab(muP)) {
         mu = muP.getValue();
       }
 
-      IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(0), true);
+      IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(new GreaterConstraint(0));
+      kP.setOptional(true);
       final int k;
       if (config.grab(kP)) {
         k = kP.getValue();
@@ -173,16 +177,19 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
         k = mu;
       }
 
-      DoubleParameter deltaP = new DoubleParameter(DELTA_ID, new GreaterEqualConstraint(0), DEFAULT_DELTA);
+      DoubleParameter deltaP = new DoubleParameter(DELTA_ID, DEFAULT_DELTA);
+      deltaP.addConstraint(new GreaterEqualConstraint(0));
       double delta = DEFAULT_DELTA;
       if (config.grab(deltaP)) {
-        delta = deltaP.getValue();
+        delta = deltaP.doubleValue();
       }
 
-      DoubleParameter alphaP = new DoubleParameter(ALPHA_ID, new IntervalConstraint(0.0, IntervalConstraint.IntervalBoundary.OPEN, 1.0, IntervalConstraint.IntervalBoundary.OPEN), DEFAULT_ALPHA);
+      DoubleParameter alphaP = new DoubleParameter(ALPHA_ID, DEFAULT_ALPHA);
+      alphaP.addConstraint(new GreaterConstraint(0.0));
+      alphaP.addConstraint(new LessConstraint(1.0));
       double alpha = DEFAULT_ALPHA;
       if (config.grab(alphaP)) {
-        alpha = alphaP.getValue();
+        alpha = alphaP.doubleValue();
       }
 
       // Configure Distance function
@@ -203,4 +210,4 @@ public class HiCO<V extends NumberVector<V, ?>> extends OPTICS<V, PCACorrelation
       return new HiCO<V>(distance, mu);
     }
   }
-}
\ No newline at end of file
+}
