@@ -37,33 +37,35 @@ import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
-import de.lmu.ifi.dbs.elki.database.query.GenericDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.GenericDistanceDBIDList;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.SortedEigenPairs;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAResult;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCARunner;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 
 /**
  * ORCLUS provides the ORCLUS algorithm, an algorithm to find clusters in high
@@ -83,38 +85,21 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
 @Title("ORCLUS: Arbitrarily ORiented projected CLUSter generation")
 @Description("Algorithm to find correlation clusters in high dimensional spaces.")
 @Reference(authors = "C. C. Aggarwal, P. S. Yu", title = "Finding Generalized Projected Clusters in High Dimensional Spaces", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '00)", url = "http://dx.doi.org/10.1145/342009.335383")
-public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClustering<Clustering<Model>, V> {
+public class ORCLUS<V extends NumberVector<?>> extends AbstractProjectedClustering<Clustering<Model>, V> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ORCLUS.class);
+  private static final Logging LOG = Logging.getLogger(ORCLUS.class);
 
   /**
-   * Parameter to specify the factor for reducing the number of current clusters
-   * in each iteration, must be an integer greater than 0 and less than 1.
-   * <p>
-   * Default value: {@code 0.5}
-   * </p>
-   * <p>
-   * Key: {@code -orclus.alpha}
-   * </p>
-   */
-  public static final OptionID ALPHA_ID = OptionID.getOrCreateOptionID("orclus.alpha", "The factor for reducing the number of current clusters in each iteration.");
-
-  /**
-   * Parameter to specify the random generator seed.
-   */
-  public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("orclus.seed", "The random number generator seed.");
-
-  /**
-   * Holds the value of {@link #ALPHA_ID}.
+   * Holds the value of {@link Parameterizer#ALPHA_ID}.
    */
   private double alpha;
 
   /**
-   * Holds the value of {@link #SEED_ID}.
+   * Random generator
    */
-  private Long seed;
+  private RandomFactory rnd;
 
   /**
    * The PCA utility object.
@@ -128,13 +113,13 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    * @param k_i k_i Parameter
    * @param l l Parameter
    * @param alpha Alpha Parameter
-   * @param seed Seed parameter
+   * @param rnd Random generator
    * @param pca PCA runner
    */
-  public ORCLUS(int k, int k_i, int l, double alpha, long seed, PCARunner<V> pca) {
+  public ORCLUS(int k, int k_i, int l, double alpha, RandomFactory rnd, PCARunner<V> pca) {
     super(k, k_i, l);
     this.alpha = alpha;
-    this.seed = seed;
+    this.rnd = rnd;
     this.pca = pca;
   }
 
@@ -148,9 +133,9 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
     try {
       DistanceQuery<V, DoubleDistance> distFunc = this.getDistanceQuery(database);
       // current dimensionality associated with each seed
-      int dim_c = DatabaseUtil.dimensionality(relation);
+      int dim_c = RelationUtil.dimensionality(relation);
 
-      if(dim_c < l) {
+      if (dim_c < l) {
         throw new IllegalStateException("Dimensionality of data < parameter l! " + "(" + dim_c + " < " + l + ")");
       }
 
@@ -162,19 +147,19 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
 
       double beta = StrictMath.exp(-StrictMath.log((double) dim_c / (double) l) * StrictMath.log(1 / alpha) / StrictMath.log((double) k_c / (double) k));
 
-      IndefiniteProgress cprogress = logger.isVerbose() ? new IndefiniteProgress("Current number of clusters:", logger) : null;
+      IndefiniteProgress cprogress = LOG.isVerbose() ? new IndefiniteProgress("Current number of clusters:", LOG) : null;
 
-      while(k_c > k) {
-        if(cprogress != null) {
-          cprogress.setProcessed(clusters.size(), logger);
+      while (k_c > k) {
+        if (cprogress != null) {
+          cprogress.setProcessed(clusters.size(), LOG);
         }
 
         // find partitioning induced by the seeds of the clusters
         assign(relation, distFunc, clusters);
 
         // determine current subspace associated with each cluster
-        for(ORCLUSCluster cluster : clusters) {
-          if(cluster.objectIDs.size() > 0) {
+        for (ORCLUSCluster cluster : clusters) {
+          if (cluster.objectIDs.size() > 0) {
             cluster.basis = findBasis(relation, distFunc, cluster, dim_c);
           }
         }
@@ -187,19 +172,18 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
       }
       assign(relation, distFunc, clusters);
 
-      if(cprogress != null) {
+      if (cprogress != null) {
         cprogress.setProcessed(clusters.size());
-        cprogress.setCompleted(logger);
+        cprogress.setCompleted(LOG);
       }
 
       // get the result
       Clustering<Model> r = new Clustering<Model>("ORCLUS clustering", "orclus-clustering");
-      for(ORCLUSCluster c : clusters) {
+      for (ORCLUSCluster c : clusters) {
         r.addCluster(new Cluster<Model>(c.objectIDs, ClusterModel.CLUSTER));
       }
       return r;
-    }
-    catch(Exception e) {
+    } catch (Exception e) {
       throw new IllegalStateException(e);
     }
   }
@@ -212,11 +196,11 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    * @return the initial seed list
    */
   private List<ORCLUSCluster> initialSeeds(Relation<V> database, int k) {
-    DBIDs randomSample = DBIDUtil.randomSample(database.getDBIDs(), k, seed);
-    V factory = DatabaseUtil.assumeVectorField(database).getFactory();
+    DBIDs randomSample = DBIDUtil.randomSample(database.getDBIDs(), k, rnd);
+    NumberVector.Factory<V, ?> factory = RelationUtil.getNumberVectorFactory(database);
     List<ORCLUSCluster> seeds = new ArrayList<ORCLUSCluster>();
-    for(DBIDIter iter = randomSample.iter(); iter.valid(); iter.advance()) {
-      seeds.add(new ORCLUSCluster(database.get(iter), iter.getDBID(), factory));
+    for (DBIDIter iter = randomSample.iter(); iter.valid(); iter.advance()) {
+      seeds.add(new ORCLUSCluster(database.get(iter), iter, factory));
     }
     return seeds;
   }
@@ -231,15 +215,15 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    *        assigned to
    */
   private void assign(Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, List<ORCLUSCluster> clusters) {
-    V factory = DatabaseUtil.assumeVectorField(database).getFactory();
+    NumberVector.Factory<V, ?> factory = RelationUtil.getNumberVectorFactory(database);
     // clear the current clusters
-    for(ORCLUSCluster cluster : clusters) {
+    for (ORCLUSCluster cluster : clusters) {
       cluster.objectIDs.clear();
     }
 
     // projected centroids of the clusters
     List<V> projectedCentroids = new ArrayList<V>(clusters.size());
-    for(ORCLUSCluster c : clusters) {
+    for (ORCLUSCluster c : clusters) {
       projectedCentroids.add(projection(c, c.centroid, factory));
     }
 
@@ -251,11 +235,11 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
       ORCLUSCluster minCluster = null;
 
       // determine projected distance between o and cluster
-      for(int i = 0; i < clusters.size(); i++) {
+      for (int i = 0; i < clusters.size(); i++) {
         ORCLUSCluster c = clusters.get(i);
         V o_proj = projection(c, o, factory);
         DoubleDistance dist = distFunc.distance(o_proj, projectedCentroids.get(i));
-        if(minDist == null || minDist.compareTo(dist) > 0) {
+        if (minDist == null || minDist.compareTo(dist) > 0) {
           minDist = dist;
           minCluster = c;
         }
@@ -266,9 +250,9 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
     }
 
     // recompute the seed in each clusters
-    for(ORCLUSCluster cluster : clusters) {
-      if(cluster.objectIDs.size() > 0) {
-        cluster.centroid = DatabaseUtil.centroid(database, cluster.objectIDs);
+    for (ORCLUSCluster cluster : clusters) {
+      if (cluster.objectIDs.size() > 0) {
+        cluster.centroid = Centroid.make(database, cluster.objectIDs).toVector(database);
       }
     }
   }
@@ -286,13 +270,12 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
   private Matrix findBasis(Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, ORCLUSCluster cluster, int dim) {
     // covariance matrix of cluster
     // Matrix covariance = Util.covarianceMatrix(database, cluster.objectIDs);
-    List<DistanceResultPair<DoubleDistance>> results = new ArrayList<DistanceResultPair<DoubleDistance>>(cluster.objectIDs.size());
-    for(DBIDIter it = cluster.objectIDs.iter(); it.valid(); it.advance()) {
+    GenericDistanceDBIDList<DoubleDistance> results = new GenericDistanceDBIDList<DoubleDistance>(cluster.objectIDs.size());
+    for (DBIDIter it = cluster.objectIDs.iter(); it.valid(); it.advance()) {
       DoubleDistance distance = distFunc.distance(cluster.centroid, database.get(it));
-      DistanceResultPair<DoubleDistance> qr = new GenericDistanceResultPair<DoubleDistance>(distance, it.getDBID());
-      results.add(qr);
+      results.add(distance, it);
     }
-    Collections.sort(results);
+    results.sort();
     PCAResult pcares = pca.processQueryResult(results, database);
     SortedEigenPairs eigenPairs = pcares.getEigenPairs();
     return eigenPairs.reverseEigenVectors(dim);
@@ -321,9 +304,9 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    */
   private void merge(Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, List<ORCLUSCluster> clusters, int k_new, int d_new, IndefiniteProgress cprogress) {
     ArrayList<ProjectedEnergy> projectedEnergies = new ArrayList<ProjectedEnergy>();
-    for(int i = 0; i < clusters.size(); i++) {
-      for(int j = 0; j < clusters.size(); j++) {
-        if(i >= j) {
+    for (int i = 0; i < clusters.size(); i++) {
+      for (int j = 0; j < clusters.size(); j++) {
+        if (i >= j) {
           continue;
         }
         // projected energy of c_ij in subspace e_ij
@@ -335,21 +318,21 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
       }
     }
 
-    while(clusters.size() > k_new) {
-      if(cprogress != null) {
-        cprogress.setProcessed(clusters.size(), logger);
+    while (clusters.size() > k_new) {
+      if (cprogress != null) {
+        cprogress.setProcessed(clusters.size(), LOG);
       }
       // find the smallest value of r_ij
       ProjectedEnergy minPE = Collections.min(projectedEnergies);
 
       // renumber the clusters by replacing cluster c_i with cluster c_ij
       // and discarding cluster c_j
-      for(int c = 0; c < clusters.size(); c++) {
-        if(c == minPE.i) {
+      for (int c = 0; c < clusters.size(); c++) {
+        if (c == minPE.i) {
           clusters.remove(c);
           clusters.add(c, minPE.cluster);
         }
-        if(c == minPE.j) {
+        if (c == minPE.j) {
           clusters.remove(c);
         }
       }
@@ -358,16 +341,15 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
       int i = minPE.i;
       int j = minPE.j;
       Iterator<ProjectedEnergy> it = projectedEnergies.iterator();
-      while(it.hasNext()) {
+      while (it.hasNext()) {
         ProjectedEnergy pe = it.next();
-        if(pe.i == i || pe.i == j || pe.j == i || pe.j == j) {
+        if (pe.i == i || pe.i == j || pe.j == i || pe.j == j) {
           it.remove();
-        }
-        else {
-          if(pe.i > j) {
+        } else {
+          if (pe.i > j) {
             pe.i -= 1;
           }
-          if(pe.j > j) {
+          if (pe.j > j) {
             pe.j -= 1;
           }
         }
@@ -375,11 +357,10 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
 
       // ... and recompute them
       ORCLUSCluster c_ij = minPE.cluster;
-      for(int c = 0; c < clusters.size(); c++) {
-        if(c < i) {
+      for (int c = 0; c < clusters.size(); c++) {
+        if (c < i) {
           projectedEnergies.add(projectedEnergy(database, distFunc, clusters.get(c), c_ij, c, i, d_new));
-        }
-        else if(c > i) {
+        } else if (c > i) {
           projectedEnergies.add(projectedEnergy(database, distFunc, clusters.get(c), c_ij, i, c, d_new));
         }
       }
@@ -404,11 +385,11 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
   private ProjectedEnergy projectedEnergy(Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, ORCLUSCluster c_i, ORCLUSCluster c_j, int i, int j, int dim) {
     // union of cluster c_i and c_j
     ORCLUSCluster c_ij = union(database, distFunc, c_i, c_j, dim);
-    V factory = DatabaseUtil.assumeVectorField(database).getFactory();
+    NumberVector.Factory<V, ?> factory = RelationUtil.getNumberVectorFactory(database);
 
     DoubleDistance sum = getDistanceFunction().getDistanceFactory().nullDistance();
     V c_proj = projection(c_ij, c_ij.centroid, factory);
-    for(DBIDIter iter = c_ij.objectIDs.iter(); iter.valid(); iter.advance()) {
+    for (DBIDIter iter = c_ij.objectIDs.iter(); iter.valid(); iter.advance()) {
       V o_proj = projection(c_ij, database.get(iter), factory);
       DoubleDistance dist = distFunc.distance(o_proj, c_proj);
       sum = sum.plus(dist.times(dist));
@@ -436,16 +417,15 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
     // convert into array.
     c.objectIDs = DBIDUtil.newArray(c.objectIDs);
 
-    if(c.objectIDs.size() > 0) {
-      c.centroid = DatabaseUtil.centroid(relation, c.objectIDs);
+    if (c.objectIDs.size() > 0) {
+      c.centroid = Centroid.make(relation, c.objectIDs).toVector(relation);
       c.basis = findBasis(relation, distFunc, c, dim);
-    }
-    else {
-      V factory = DatabaseUtil.assumeVectorField(relation).getFactory();
-      Vector cent = c1.centroid.getColumnVector().plusEquals(c2.centroid.getColumnVector()).timesEquals(0.5); 
+    } else {
+      NumberVector.Factory<V, ?> factory = RelationUtil.getNumberVectorFactory(relation);
+      Vector cent = c1.centroid.getColumnVector().plusEquals(c2.centroid.getColumnVector()).timesEquals(0.5);
       c.centroid = factory.newNumberVector(cent.getArrayRef());
       double[][] doubles = new double[c1.basis.getRowDimensionality()][dim];
-      for(int i = 0; i < dim; i++) {
+      for (int i = 0; i < dim; i++) {
         doubles[i][i] = 1;
       }
       c.basis = new Matrix(doubles);
@@ -462,7 +442,7 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    * @param factory Factory object / prototype
    * @return the projection of double vector o in the subspace of cluster c
    */
-  private V projection(ORCLUSCluster c, V o, V factory) {
+  private V projection(ORCLUSCluster c, V o, NumberVector.Factory<V, ?> factory) {
     Matrix o_proj = o.getColumnVector().transposeTimes(c.basis);
     double[] values = o_proj.getColumnPackedCopy();
     return factory.newNumberVector(values);
@@ -475,7 +455,7 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -511,21 +491,19 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
      * Creates a new cluster containing the specified object o.
      * 
      * @param o the object belonging to this cluster.
+     * @param id Object id
      * @param factory Factory object / prototype
      */
-    ORCLUSCluster(V o, DBID id, V factory) {
+    ORCLUSCluster(V o, DBIDRef id, NumberVector.Factory<V, ?> factory) {
       this.objectIDs.add(id);
 
       // initially the basis ist the original axis-system
       int dim = o.getDimensionality();
       this.basis = Matrix.unitMatrix(dim);
 
-      // TODO: can we replace this with some kind of clone() statement?
       // initially the centroid is the value array of o
-      double[] values = new double[o.getDimensionality()];
-      for(int d = 1; d <= o.getDimensionality(); d++) {
-        values[d - 1] = o.doubleValue(d);
-      }
+      double[] values = o.getColumnVector().getArrayRef();
+      // FIXME: avoid going through 'values'
       this.centroid = factory.newNumberVector(values);
     }
   }
@@ -571,10 +549,28 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractProjectedClustering.Parameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractProjectedClustering.Parameterizer {
+    /**
+     * Parameter to specify the factor for reducing the number of current
+     * clusters in each iteration, must be an integer greater than 0 and less
+     * than 1.
+     * <p>
+     * Default value: {@code 0.5}
+     * </p>
+     * <p>
+     * Key: {@code -orclus.alpha}
+     * </p>
+     */
+    public static final OptionID ALPHA_ID = new OptionID("orclus.alpha", "The factor for reducing the number of current clusters in each iteration.");
+
+    /**
+     * Parameter to specify the random generator seed.
+     */
+    public static final OptionID SEED_ID = new OptionID("orclus.seed", "The random number generator seed.");
+
     protected double alpha = -1;
 
-    protected Long seed = null;
+    protected RandomFactory rnd;
 
     protected PCARunner<V> pca = null;
 
@@ -593,22 +589,24 @@ public class ORCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClust
     }
 
     protected void configAlpha(Parameterization config) {
-      DoubleParameter alphaP = new DoubleParameter(ALPHA_ID, new IntervalConstraint(0, IntervalConstraint.IntervalBoundary.OPEN, 1, IntervalConstraint.IntervalBoundary.CLOSE), 0.5);
-      if(config.grab(alphaP)) {
-        alpha = alphaP.getValue();
+      DoubleParameter alphaP = new DoubleParameter(ALPHA_ID, 0.5);
+      alphaP.addConstraint(new GreaterConstraint(0));
+      alphaP.addConstraint(new LessEqualConstraint(1));
+      if (config.grab(alphaP)) {
+        alpha = alphaP.doubleValue();
       }
     }
 
     protected void configSeed(Parameterization config) {
-      LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        seed = seedP.getValue();
+      RandomParameter rndP = new RandomParameter(SEED_ID);
+      if (config.grab(rndP)) {
+        rnd = rndP.getValue();
       }
     }
 
     @Override
     protected ORCLUS<V> makeInstance() {
-      return new ORCLUS<V>(k, k_i, l, alpha, seed, pca);
+      return new ORCLUS<V>(k, k_i, l, alpha, rnd, pca);
     }
   }
-}
\ No newline at end of file
+}
