@@ -42,7 +42,7 @@ import de.lmu.ifi.dbs.elki.logging.LoggingConfiguration;
  */
 public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInterval> {
   /**
-   * Serial version number
+   * Serial version number.
    */
   private static final long serialVersionUID = 1;
 
@@ -141,14 +141,14 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
   /**
    * Removes the specified ids from this interval.
    * 
-   * @param ids the set of ids to be removed
+   * @param ids2 the set of ids to be removed
    */
-  public void removeIDs(DBIDs ids) {
-    this.ids.removeDBIDs(ids);
+  public void removeIDs(DBIDs ids2) {
+    this.ids.removeDBIDs(ids2);
   }
 
   /**
-   * Returns the number of objects associated with this interval
+   * Returns the number of objects associated with this interval.
    * 
    * @return the number of objects associated with this interval
    */
@@ -156,18 +156,6 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
     return ids.size();
   }
 
-  /**
-   * Returns true if this interval has already been split in the specified
-   * dimension.
-   * 
-   * @param d the dimension to be tested
-   * @return true if this interval has already been split in the specified
-   *         dimension
-   */
-  public boolean isSplit(int d) {
-    return maxSplitDimension >= d;
-  }
-
   /**
    * Returns a String representation of the HyperBoundingBox.
    * 
@@ -286,9 +274,6 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
     }
   }
 
-  /**
-   * @see Object#equals(Object)
-   */
   @Override
   public boolean equals(Object o) {
     if(this == o) {
@@ -305,9 +290,6 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
     return super.equals(o);
   }
 
-  /**
-   * Returns the unique id of this interval as hash code.
-   */
   @Override
   public int hashCode() {
     return intervalID.hashCode();
@@ -330,24 +312,23 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
       return;
     }
 
-    int dim = getDimensionality();
-    int childLevel = isSplit(dim) ? level + 1 : level;
-
-    int splitDim = isSplit(dim) ? 1 : maxSplitDimension + 1;
-    double splitPoint = getMin(splitDim) + (getMax(splitDim) - getMin(splitDim)) / 2;
+    final boolean issplit = (maxSplitDimension >= (getDimensionality() - 1));
+    final int childLevel = issplit ? level + 1 : level;
+    final int splitDim = issplit ? 0 : maxSplitDimension + 1;
+    final double splitPoint = getMin(splitDim) + (getMax(splitDim) - getMin(splitDim)) * .5;
 
     // left and right child
     for(int i = 0; i < 2; i++) {
-      double[] min = SpatialUtil.getMin(this);
-      double[] max = SpatialUtil.getMax(this);
+      double[] min = SpatialUtil.getMin(this); // clone
+      double[] max = SpatialUtil.getMax(this); // clone
 
       // right child
       if(i == 0) {
-        min[splitDim - 1] = splitPoint;
+        min[splitDim] = splitPoint;
       }
       // left child
       else {
-        max[splitDim - 1] = splitPoint;
+        max[splitDim] = splitPoint;
       }
 
       ModifiableDBIDs childIDs = split.determineIDs(getIDs(), new HyperBoundingBox(min, max), d_min, d_max);
@@ -364,7 +345,7 @@ public class CASHInterval extends HyperBoundingBox implements Comparable<CASHInt
     }
 
     if(LoggingConfiguration.DEBUG) {
-      StringBuffer msg = new StringBuffer();
+      StringBuilder msg = new StringBuilder();
       msg.append("\nchild level ").append(childLevel).append(",  split Dim   ").append(splitDim);
       if(leftChild != null) {
         msg.append("\nleft   ").append(leftChild);
