@@ -47,6 +47,7 @@ import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.IndexBasedDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.ProxyDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.DiSHDistanceFunction;
@@ -55,10 +56,11 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.PreferenceVectorBasedCorrelati
 import de.lmu.ifi.dbs.elki.index.preprocessed.preference.DiSHPreferenceVectorIndex;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.ProjectedCentroid;
 import de.lmu.ifi.dbs.elki.result.optics.ClusterOrderEntry;
 import de.lmu.ifi.dbs.elki.result.optics.ClusterOrderResult;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.hierarchy.HierarchyReferenceLists;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
@@ -99,11 +101,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("DiSH: Detecting Subspace cluster Hierarchies")
 @Description("Algorithm to find hierarchical correlation clusters in subspaces.")
 @Reference(authors = "E. Achtert, C. Böhm, H.-P. Kriegel, P. Kröger, I. Müller-Gorman, A. Zimek", title = "Detection and Visualization of Subspace Cluster Hierarchies", booktitle = "Proc. 12th International Conference on Database Systems for Advanced Applications (DASFAA), Bangkok, Thailand, 2007", url = "http://dx.doi.org/10.1007/978-3-540-71703-4_15")
-public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
+public class DiSH<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(DiSH.class);
+  private static final Logging LOG = Logging.getLogger(DiSH.class);
 
   /**
    * Parameter that specifies the maximum radius of the neighborhood to be
@@ -116,7 +118,7 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * Key: {@code -dish.epsilon}
    * </p>
    */
-  public static final OptionID EPSILON_ID = OptionID.getOrCreateOptionID("dish.epsilon", "The maximum radius of the neighborhood " + "to be considered in each dimension for determination of " + "the preference vector.");
+  public static final OptionID EPSILON_ID = new OptionID("dish.epsilon", "The maximum radius of the neighborhood " + "to be considered in each dimension for determination of " + "the preference vector.");
 
   /**
    * Parameter that specifies the a minimum number of points as a smoothing
@@ -128,7 +130,7 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * Key: {@code -dish.mu}
    * </p>
    */
-  public static final OptionID MU_ID = OptionID.getOrCreateOptionID("dish.mu", "The minimum number of points as a smoothing factor to avoid the single-link-effekt.");
+  public static final OptionID MU_ID = new OptionID("dish.mu", "The minimum number of points as a smoothing factor to avoid the single-link-effekt.");
 
   /**
    * Holds the value of {@link #EPSILON_ID}.
@@ -167,13 +169,13 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    */
   public Clustering<SubspaceModel<V>> run(Database database, Relation<V> relation) {
     // Instantiate DiSH distance (and thus run the preprocessor)
-    if(logger.isVerbose()) {
-      logger.verbose("*** Run DiSH preprocessor.");
+    if (LOG.isVerbose()) {
+      LOG.verbose("*** Run DiSH preprocessor.");
     }
     DiSHDistanceFunction.Instance<V> dishDistanceQuery = dishDistance.instantiate(relation);
     // Configure and run OPTICS.
-    if(logger.isVerbose()) {
-      logger.verbose("*** Run OPTICS algorithm.");
+    if (LOG.isVerbose()) {
+      LOG.verbose("*** Run OPTICS algorithm.");
     }
     ListParameterization opticsconfig = new ListParameterization(opticsAlgorithmParameters);
     opticsconfig.addParameter(OPTICS.DISTANCE_FUNCTION_ID, ProxyDistanceFunction.proxy(dishDistanceQuery));
@@ -183,8 +185,8 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
     optics = opticsconfig.tryInstantiate(cls);
     ClusterOrderResult<PreferenceVectorBasedCorrelationDistance> opticsResult = optics.run(database, relation);
 
-    if(logger.isVerbose()) {
-      logger.verbose("*** Compute Clusters.");
+    if (LOG.isVerbose()) {
+      LOG.verbose("*** Compute Clusters.");
     }
     return computeClusters(relation, opticsResult, dishDistanceQuery);
   }
@@ -197,64 +199,64 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @param distFunc Distance function
    */
   private Clustering<SubspaceModel<V>> computeClusters(Relation<V> database, ClusterOrderResult<PreferenceVectorBasedCorrelationDistance> clusterOrder, DiSHDistanceFunction.Instance<V> distFunc) {
-    int dimensionality = DatabaseUtil.dimensionality(database);
+    int dimensionality = RelationUtil.dimensionality(database);
     int minpts = dishDistance.getMinpts();
 
     // extract clusters
     Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> clustersMap = extractClusters(database, distFunc, clusterOrder);
 
-    if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer("Step 1: extract clusters");
-      for(List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
-        for(Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
-          msg.append("\n").append(FormatUtil.format(dimensionality, c.first)).append(" ids ").append(c.second.size());
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder("Step 1: extract clusters");
+      for (List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
+        for (Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
+          msg.append('\n').append(FormatUtil.format(dimensionality, c.first)).append(" ids ").append(c.second.size());
         }
       }
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // check if there are clusters < minpts
     checkClusters(database, distFunc, clustersMap, minpts);
-    if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer("Step 2: check clusters");
-      for(List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
-        for(Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
-          msg.append("\n").append(FormatUtil.format(dimensionality, c.first)).append(" ids ").append(c.second.size());
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder("Step 2: check clusters");
+      for (List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
+        for (Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
+          msg.append('\n').append(FormatUtil.format(dimensionality, c.first)).append(" ids ").append(c.second.size());
         }
       }
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // sort the clusters
     List<Cluster<SubspaceModel<V>>> clusters = sortClusters(database, clustersMap);
-    if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer("Step 3: sort clusters");
-      for(Cluster<SubspaceModel<V>> c : clusters) {
-        msg.append("\n").append(FormatUtil.format(dimensionality, c.getModel().getSubspace().getDimensions())).append(" ids ").append(c.size());
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder("Step 3: sort clusters");
+      for (Cluster<SubspaceModel<V>> c : clusters) {
+        msg.append('\n').append(FormatUtil.format(dimensionality, c.getModel().getSubspace().getDimensions())).append(" ids ").append(c.size());
       }
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // build the hierarchy
     buildHierarchy(database, distFunc, clusters, dimensionality);
-    if(logger.isVerbose()) {
-      StringBuffer msg = new StringBuffer("Step 4: build hierarchy");
-      for(Cluster<SubspaceModel<V>> c : clusters) {
-        msg.append("\n").append(FormatUtil.format(dimensionality, c.getModel().getDimensions())).append(" ids ").append(c.size());
-        for(Cluster<SubspaceModel<V>> cluster : c.getParents()) {
+    if (LOG.isVerbose()) {
+      StringBuilder msg = new StringBuilder("Step 4: build hierarchy");
+      for (Cluster<SubspaceModel<V>> c : clusters) {
+        msg.append('\n').append(FormatUtil.format(dimensionality, c.getModel().getDimensions())).append(" ids ").append(c.size());
+        for (Cluster<SubspaceModel<V>> cluster : c.getParents()) {
           msg.append("\n   parent ").append(cluster);
         }
-        for(Cluster<SubspaceModel<V>> cluster : c.getChildren()) {
+        for (Cluster<SubspaceModel<V>> cluster : c.getChildren()) {
           msg.append("\n   child ").append(cluster);
         }
       }
-      logger.verbose(msg.toString());
+      LOG.verbose(msg.toString());
     }
 
     // build result
     Clustering<SubspaceModel<V>> result = new Clustering<SubspaceModel<V>>("DiSH clustering", "dish-clustering");
-    for(Cluster<SubspaceModel<V>> c : clusters) {
-      if(c.getParents() == null || c.getParents().isEmpty()) {
+    for (Cluster<SubspaceModel<V>> c : clusters) {
+      if (c.getParents() == null || c.getParents().isEmpty()) {
         result.addCluster(c);
       }
     }
@@ -270,12 +272,12 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @return the extracted clusters
    */
   private Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> extractClusters(Relation<V> database, DiSHDistanceFunction.Instance<V> distFunc, ClusterOrderResult<PreferenceVectorBasedCorrelationDistance> clusterOrder) {
-    FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Extract Clusters", database.size(), logger) : null;
+    FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Extract Clusters", database.size(), LOG) : null;
     int processed = 0;
     Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> clustersMap = new HashMap<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>>();
     Map<DBID, ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance>> entryMap = new HashMap<DBID, ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance>>();
     Map<DBID, Pair<BitSet, ArrayModifiableDBIDs>> entryToClusterMap = new HashMap<DBID, Pair<BitSet, ArrayModifiableDBIDs>>();
-    for(Iterator<ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance>> it = clusterOrder.iterator(); it.hasNext();) {
+    for (Iterator<ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance>> it = clusterOrder.iterator(); it.hasNext();) {
       ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance> entry = it.next();
       entryMap.put(entry.getID(), entry);
 
@@ -284,68 +286,68 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
 
       // get the list of (parallel) clusters for the preference vector
       List<Pair<BitSet, ArrayModifiableDBIDs>> parallelClusters = clustersMap.get(preferenceVector);
-      if(parallelClusters == null) {
+      if (parallelClusters == null) {
         parallelClusters = new ArrayList<Pair<BitSet, ArrayModifiableDBIDs>>();
         clustersMap.put(preferenceVector, parallelClusters);
       }
 
       // look for the proper cluster
       Pair<BitSet, ArrayModifiableDBIDs> cluster = null;
-      for(Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
-        V c_centroid = DatabaseUtil.centroid(database, c.second, c.first);
+      for (Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
+        V c_centroid = ProjectedCentroid.make(c.first, database, c.second).toVector(database);
         PreferenceVectorBasedCorrelationDistance dist = distFunc.correlationDistance(object, c_centroid, preferenceVector, preferenceVector);
-        if(dist.getCorrelationValue() == entry.getReachability().getCorrelationValue()) {
+        if (dist.getCorrelationValue() == entry.getReachability().getCorrelationValue()) {
           double d = distFunc.weightedDistance(object, c_centroid, dist.getCommonPreferenceVector());
-          if(d <= 2 * epsilon) {
+          if (d <= 2 * epsilon) {
             cluster = c;
             break;
           }
         }
       }
-      if(cluster == null) {
+      if (cluster == null) {
         cluster = new Pair<BitSet, ArrayModifiableDBIDs>(preferenceVector, DBIDUtil.newArray());
         parallelClusters.add(cluster);
       }
       cluster.second.add(entry.getID());
       entryToClusterMap.put(entry.getID(), cluster);
 
-      if(progress != null) {
-        progress.setProcessed(++processed, logger);
+      if (progress != null) {
+        progress.setProcessed(++processed, LOG);
       }
     }
-    if(progress != null) {
-      progress.ensureCompleted(logger);
+    if (progress != null) {
+      progress.ensureCompleted(LOG);
     }
 
-    if(logger.isDebuggingFiner()) {
-      StringBuffer msg = new StringBuffer("Step 0");
-      for(List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
-        for(Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
-          msg.append("\n").append(FormatUtil.format(DatabaseUtil.dimensionality(database), c.first)).append(" ids ").append(c.second.size());
+    if (LOG.isDebuggingFiner()) {
+      StringBuilder msg = new StringBuilder("Step 0");
+      for (List<Pair<BitSet, ArrayModifiableDBIDs>> clusterList : clustersMap.values()) {
+        for (Pair<BitSet, ArrayModifiableDBIDs> c : clusterList) {
+          msg.append('\n').append(FormatUtil.format(RelationUtil.dimensionality(database), c.first)).append(" ids ").append(c.second.size());
         }
       }
-      logger.debugFiner(msg.toString());
+      LOG.debugFiner(msg.toString());
     }
 
     // add the predecessor to the cluster
-    for(BitSet pv : clustersMap.keySet()) {
+    for (BitSet pv : clustersMap.keySet()) {
       List<Pair<BitSet, ArrayModifiableDBIDs>> parallelClusters = clustersMap.get(pv);
-      for(Pair<BitSet, ArrayModifiableDBIDs> cluster : parallelClusters) {
-        if(cluster.second.isEmpty()) {
+      for (Pair<BitSet, ArrayModifiableDBIDs> cluster : parallelClusters) {
+        if (cluster.second.isEmpty()) {
           continue;
         }
         DBID firstID = cluster.second.get(0);
         ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance> entry = entryMap.get(firstID);
         DBID predecessorID = entry.getPredecessorID();
-        if(predecessorID == null) {
+        if (predecessorID == null) {
           continue;
         }
         ClusterOrderEntry<PreferenceVectorBasedCorrelationDistance> predecessor = entryMap.get(predecessorID);
         // parallel cluster
-        if(predecessor.getReachability().getCommonPreferenceVector().equals(entry.getReachability().getCommonPreferenceVector())) {
+        if (predecessor.getReachability().getCommonPreferenceVector().equals(entry.getReachability().getCommonPreferenceVector())) {
           continue;
         }
-        if(predecessor.getReachability().compareTo(entry.getReachability()) < 0) {
+        if (predecessor.getReachability().compareTo(entry.getReachability()) < 0) {
           continue;
         }
 
@@ -369,22 +371,21 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @return a sorted list of the clusters
    */
   private List<Cluster<SubspaceModel<V>>> sortClusters(Relation<V> database, Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> clustersMap) {
-    final int db_dim = DatabaseUtil.dimensionality(database);
+    final int db_dim = RelationUtil.dimensionality(database);
     // int num = 1;
     List<Cluster<SubspaceModel<V>>> clusters = new ArrayList<Cluster<SubspaceModel<V>>>();
-    for(BitSet pv : clustersMap.keySet()) {
+    for (BitSet pv : clustersMap.keySet()) {
       List<Pair<BitSet, ArrayModifiableDBIDs>> parallelClusters = clustersMap.get(pv);
-      for(int i = 0; i < parallelClusters.size(); i++) {
+      for (int i = 0; i < parallelClusters.size(); i++) {
         Pair<BitSet, ArrayModifiableDBIDs> c = parallelClusters.get(i);
         Cluster<SubspaceModel<V>> cluster = new Cluster<SubspaceModel<V>>(c.second);
-        cluster.setModel(new SubspaceModel<V>(new Subspace<V>(c.first), DatabaseUtil.centroid(database, c.second)));
+        cluster.setModel(new SubspaceModel<V>(new Subspace(c.first), Centroid.make(database, c.second).toVector(database)));
         cluster.setHierarchy(new HierarchyReferenceLists<Cluster<SubspaceModel<V>>>(cluster, new ArrayList<Cluster<SubspaceModel<V>>>(), new ArrayList<Cluster<SubspaceModel<V>>>()));
         // cluster.setName("Cluster_" + num++);
         String subspace = FormatUtil.format(cluster.getModel().getSubspace().getDimensions(), db_dim, "");
-        if(parallelClusters.size() > 1) {
+        if (parallelClusters.size() > 1) {
           cluster.setName("Cluster_" + subspace + "_" + i);
-        }
-        else {
+        } else {
           cluster.setName("Cluster_" + subspace);
         }
         clusters.add(cluster);
@@ -417,11 +418,11 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
     List<Pair<BitSet, ArrayModifiableDBIDs>> notAssigned = new ArrayList<Pair<BitSet, ArrayModifiableDBIDs>>();
     Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> newClustersMap = new HashMap<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>>();
     Pair<BitSet, ArrayModifiableDBIDs> noise = new Pair<BitSet, ArrayModifiableDBIDs>(new BitSet(), DBIDUtil.newArray());
-    for(BitSet pv : clustersMap.keySet()) {
+    for (BitSet pv : clustersMap.keySet()) {
       // noise
-      if(pv.cardinality() == 0) {
+      if (pv.cardinality() == 0) {
         List<Pair<BitSet, ArrayModifiableDBIDs>> parallelClusters = clustersMap.get(pv);
-        for(Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
+        for (Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
           noise.second.addDBIDs(c.second);
         }
       }
@@ -429,11 +430,10 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
       else {
         List<Pair<BitSet, ArrayModifiableDBIDs>> parallelClusters = clustersMap.get(pv);
         List<Pair<BitSet, ArrayModifiableDBIDs>> newParallelClusters = new ArrayList<Pair<BitSet, ArrayModifiableDBIDs>>(parallelClusters.size());
-        for(Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
-          if(!pv.equals(new BitSet()) && c.second.size() < minpts) {
+        for (Pair<BitSet, ArrayModifiableDBIDs> c : parallelClusters) {
+          if (!pv.equals(new BitSet()) && c.second.size() < minpts) {
             notAssigned.add(c);
-          }
-          else {
+          } else {
             newParallelClusters.add(c);
           }
         }
@@ -444,15 +444,14 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
     clustersMap.clear();
     clustersMap.putAll(newClustersMap);
 
-    for(Pair<BitSet, ArrayModifiableDBIDs> c : notAssigned) {
-      if(c.second.isEmpty()) {
+    for (Pair<BitSet, ArrayModifiableDBIDs> c : notAssigned) {
+      if (c.second.isEmpty()) {
         continue;
       }
       Pair<BitSet, ArrayModifiableDBIDs> parent = findParent(database, distFunc, c, clustersMap);
-      if(parent != null) {
+      if (parent != null) {
         parent.second.addDBIDs(c.second);
-      }
-      else {
+      } else {
         noise.second.addDBIDs(c.second);
       }
     }
@@ -472,30 +471,30 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @return the parent of the specified cluster
    */
   private Pair<BitSet, ArrayModifiableDBIDs> findParent(Relation<V> database, DiSHDistanceFunction.Instance<V> distFunc, Pair<BitSet, ArrayModifiableDBIDs> child, Map<BitSet, List<Pair<BitSet, ArrayModifiableDBIDs>>> clustersMap) {
-    V child_centroid = DatabaseUtil.centroid(database, child.second, child.first);
+    V child_centroid = ProjectedCentroid.make(child.first, database, child.second).toVector(database);
 
     Pair<BitSet, ArrayModifiableDBIDs> result = null;
     int resultCardinality = -1;
 
     BitSet childPV = child.first;
     int childCardinality = childPV.cardinality();
-    for(BitSet parentPV : clustersMap.keySet()) {
+    for (BitSet parentPV : clustersMap.keySet()) {
       int parentCardinality = parentPV.cardinality();
-      if(parentCardinality >= childCardinality) {
+      if (parentCardinality >= childCardinality) {
         continue;
       }
-      if(resultCardinality != -1 && parentCardinality <= resultCardinality) {
+      if (resultCardinality != -1 && parentCardinality <= resultCardinality) {
         continue;
       }
 
       BitSet pv = (BitSet) childPV.clone();
       pv.and(parentPV);
-      if(pv.equals(parentPV)) {
+      if (pv.equals(parentPV)) {
         List<Pair<BitSet, ArrayModifiableDBIDs>> parentList = clustersMap.get(parentPV);
-        for(Pair<BitSet, ArrayModifiableDBIDs> parent : parentList) {
-          V parent_centroid = DatabaseUtil.centroid(database, parent.second, parentPV);
+        for (Pair<BitSet, ArrayModifiableDBIDs> parent : parentList) {
+          V parent_centroid = ProjectedCentroid.make(parentPV, database, parent.second).toVector(database);
           double d = distFunc.weightedDistance(child_centroid, parent_centroid, parentPV);
-          if(d <= 2 * epsilon) {
+          if (d <= 2 * epsilon) {
             result = parent;
             resultCardinality = parentCardinality;
             break;
@@ -516,64 +515,62 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * @param database the database containing the data objects
    */
   private void buildHierarchy(Relation<V> database, DiSHDistanceFunction.Instance<V> distFunc, List<Cluster<SubspaceModel<V>>> clusters, int dimensionality) {
-    StringBuffer msg = new StringBuffer();
-    final int db_dim = DatabaseUtil.dimensionality(database);
+    StringBuilder msg = new StringBuilder();
+    final int db_dim = RelationUtil.dimensionality(database);
 
-    for(int i = 0; i < clusters.size() - 1; i++) {
+    for (int i = 0; i < clusters.size() - 1; i++) {
       Cluster<SubspaceModel<V>> c_i = clusters.get(i);
       int subspaceDim_i = dimensionality - c_i.getModel().getSubspace().dimensionality();
-      V ci_centroid = DatabaseUtil.centroid(database, c_i.getIDs(), c_i.getModel().getDimensions());
+      V ci_centroid = ProjectedCentroid.make(c_i.getModel().getDimensions(), database, c_i.getIDs()).toVector(database);
 
-      for(int j = i + 1; j < clusters.size(); j++) {
+      for (int j = i + 1; j < clusters.size(); j++) {
         Cluster<SubspaceModel<V>> c_j = clusters.get(j);
         int subspaceDim_j = dimensionality - c_j.getModel().getSubspace().dimensionality();
 
-        if(subspaceDim_i < subspaceDim_j) {
-          if(logger.isDebugging()) {
-            msg.append("\n l_i=").append(subspaceDim_i).append(" pv_i=[").append(FormatUtil.format(db_dim, c_i.getModel().getSubspace().getDimensions())).append("]");
-            msg.append("\n l_j=").append(subspaceDim_j).append(" pv_j=[").append(FormatUtil.format(db_dim, c_j.getModel().getSubspace().getDimensions())).append("]");
+        if (subspaceDim_i < subspaceDim_j) {
+          if (LOG.isDebugging()) {
+            msg.append("\n l_i=").append(subspaceDim_i).append(" pv_i=[").append(FormatUtil.format(db_dim, c_i.getModel().getSubspace().getDimensions())).append(']');
+            msg.append("\n l_j=").append(subspaceDim_j).append(" pv_j=[").append(FormatUtil.format(db_dim, c_j.getModel().getSubspace().getDimensions())).append(']');
           }
 
           // noise level reached
-          if(c_j.getModel().getSubspace().dimensionality() == 0) {
+          if (c_j.getModel().getSubspace().dimensionality() == 0) {
             // no parents exists -> parent is noise
-            if(c_i.getParents().isEmpty()) {
+            if (c_i.getParents().isEmpty()) {
               c_j.getChildren().add(c_i);
               c_i.getParents().add(c_j);
-              if(logger.isDebugging()) {
+              if (LOG.isDebugging()) {
                 msg.append("\n [").append(FormatUtil.format(db_dim, c_j.getModel().getSubspace().getDimensions()));
                 msg.append("] is parent of [").append(FormatUtil.format(db_dim, c_i.getModel().getSubspace().getDimensions()));
-                msg.append("]");
+                msg.append(']');
               }
             }
-          }
-          else {
-            V cj_centroid = DatabaseUtil.centroid(database, c_j.getIDs(), c_j.getModel().getDimensions());
+          } else {
+            V cj_centroid = ProjectedCentroid.make(c_j.getModel().getDimensions(), database, c_j.getIDs()).toVector(database);
             PreferenceVectorBasedCorrelationDistance distance = distFunc.correlationDistance(ci_centroid, cj_centroid, c_i.getModel().getSubspace().getDimensions(), c_j.getModel().getSubspace().getDimensions());
             double d = distFunc.weightedDistance(ci_centroid, cj_centroid, distance.getCommonPreferenceVector());
-            if(logger.isDebugging()) {
+            if (LOG.isDebugging()) {
               msg.append("\n dist = ").append(distance.getCorrelationValue());
             }
 
-            if(distance.getCorrelationValue() == subspaceDim_j) {
-              if(logger.isDebugging()) {
+            if (distance.getCorrelationValue() == subspaceDim_j) {
+              if (LOG.isDebugging()) {
                 msg.append("\n d = ").append(d);
               }
-              if(d <= 2 * epsilon) {
+              if (d <= 2 * epsilon) {
                 // no parent exists or c_j is not a parent of the already
                 // existing parents
-                if(c_i.getParents().isEmpty() || !isParent(database, distFunc, c_j, c_i.getParents())) {
+                if (c_i.getParents().isEmpty() || !isParent(database, distFunc, c_j, c_i.getParents())) {
                   c_j.getChildren().add(c_i);
                   c_i.getParents().add(c_j);
-                  if(logger.isDebugging()) {
+                  if (LOG.isDebugging()) {
                     msg.append("\n [").append(FormatUtil.format(db_dim, c_j.getModel().getSubspace().getDimensions()));
                     msg.append("] is parent of [");
                     msg.append(FormatUtil.format(db_dim, c_i.getModel().getSubspace().getDimensions()));
-                    msg.append("]");
+                    msg.append(']');
                   }
                 }
-              }
-              else {
+              } else {
                 throw new RuntimeException("Should never happen: d = " + d);
               }
             }
@@ -581,8 +578,8 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
         }
       }
     }
-    if(logger.isDebugging()) {
-      logger.debug(msg.toString());
+    if (LOG.isDebugging()) {
+      LOG.debug(msg.toString());
     }
   }
 
@@ -599,14 +596,14 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    *         the children clusters, false otherwise
    */
   private boolean isParent(Relation<V> database, DiSHDistanceFunction.Instance<V> distFunc, Cluster<SubspaceModel<V>> parent, List<Cluster<SubspaceModel<V>>> children) {
-    V parent_centroid = DatabaseUtil.centroid(database, parent.getIDs(), parent.getModel().getDimensions());
-    int dimensionality = DatabaseUtil.dimensionality(database);
+    V parent_centroid = ProjectedCentroid.make(parent.getModel().getDimensions(), database, parent.getIDs()).toVector(database);
+    int dimensionality = RelationUtil.dimensionality(database);
     int subspaceDim_parent = dimensionality - parent.getModel().getSubspace().dimensionality();
 
-    for(Cluster<SubspaceModel<V>> child : children) {
-      V child_centroid = DatabaseUtil.centroid(database, child.getIDs(), child.getModel().getDimensions());
+    for (Cluster<SubspaceModel<V>> child : children) {
+      V child_centroid = ProjectedCentroid.make(child.getModel().getDimensions(), database, child.getIDs()).toVector(database);
       PreferenceVectorBasedCorrelationDistance distance = distFunc.correlationDistance(parent_centroid, child_centroid, parent.getModel().getSubspace().getDimensions(), child.getModel().getSubspace().getDimensions());
-      if(distance.getCorrelationValue() == subspaceDim_parent) {
+      if (distance.getCorrelationValue() == subspaceDim_parent) {
         return true;
       }
     }
@@ -621,7 +618,7 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -631,7 +628,7 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     protected double epsilon = 0.0;
 
     protected int mu = 1;
@@ -644,14 +641,16 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
 
-      DoubleParameter epsilonP = new DoubleParameter(EPSILON_ID, new GreaterEqualConstraint(0), 0.001);
-      if(config.grab(epsilonP)) {
-        epsilon = epsilonP.getValue();
+      DoubleParameter epsilonP = new DoubleParameter(EPSILON_ID, 0.001);
+      epsilonP.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(epsilonP)) {
+        epsilon = epsilonP.doubleValue();
       }
 
-      IntParameter muP = new IntParameter(MU_ID, new GreaterConstraint(0), 1);
-      if(config.grab(muP)) {
-        mu = muP.getValue();
+      IntParameter muP = new IntParameter(MU_ID, 1);
+      muP.addConstraint(new GreaterConstraint(0));
+      if (config.grab(muP)) {
+        mu = muP.intValue();
       }
 
       configDiSHDistance(config, epsilon, mu);
@@ -703,4 +702,4 @@ public class DiSH<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Cluste
       return new DiSH<V>(epsilon, dishDistance, opticsO);
     }
   }
-}
\ No newline at end of file
+}
