@@ -23,15 +23,17 @@ package de.lmu.ifi.dbs.elki.algorithm.clustering.subspace;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.iterator.TIntIterator;
+import gnu.trove.set.TIntSet;
+import gnu.trove.set.hash.TIntHashSet;
+
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Random;
-import java.util.Set;
 
 import de.lmu.ifi.dbs.elki.algorithm.clustering.AbstractProjectedClustering;
 import de.lmu.ifi.dbs.elki.data.Cluster;
@@ -47,16 +49,20 @@ import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
-import de.lmu.ifi.dbs.elki.database.query.GenericDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.IndefiniteProgress;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.math.Mean;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -64,7 +70,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.CTriple;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
@@ -90,11 +96,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("PROCLUS: PROjected CLUStering")
 @Description("Algorithm to find subspace clusters in high dimensional spaces.")
 @Reference(authors = "C. C. Aggarwal, C. Procopiuc, J. L. Wolf, P. S. Yu, J. S. Park", title = "Fast Algorithms for Projected Clustering", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD '99)", url = "http://dx.doi.org/10.1145/304181.304188")
-public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClustering<Clustering<SubspaceModel<V>>, V> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
+public class PROCLUS<V extends NumberVector<?>> extends AbstractProjectedClustering<Clustering<SubspaceModel<V>>, V> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(PROCLUS.class);
+  private static final Logging LOG = Logging.getLogger(PROCLUS.class);
 
   /**
    * Parameter to specify the multiplier for the initial number of medoids, must
@@ -106,12 +112,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * Key: {@code -proclus.mi}
    * </p>
    */
-  public static final OptionID M_I_ID = OptionID.getOrCreateOptionID("proclus.mi", "The multiplier for the initial number of medoids.");
-
-  /**
-   * Parameter to specify the random generator seed.
-   */
-  public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("proclus.seed", "The random number generator seed.");
+  public static final OptionID M_I_ID = new OptionID("proclus.mi", "The multiplier for the initial number of medoids.");
 
   /**
    * Holds the value of {@link #M_I_ID}.
@@ -119,9 +120,9 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
   private int m_i;
 
   /**
-   * Holds the value of {@link #SEED_ID}.
+   * Random generator
    */
-  private Long seed;
+  private RandomFactory rnd;
 
   /**
    * Java constructor.
@@ -130,12 +131,12 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @param k_i k_i Parameter
    * @param l l Parameter
    * @param m_i m_i Parameter
-   * @param seed Random generator seed
+   * @param rnd Random generator
    */
-  public PROCLUS(int k, int k_i, int l, int m_i, Long seed) {
+  public PROCLUS(int k, int k_i, int l, int m_i, RandomFactory rnd) {
     super(k, k_i, l);
     this.m_i = m_i;
-    this.seed = seed;
+    this.rnd = rnd;
   }
 
   /**
@@ -147,19 +148,16 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
   public Clustering<SubspaceModel<V>> run(Database database, Relation<V> relation) {
     DistanceQuery<V, DoubleDistance> distFunc = this.getDistanceQuery(database);
     RangeQuery<V, DoubleDistance> rangeQuery = database.getRangeQuery(distFunc);
-    final Random random = new Random();
-    if(seed != null) {
-      random.setSeed(seed);
-    }
+    final Random random = rnd.getRandom();
 
-    if(DatabaseUtil.dimensionality(relation) < l) {
-      throw new IllegalStateException("Dimensionality of data < parameter l! " + "(" + DatabaseUtil.dimensionality(relation) + " < " + l + ")");
+    if(RelationUtil.dimensionality(relation) < l) {
+      throw new IllegalStateException("Dimensionality of data < parameter l! " + "(" + RelationUtil.dimensionality(relation) + " < " + l + ")");
     }
 
     // TODO: use a StepProgress!
     // initialization phase
-    if(logger.isVerbose()) {
-      logger.verbose("1. Initialization phase...");
+    if(LOG.isVerbose()) {
+      LOG.verbose("1. Initialization phase...");
     }
     int sampleSize = Math.min(relation.size(), k_i * k);
     DBIDs sampleSet = DBIDUtil.randomSample(relation.getDBIDs(), sampleSize, random.nextLong());
@@ -167,39 +165,39 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     int medoidSize = Math.min(relation.size(), m_i * k);
     DBIDs medoids = greedy(distFunc, sampleSet, medoidSize, random);
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
-      msg.append("\n");
-      msg.append("sampleSize ").append(sampleSize).append("\n");
-      msg.append("sampleSet ").append(sampleSet).append("\n");
-      msg.append("medoidSize ").append(medoidSize).append("\n");
-      msg.append("m ").append(medoids).append("\n");
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
+      msg.append('\n');
+      msg.append("sampleSize ").append(sampleSize).append('\n');
+      msg.append("sampleSet ").append(sampleSet).append('\n');
+      msg.append("medoidSize ").append(medoidSize).append('\n');
+      msg.append("m ").append(medoids).append('\n');
+      LOG.debugFine(msg.toString());
     }
 
     // iterative phase
-    if(logger.isVerbose()) {
-      logger.verbose("2. Iterative phase...");
+    if(LOG.isVerbose()) {
+      LOG.verbose("2. Iterative phase...");
     }
     double bestObjective = Double.POSITIVE_INFINITY;
     ModifiableDBIDs m_best = null;
     ModifiableDBIDs m_bad = null;
     ModifiableDBIDs m_current = initialSet(medoids, k, random);
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
-      msg.append("\n");
-      msg.append("m_c ").append(m_current).append("\n");
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
+      msg.append('\n');
+      msg.append("m_c ").append(m_current).append('\n');
+      LOG.debugFine(msg.toString());
     }
 
-    IndefiniteProgress cprogress = logger.isVerbose() ? new IndefiniteProgress("Current number of clusters:", logger) : null;
+    IndefiniteProgress cprogress = LOG.isVerbose() ? new IndefiniteProgress("Current number of clusters:", LOG) : null;
 
     // TODO: Use DataStore and Trove for performance
     Map<DBID, PROCLUSCluster> clusters = null;
     int loops = 0;
     while(loops < 10) {
-      Map<DBID, Set<Integer>> dimensions = findDimensions(m_current, relation, distFunc, rangeQuery);
+      Map<DBID, TIntSet> dimensions = findDimensions(m_current, relation, distFunc, rangeQuery);
       clusters = assignPoints(dimensions, relation);
       double objectiveFunction = evaluateClusters(clusters, dimensions, relation);
 
@@ -214,20 +212,20 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       m_current = computeM_current(medoids, m_best, m_bad, random);
       loops++;
       if(cprogress != null) {
-        cprogress.setProcessed(clusters.size(), logger);
+        cprogress.setProcessed(clusters.size(), LOG);
       }
     }
 
     if(cprogress != null) {
-      cprogress.setCompleted(logger);
+      cprogress.setCompleted(LOG);
     }
 
     // refinement phase
-    if(logger.isVerbose()) {
-      logger.verbose("3. Refinement phase...");
+    if(LOG.isVerbose()) {
+      LOG.verbose("3. Refinement phase...");
     }
 
-    List<Pair<V, Set<Integer>>> dimensions = findDimensions(new ArrayList<PROCLUSCluster>(clusters.values()), relation);
+    List<Pair<V, TIntSet>> dimensions = findDimensions(new ArrayList<PROCLUSCluster>(clusters.values()), relation);
     List<PROCLUSCluster> finalClusters = finalAssignment(dimensions, relation);
 
     // build result
@@ -235,7 +233,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     Clustering<SubspaceModel<V>> result = new Clustering<SubspaceModel<V>>("ProClus clustering", "proclus-clustering");
     for(PROCLUSCluster c : finalClusters) {
       Cluster<SubspaceModel<V>> cluster = new Cluster<SubspaceModel<V>>(c.objectIDs);
-      cluster.setModel(new SubspaceModel<V>(new Subspace<V>(c.getDimensions()), c.centroid));
+      cluster.setModel(new SubspaceModel<V>(new Subspace(c.getDimensions()), c.centroid));
       cluster.setName("cluster_" + numClusters++);
 
       result.addCluster(cluster);
@@ -259,40 +257,41 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     // m_1 is random point of S
     DBID m_i = s.remove(random.nextInt(s.size()));
     medoids.add(m_i);
-    if(logger.isDebugging()) {
-      logger.debugFiner("medoids " + medoids);
+    if(LOG.isDebugging()) {
+      LOG.debugFiner("medoids " + medoids);
     }
 
     // compute distances between each point in S and m_i
-    Map<DBID, DistanceResultPair<DoubleDistance>> distances = new HashMap<DBID, DistanceResultPair<DoubleDistance>>();
+    // FIXME: don't use maps, so we can work with DBIDRef
+    Map<DBID, DistanceDBIDPair<DoubleDistance>> distances = new HashMap<DBID, DistanceDBIDPair<DoubleDistance>>();
     for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
-      DBID id = iter.getDBID();
+      DBID id = DBIDUtil.deref(iter);
       DoubleDistance dist = distFunc.distance(id, m_i);
-      distances.put(id, new GenericDistanceResultPair<DoubleDistance>(dist, id));
+      distances.put(id, DBIDUtil.newDistancePair(dist, id));
     }
 
     for(int i = 1; i < m; i++) {
       // choose medoid m_i to be far from prevois medoids
-      List<DistanceResultPair<DoubleDistance>> d = new ArrayList<DistanceResultPair<DoubleDistance>>(distances.values());
-      Collections.sort(d);
+      List<DistanceDBIDPair<DoubleDistance>> d = new ArrayList<DistanceDBIDPair<DoubleDistance>>(distances.values());
+      DistanceDBIDResultUtil.sortByDistance(d);
 
-      m_i = d.get(d.size() - 1).getDBID();
+      m_i = DBIDUtil.deref(d.get(d.size() - 1));
       medoids.add(m_i);
       s.remove(m_i);
       distances.remove(m_i);
 
       // compute distances of each point to closest medoid
       for(DBIDIter iter = s.iter(); iter.valid(); iter.advance()) {
-        DBID id = iter.getDBID();
+        DBID id = DBIDUtil.deref(iter);
         DoubleDistance dist_new = distFunc.distance(id, m_i);
         DoubleDistance dist_old = distances.get(id).getDistance();
 
         DoubleDistance dist = dist_new.compareTo(dist_old) < 0 ? dist_new : dist_old;
-        distances.put(id, new GenericDistanceResultPair<DoubleDistance>(dist, id));
+        distances.put(id, DBIDUtil.newDistancePair(dist, id));
       }
 
-      if(logger.isDebugging()) {
-        logger.debugFiner("medoids " + medoids);
+      if(LOG.isDebugging()) {
+        LOG.debugFiner("medoids " + medoids);
       }
     }
 
@@ -332,7 +331,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     ModifiableDBIDs m_current = DBIDUtil.newHashSet();
     for(DBIDIter iter = m_best.iter(); iter.valid(); iter.advance()) {
-      DBID m_i = iter.getDBID();
+      DBID m_i = DBIDUtil.deref(iter);
       if(m_bad.contains(m_i)) {
         int currentSize = m_current.size();
         while(m_current.size() == currentSize) {
@@ -359,17 +358,17 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @param distFunc the distance function
    * @return a mapping of the medoid's id to its locality
    */
-  private Map<DBID, List<DistanceResultPair<DoubleDistance>>> getLocalities(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
-    Map<DBID, List<DistanceResultPair<DoubleDistance>>> result = new HashMap<DBID, List<DistanceResultPair<DoubleDistance>>>();
+  private Map<DBID, DistanceDBIDResult<DoubleDistance>> getLocalities(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
+    Map<DBID, DistanceDBIDResult<DoubleDistance>> result = new HashMap<DBID, DistanceDBIDResult<DoubleDistance>>();
 
     for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
-      DBID m = iter.getDBID();
+      DBID m = DBIDUtil.deref(iter);
       // determine minimum distance between current medoid m and any other
       // medoid m_i
       DoubleDistance minDist = null;
       for(DBIDIter iter2 = medoids.iter(); iter2.valid(); iter2.advance()) {
-        DBID m_i = iter2.getDBID();
-        if(m_i == m) {
+        DBID m_i = DBIDUtil.deref(iter2);
+        if(DBIDUtil.equal(m_i, m)) {
           continue;
         }
         DoubleDistance currentDist = distFunc.distance(m, m_i);
@@ -380,7 +379,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
       // determine points in sphere centered at m with radius minDist
       assert minDist != null;
-      List<DistanceResultPair<DoubleDistance>> qr = rangeQuery.getRangeForDBID(m, minDist);
+      DistanceDBIDResult<DoubleDistance> qr = rangeQuery.getRangeForDBID(m, minDist);
       result.put(m, qr);
     }
 
@@ -397,23 +396,23 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @return the set of correlated dimensions for each medoid in the specified
    *         medoid set
    */
-  private Map<DBID, Set<Integer>> findDimensions(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
+  private Map<DBID, TIntSet> findDimensions(DBIDs medoids, Relation<V> database, DistanceQuery<V, DoubleDistance> distFunc, RangeQuery<V, DoubleDistance> rangeQuery) {
     // get localities
-    Map<DBID, List<DistanceResultPair<DoubleDistance>>> localities = getLocalities(medoids, database, distFunc, rangeQuery);
+    Map<DBID, DistanceDBIDResult<DoubleDistance>> localities = getLocalities(medoids, database, distFunc, rangeQuery);
 
     // compute x_ij = avg distance from points in l_i to medoid m_i
-    int dim = DatabaseUtil.dimensionality(database);
+    int dim = RelationUtil.dimensionality(database);
     Map<DBID, double[]> averageDistances = new HashMap<DBID, double[]>();
 
     for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
-      DBID m_i = iter.getDBID();
+      DBID m_i = DBIDUtil.deref(iter);
       V medoid_i = database.get(m_i);
-      List<DistanceResultPair<DoubleDistance>> l_i = localities.get(m_i);
+      DistanceDBIDResult<DoubleDistance> l_i = localities.get(m_i);
       double[] x_i = new double[dim];
-      for(DistanceResultPair<DoubleDistance> qr : l_i) {
-        V o = database.get(qr.getDBID());
+      for(DBIDIter qr = l_i.iter(); qr.valid(); qr.advance()) {
+        V o = database.get(qr);
         for(int d = 0; d < dim; d++) {
-          x_i[d] += Math.abs(medoid_i.doubleValue(d + 1) - o.doubleValue(d + 1));
+          x_i[d] += Math.abs(medoid_i.doubleValue(d) - o.doubleValue(d));
         }
       }
       for(int d = 0; d < dim; d++) {
@@ -422,11 +421,11 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       averageDistances.put(m_i, x_i);
     }
 
-    Map<DBID, Set<Integer>> dimensionMap = new HashMap<DBID, Set<Integer>>();
+    Map<DBID, TIntSet> dimensionMap = new HashMap<DBID, TIntSet>();
     List<CTriple<Double, DBID, Integer>> z_ijs = new ArrayList<CTriple<Double, DBID, Integer>>();
     for(DBIDIter iter = medoids.iter(); iter.valid(); iter.advance()) {
-      DBID m_i = iter.getDBID();
-      Set<Integer> dims_i = new HashSet<Integer>();
+      DBID m_i = DBIDUtil.deref(iter);
+      TIntSet dims_i = new TIntHashSet();
       dimensionMap.put(m_i, dims_i);
 
       double[] x_i = averageDistances.get(m_i);
@@ -447,7 +446,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       sigma_i = Math.sqrt(sigma_i);
 
       for(int j = 0; j < dim; j++) {
-        z_ijs.add(new CTriple<Double, DBID, Integer>((x_i[j] - y_i) / sigma_i, m_i, j + 1));
+        z_ijs.add(new CTriple<Double, DBID, Integer>((x_i[j] - y_i) / sigma_i, m_i, j));
       }
     }
     Collections.sort(z_ijs);
@@ -455,15 +454,15 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     int max = Math.max(k * l, 2);
     for(int m = 0; m < max; m++) {
       CTriple<Double, DBID, Integer> z_ij = z_ijs.get(m);
-      Set<Integer> dims_i = dimensionMap.get(z_ij.getSecond());
+      TIntSet dims_i = dimensionMap.get(z_ij.getSecond());
       dims_i.add(z_ij.getThird());
 
-      if(logger.isDebugging()) {
-        StringBuffer msg = new StringBuffer();
-        msg.append("\n");
-        msg.append("z_ij ").append(z_ij).append("\n");
-        msg.append("D_i ").append(dims_i).append("\n");
-        logger.debugFiner(msg.toString());
+      if(LOG.isDebugging()) {
+        StringBuilder msg = new StringBuilder();
+        msg.append('\n');
+        msg.append("z_ij ").append(z_ij).append('\n');
+        msg.append("D_i ").append(dims_i).append('\n');
+        LOG.debugFiner(msg.toString());
       }
     }
     return dimensionMap;
@@ -478,9 +477,9 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @return the set of correlated dimensions for each specified cluster
    *         centroid
    */
-  private List<Pair<V, Set<Integer>>> findDimensions(List<PROCLUSCluster> clusters, Relation<V> database) {
+  private List<Pair<V, TIntSet>> findDimensions(List<PROCLUSCluster> clusters, Relation<V> database) {
     // compute x_ij = avg distance from points in c_i to c_i.centroid
-    int dim = DatabaseUtil.dimensionality(database);
+    int dim = RelationUtil.dimensionality(database);
     Map<Integer, double[]> averageDistances = new HashMap<Integer, double[]>();
 
     for(int i = 0; i < clusters.size(); i++) {
@@ -489,7 +488,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       for(DBIDIter iter = c_i.objectIDs.iter(); iter.valid(); iter.advance()) {
         V o = database.get(iter);
         for(int d = 0; d < dim; d++) {
-          x_i[d] += Math.abs(c_i.centroid.doubleValue(d + 1) - o.doubleValue(d + 1));
+          x_i[d] += Math.abs(c_i.centroid.doubleValue(d) - o.doubleValue(d));
         }
       }
       for(int d = 0; d < dim; d++) {
@@ -518,38 +517,38 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
       sigma_i = Math.sqrt(sigma_i);
 
       for(int j = 0; j < dim; j++) {
-        z_ijs.add(new CTriple<Double, Integer, Integer>((x_i[j] - y_i) / sigma_i, i, j + 1));
+        z_ijs.add(new CTriple<Double, Integer, Integer>((x_i[j] - y_i) / sigma_i, i, j));
       }
     }
     Collections.sort(z_ijs);
 
     // mapping cluster index -> dimensions
-    Map<Integer, Set<Integer>> dimensionMap = new HashMap<Integer, Set<Integer>>();
+    Map<Integer, TIntSet> dimensionMap = new HashMap<Integer, TIntSet>();
     int max = Math.max(k * l, 2);
     for(int m = 0; m < max; m++) {
       CTriple<Double, Integer, Integer> z_ij = z_ijs.get(m);
-      Set<Integer> dims_i = dimensionMap.get(z_ij.getSecond());
+      TIntSet dims_i = dimensionMap.get(z_ij.getSecond());
       if(dims_i == null) {
-        dims_i = new HashSet<Integer>();
+        dims_i = new TIntHashSet();
         dimensionMap.put(z_ij.getSecond(), dims_i);
       }
       dims_i.add(z_ij.getThird());
 
-      if(logger.isDebugging()) {
-        StringBuffer msg = new StringBuffer();
-        msg.append("\n");
-        msg.append("z_ij ").append(z_ij).append("\n");
-        msg.append("D_i ").append(dims_i).append("\n");
-        logger.debugFiner(msg.toString());
+      if(LOG.isDebugging()) {
+        StringBuilder msg = new StringBuilder();
+        msg.append('\n');
+        msg.append("z_ij ").append(z_ij).append('\n');
+        msg.append("D_i ").append(dims_i).append('\n');
+        LOG.debugFiner(msg.toString());
       }
     }
 
     // mapping cluster -> dimensions
-    List<Pair<V, Set<Integer>>> result = new ArrayList<Pair<V, Set<Integer>>>();
+    List<Pair<V, TIntSet>> result = new ArrayList<Pair<V, TIntSet>>();
     for(int i : dimensionMap.keySet()) {
-      Set<Integer> dims_i = dimensionMap.get(i);
+      TIntSet dims_i = dimensionMap.get(i);
       PROCLUSCluster c_i = clusters.get(i);
-      result.add(new Pair<V, Set<Integer>>(c_i.centroid, dims_i));
+      result.add(new Pair<V, TIntSet>(c_i.centroid, dims_i));
     }
     return result;
   }
@@ -562,26 +561,26 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @param database the database containing the objects
    * @return the assignments of the object to the clusters
    */
-  private Map<DBID, PROCLUSCluster> assignPoints(Map<DBID, Set<Integer>> dimensions, Relation<V> database) {
+  private Map<DBID, PROCLUSCluster> assignPoints(Map<DBID, TIntSet> dimensions, Relation<V> database) {
     Map<DBID, ModifiableDBIDs> clusterIDs = new HashMap<DBID, ModifiableDBIDs>();
     for(DBID m_i : dimensions.keySet()) {
       clusterIDs.put(m_i, DBIDUtil.newHashSet());
     }
 
     for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
-      DBID p_id = it.getDBID();
+      DBID p_id = DBIDUtil.deref(it);
       V p = database.get(p_id);
-      DistanceResultPair<DoubleDistance> minDist = null;
+      DistanceDBIDPair<DoubleDistance> minDist = null;
       for(DBID m_i : dimensions.keySet()) {
         V m = database.get(m_i);
-        DistanceResultPair<DoubleDistance> currentDist = new GenericDistanceResultPair<DoubleDistance>(manhattanSegmentalDistance(p, m, dimensions.get(m_i)), m_i);
-        if(minDist == null || currentDist.compareTo(minDist) < 0) {
+        DistanceDBIDPair<DoubleDistance> currentDist = DBIDUtil.newDistancePair(manhattanSegmentalDistance(p, m, dimensions.get(m_i)), m_i);
+        if(minDist == null || currentDist.compareByDistance(minDist) < 0) {
           minDist = currentDist;
         }
       }
       // add p to cluster with mindist
       assert minDist != null;
-      ModifiableDBIDs ids = clusterIDs.get(minDist.getDBID());
+      ModifiableDBIDs ids = clusterIDs.get(DBIDUtil.deref(minDist));
       ids.add(p_id);
     }
 
@@ -589,17 +588,17 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     for(DBID m_i : dimensions.keySet()) {
       ModifiableDBIDs objectIDs = clusterIDs.get(m_i);
       if(!objectIDs.isEmpty()) {
-        Set<Integer> clusterDimensions = dimensions.get(m_i);
-        V centroid = DatabaseUtil.centroid(database, objectIDs);
+        TIntSet clusterDimensions = dimensions.get(m_i);
+        V centroid = Centroid.make(database, objectIDs).toVector(database);
         clusters.put(m_i, new PROCLUSCluster(objectIDs, clusterDimensions, centroid));
       }
     }
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
-      msg.append("\n");
-      msg.append("clusters ").append(clusters).append("\n");
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
+      msg.append('\n');
+      msg.append("clusters ").append(clusters).append('\n');
+      LOG.debugFine(msg.toString());
     }
     return clusters;
   }
@@ -612,20 +611,20 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @param database the database containing the objects
    * @return the assignments of the object to the clusters
    */
-  private List<PROCLUSCluster> finalAssignment(List<Pair<V, Set<Integer>>> dimensions, Relation<V> database) {
+  private List<PROCLUSCluster> finalAssignment(List<Pair<V, TIntSet>> dimensions, Relation<V> database) {
     Map<Integer, ModifiableDBIDs> clusterIDs = new HashMap<Integer, ModifiableDBIDs>();
     for(int i = 0; i < dimensions.size(); i++) {
       clusterIDs.put(i, DBIDUtil.newHashSet());
     }
 
     for(DBIDIter it = database.iterDBIDs(); it.valid(); it.advance()) {
-      DBID p_id = it.getDBID();
+      DBID p_id = DBIDUtil.deref(it);
       V p = database.get(p_id);
       Pair<DoubleDistance, Integer> minDist = null;
       for(int i = 0; i < dimensions.size(); i++) {
-        Pair<V, Set<Integer>> pair_i = dimensions.get(i);
+        Pair<V, TIntSet> pair_i = dimensions.get(i);
         V c_i = pair_i.first;
-        Set<Integer> dimensions_i = pair_i.second;
+        TIntSet dimensions_i = pair_i.second;
         DoubleDistance currentDist = manhattanSegmentalDistance(p, c_i, dimensions_i);
         if(minDist == null || currentDist.compareTo(minDist.first) < 0) {
           minDist = new Pair<DoubleDistance, Integer>(currentDist, i);
@@ -641,17 +640,17 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     for(int i = 0; i < dimensions.size(); i++) {
       ModifiableDBIDs objectIDs = clusterIDs.get(i);
       if(!objectIDs.isEmpty()) {
-        Set<Integer> clusterDimensions = dimensions.get(i).second;
-        V centroid = DatabaseUtil.centroid(database, objectIDs);
+        TIntSet clusterDimensions = dimensions.get(i).second;
+        V centroid = Centroid.make(database, objectIDs).toVector(database);
         clusters.add(new PROCLUSCluster(objectIDs, clusterDimensions, centroid));
       }
     }
 
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
-      msg.append("\n");
-      msg.append("clusters ").append(clusters).append("\n");
-      logger.debugFine(msg.toString());
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
+      msg.append('\n');
+      msg.append("clusters ").append(clusters).append('\n');
+      LOG.debugFine(msg.toString());
     }
     return clusters;
   }
@@ -666,9 +665,10 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @return the Manhattan segmental distance between o1 and o2 relative to the
    *         specified dimensions
    */
-  private DoubleDistance manhattanSegmentalDistance(V o1, V o2, Set<Integer> dimensions) {
+  private DoubleDistance manhattanSegmentalDistance(V o1, V o2, TIntSet dimensions) {
     double result = 0;
-    for(Integer d : dimensions) {
+    for (TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
+      final int d = iter.next();
       result += Math.abs(o1.doubleValue(d) - o2.doubleValue(d));
     }
     result /= dimensions.size();
@@ -683,15 +683,16 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * @param database the database holding the objects
    * @return a measure for the cluster quality
    */
-  private double evaluateClusters(Map<DBID, PROCLUSCluster> clusters, Map<DBID, Set<Integer>> dimensions, Relation<V> database) {
+  private double evaluateClusters(Map<DBID, PROCLUSCluster> clusters, Map<DBID, TIntSet> dimensions, Relation<V> database) {
     double result = 0;
     for(DBID m_i : clusters.keySet()) {
       PROCLUSCluster c_i = clusters.get(m_i);
       V centroid_i = c_i.centroid;
 
-      Set<Integer> dims_i = dimensions.get(m_i);
+      TIntSet dims_i = dimensions.get(m_i);
       double w_i = 0;
-      for(Integer j : dims_i) {
+      for (TIntIterator iter = dims_i.iterator(); iter.hasNext(); ) {
+        final int j = iter.next();
         w_i += avgDistance(centroid_i, c_i.objectIDs, database, j);
       }
 
@@ -714,12 +715,12 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    *         specified dimension
    */
   private double avgDistance(V centroid, DBIDs objectIDs, Relation<V> database, int dimension) {
-    double avg = 0;
+    Mean avg = new Mean();
     for(DBIDIter iter = objectIDs.iter(); iter.valid(); iter.advance()) {
       V o = database.get(iter);
-      avg += Math.abs(centroid.doubleValue(dimension) - o.doubleValue(dimension));
+      avg.put(Math.abs(centroid.doubleValue(dimension) - o.doubleValue(dimension)));
     }
-    return avg / objectIDs.size();
+    return avg.getMean();
   }
 
   /**
@@ -748,7 +749,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -765,7 +766,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
     /**
      * The correlated dimensions of this cluster.
      */
-    Set<Integer> dimensions;
+    TIntSet dimensions;
 
     /**
      * The centroids of this cluster along each dimension.
@@ -779,7 +780,7 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
      * @param dimensions the correlated dimensions of this cluster
      * @param centroid the centroid of this cluster
      */
-    public PROCLUSCluster(ModifiableDBIDs objectIDs, Set<Integer> dimensions, V centroid) {
+    public PROCLUSCluster(ModifiableDBIDs objectIDs, TIntSet dimensions, V centroid) {
       this.objectIDs = objectIDs;
       this.dimensions = dimensions;
       this.centroid = centroid;
@@ -787,19 +788,19 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
 
     @Override
     public String toString() {
-      StringBuffer result = new StringBuffer();
+      StringBuilder result = new StringBuilder();
       result.append("Dimensions: [");
       boolean notFirst = false;
-      for(Integer d : dimensions) {
+      for(TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
         if(notFirst) {
-          result.append(",");
+          result.append(',');
         }
         else {
           notFirst = true;
         }
-        result.append(d);
+        result.append(iter.next());
       }
-      result.append("]");
+      result.append(']');
 
       result.append("\nCentroid: ").append(centroid);
       return result.toString();
@@ -812,8 +813,8 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
      */
     public BitSet getDimensions() {
       BitSet result = new BitSet();
-      for(int d : dimensions) {
-        result.set(d - 1);
+      for(TIntIterator iter = dimensions.iterator(); iter.hasNext(); ) {
+        result.set(iter.next());
       }
       return result;
     }
@@ -826,10 +827,15 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractProjectedClustering.Parameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractProjectedClustering.Parameterizer {
+    /**
+     * Parameter to specify the random generator seed.
+     */
+    public static final OptionID SEED_ID = new OptionID("proclus.seed", "The random number generator seed.");
+
     protected int m_i = -1;
 
-    protected Long seed = null;
+    protected RandomFactory rnd;
 
     @Override
     protected void makeOptions(Parameterization config) {
@@ -845,15 +851,15 @@ public class PROCLUS<V extends NumberVector<V, ?>> extends AbstractProjectedClus
         m_i = m_iP.getValue();
       }
 
-      LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        seed = seedP.getValue();
+      RandomParameter rndP = new RandomParameter(SEED_ID);
+      if(config.grab(rndP)) {
+        rnd = rndP.getValue();
       }
     }
 
     @Override
     protected PROCLUS<V> makeInstance() {
-      return new PROCLUS<V>(k, k_i, l, m_i, seed);
+      return new PROCLUS<V>(k, k_i, l, m_i, rnd);
     }
   }
 }
\ No newline at end of file
