@@ -42,12 +42,13 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.ProxyDatabase;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.AbstractDimensionsSelectingDoubleDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.SubspaceEuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.Centroid;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -84,11 +85,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 @Title("SUBCLU: Density connected Subspace Clustering")
 @Description("Algorithm to detect arbitrarily shaped and positioned clusters in subspaces. SUBCLU delivers for each subspace the same clusters DBSCAN would have found, when applied to this subspace seperately.")
 @Reference(authors = "K. Kailing, H.-P. Kriegel, P. Kröger", title = "Density connected Subspace Clustering for High Dimensional Data. ", booktitle = "Proc. SIAM Int. Conf. on Data Mining (SDM'04), Lake Buena Vista, FL, 2004")
-public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
+public class SUBCLU<V extends NumberVector<?>> extends AbstractAlgorithm<Clustering<SubspaceModel<V>>> implements SubspaceClusteringAlgorithm<SubspaceModel<V>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(SUBCLU.class);
+  private static final Logging LOG = Logging.getLogger(SUBCLU.class);
 
   /**
    * The distance function to determine the distance between database objects.
@@ -99,7 +100,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * Key: {@code -subclu.distancefunction}
    * </p>
    */
-  public static final OptionID DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("subclu.distancefunction", "Distance function to determine the distance between database objects.");
+  public static final OptionID DISTANCE_FUNCTION_ID = new OptionID("subclu.distancefunction", "Distance function to determine the distance between database objects.");
 
   /**
    * Parameter to specify the maximum radius of the neighborhood to be
@@ -109,7 +110,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * Key: {@code -subclu.epsilon}
    * </p>
    */
-  public static final OptionID EPSILON_ID = OptionID.getOrCreateOptionID("subclu.epsilon", "The maximum radius of the neighborhood to be considered.");
+  public static final OptionID EPSILON_ID = new OptionID("subclu.epsilon", "The maximum radius of the neighborhood to be considered.");
 
   /**
    * Parameter to specify the threshold for minimum number of points in the
@@ -118,7 +119,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * Key: {@code -subclu.minpts}
    * </p>
    */
-  public static final OptionID MINPTS_ID = OptionID.getOrCreateOptionID("subclu.minpts", "Threshold for minimum number of points in the epsilon-neighborhood of a point.");
+  public static final OptionID MINPTS_ID = new OptionID("subclu.minpts", "Threshold for minimum number of points in the epsilon-neighborhood of a point.");
 
   /**
    * Holds the instance of the distance function specified by
@@ -162,36 +163,36 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * @return Clustering result
    */
   public Clustering<SubspaceModel<V>> run(Relation<V> relation) {
-    final int dimensionality = DatabaseUtil.dimensionality(relation);
+    final int dimensionality = RelationUtil.dimensionality(relation);
 
-    StepProgress stepprog = logger.isVerbose() ? new StepProgress(dimensionality) : null;
+    StepProgress stepprog = LOG.isVerbose() ? new StepProgress(dimensionality) : null;
 
     // Generate all 1-dimensional clusters
     if(stepprog != null) {
-      stepprog.beginStep(1, "Generate all 1-dimensional clusters.", logger);
+      stepprog.beginStep(1, "Generate all 1-dimensional clusters.", LOG);
     }
 
     // mapping of dimensionality to set of subspaces
-    HashMap<Integer, List<Subspace<V>>> subspaceMap = new HashMap<Integer, List<Subspace<V>>>();
+    HashMap<Integer, List<Subspace>> subspaceMap = new HashMap<Integer, List<Subspace>>();
 
     // list of 1-dimensional subspaces containing clusters
-    List<Subspace<V>> s_1 = new ArrayList<Subspace<V>>();
+    List<Subspace> s_1 = new ArrayList<Subspace>();
     subspaceMap.put(0, s_1);
 
     // mapping of subspaces to list of clusters
-    TreeMap<Subspace<V>, List<Cluster<Model>>> clusterMap = new TreeMap<Subspace<V>, List<Cluster<Model>>>(new Subspace.DimensionComparator());
+    TreeMap<Subspace, List<Cluster<Model>>> clusterMap = new TreeMap<Subspace, List<Cluster<Model>>>(new Subspace.DimensionComparator());
 
     for(int d = 0; d < dimensionality; d++) {
-      Subspace<V> currentSubspace = new Subspace<V>(d);
+      Subspace currentSubspace = new Subspace(d);
       List<Cluster<Model>> clusters = runDBSCAN(relation, null, currentSubspace);
 
-      if(logger.isDebuggingFiner()) {
-        StringBuffer msg = new StringBuffer();
-        msg.append("\n").append(clusters.size()).append(" clusters in subspace ").append(currentSubspace.dimensonsToString()).append(": \n");
+      if(LOG.isDebuggingFiner()) {
+        StringBuilder msg = new StringBuilder();
+        msg.append('\n').append(clusters.size()).append(" clusters in subspace ").append(currentSubspace.dimensonsToString()).append(": \n");
         for(Cluster<Model> cluster : clusters) {
           msg.append("      " + cluster.getIDs() + "\n");
         }
-        logger.debugFiner(msg.toString());
+        LOG.debugFiner(msg.toString());
       }
 
       if(!clusters.isEmpty()) {
@@ -203,26 +204,26 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
     // Generate (d+1)-dimensional clusters from d-dimensional clusters
     for(int d = 0; d < dimensionality - 1; d++) {
       if(stepprog != null) {
-        stepprog.beginStep(d + 2, "Generate " + (d + 2) + "-dimensional clusters from " + (d + 1) + "-dimensional clusters.", logger);
+        stepprog.beginStep(d + 2, "Generate " + (d + 2) + "-dimensional clusters from " + (d + 1) + "-dimensional clusters.", LOG);
       }
 
-      List<Subspace<V>> subspaces = subspaceMap.get(d);
+      List<Subspace> subspaces = subspaceMap.get(d);
       if(subspaces == null || subspaces.isEmpty()) {
         if(stepprog != null) {
           for(int dim = d + 1; dim < dimensionality - 1; dim++) {
-            stepprog.beginStep(dim + 2, "Generation of" + (dim + 2) + "-dimensional clusters not applicable, because no more " + (d + 2) + "-dimensional subspaces found.", logger);
+            stepprog.beginStep(dim + 2, "Generation of" + (dim + 2) + "-dimensional clusters not applicable, because no more " + (d + 2) + "-dimensional subspaces found.", LOG);
           }
         }
         break;
       }
 
-      List<Subspace<V>> candidates = generateSubspaceCandidates(subspaces);
-      List<Subspace<V>> s_d = new ArrayList<Subspace<V>>();
+      List<Subspace> candidates = generateSubspaceCandidates(subspaces);
+      List<Subspace> s_d = new ArrayList<Subspace>();
 
-      for(Subspace<V> candidate : candidates) {
-        Subspace<V> bestSubspace = bestSubspace(subspaces, candidate, clusterMap);
-        if(logger.isDebuggingFine()) {
-          logger.debugFine("best subspace of " + candidate.dimensonsToString() + ": " + bestSubspace.dimensonsToString());
+      for(Subspace candidate : candidates) {
+        Subspace bestSubspace = bestSubspace(subspaces, candidate, clusterMap);
+        if(LOG.isDebuggingFine()) {
+          LOG.debugFine("best subspace of " + candidate.dimensonsToString() + ": " + bestSubspace.dimensonsToString());
         }
 
         List<Cluster<Model>> bestSubspaceClusters = clusterMap.get(bestSubspace);
@@ -234,13 +235,13 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
           }
         }
 
-        if(logger.isDebuggingFine()) {
-          StringBuffer msg = new StringBuffer();
+        if(LOG.isDebuggingFine()) {
+          StringBuilder msg = new StringBuilder();
           msg.append(clusters.size() + " cluster(s) in subspace " + candidate + ": \n");
           for(Cluster<Model> c : clusters) {
             msg.append("      " + c.getIDs() + "\n");
           }
-          logger.debugFine(msg.toString());
+          LOG.debugFine(msg.toString());
         }
 
         if(!clusters.isEmpty()) {
@@ -257,18 +258,18 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
     // build result
     int numClusters = 1;
     result = new Clustering<SubspaceModel<V>>("SUBCLU clustering", "subclu-clustering");
-    for(Subspace<V> subspace : clusterMap.descendingKeySet()) {
+    for(Subspace subspace : clusterMap.descendingKeySet()) {
       List<Cluster<Model>> clusters = clusterMap.get(subspace);
       for(Cluster<Model> cluster : clusters) {
         Cluster<SubspaceModel<V>> newCluster = new Cluster<SubspaceModel<V>>(cluster.getIDs());
-        newCluster.setModel(new SubspaceModel<V>(subspace, DatabaseUtil.centroid(relation, cluster.getIDs())));
+        newCluster.setModel(new SubspaceModel<V>(subspace, Centroid.make(relation, cluster.getIDs()).toVector(relation)));
         newCluster.setName("cluster_" + numClusters++);
         result.addCluster(newCluster);
       }
     }
 
     if(stepprog != null) {
-      stepprog.setCompleted(logger);
+      stepprog.setCompleted(LOG);
     }
     return result;
   }
@@ -294,7 +295,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * @param subspace the subspace to run DBSCAN on
    * @return the clustering result of the DBSCAN run
    */
-  private List<Cluster<Model>> runDBSCAN(Relation<V> relation, DBIDs ids, Subspace<V> subspace) {
+  private List<Cluster<Model>> runDBSCAN(Relation<V> relation, DBIDs ids, Subspace subspace) {
     // distance function
     distanceFunction.setSelectedDimensions(subspace.getDimensions());
 
@@ -309,8 +310,8 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
 
     DBSCAN<V, DoubleDistance> dbscan = new DBSCAN<V, DoubleDistance>(distanceFunction, epsilon, minpts);
     // run DBSCAN
-    if(logger.isVerbose()) {
-      logger.verbose("\nRun DBSCAN on subspace " + subspace.dimensonsToString());
+    if(LOG.isVerbose()) {
+      LOG.verbose("\nRun DBSCAN on subspace " + subspace.dimensonsToString());
     }
     Clustering<Model> dbsres = dbscan.run(proxy);
 
@@ -332,8 +333,8 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * @param subspaces the {@code d}-dimensional subspaces
    * @return the {@code d+1}-dimensional subspace candidates
    */
-  private List<Subspace<V>> generateSubspaceCandidates(List<Subspace<V>> subspaces) {
-    List<Subspace<V>> candidates = new ArrayList<Subspace<V>>();
+  private List<Subspace> generateSubspaceCandidates(List<Subspace> subspaces) {
+    List<Subspace> candidates = new ArrayList<Subspace>();
 
     if(subspaces.isEmpty()) {
       return candidates;
@@ -342,28 +343,28 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
     // Generate (d+1)-dimensional candidate subspaces
     int d = subspaces.get(0).dimensionality();
 
-    StringBuffer msgFine = new StringBuffer("\n");
-    if(logger.isDebuggingFiner()) {
-      msgFine.append("subspaces ").append(subspaces).append("\n");
+    StringBuilder msgFine = new StringBuilder("\n");
+    if(LOG.isDebuggingFiner()) {
+      msgFine.append("subspaces ").append(subspaces).append('\n');
     }
 
     for(int i = 0; i < subspaces.size(); i++) {
-      Subspace<V> s1 = subspaces.get(i);
+      Subspace s1 = subspaces.get(i);
       for(int j = i + 1; j < subspaces.size(); j++) {
-        Subspace<V> s2 = subspaces.get(j);
-        Subspace<V> candidate = s1.join(s2);
+        Subspace s2 = subspaces.get(j);
+        Subspace candidate = s1.join(s2);
 
         if(candidate != null) {
-          if(logger.isDebuggingFiner()) {
-            msgFine.append("candidate: ").append(candidate.dimensonsToString()).append("\n");
+          if(LOG.isDebuggingFiner()) {
+            msgFine.append("candidate: ").append(candidate.dimensonsToString()).append('\n');
           }
           // prune irrelevant candidate subspaces
-          List<Subspace<V>> lowerSubspaces = lowerSubspaces(candidate);
-          if(logger.isDebuggingFiner()) {
-            msgFine.append("lowerSubspaces: ").append(lowerSubspaces).append("\n");
+          List<Subspace> lowerSubspaces = lowerSubspaces(candidate);
+          if(LOG.isDebuggingFiner()) {
+            msgFine.append("lowerSubspaces: ").append(lowerSubspaces).append('\n');
           }
           boolean irrelevantCandidate = false;
-          for(Subspace<V> s : lowerSubspaces) {
+          for(Subspace s : lowerSubspaces) {
             if(!subspaces.contains(s)) {
               irrelevantCandidate = true;
               break;
@@ -376,16 +377,16 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
       }
     }
 
-    if(logger.isDebuggingFiner()) {
-      logger.debugFiner(msgFine.toString());
+    if(LOG.isDebuggingFiner()) {
+      LOG.debugFiner(msgFine.toString());
     }
-    if(logger.isDebugging()) {
-      StringBuffer msg = new StringBuffer();
+    if(LOG.isDebugging()) {
+      StringBuilder msg = new StringBuilder();
       msg.append(d + 1).append("-dimensional candidate subspaces: ");
-      for(Subspace<V> candidate : candidates) {
-        msg.append(candidate.dimensonsToString()).append(" ");
+      for(Subspace candidate : candidates) {
+        msg.append(candidate.dimensonsToString()).append(' ');
       }
-      logger.debug(msg.toString());
+      LOG.debug(msg.toString());
     }
 
     return candidates;
@@ -398,19 +399,19 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * @param subspace the {@code d}-dimensional subspace
    * @return a list of all {@code (d-1)}-dimensional subspaces
    */
-  private List<Subspace<V>> lowerSubspaces(Subspace<V> subspace) {
+  private List<Subspace> lowerSubspaces(Subspace subspace) {
     int dimensionality = subspace.dimensionality();
     if(dimensionality <= 1) {
       return null;
     }
 
     // order result according to the dimensions
-    List<Subspace<V>> result = new ArrayList<Subspace<V>>();
+    List<Subspace> result = new ArrayList<Subspace>();
     BitSet dimensions = subspace.getDimensions();
     for(int dim = dimensions.nextSetBit(0); dim >= 0; dim = dimensions.nextSetBit(dim + 1)) {
       BitSet newDimensions = (BitSet) dimensions.clone();
       newDimensions.set(dim, false);
-      result.add(new Subspace<V>(newDimensions));
+      result.add(new Subspace(newDimensions));
     }
 
     return result;
@@ -428,10 +429,10 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    *         -dimensional candidate with minimal number of objects in the
    *         cluster
    */
-  private Subspace<V> bestSubspace(List<Subspace<V>> subspaces, Subspace<V> candidate, TreeMap<Subspace<V>, List<Cluster<Model>>> clusterMap) {
-    Subspace<V> bestSubspace = null;
+  private Subspace bestSubspace(List<Subspace> subspaces, Subspace candidate, TreeMap<Subspace, List<Cluster<Model>>> clusterMap) {
+    Subspace bestSubspace = null;
 
-    for(Subspace<V> subspace : subspaces) {
+    for(Subspace subspace : subspaces) {
       int min = Integer.MAX_VALUE;
 
       if(subspace.isSubspace(candidate)) {
@@ -456,7 +457,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -466,7 +467,7 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     protected int minpts = 0;
 
     protected DoubleDistance epsilon = null;
@@ -486,7 +487,8 @@ public class SUBCLU<V extends NumberVector<V, ?>> extends AbstractAlgorithm<Clus
         epsilon = epsilonP.getValue();
       }
 
-      IntParameter minptsP = new IntParameter(MINPTS_ID, new GreaterConstraint(0));
+      IntParameter minptsP = new IntParameter(MINPTS_ID);
+      minptsP.addConstraint(new GreaterConstraint(0));
       if(config.grab(minptsP)) {
         minpts = minptsP.getValue();
       }
