@@ -46,7 +46,7 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * 
  * @param <V> the type of NumberVector this subspace contains
  */
-public class CLIQUESubspace<V extends NumberVector<V, ?>> extends Subspace<V> {
+public class CLIQUESubspace<V extends NumberVector<?>> extends Subspace {
   /**
    * The dense units belonging to this subspace.
    */
@@ -103,14 +103,14 @@ public class CLIQUESubspace<V extends NumberVector<V, ?>> extends Subspace<V> {
    * 
    * @return the clusters in this subspace and the corresponding cluster models
    */
-  public List<Pair<Subspace<V>, ModifiableDBIDs>> determineClusters() {
-    List<Pair<Subspace<V>, ModifiableDBIDs>> clusters = new ArrayList<Pair<Subspace<V>, ModifiableDBIDs>>();
+  public List<Pair<Subspace, ModifiableDBIDs>> determineClusters() {
+    List<Pair<Subspace, ModifiableDBIDs>> clusters = new ArrayList<Pair<Subspace, ModifiableDBIDs>>();
 
     for(CLIQUEUnit<V> unit : getDenseUnits()) {
       if(!unit.isAssigned()) {
         ModifiableDBIDs cluster = DBIDUtil.newHashSet();
         CLIQUESubspace<V> model = new CLIQUESubspace<V>(getDimensions());
-        clusters.add(new Pair<Subspace<V>, ModifiableDBIDs>(model, cluster));
+        clusters.add(new Pair<Subspace, ModifiableDBIDs>(model, cluster));
         dfs(unit, cluster, model);
       }
     }
@@ -151,8 +151,8 @@ public class CLIQUESubspace<V extends NumberVector<V, ?>> extends Subspace<V> {
    * @param dim the dimension
    * @return the left neighbor of the given unit in the specified dimension
    */
-  public CLIQUEUnit<V> leftNeighbor(CLIQUEUnit<V> unit, Integer dim) {
-    Interval i = unit.getInterval(dim);
+  public CLIQUEUnit<V> leftNeighbor(CLIQUEUnit<V> unit, int dim) {
+    Interval i = unit.getInterval(Integer.valueOf(dim));
 
     for(CLIQUEUnit<V> u : getDenseUnits()) {
       if(u.containsLeftNeighbor(i)) {
@@ -238,10 +238,10 @@ public class CLIQUESubspace<V extends NumberVector<V, ?>> extends Subspace<V> {
    */
   @Override
   public String toString(String pre) {
-    StringBuffer result = new StringBuffer();
+    StringBuilder result = new StringBuilder();
     result.append(super.toString(pre));
-    result.append("\n").append(pre).append("Coverage: ").append(coverage);
-    result.append("\n").append(pre).append("Units: " + "\n");
+    result.append('\n').append(pre).append("Coverage: ").append(coverage);
+    result.append('\n').append(pre).append("Units: " + "\n");
     for(CLIQUEUnit<V> denseUnit : getDenseUnits()) {
       result.append(pre).append("   ").append(denseUnit.toString()).append("   ").append(denseUnit.getIds().size()).append(" objects\n");
     }
