@@ -79,19 +79,19 @@ public class ByLabelClustering extends AbstractAlgorithm<Clustering<Model>> impl
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ByLabelClustering.class);
+  private static final Logging LOG = Logging.getLogger(ByLabelClustering.class);
 
   /**
    * Flag to indicate that multiple cluster assignment is possible. If an
    * assignment to multiple clusters is desired, the labels indicating the
    * clusters need to be separated by blanks.
    */
-  public static final OptionID MULTIPLE_ID = OptionID.getOrCreateOptionID("bylabelclustering.multiple", "Flag to indicate that only subspaces with large coverage " + "(i.e. the fraction of the database that is covered by the dense units) " + "are selected, the rest will be pruned.");
+  public static final OptionID MULTIPLE_ID = new OptionID("bylabelclustering.multiple", "Flag to indicate that only subspaces with large coverage " + "(i.e. the fraction of the database that is covered by the dense units) " + "are selected, the rest will be pruned.");
 
   /**
    * Pattern to recognize noise clusters by.
    */
-  public static final OptionID NOISE_ID = OptionID.getOrCreateOptionID("bylabelclustering.noise", "Pattern to recognize noise classes by their label.");
+  public static final OptionID NOISE_ID = new OptionID("bylabelclustering.noise", "Pattern to recognize noise classes by their label.");
 
   /**
    * Holds the value of {@link #MULTIPLE_ID}.
@@ -226,7 +226,7 @@ public class ByLabelClustering extends AbstractAlgorithm<Clustering<Model>> impl
       }
     }
     else {
-      labelMap.put(label, id.getDBID());
+      labelMap.put(label, DBIDUtil.deref(id));
     }
   }
 
@@ -237,7 +237,7 @@ public class ByLabelClustering extends AbstractAlgorithm<Clustering<Model>> impl
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -260,7 +260,8 @@ public class ByLabelClustering extends AbstractAlgorithm<Clustering<Model>> impl
         multiple = multipleF.getValue();
       }
 
-      PatternParameter noisepatP = new PatternParameter(NOISE_ID, true);
+      PatternParameter noisepatP = new PatternParameter(NOISE_ID);
+      noisepatP.setOptional(true);
       if(config.grab(noisepatP)) {
         noisepat = noisepatP.getValue();
       }
