@@ -73,7 +73,7 @@ public class ByLabelHierarchicalClustering extends AbstractAlgorithm<Clustering<
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ByLabelHierarchicalClustering.class);
+  private static final Logging LOG = Logging.getLogger(ByLabelHierarchicalClustering.class);
 
   /**
    * Constructor without parameters
@@ -178,7 +178,7 @@ public class ByLabelHierarchicalClustering extends AbstractAlgorithm<Clustering<
       }
     }
     else {
-      labelMap.put(label, id.getDBID());
+      labelMap.put(label, DBIDUtil.deref(id));
     }
   }
 
@@ -189,6 +189,6 @@ public class ByLabelHierarchicalClustering extends AbstractAlgorithm<Clustering<
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 }
\ No newline at end of file
