@@ -25,13 +25,11 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
 
 import java.util.Collections;
 import java.util.HashMap;
-import java.util.Iterator;
 
 import de.lmu.ifi.dbs.elki.algorithm.AbstractDistanceBasedAlgorithm;
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
-import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.QueryUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
@@ -42,13 +40,13 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.distance.similarityfunction.PrimitiveSimilarityFunction;
 import de.lmu.ifi.dbs.elki.distance.similarityfunction.kernel.KernelMatrix;
@@ -66,11 +64,11 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 
 /**
  * Angle-Based Outlier Detection
@@ -92,39 +90,39 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 @Title("ABOD: Angle-Based Outlier Detection")
 @Description("Outlier detection using variance analysis on angles, especially for high dimensional data sets.")
 @Reference(authors = "H.-P. Kriegel, M. Schubert, and A. Zimek", title = "Angle-Based Outlier Detection in High-dimensional Data", booktitle = "Proc. 14th ACM SIGKDD Int. Conf. on Knowledge Discovery and Data Mining (KDD '08), Las Vegas, NV, 2008", url = "http://dx.doi.org/10.1145/1401890.1401946")
-public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlgorithm<V, DoubleDistance, OutlierResult> implements OutlierAlgorithm {
+public class ABOD<V extends NumberVector<?>> extends AbstractDistanceBasedAlgorithm<V, DoubleDistance, OutlierResult> implements OutlierAlgorithm {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ABOD.class);
+  private static final Logging LOG = Logging.getLogger(ABOD.class);
 
   /**
    * Parameter for k, the number of neighbors used in kNN queries.
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("abod.k", "Parameter k for kNN queries.");
+  public static final OptionID K_ID = new OptionID("abod.k", "Parameter k for kNN queries.");
 
   /**
    * Parameter for sample size to be used in fast mode.
    */
-  public static final OptionID FAST_SAMPLE_ID = OptionID.getOrCreateOptionID("abod.samplesize", "Sample size to enable fast mode.");
+  public static final OptionID FAST_SAMPLE_ID = new OptionID("abod.samplesize", "Sample size to enable fast mode.");
 
   /**
    * Parameter for the kernel function.
    */
-  public static final OptionID KERNEL_FUNCTION_ID = OptionID.getOrCreateOptionID("abod.kernelfunction", "Kernel function to use.");
+  public static final OptionID KERNEL_FUNCTION_ID = new OptionID("abod.kernelfunction", "Kernel function to use.");
 
   /**
    * The preprocessor used to materialize the kNN neighborhoods.
    */
-  public static final OptionID PREPROCESSOR_ID = OptionID.getOrCreateOptionID("abod.knnquery", "Processor to compute the kNN neighborhoods.");
+  public static final OptionID PREPROCESSOR_ID = new OptionID("abod.knnquery", "Processor to compute the kNN neighborhoods.");
 
   /**
-   * use alternate code below
+   * use alternate code below.
    */
-  private static final boolean useRNDSample = false;
+  private static final boolean USE_RND_SAMPLE = false;
 
   /**
-   * k parameter
+   * k parameter.
    */
   private int k;
 
@@ -134,10 +132,13 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
   int sampleSize = 0;
 
   /**
-   * Store the configured Kernel version
+   * Store the configured Kernel version.
    */
   private PrimitiveSimilarityFunction<? super V, DoubleDistance> primitiveKernelFunction;
 
+  /**
+   * Static DBID map.
+   */
   private ArrayModifiableDBIDs staticids = null;
 
   /**
@@ -173,41 +174,32 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * Main part of the algorithm. Exact version.
    * 
    * @param relation Relation to query
-   * @param k k for kNN queries
    * @return result
    */
-  public OutlierResult getRanking(Relation<V> relation, int k) {
+  public OutlierResult getRanking(Relation<V> relation) {
     // Fix a static set of IDs
     staticids = DBIDUtil.newArray(relation.getDBIDs());
     staticids.sort();
 
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, relation, staticids);
-    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(relation.size(), Collections.reverseOrder());
+    Heap<DoubleDBIDPair> pq = new Heap<DoubleDBIDPair>(relation.size(), Collections.reverseOrder());
 
     // preprocess kNN neighborhoods
-    assert (k == this.k);
     KNNQuery<V, DoubleDistance> knnQuery = QueryUtil.getKNNQuery(relation, getDistanceFunction(), k);
 
     MeanVariance s = new MeanVariance();
-    for(DBIDIter objKey = relation.iterDBIDs(); objKey.valid(); objKey.advance()) {
+    for (DBIDIter objKey = relation.iterDBIDs(); objKey.valid(); objKey.advance()) {
       s.reset();
 
-      // System.out.println("Processing: " +objKey);
       KNNResult<DoubleDistance> neighbors = knnQuery.getKNNForDBID(objKey, k);
-      Iterator<DistanceResultPair<DoubleDistance>> iter = neighbors.iterator();
-      while(iter.hasNext()) {
-        DistanceResultPair<DoubleDistance> key1 = iter.next();
-        // Iterator iter2 = data.keyIterator();
-        Iterator<DistanceResultPair<DoubleDistance>> iter2 = neighbors.iterator();
-        // PriorityQueue best = new PriorityQueue(false, k);
-        while(iter2.hasNext()) {
-          DistanceResultPair<DoubleDistance> key2 = iter2.next();
-          if(key2.sameDBID(key1) || key1.sameDBID(objKey) || key2.sameDBID(objKey)) {
+      for (DBIDIter key1 = neighbors.iter(); key1.valid(); key1.advance()) {
+        for (DBIDIter key2 = neighbors.iter(); key2.valid(); key2.advance()) {
+          if (DBIDUtil.equal(key2, key1) || DBIDUtil.equal(key1, objKey) || DBIDUtil.equal(key2, objKey)) {
             continue;
           }
           double nenner = calcDenominator(kernelMatrix, objKey, key1, key2);
 
-          if(nenner != 0) {
+          if (nenner != 0) {
             double sqrtnenner = Math.sqrt(nenner);
             double tmp = calcNumerator(kernelMatrix, objKey, key1, key2) / nenner;
             s.put(tmp, 1 / sqrtnenner);
@@ -217,14 +209,14 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       }
       // Sample variance probably would be correct, however the numerical
       // instabilities can actually break ABOD here.
-      pq.add(new DoubleObjPair<DBID>(s.getNaiveVariance(), objKey.getDBID()));
+      pq.add(DBIDUtil.newPair(s.getNaiveVariance(), objKey));
     }
 
     DoubleMinMax minmaxabod = new DoubleMinMax();
     WritableDoubleDataStore abodvalues = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
-    for(DoubleObjPair<DBID> pair : pq) {
-      abodvalues.putDouble(pair.getSecond(), pair.first);
-      minmaxabod.put(pair.first);
+    for (DoubleDBIDPair pair : pq) {
+      abodvalues.putDouble(pair, pair.doubleValue());
+      minmaxabod.put(pair.doubleValue());
     }
     // Build result representation.
     Relation<Double> scoreResult = new MaterializedRelation<Double>("Angle-based Outlier Degree", "abod-outlier", TypeUtil.DOUBLE, abodvalues, relation.getDBIDs());
@@ -236,11 +228,9 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * Main part of the algorithm. Fast version.
    * 
    * @param relation Relation to use
-   * @param k k for kNN queries
-   * @param sampleSize Sample size
    * @return result
    */
-  public OutlierResult getFastRanking(Relation<V> relation, int k, int sampleSize) {
+  public OutlierResult getFastRanking(Relation<V> relation) {
     final DBIDs ids = relation.getDBIDs();
     // Fix a static set of IDs
     // TODO: add a DBIDUtil.ensureSorted?
@@ -249,92 +239,72 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
 
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, relation, staticids);
 
-    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(relation.size(), Collections.reverseOrder());
+    Heap<DoubleDBIDPair> pq = new Heap<DoubleDBIDPair>(relation.size(), Collections.reverseOrder());
     // get Candidate Ranking
-    for(DBIDIter aKey = relation.iterDBIDs(); aKey.valid(); aKey.advance()) {
+    for (DBIDIter aKey = relation.iterDBIDs(); aKey.valid(); aKey.advance()) {
       WritableDoubleDataStore dists = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_TEMP | DataStoreFactory.HINT_HOT);
       // determine kNearestNeighbors and pairwise distances
-      Heap<DoubleObjPair<DBID>> nn;
-      if(!useRNDSample) {
+      Heap<DoubleDBIDPair> nn;
+      if (!USE_RND_SAMPLE) {
         nn = calcDistsandNN(relation, kernelMatrix, sampleSize, aKey, dists);
-      }
-      else {
+      } else {
         // alternative:
         nn = calcDistsandRNDSample(relation, kernelMatrix, sampleSize, aKey, dists);
       }
 
       // get normalization
       double[] counter = calcFastNormalization(aKey, dists, staticids);
-      // System.out.println(counter[0] + " " + counter2[0] + " " + counter[1] +
-      // " " + counter2[1]);
       // umsetzen von Pq zu list
       ModifiableDBIDs neighbors = DBIDUtil.newArray(nn.size());
-      while(!nn.isEmpty()) {
-        neighbors.add(nn.remove().getSecond());
+      while (!nn.isEmpty()) {
+        neighbors.add(nn.poll());
       }
       // getFilter
       double var = getAbofFilter(kernelMatrix, aKey, dists, counter[1], counter[0], neighbors);
-      pq.add(new DoubleObjPair<DBID>(var, aKey.getDBID()));
-      // System.out.println("prog "+(prog++));
+      pq.add(DBIDUtil.newPair(var, aKey));
     }
     // refine Candidates
-    Heap<DoubleObjPair<DBID>> resqueue = new Heap<DoubleObjPair<DBID>>(k);
-    // System.out.println(pq.size() + " objects ordered into candidate list.");
-    // int v = 0;
+    Heap<DoubleDBIDPair> resqueue = new Heap<DoubleDBIDPair>(k);
     MeanVariance s = new MeanVariance();
-    while(!pq.isEmpty()) {
-      if(resqueue.size() == k && pq.peek().first > resqueue.peek().first) {
+    while (!pq.isEmpty()) {
+      if (resqueue.size() == k && pq.peek().doubleValue() > resqueue.peek().doubleValue()) {
         break;
       }
       // double approx = pq.peek().getFirst();
-      DBID aKey = pq.remove().getSecond();
-      // if(!result.isEmpty()) {
-      // System.out.println("Best Candidate " + aKey+" : " + pq.firstPriority()
-      // + " worst result: " + result.firstPriority());
-      // } else {
-      // System.out.println("Best Candidate " + aKey+" : " + pq.firstPriority()
-      // + " worst result: " + Double.MAX_VALUE);
-      // }
-      // v++;
+      DBIDRef aKey = pq.poll();
       s.reset();
-      for(DBIDIter bKey = relation.iterDBIDs(); bKey.valid(); bKey.advance()) {
-        if(bKey.sameDBID(aKey)) {
+      for (DBIDIter bKey = relation.iterDBIDs(); bKey.valid(); bKey.advance()) {
+        if (DBIDUtil.equal(bKey, aKey)) {
           continue;
         }
-        for(DBIDIter cKey = relation.iterDBIDs(); cKey.valid(); cKey.advance()) {
-          if(cKey.sameDBID(aKey)) {
+        for (DBIDIter cKey = relation.iterDBIDs(); cKey.valid(); cKey.advance()) {
+          if (DBIDUtil.equal(cKey, aKey)) {
             continue;
           }
           // double nenner = dists[y]*dists[z];
           double nenner = calcDenominator(kernelMatrix, aKey, bKey, cKey);
-          if(nenner != 0) {
+          if (nenner != 0) {
             double tmp = calcNumerator(kernelMatrix, aKey, bKey, cKey) / nenner;
             double sqrtNenner = Math.sqrt(nenner);
             s.put(tmp, 1 / sqrtNenner);
           }
         }
       }
-      // System.out.println( aKey + "Sum " + sum + " SQRSum " +sqrSum +
-      // " Counter " + counter);
       double var = s.getSampleVariance();
-      // System.out.println(aKey+ " : " + approx +" " + var);
-      if(resqueue.size() < k) {
-        resqueue.add(new DoubleObjPair<DBID>(var, aKey));
-      }
-      else {
-        if(resqueue.peek().first > var) {
-          resqueue.remove();
-          resqueue.add(new DoubleObjPair<DBID>(var, aKey));
+      if (resqueue.size() < k) {
+        resqueue.add(DBIDUtil.newPair(var, aKey));
+      } else {
+        if (resqueue.peek().doubleValue() > var) {
+          resqueue.replaceTopElement(DBIDUtil.newPair(var, aKey));
         }
       }
 
     }
-    // System.out.println(v + " Punkte von " + data.size() + " verfeinert !!");
     DoubleMinMax minmaxabod = new DoubleMinMax();
     WritableDoubleDataStore abodvalues = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_STATIC);
-    for(DoubleObjPair<DBID> pair : pq) {
-      abodvalues.putDouble(pair.getSecond(), pair.first);
-      minmaxabod.put(pair.first);
+    for (DoubleDBIDPair pair : pq) {
+      abodvalues.putDouble(pair, pair.doubleValue());
+      minmaxabod.put(pair.doubleValue());
     }
     // Build result representation.
     Relation<Double> scoreResult = new MaterializedRelation<Double>("Angle-based Outlier Detection", "abod-outlier", TypeUtil.DOUBLE, abodvalues, ids);
@@ -348,7 +318,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
     double sum = 0;
     double sumF = 0;
     for (DBIDIter yKey = ids.iter(); yKey.valid(); yKey.advance()) {
-      if(dists.doubleValue(yKey) != 0) {
+      if (dists.doubleValue(yKey) != 0) {
         double tmp = 1 / Math.sqrt(dists.doubleValue(yKey));
         sum += tmp;
         sumF += (1 / dists.doubleValue(yKey)) * tmp;
@@ -357,7 +327,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
     double sofar = 0;
     double sofarF = 0;
     for (DBIDIter zKey = ids.iter(); zKey.valid(); zKey.advance()) {
-      if(dists.doubleValue(zKey) != 0) {
+      if (dists.doubleValue(zKey) != 0) {
         double tmp = 1 / Math.sqrt(dists.doubleValue(zKey));
         sofar += tmp;
         double rest = sum - sofar;
@@ -375,17 +345,17 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
     double sum = 0.0;
     double sqrSum = 0.0;
     double partCounter = 0;
-    for(DBIDIter bKey = neighbors.iter(); bKey.valid(); bKey.advance()) {
-      if(bKey.sameDBID(aKey)) {
+    for (DBIDIter bKey = neighbors.iter(); bKey.valid(); bKey.advance()) {
+      if (DBIDUtil.equal(bKey, aKey)) {
         continue;
       }
-      for(DBIDIter cKey = neighbors.iter(); cKey.valid(); cKey.advance()) {
-        if(cKey.sameDBID(aKey)) {
+      for (DBIDIter cKey = neighbors.iter(); cKey.valid(); cKey.advance()) {
+        if (DBIDUtil.equal(cKey, aKey)) {
           continue;
         }
-        if(bKey.compareDBID(cKey) > 0) {
+        if (DBIDUtil.compare(bKey, cKey) > 0) {
           double nenner = dists.doubleValue(bKey) * dists.doubleValue(cKey);
-          if(nenner != 0) {
+          if (nenner != 0) {
             double tmp = calcNumerator(kernelMatrix, aKey, bKey, cKey) / nenner;
             double sqrtNenner = Math.sqrt(nenner);
             sum += tmp * (1 / sqrtNenner);
@@ -417,7 +387,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
   private int mapDBID(DBIDRef aKey) {
     // TODO: this is not the most efficient...
     int off = staticids.binarySearch(aKey);
-    if(off < 0) {
+    if (off < 0) {
       throw new AbortException("Did not find id " + aKey.toString() + " in staticids. " + staticids.contains(aKey));
     }
     return off + 1;
@@ -434,33 +404,31 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
     return (kernelMatrix.getDistance(ai, ai) + kernelMatrix.getDistance(bi, ci) - kernelMatrix.getDistance(ai, ci) - kernelMatrix.getDistance(ai, bi));
   }
 
-  private Heap<DoubleObjPair<DBID>> calcDistsandNN(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
-    Heap<DoubleObjPair<DBID>> nn = new Heap<DoubleObjPair<DBID>>(sampleSize);
-    for(DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
+  private Heap<DoubleDBIDPair> calcDistsandNN(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
+    Heap<DoubleDBIDPair> nn = new Heap<DoubleDBIDPair>(sampleSize);
+    for (DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
       double val = calcCos(kernelMatrix, aKey, bKey);
       dists.putDouble(bKey, val);
-      if(nn.size() < sampleSize) {
-        nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
-      }
-      else {
-        if(val < nn.peek().first) {
-          nn.remove();
-          nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
+      if (nn.size() < sampleSize) {
+        nn.add(DBIDUtil.newPair(val, bKey));
+      } else {
+        if (val < nn.peek().doubleValue()) {
+          nn.replaceTopElement(DBIDUtil.newPair(val, bKey));
         }
       }
     }
     return nn;
   }
 
-  private Heap<DoubleObjPair<DBID>> calcDistsandRNDSample(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
-    Heap<DoubleObjPair<DBID>> nn = new Heap<DoubleObjPair<DBID>>(sampleSize);
+  private Heap<DoubleDBIDPair> calcDistsandRNDSample(Relation<V> data, KernelMatrix kernelMatrix, int sampleSize, DBIDRef aKey, WritableDoubleDataStore dists) {
+    Heap<DoubleDBIDPair> nn = new Heap<DoubleDBIDPair>(sampleSize);
     int step = (int) ((double) data.size() / (double) sampleSize);
     int counter = 0;
-    for(DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
+    for (DBIDIter bKey = data.iterDBIDs(); bKey.valid(); bKey.advance()) {
       double val = calcCos(kernelMatrix, aKey, bKey);
       dists.putDouble(bKey, val);
-      if(counter % step == 0) {
-        nn.add(new DoubleObjPair<DBID>(val, bKey.getDBID()));
+      if (counter % step == 0) {
+        nn.add(DBIDUtil.newPair(val, bKey));
       }
       counter++;
     }
@@ -471,112 +439,108 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * Get explanations for points in the database.
    * 
    * @param data to get explanations for
+   * @return String explanation
    */
   // TODO: this should be done by the result classes.
-  public void getExplanations(Relation<V> data) {
+  public String getExplanations(Relation<V> data) {
     KernelMatrix kernelMatrix = new KernelMatrix(primitiveKernelFunction, data, staticids);
     // PQ for Outlier Ranking
-    Heap<DoubleObjPair<DBID>> pq = new Heap<DoubleObjPair<DBID>>(data.size(), Collections.reverseOrder());
+    Heap<DoubleDBIDPair> pq = new Heap<DoubleDBIDPair>(data.size(), Collections.reverseOrder());
     HashMap<DBID, DBIDs> explaintab = new HashMap<DBID, DBIDs>();
     // test all objects
     MeanVariance s = new MeanVariance(), s2 = new MeanVariance();
-    for(DBIDIter objKey = data.iterDBIDs(); objKey.valid(); objKey.advance()) {
+    for (DBIDIter objKey = data.iterDBIDs(); objKey.valid(); objKey.advance()) {
       s.reset();
       // Queue for the best explanation
-      Heap<DoubleObjPair<DBID>> explain = new Heap<DoubleObjPair<DBID>>();
+      Heap<DoubleDBIDPair> explain = new Heap<DoubleDBIDPair>();
       // determine Object
       // for each pair of other objects
       for (DBIDIter key1 = data.iterDBIDs(); key1.valid(); key1.advance()) {
-      // Collect Explanation Vectors
+        // Collect Explanation Vectors
         s2.reset();
-        if(objKey.sameDBID(key1)) {
+        if (DBIDUtil.equal(objKey, key1)) {
           continue;
         }
         for (DBIDIter key2 = data.iterDBIDs(); key2.valid(); key2.advance()) {
-          if(key2.sameDBID(key1) || objKey.sameDBID(key2)) {
+          if (DBIDUtil.equal(key2, key1) || DBIDUtil.equal(objKey, key2)) {
             continue;
           }
           double nenner = calcDenominator(kernelMatrix, objKey, key1, key2);
-          if(nenner != 0) {
+          if (nenner != 0) {
             double tmp = calcNumerator(kernelMatrix, objKey, key1, key2) / nenner;
             double sqr = Math.sqrt(nenner);
             s2.put(tmp, 1 / sqr);
           }
         }
-        explain.add(new DoubleObjPair<DBID>(s2.getSampleVariance(), key1.getDBID()));
+        explain.add(DBIDUtil.newPair(s2.getSampleVariance(), key1));
         s.put(s2);
       }
       // build variance of the observed vectors
-      pq.add(new DoubleObjPair<DBID>(s.getSampleVariance(), objKey.getDBID()));
+      pq.add(DBIDUtil.newPair(s.getSampleVariance(), objKey));
       //
       ModifiableDBIDs expList = DBIDUtil.newArray();
-      expList.add(explain.remove().getSecond());
-      while(!explain.isEmpty()) {
-        DBID nextKey = explain.remove().getSecond();
-        if(nextKey.sameDBID(objKey)) {
+      expList.add(explain.poll());
+      while (!explain.isEmpty()) {
+        DBIDRef nextKey = explain.poll();
+        if (DBIDUtil.equal(nextKey, objKey)) {
           continue;
         }
         double max = Double.MIN_VALUE;
-        for(DBIDIter exp = expList.iter(); exp.valid(); exp.advance()) {
-          if(exp.sameDBID(objKey) || nextKey.sameDBID(exp)) {
+        for (DBIDIter exp = expList.iter(); exp.valid(); exp.advance()) {
+          if (DBIDUtil.equal(exp, objKey) || DBIDUtil.equal(nextKey, exp)) {
             continue;
           }
           double nenner = Math.sqrt(calcCos(kernelMatrix, objKey, nextKey)) * Math.sqrt(calcCos(kernelMatrix, objKey, exp));
           double angle = calcNumerator(kernelMatrix, objKey, nextKey, exp) / nenner;
           max = Math.max(angle, max);
         }
-        if(max < 0.5) {
+        if (max < 0.5) {
           expList.add(nextKey);
         }
       }
-      explaintab.put(objKey.getDBID(), expList);
+      explaintab.put(DBIDUtil.deref(objKey), expList);
     }
-    System.out.println("--------------------------------------------");
-    System.out.println("Result: ABOD");
+    StringBuilder buf = new StringBuilder();
+    buf.append("Result: ABOD\n");
     int count = 0;
-    while(!pq.isEmpty()) {
-      if(count > 10) {
+    while (!pq.isEmpty()) {
+      if (count > 10) {
         break;
       }
-      double factor = pq.peek().first;
-      DBID key = pq.remove().getSecond();
-      System.out.print(data.get(key) + " ");
-      System.out.println(count + " Factor=" + factor + " " + key);
+      double factor = pq.peek().doubleValue();
+      DBIDRef key = pq.poll();
+      buf.append(data.get(key)).append(' ');
+      buf.append(count).append(" Factor=").append(factor).append(' ').append(key).append('\n');
       DBIDs expList = explaintab.get(key);
-      generateExplanation(data, key, expList);
+      generateExplanation(buf, data, key, expList);
       count++;
     }
-    System.out.println("--------------------------------------------");
+    return buf.toString();
   }
 
-  private void generateExplanation(Relation<V> data, DBID key, DBIDs expList) {
+  private void generateExplanation(StringBuilder buf, Relation<V> data, DBIDRef key, DBIDs expList) {
     Vector vect1 = data.get(key).getColumnVector();
-    for(DBIDIter iter = expList.iter(); iter.valid(); iter.advance()) {
-      System.out.println("Outlier: " + vect1);
+    for (DBIDIter iter = expList.iter(); iter.valid(); iter.advance()) {
+      buf.append("Outlier: ").append(vect1).append('\n');
       Vector exp = data.get(iter).getColumnVector();
-      System.out.println("Most common neighbor: " + exp);
+      buf.append("Most common neighbor: ").append(exp).append('\n');
       // determine difference Vector
       Vector vals = exp.minus(vect1);
-      System.out.println(vals);
-      // System.out.println(new FeatureVector(
-      // "Diff-"+vect1.getPrimaryKey(),vals ));
+      buf.append(vals).append('\n');
     }
-    System.out.println();
   }
 
   /**
-   * Run ABOD on the data set
+   * Run ABOD on the data set.
    * 
-   * @param database
-   * @param relation
+   * @param relation Relation to process
    * @return Outlier detection result
    */
-  public OutlierResult run(Database database, Relation<V> relation) {
-    if(sampleSize > 0) {
-      return getFastRanking(relation, k, sampleSize);
-    }
-    else {
-      return getRanking(relation, k);
+  public OutlierResult run(Relation<V> relation) {
+    if (sampleSize > 0) {
+      return getFastRanking(relation);
+    } else {
+      return getRanking(relation);
     }
   }
 
@@ -587,7 +551,7 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -597,26 +561,38 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, DoubleDistance> {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, DoubleDistance> {
+    /**
+     * k Parameter.
+     */
     protected int k = 0;
 
+    /**
+     * Sample size.
+     */
     protected int sampleSize = 0;
 
+    /**
+     * Distance function.
+     */
     protected PrimitiveSimilarityFunction<V, DoubleDistance> primitiveKernelFunction = null;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter kP = new IntParameter(K_ID, new GreaterEqualConstraint(1), 30);
-      if(config.grab(kP)) {
+      final IntParameter kP = new IntParameter(K_ID, 30);
+      kP.addConstraint(new GreaterEqualConstraint(1));
+      if (config.grab(kP)) {
         k = kP.getValue();
       }
-      final IntParameter sampleSizeP = new IntParameter(FAST_SAMPLE_ID, new GreaterEqualConstraint(1), true);
-      if(config.grab(sampleSizeP)) {
+      final IntParameter sampleSizeP = new IntParameter(FAST_SAMPLE_ID);
+      sampleSizeP.addConstraint(new GreaterEqualConstraint(1));
+      sampleSizeP.setOptional(true);
+      if (config.grab(sampleSizeP)) {
         sampleSize = sampleSizeP.getValue();
       }
       final ObjectParameter<PrimitiveSimilarityFunction<V, DoubleDistance>> param = new ObjectParameter<PrimitiveSimilarityFunction<V, DoubleDistance>>(KERNEL_FUNCTION_ID, PrimitiveSimilarityFunction.class, PolynomialKernelFunction.class);
-      if(config.grab(param)) {
+      if (config.grab(param)) {
         primitiveKernelFunction = param.instantiateClass(config);
       }
     }
@@ -626,4 +602,4 @@ public class ABOD<V extends NumberVector<V, ?>> extends AbstractDistanceBasedAlg
       return new ABOD<V>(k, sampleSize, primitiveKernelFunction, distanceFunction);
     }
   }
-}
\ No newline at end of file
+}
