@@ -36,10 +36,12 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.NumberVectorDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
@@ -51,6 +53,7 @@ import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.QuotientOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -58,8 +61,8 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
 /**
@@ -78,17 +81,19 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * @author Jonathan von Brünken
  * @author Erich Schubert
  * 
+ * @apiviz.composedOf ALOCIQuadTree
+ * 
  * @param <O> Object type
  * @param <D> Distance type
  */
 @Title("LOCI: Fast Outlier Detection Using the Local Correlation Integral")
 @Description("Algorithm to compute outliers based on the Local Correlation Integral")
 @Reference(authors = "S. Papadimitriou, H. Kitagawa, P. B. Gibbons, C. Faloutsos", title = "LOCI: Fast Outlier Detection Using the Local Correlation Integral", booktitle = "Proc. 19th IEEE Int. Conf. on Data Engineering (ICDE '03), Bangalore, India, 2003", url = "http://dx.doi.org/10.1109/ICDE.2003.1260802")
-public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<OutlierResult> implements OutlierAlgorithm {
+public class ALOCI<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<OutlierResult> implements OutlierAlgorithm {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(ALOCI.class);
+  private static final Logging LOG = Logging.getLogger(ALOCI.class);
 
   /**
    * Minimum size for a leaf.
@@ -108,7 +113,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
   /**
    * Random generator
    */
-  private Random random;
+  private RandomFactory rnd;
 
   /**
    * Distance function
@@ -122,20 +127,21 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
    * @param nmin Minimum neighborhood size
    * @param alpha Alpha value
    * @param g Number of grids to use
-   * @param seed Random generator seed.
+   * @param rnd Random generator.
    */
-  public ALOCI(NumberVectorDistanceFunction<D> distanceFunction, int nmin, int alpha, int g, Long seed) {
+  public ALOCI(NumberVectorDistanceFunction<D> distanceFunction, int nmin, int alpha, int g, RandomFactory rnd) {
     super();
     this.distFunc = distanceFunction;
     this.nmin = nmin;
     this.alpha = alpha;
     this.g = g;
-    this.random = (seed != null) ? new Random(seed) : new Random(0);
+    this.rnd = rnd;
   }
 
   public OutlierResult run(Database database, Relation<O> relation) {
-    final int dim = DatabaseUtil.dimensionality(relation);
-    FiniteProgress progressPreproc = logger.isVerbose() ? new FiniteProgress("Build aLOCI quadtress", g, logger) : null;
+    final int dim = RelationUtil.dimensionality(relation);
+    final Random random = rnd.getRandom();
+    FiniteProgress progressPreproc = LOG.isVerbose() ? new FiniteProgress("Build aLOCI quadtress", g, LOG) : null;
 
     // Compute extend of dataset.
     double[] min, max;
@@ -145,13 +151,13 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
       min = new double[dim];
       max = new double[dim];
       for(int i = 0; i < dim; i++) {
-        min[i] = hbbs.first.doubleValue(i + 1);
-        max[i] = hbbs.second.doubleValue(i + 1);
+        min[i] = hbbs.first.doubleValue(i);
+        max[i] = hbbs.second.doubleValue(i);
         maxd = Math.max(maxd, max[i] - min[i]);
       }
       // Enlarge bounding box to have equal lengths.
       for(int i = 0; i < dim; i++) {
-        double diff = (maxd - (max[i] - min[i])) / 2;
+        double diff = (maxd - (max[i] - min[i])) * .5;
         min[i] -= diff;
         max[i] += diff;
       }
@@ -163,7 +169,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
     ALOCIQuadTree qt = new ALOCIQuadTree(min, max, nshift, nmin, relation);
     qts.add(qt);
     if(progressPreproc != null) {
-      progressPreproc.incrementProcessed(logger);
+      progressPreproc.incrementProcessed(LOG);
     }
     /*
      * create the remaining g-1 shifted QuadTrees. This not clearly described in
@@ -178,15 +184,15 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
       qt = new ALOCIQuadTree(min, max, svec, nmin, relation);
       qts.add(qt);
       if(progressPreproc != null) {
-        progressPreproc.incrementProcessed(logger);
+        progressPreproc.incrementProcessed(LOG);
       }
     }
     if(progressPreproc != null) {
-      progressPreproc.ensureCompleted(logger);
+      progressPreproc.ensureCompleted(LOG);
     }
 
     // aLOCI main loop: evaluate
-    FiniteProgress progressLOCI = logger.isVerbose() ? new FiniteProgress("Compute aLOCI scores", relation.size(), logger) : null;
+    FiniteProgress progressLOCI = LOG.isVerbose() ? new FiniteProgress("Compute aLOCI scores", relation.size(), LOG) : null;
     WritableDoubleDataStore mdef_norm = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
     DoubleMinMax minmax = new DoubleMinMax();
 
@@ -239,11 +245,11 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
       mdef_norm.putDouble(iditer, maxmdefnorm);
       minmax.put(maxmdefnorm);
       if(progressLOCI != null) {
-        progressLOCI.incrementProcessed(logger);
+        progressLOCI.incrementProcessed(LOG);
       }
     }
     if(progressLOCI != null) {
-      progressLOCI.ensureCompleted(logger);
+      progressLOCI.ensureCompleted(LOG);
     }
     Relation<Double> scoreResult = new MaterializedRelation<Double>("aLOCI normalized MDEF", "aloci-mdef-outlier", TypeUtil.DOUBLE, mdef_norm, relation.getDBIDs());
     OutlierScoreMeta scoreMeta = new QuotientOutlierScoreMeta(minmax.getMin(), minmax.getMax(), 0.0, Double.POSITIVE_INFINITY);
@@ -291,7 +297,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   @Override
@@ -329,7 +335,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
     /**
      * Relation indexed.
      */
-    private Relation<? extends NumberVector<?, ?>> relation;
+    private Relation<? extends NumberVector<?>> relation;
 
     /**
      * Constructor.
@@ -340,7 +346,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
      * @param nmin Maximum size for a page to split
      * @param relation Relation to index
      */
-    public ALOCIQuadTree(double[] min, double[] max, double[] shift, int nmin, Relation<? extends NumberVector<?, ?>> relation) {
+    public ALOCIQuadTree(double[] min, double[] max, double[] shift, int nmin, Relation<? extends NumberVector<?>> relation) {
       super();
       assert (min.length <= 32) : "Quadtrees are only supported for up to 32 dimensions";
       this.shift = shift;
@@ -386,11 +392,14 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
       // logger.warning(FormatUtil.format(lmin)+" "+FormatUtil.format(lmax)+" "+start+"->"+end+" "+(end-start));
       // Hack: Check degenerate cases that won't split
       if(dim == 0) {
-        NumberVector<?, ?> first = relation.get(ids.get(start));
+        DBIDArrayIter iter = ids.iter();
+        iter.seek(start);
+        NumberVector<?> first = relation.get(iter);
+        iter.advance();
         boolean degenerate = true;
-        loop: for(int pos = start + 1; pos < end; pos++) {
-          NumberVector<?, ?> other = relation.get(ids.get(pos));
-          for(int d = 1; d <= lmin.length; d++) {
+        loop: for(; iter.getOffset() < end; iter.advance()) {
+          NumberVector<?> other = relation.get(iter);
+          for(int d = 0; d < lmin.length; d++) {
             if(Math.abs(first.doubleValue(d) - other.doubleValue(d)) > 1E-15) {
               degenerate = false;
               break loop;
@@ -431,20 +440,23 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
       }
       else {
         // Partially sort data, by dimension dim < mid
-        int spos = start, epos = end;
-        while(spos < epos) {
-          if(getShiftedDim(relation.get(ids.get(spos)), dim, level) <= .5) {
-            spos++;
+        DBIDArrayIter siter = ids.iter(), eiter = ids.iter();
+        siter.seek(start);
+        eiter.seek(end - 1);
+        while(siter.getOffset() < eiter.getOffset()) {
+          if(getShiftedDim(relation.get(siter), dim, level) <= .5) {
+            siter.advance();
             continue;
           }
-          if(getShiftedDim(relation.get(ids.get(epos - 1)), dim, level) > 0.5) {
-            epos--;
+          if(getShiftedDim(relation.get(eiter), dim, level) > 0.5) {
+            eiter.retract();
             continue;
           }
-          ids.swap(spos, epos - 1);
-          spos++;
-          epos--;
+          ids.swap(siter.getOffset(), eiter.getOffset() - 1);
+          siter.advance();
+          eiter.retract();
         }
+        final int spos = siter.getOffset();
         if(start < spos) {
           final double tmp = lmax[dim];
           lmax[dim] = lmax[dim] * .5 + lmin[dim] * .5;
@@ -468,8 +480,8 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
      * @param level Level (controls scaling/wraping!)
      * @return Shifted position
      */
-    private double getShiftedDim(NumberVector<?, ?> obj, int dim, int level) {
-      double pos = obj.doubleValue(dim + 1) + shift[dim];
+    private double getShiftedDim(NumberVector<?> obj, int dim, int level) {
+      double pos = obj.doubleValue(dim) + shift[dim];
       pos = (pos - min[dim]) / width[dim] * (1 + level);
       return pos - Math.floor(pos);
     }
@@ -482,7 +494,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
      * @param tlevel Target level
      * @return Node
      */
-    public Node findClosestNode(NumberVector<?, ?> vec, int tlevel) {
+    public Node findClosestNode(NumberVector<?> vec, int tlevel) {
       Node cur = root;
       for(int level = 0; level <= tlevel; level++) {
         if(cur.children == null) {
@@ -637,26 +649,26 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractParameterizer {
     /**
      * Parameter to specify the minimum neighborhood size
      */
-    public static final OptionID NMIN_ID = OptionID.getOrCreateOptionID("loci.nmin", "Minimum neighborhood size to be considered.");
+    public static final OptionID NMIN_ID = new OptionID("loci.nmin", "Minimum neighborhood size to be considered.");
 
     /**
      * Parameter to specify the averaging neighborhood scaling.
      */
-    public static final OptionID ALPHA_ID = OptionID.getOrCreateOptionID("loci.alpha", "Scaling factor for averaging neighborhood");
+    public static final OptionID ALPHA_ID = new OptionID("loci.alpha", "Scaling factor for averaging neighborhood");
 
     /**
      * Parameter to specify the number of Grids to use.
      */
-    public static final OptionID GRIDS_ID = OptionID.getOrCreateOptionID("loci.g", "The number of Grids to use.");
+    public static final OptionID GRIDS_ID = new OptionID("loci.g", "The number of Grids to use.");
 
     /**
      * Parameter to specify the seed to initialize Random.
      */
-    public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("loci.seed", "The seed to use for initializing Random.");
+    public static final OptionID SEED_ID = new OptionID("loci.seed", "The seed to use for initializing Random.");
 
     /**
      * Neighborhood minimum size
@@ -674,9 +686,9 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
     protected int g = 1;
 
     /**
-     * Random generator seed
+     * Random generator
      */
-    protected Long seed = null;
+    protected RandomFactory rnd;
 
     /**
      * The distance function
@@ -702,9 +714,9 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
         this.g = g.getValue();
       }
 
-      final LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        this.seed = seedP.getValue();
+      final RandomParameter rndP = new RandomParameter(SEED_ID);
+      if(config.grab(rndP)) {
+        this.rnd = rndP.getValue();
       }
 
       final IntParameter alphaP = new IntParameter(ALPHA_ID, 4);
@@ -718,7 +730,7 @@ public class ALOCI<O extends NumberVector<O, ?>, D extends NumberDistance<D, ?>>
 
     @Override
     protected ALOCI<O, D> makeInstance() {
-      return new ALOCI<O, D>(distanceFunction, nmin, alpha, g, seed);
+      return new ALOCI<O, D>(distanceFunction, nmin, alpha, g, rnd);
     }
   }
-}
\ No newline at end of file
+}
