@@ -37,18 +37,18 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.result.outlier.InvertedOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.TopBoundedHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
@@ -58,7 +58,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.FCPair;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
@@ -85,40 +85,26 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("EAFOD: the evolutionary outlier detection algorithm")
 @Description("Outlier detection for high dimensional data")
 @Reference(authors = "C.C. Aggarwal, P. S. Yu", title = "Outlier detection for high dimensional data", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD 2001), Santa Barbara, CA, 2001", url = "http://dx.doi.org/10.1145/375663.375668")
-public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends AbstractAggarwalYuOutlier<V> {
+public class AggarwalYuEvolutionary<V extends NumberVector<?>> extends AbstractAggarwalYuOutlier<V> {
   /**
    * The logger for this class.
    */
-  protected static final Logging logger = Logging.getLogger(AggarwalYuEvolutionary.class);
-
-  /**
-   * Parameter to specify the number of solutions must be an integer greater
-   * than 1.
-   * <p>
-   * Key: {@code -eafod.m}
-   * </p>
-   */
-  public static final OptionID M_ID = OptionID.getOrCreateOptionID("ay.m", "Population size for evolutionary algorithm.");
-
-  /**
-   * Parameter to specify the random generator seed.
-   */
-  public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("ay.seed", "The random number generator seed.");
+  private static final Logging LOG = Logging.getLogger(AggarwalYuEvolutionary.class);
 
   /**
    * Maximum iteration count for evolutionary search.
    */
-  protected final int MAX_ITERATIONS = 1000;
+  protected final static int MAX_ITERATIONS = 1000;
 
   /**
-   * Holds the value of {@link #M_ID}.
+   * Holds the value of {@link Parameterizer#M_ID}.
    */
   private int m;
 
   /**
-   * Holds the value of {@link #SEED_ID}.
+   * Random generator.
    */
-  private Long seed;
+  private RandomFactory rnd;
 
   /**
    * Constructor.
@@ -126,12 +112,12 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
    * @param k K
    * @param phi Phi
    * @param m M
-   * @param seed Seed
+   * @param rnd Random generator
    */
-  public AggarwalYuEvolutionary(int k, int phi, int m, Long seed) {
+  public AggarwalYuEvolutionary(int k, int phi, int m, RandomFactory rnd) {
     super(k, phi);
     this.m = m;
-    this.seed = seed;
+    this.rnd = rnd;
   }
 
   /**
@@ -145,27 +131,25 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
     final int dbsize = relation.size();
     ArrayList<ArrayList<DBIDs>> ranges = buildRanges(relation);
 
-    Collection<Individuum> individuums = (new EvolutionarySearch(relation, ranges, m, seed)).run();
+    Iterable<Individuum> individuums = (new EvolutionarySearch(relation, ranges, m, rnd.getRandom())).run();
 
     WritableDoubleDataStore outlierScore = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC);
     for(Individuum ind : individuums) {
       DBIDs ids = computeSubspaceForGene(ind.getGene(), ranges);
-      double sparsityC = sparsity(ids.size(), dbsize, k);
+      double sparsityC = sparsity(ids.size(), dbsize, k, phi);
       for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-        DBID id = iter.getDBID();
-        double prev = outlierScore.doubleValue(id);
+        double prev = outlierScore.doubleValue(iter);
         if(Double.isNaN(prev) || sparsityC < prev) {
-          outlierScore.putDouble(id, sparsityC);
+          outlierScore.putDouble(iter, sparsityC);
         }
       }
     }
 
     DoubleMinMax minmax = new DoubleMinMax();
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id  = iditer.getDBID();
-      double val = outlierScore.doubleValue(id);
+      double val = outlierScore.doubleValue(iditer);
       if(Double.isNaN(val)) {
-        outlierScore.putDouble(id, 0.0);
+        outlierScore.putDouble(iditer, 0.0);
         val = 0.0;
       }
       minmax.put(val);
@@ -177,7 +161,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -189,17 +173,17 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
    */
   private class EvolutionarySearch {
     /**
-     * Database size
+     * Database size.
      */
     final int dbsize;
 
     /**
-     * Database dimensionality
+     * Database dimensionality.
      */
     final int dim;
 
     /**
-     * Database ranges
+     * Database ranges.
      */
     final ArrayList<ArrayList<DBIDs>> ranges;
 
@@ -209,36 +193,34 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
     final int m;
 
     /**
-     * random generator
+     * random generator.
      */
     final private Random random;
 
     /**
      * Constructor.
      * 
-     * @param database Database to use
+     * @param relation Database to use
+     * @param ranges DBID ranges to process
      * @param m Population size
-     * @param seed Random generator seed
+     * @param random Random generator
      */
-    public EvolutionarySearch(Relation<V> database, ArrayList<ArrayList<DBIDs>> ranges, int m, Long seed) {
+    public EvolutionarySearch(Relation<V> relation, ArrayList<ArrayList<DBIDs>> ranges, int m, Random random) {
       super();
       this.ranges = ranges;
       this.m = m;
-      this.dbsize = database.size();
-      this.dim = DatabaseUtil.dimensionality(database);
-      if(seed != null) {
-        this.random = new Random(seed);
-      }
-      else {
-        this.random = new Random();
-      }
+      this.dbsize = relation.size();
+      this.dim = RelationUtil.dimensionality(relation);
+      this.random = random;
     }
 
-    public Collection<Individuum> run() {
+    public Iterable<Individuum> run() {
       ArrayList<Individuum> pop = initialPopulation(m);
       // best Population
       TopBoundedHeap<Individuum> bestSol = new TopBoundedHeap<Individuum>(m, Collections.reverseOrder());
-      bestSol.addAll(pop);
+      for (Individuum ind : pop) {
+        bestSol.add(ind);
+      }
 
       int iterations = 0;
       while(!checkConvergence(pop)) {
@@ -249,26 +231,29 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
         // Mutation with probability 0.25 , 0.25
         pop = mutation(pop, 0.5, 0.5);
         // Avoid duplicates
-        for(Individuum ind : pop) {
-          if(!bestSol.contains(ind)) {
-            bestSol.add(ind);
+        ind: for(Individuum ind : pop) {
+          for (Individuum b : bestSol) {
+            if (b.equals(ind)) {
+              continue ind;
+            }
           }
+          bestSol.add(ind);
         }
-        if(logger.isDebuggingFinest()) {
-          StringBuffer buf = new StringBuffer();
+        if(LOG.isDebuggingFinest()) {
+          StringBuilder buf = new StringBuilder();
           buf.append("Top solutions:\n");
           for(Individuum ind : bestSol) {
-            buf.append(ind.toString()).append("\n");
+            buf.append(ind.toString()).append('\n');
           }
           buf.append("Population:\n");
           for(Individuum ind : pop) {
-            buf.append(ind.toString()).append("\n");
+            buf.append(ind.toString()).append('\n');
           }
-          logger.debugFinest(buf.toString());
+          LOG.debugFinest(buf.toString());
         }
         iterations++;
         if(iterations > MAX_ITERATIONS) {
-          logger.warning("Maximum iterations reached.");
+          LOG.warning("Maximum iterations reached.");
           break;
         }
       }
@@ -276,7 +261,10 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
     }
 
     /**
-     * check the termination criterion
+     * check the termination criterion.
+     * 
+     * @param pop Population
+     * @return Convergence
      */
     private boolean checkConvergence(Collection<Individuum> pop) {
       if(pop.size() == 0) {
@@ -291,7 +279,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
         for(int d = 0; d < dim; d++) {
           int val = gene[d] + DONT_CARE;
           if(val < 0 || val >= phi + 1) {
-            logger.warning("Invalid gene value encountered: " + val + " in " + ind.toString());
+            LOG.warning("Invalid gene value encountered: " + val + " in " + ind.toString());
             continue;
           }
           occur[d][val] += 1;
@@ -299,8 +287,8 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
       }
 
       int conv = (int) (pop.size() * 0.95);
-      if(logger.isDebuggingFine()) {
-        logger.debugFine("Convergence at " + conv + " of " + pop.size() + " individuums.");
+      if(LOG.isDebuggingFine()) {
+        LOG.debugFine("Convergence at " + conv + " of " + pop.size() + " individuums.");
       }
       for(int d = 0; d < dim; d++) {
         boolean converged = false;
@@ -353,18 +341,21 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
     }
 
     /**
+     * Select surviving individuums weighted by rank.
+     * 
      * the selection criterion for the genetic algorithm: <br>
      * roulette wheel mechanism: <br>
      * where the probability of sampling an individual of the population was
      * proportional to p - r(i), where p is the size of population and r(i) the
      * rank of i-th individual
      * 
-     * @param population
+     * @param population Population
+     * @return Survivors
      */
     private ArrayList<Individuum> rouletteRankSelection(ArrayList<Individuum> population) {
       final int popsize = population.size();
       // Relative weight := popsize - position => sum(1..popsize)
-      int totalweight = popsize * (popsize + 1) / 2;
+      int totalweight = (popsize * (popsize + 1)) >> 1;
       // Survivors
       ArrayList<Individuum> survivors = new ArrayList<Individuum>(popsize);
 
@@ -392,7 +383,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
     }
 
     /**
-     * method implements the mutation algorithm
+     * Apply the mutation alogrithm.
      */
     private ArrayList<Individuum> mutation(ArrayList<Individuum> population, double perc1, double perc2) {
       // the Mutations
@@ -470,7 +461,7 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
      */
     private Individuum makeIndividuum(int[] gene) {
       final DBIDs ids = computeSubspaceForGene(gene, ranges);
-      final double fitness = (ids.size() > 0) ? sparsity(ids.size(), dbsize, k) : Double.MAX_VALUE;
+      final double fitness = (ids.size() > 0) ? sparsity(ids.size(), dbsize, k, phi) : Double.MAX_VALUE;
       return new Individuum(fitness, gene);
     }
 
@@ -543,8 +534,8 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
             l1[next] = parent1.getGene()[next];
             l2[next] = parent2.getGene()[next];
 
-            final double sparsityL1 = sparsity(computeSubspaceForGene(l1, ranges).size(), dbsize, k);
-            final double sparsityL2 = sparsity(computeSubspaceForGene(l2, ranges).size(), dbsize, k);
+            final double sparsityL1 = sparsity(computeSubspaceForGene(l1, ranges).size(), dbsize, k, phi);
+            final double sparsityL2 = sparsity(computeSubspaceForGene(l2, ranges).size(), dbsize, k, phi);
 
             if(sparsityL1 <= sparsityL2) {
               b = l1.clone();
@@ -619,6 +610,8 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
    * Individuum for the evolutionary search.
    * 
    * @author Erich Schubert
+   *
+   * @apiviz.exclude de.lmu.ifi.dbs.elki.utilities.pairs.FCPair
    */
   private static class Individuum extends FCPair<Double, int[]> {
     /**
@@ -691,27 +684,42 @@ public class AggarwalYuEvolutionary<V extends NumberVector<?, ?>> extends Abstra
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<?, ?>> extends AbstractAggarwalYuOutlier.Parameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractAggarwalYuOutlier.Parameterizer {
+    /**
+     * Parameter to specify the number of solutions must be an integer greater
+     * than 1.
+     * <p>
+     * Key: {@code -eafod.m}
+     * </p>
+     */
+    public static final OptionID M_ID = new OptionID("ay.m", "Population size for evolutionary algorithm.");
+
+    /**
+     * Parameter to specify the random generator seed.
+     */
+    public static final OptionID SEED_ID = new OptionID("ay.seed", "The random number generator seed.");
+
     protected int m = 0;
 
-    protected Long seed = null;
+    protected RandomFactory rnd;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter mP = new IntParameter(M_ID, new GreaterEqualConstraint(2));
+      final IntParameter mP = new IntParameter(M_ID);
+      mP.addConstraint(new GreaterEqualConstraint(2));
       if(config.grab(mP)) {
         m = mP.getValue();
       }
-      final LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        seed = seedP.getValue();
+      final RandomParameter rndP = new RandomParameter(SEED_ID);
+      if(config.grab(rndP)) {
+        rnd = rndP.getValue();
       }
     }
 
     @Override
     protected AggarwalYuEvolutionary<V> makeInstance() {
-      return new AggarwalYuEvolutionary<V>(k, phi, m, seed);
+      return new AggarwalYuEvolutionary<V>(k, phi, m, rnd);
     }
   }
-}
\ No newline at end of file
+}
