@@ -24,7 +24,6 @@ package de.lmu.ifi.dbs.elki.algorithm.outlier;
  */
 
 import java.util.ArrayList;
-import java.util.Vector;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
@@ -35,12 +34,12 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.result.outlier.InvertedOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
@@ -65,16 +64,18 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.IntIntPair;
  * 
  * @author Ahmed Hettab
  * @author Erich Schubert
+ * 
+ * @param <V> Vector type
  */
 // TODO: progress logging!
 @Title("BruteForce: Outlier detection for high dimensional data")
 @Description("Examines all possible sets of k dimensional projections")
 @Reference(authors = "C.C. Aggarwal, P. S. Yu", title = "Outlier detection for high dimensional data", booktitle = "Proc. ACM SIGMOD Int. Conf. on Management of Data (SIGMOD 2001), Santa Barbara, CA, 2001", url = "http://dx.doi.org/10.1145/375663.375668")
-public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggarwalYuOutlier<V> {
+public class AggarwalYuNaive<V extends NumberVector<?>> extends AbstractAggarwalYuOutlier<V> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(AggarwalYuNaive.class);
+  private static final Logging LOG = Logging.getLogger(AggarwalYuNaive.class);
 
   /**
    * Constructor.
@@ -93,23 +94,23 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
    * @return Outlier detection result
    */
   public OutlierResult run(Relation<V> relation) {
-    final int dimensionality = DatabaseUtil.dimensionality(relation);
+    final int dimensionality = RelationUtil.dimensionality(relation);
     final int size = relation.size();
     ArrayList<ArrayList<DBIDs>> ranges = buildRanges(relation);
 
-    ArrayList<Vector<IntIntPair>> Rk;
+    ArrayList<ArrayList<IntIntPair>> Rk;
     // Build a list of all subspaces
     {
       // R1 initial one-dimensional subspaces.
-      Rk = new ArrayList<Vector<IntIntPair>>();
+      Rk = new ArrayList<ArrayList<IntIntPair>>();
       // Set of all dim*phi ranges
       ArrayList<IntIntPair> q = new ArrayList<IntIntPair>();
-      for(int i = 1; i <= dimensionality; i++) {
+      for(int i = 0; i < dimensionality; i++) {
         for(int j = 1; j <= phi; j++) {
           IntIntPair s = new IntIntPair(i, j);
           q.add(s);
           // Add to first Rk
-          Vector<IntIntPair> v = new Vector<IntIntPair>();
+          ArrayList<IntIntPair> v = new ArrayList<IntIntPair>();
           v.add(s);
           Rk.add(v);
         }
@@ -117,10 +118,10 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
 
       // build Ri
       for(int i = 2; i <= k; i++) {
-        ArrayList<Vector<IntIntPair>> Rnew = new ArrayList<Vector<IntIntPair>>();
+        ArrayList<ArrayList<IntIntPair>> Rnew = new ArrayList<ArrayList<IntIntPair>>();
 
         for(int j = 0; j < Rk.size(); j++) {
-          Vector<IntIntPair> c = Rk.get(j);
+          ArrayList<IntIntPair> c = Rk.get(j);
           for(IntIntPair pair : q) {
             boolean invalid = false;
             for(int t = 0; t < c.size(); t++) {
@@ -130,7 +131,7 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
               }
             }
             if(!invalid) {
-              Vector<IntIntPair> neu = new Vector<IntIntPair>(c);
+              ArrayList<IntIntPair> neu = new ArrayList<IntIntPair>(c);
               neu.add(pair);
               Rnew.add(neu);
             }
@@ -142,9 +143,9 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
 
     WritableDoubleDataStore sparsity = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC);
     // calculate the sparsity coefficient
-    for(Vector<IntIntPair> sub : Rk) {
+    for(ArrayList<IntIntPair> sub : Rk) {
       DBIDs ids = computeSubspace(sub, ranges);
-      final double sparsityC = sparsity(ids.size(), size, k);
+      final double sparsityC = sparsity(ids.size(), size, k, phi);
 
       if(sparsityC < 0) {
         for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
@@ -171,7 +172,7 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -181,7 +182,7 @@ public class AggarwalYuNaive<V extends NumberVector<?, ?>> extends AbstractAggar
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<?, ?>> extends AbstractAggarwalYuOutlier.Parameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractAggarwalYuOutlier.Parameterizer {
     @Override
     protected AggarwalYuNaive<V> makeInstance() {
       return new AggarwalYuNaive<V>(k, phi);
