@@ -36,13 +36,15 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDFactory;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.HashSetModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DoubleDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.LPNormDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
@@ -91,11 +93,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 @Title("Fast Outlier Detection in High Dimensional Spaces")
 @Description("Algorithm to compute outliers using Hilbert space filling curves")
 @Reference(authors = "F. Angiulli, C. Pizzuti", title = "Fast Outlier Detection in High Dimensional Spaces", booktitle = "Proc. European Conference on Principles of Knowledge Discovery and Data Mining (PKDD'02)", url = "http://dx.doi.org/10.1145/375663.375668")
-public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedAlgorithm<O, DoubleDistance, OutlierResult> implements OutlierAlgorithm {
+public class HilOut<O extends NumberVector<?>> extends AbstractDistanceBasedAlgorithm<O, DoubleDistance, OutlierResult> implements OutlierAlgorithm {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(HilOut.class);
+  private static final Logging LOG = Logging.getLogger(HilOut.class);
 
   /**
    * Number of nearest neighbors
@@ -170,7 +172,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
 
   public OutlierResult run(Database database, Relation<O> relation) {
     distq = database.getDistanceQuery(relation, getDistanceFunction());
-    d = DatabaseUtil.dimensionality(relation);
+    d = RelationUtil.dimensionality(relation);
     WritableDoubleDataStore hilout_weight = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
 
     // Compute extend of dataset.
@@ -181,18 +183,18 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       min = new double[d];
       double[] max = new double[d];
       for(int i = 0; i < d; i++) {
-        min[i] = hbbs.first.doubleValue(i + 1);
-        max[i] = hbbs.second.doubleValue(i + 1);
+        min[i] = hbbs.first.doubleValue(i);
+        max[i] = hbbs.second.doubleValue(i);
         diameter = Math.max(diameter, max[i] - min[i]);
       }
       // Enlarge bounding box to have equal lengths.
       for(int i = 0; i < d; i++) {
-        double diff = (diameter - (max[i] - min[i])) / 2;
+        double diff = (diameter - (max[i] - min[i])) * .5;
         min[i] -= diff;
         max[i] += diff;
       }
-      if(logger.isVerbose()) {
-        logger.verbose("Rescaling dataset by " + (1 / diameter) + " to fit the unit cube.");
+      if(LOG.isVerbose()) {
+        LOG.verbose("Rescaling dataset by " + (1 / diameter) + " to fit the unit cube.");
       }
     }
 
@@ -200,8 +202,8 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
     capital_n_star = capital_n = relation.size();
     HilbertFeatures h = new HilbertFeatures(relation, min, diameter);
 
-    FiniteProgress progressHilOut = logger.isVerbose() ? new FiniteProgress("HilOut iterations", d + 1, logger) : null;
-    FiniteProgress progressTrueOut = logger.isVerbose() ? new FiniteProgress("True outliers found", n, logger) : null;
+    FiniteProgress progressHilOut = LOG.isVerbose() ? new FiniteProgress("HilOut iterations", d + 1, LOG) : null;
+    FiniteProgress progressTrueOut = LOG.isVerbose() ? new FiniteProgress("True outliers found", n, LOG) : null;
     // Main part: 1. Phase max. d+1 loops
     for(int j = 0; j <= d && n_star < n; j++) {
       // initialize (clear) out and wlb - not 100% clear in the paper
@@ -214,7 +216,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       // determine the true outliers (n_star)
       trueOutliers(h);
       if(progressTrueOut != null) {
-        progressTrueOut.setProcessed(n_star, logger);
+        progressTrueOut.setProcessed(n_star, LOG);
       }
       // Build the top Set as out + wlb
       h.top.clear();
@@ -230,7 +232,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
         }
       }
       if(progressHilOut != null) {
-        progressHilOut.incrementProcessed(logger);
+        progressHilOut.incrementProcessed(LOG);
       }
     }
     // 2. Phase: Additional Scan if less than n true outliers determined
@@ -241,12 +243,12 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       scan(h, capital_n);
     }
     if(progressHilOut != null) {
-      progressHilOut.setProcessed(d, logger);
-      progressHilOut.ensureCompleted(logger);
+      progressHilOut.setProcessed(d, LOG);
+      progressHilOut.ensureCompleted(LOG);
     }
     if(progressTrueOut != null) {
-      progressTrueOut.setProcessed(n, logger);
-      progressTrueOut.ensureCompleted(logger);
+      progressTrueOut.setProcessed(n, LOG);
+      progressTrueOut.ensureCompleted(LOG);
     }
     DoubleMinMax minmax = new DoubleMinMax();
     // Return weights in out
@@ -281,8 +283,8 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
    */
   private void scan(HilbertFeatures hf, int k0) {
     final int mink0 = Math.min(2 * k0, capital_n - 1);
-    if(logger.isDebuggingFine()) {
-      logger.debugFine("Scanning with k0=" + k0 + " (" + mink0 + ")" + " N*=" + capital_n_star);
+    if(LOG.isDebuggingFine()) {
+      LOG.debugFine("Scanning with k0=" + k0 + " (" + mink0 + ")" + " N*=" + capital_n_star);
     }
     for(int i = 0; i < hf.pf.length; i++) {
       if(hf.pf[i].ubound < omega_star) {
@@ -366,7 +368,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
           if(mlevel < level) {
             level = mlevel;
             final double delta = hf.minDistLevel(hf.pf[i].id, level);
-            if(delta >= hf.pf[i].nn.peek().getDoubleDistance()) {
+            if(delta >= hf.pf[i].nn.peek().doubleDistance()) {
               break; // stop = true
             }
           }
@@ -376,10 +378,10 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
     double br = hf.boxRadius(i, a - 1, b + 1);
     double newlb = 0.0;
     double newub = 0.0;
-    for(DoubleDistanceResultPair entry : hf.pf[i].nn) {
-      newub += entry.getDoubleDistance();
-      if(entry.getDoubleDistance() <= br) {
-        newlb += entry.getDoubleDistance();
+    for(DoubleDistanceDBIDPair entry : hf.pf[i].nn) {
+      newub += entry.doubleDistance();
+      if(entry.doubleDistance() <= br) {
+        newlb += entry.doubleDistance();
       }
     }
     if(newlb > hf.pf[i].lbound) {
@@ -408,7 +410,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   @Override
@@ -482,7 +484,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
 
       int pos = 0;
       for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        pf[pos++] = new HilFeature(iditer.getDBID(), new Heap<DoubleDistanceResultPair>(k, Collections.reverseOrder()));
+        pf[pos++] = new HilFeature(DBIDUtil.deref(iditer), new Heap<DoubleDistanceDBIDPair>(k, Collections.reverseOrder()));
       }
       this.out = new Heap<HilFeature>(n, new Comparator<HilFeature>() {
         @Override
@@ -513,7 +515,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       if(h >= 32) { // 32 to 63 bit
         final long scale = Long.MAX_VALUE; // = 63 bits
         for(int i = 0; i < pf.length; i++) {
-          NumberVector<?, ?> obj = relation.get(pf[i].id);
+          NumberVector<?> obj = relation.get(pf[i].id);
           long[] coord = new long[d];
           for(int dim = 0; dim < d; dim++) {
             coord[dim] = (long) (getDimForObject(obj, dim) * .5 * scale);
@@ -524,7 +526,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       else if(h >= 16) { // 16-31 bit
         final int scale = ~1 >>> 1;
         for(int i = 0; i < pf.length; i++) {
-          NumberVector<?, ?> obj = relation.get(pf[i].id);
+          NumberVector<?> obj = relation.get(pf[i].id);
           int[] coord = new int[d];
           for(int dim = 0; dim < d; dim++) {
             coord[dim] = (int) (getDimForObject(obj, dim) * .5 * scale);
@@ -535,7 +537,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       else if(h >= 8) { // 8-15 bit
         final int scale = ~1 >>> 16;
         for(int i = 0; i < pf.length; i++) {
-          NumberVector<?, ?> obj = relation.get(pf[i].id);
+          NumberVector<?> obj = relation.get(pf[i].id);
           short[] coord = new short[d];
           for(int dim = 0; dim < d; dim++) {
             coord[dim] = (short) (getDimForObject(obj, dim) * .5 * scale);
@@ -546,7 +548,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
       else { // 1-7 bit
         final int scale = ~1 >>> 8;
         for(int i = 0; i < pf.length; i++) {
-          NumberVector<?, ?> obj = relation.get(pf[i].id);
+          NumberVector<?> obj = relation.get(pf[i].id);
           byte[] coord = new byte[d];
           for(int dim = 0; dim < d; dim++) {
             coord[dim] = (byte) (getDimForObject(obj, dim) * .5 * scale);
@@ -575,15 +577,13 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      */
     private void updateOUT(int i) {
       if(out.size() < n) {
-        out.offer(pf[i]);
+        out.add(pf[i]);
       }
       else {
         HilFeature head = out.peek();
         if(pf[i].ubound > head.ubound) {
           // replace smallest
-          out.poll();
-          // assert (out.peek().ubound >= head.ubound);
-          out.offer(pf[i]);
+          out.replaceTopElement(pf[i]);
         }
       }
     }
@@ -595,15 +595,13 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      */
     private void updateWLB(int i) {
       if(wlb.size() < n) {
-        wlb.offer(pf[i]);
+        wlb.add(pf[i]);
       }
       else {
         HilFeature head = wlb.peek();
         if(pf[i].lbound > head.lbound) {
           // replace smallest
-          wlb.poll();
-          // assert (wlb.peek().lbound >= head.lbound);
-          wlb.offer(pf[i]);
+          wlb.replaceTopElement(pf[i]);
         }
       }
     }
@@ -639,7 +637,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      * @param level Level of the corresponding r-region
      */
     private double minDistLevel(DBID id, int level) {
-      final NumberVector<?, ?> obj = relation.get(id);
+      final NumberVector<?> obj = relation.get(id);
       // level 1 is supposed to have r=1 as in the original publication
       // 2 ^ - (level - 1)
       final double r = 1.0 / (1 << (level - 1));
@@ -659,7 +657,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      * @param level Level of the corresponding r-region
      */
     private double maxDistLevel(DBID id, int level) {
-      final NumberVector<?, ?> obj = relation.get(id);
+      final NumberVector<?> obj = relation.get(id);
       // level 1 is supposed to have r=1 as in the original publication
       final double r = 1.0 / (1 << (level - 1));
       double dist;
@@ -780,8 +778,8 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      * @param dim Dimension
      * @return Projected and shifted position
      */
-    private double getDimForObject(NumberVector<?, ?> obj, int dim) {
-      return (obj.doubleValue(dim + 1) - min[dim]) / diameter + shift;
+    private double getDimForObject(NumberVector<?> obj, int dim) {
+      return (obj.doubleValue(dim) - min[dim]) / diameter + shift;
     }
   }
 
@@ -824,7 +822,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
     /**
      * Heap with the nearest known neighbors
      */
-    public Heap<DoubleDistanceResultPair> nn;
+    public Heap<DoubleDistanceDBIDPair> nn;
 
     /**
      * Set representation of the nearest neighbors for faster lookups
@@ -842,7 +840,7 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
      * @param id Object ID
      * @param nn Heap for neighbors
      */
-    public HilFeature(DBID id, Heap<DoubleDistanceResultPair> nn) {
+    public HilFeature(DBID id, Heap<DoubleDistanceDBIDPair> nn) {
       super();
       this.id = id;
       this.nn = nn;
@@ -864,27 +862,26 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
     protected void insert(DBID id, double dt, int k) {
       // assert (!nn_keys.contains(id));
       if(nn.size() < k) {
-        DoubleDistanceResultPair entry = new DoubleDistanceResultPair(dt, id);
-        nn.offer(entry);
+        DoubleDistanceDBIDPair entry = DBIDFactory.FACTORY.newDistancePair(dt, id);
+        nn.add(entry);
         nn_keys.add(id);
         sum_nn += dt;
       }
       else {
-        DoubleDistanceResultPair head = nn.peek();
-        if(dt < head.getDoubleDistance()) {
+        DoubleDistanceDBIDPair head = nn.peek();
+        if(dt < head.doubleDistance()) {
           head = nn.poll(); // Remove worst
-          sum_nn -= head.getDoubleDistance();
-          nn_keys.remove(head.getDBID());
+          sum_nn -= head.doubleDistance();
+          nn_keys.remove(head);
 
-          // assert (nn.peek().getDoubleDistance() <= head.getDoubleDistance());
+          // assert (nn.peek().doubleDistance() <= head.doubleDistance());
 
-          DoubleDistanceResultPair entry = new DoubleDistanceResultPair(dt, id);
-          nn.offer(entry);
+          DoubleDistanceDBIDPair entry = DBIDFactory.FACTORY.newDistancePair(dt, id);
+          nn.add(entry);
           nn_keys.add(id);
           sum_nn += dt;
         }
       }
-
     }
   }
 
@@ -897,33 +894,33 @@ public class HilOut<O extends NumberVector<O, ?>> extends AbstractDistanceBasedA
    *
    * @param <O> Vector type
    */
-  public static class Parameterizer<O extends NumberVector<O, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<O extends NumberVector<?>> extends AbstractParameterizer {
     /**
      * Parameter to specify how many next neighbors should be used in the
      * computation
      */
-    public static final OptionID K_ID = OptionID.getOrCreateOptionID("HilOut.k", "Compute up to k next neighbors");
+    public static final OptionID K_ID = new OptionID("HilOut.k", "Compute up to k next neighbors");
 
     /**
      * Parameter to specify how many outliers should be computed
      */
-    public static final OptionID N_ID = OptionID.getOrCreateOptionID("HilOut.n", "Compute n outliers");
+    public static final OptionID N_ID = new OptionID("HilOut.n", "Compute n outliers");
 
     /**
      * Parameter to specify the maximum Hilbert-Level
      */
-    public static final OptionID H_ID = OptionID.getOrCreateOptionID("HilOut.h", "Max. Hilbert-Level");
+    public static final OptionID H_ID = new OptionID("HilOut.h", "Max. Hilbert-Level");
 
     /**
      * Parameter to specify p of LP-NormDistance
      */
-    public static final OptionID T_ID = OptionID.getOrCreateOptionID("HilOut.t", "t of Lt Metric");
+    public static final OptionID T_ID = new OptionID("HilOut.t", "t of Lt Metric");
 
     /**
      * Parameter to specify if only the Top n, or also approximations for the
      * other elements, should be returned
      */
-    public static final OptionID TN_ID = OptionID.getOrCreateOptionID("HilOut.tn", "output of Top n or all elements");
+    public static final OptionID TN_ID = new OptionID("HilOut.tn", "output of Top n or all elements");
 
     /**
      * Neighborhood size
