@@ -31,13 +31,14 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
@@ -81,13 +82,13 @@ public class LDOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBas
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(LDOF.class);
+  private static final Logging LOG = Logging.getLogger(LDOF.class);
 
   /**
    * Parameter to specify the number of nearest neighbors of an object to be
    * considered for computing its LDOF_SCORE, must be an integer greater than 1.
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("ldof.k", "The number of nearest neighbors of an object to be considered for computing its LDOF_SCORE.");
+  public static final OptionID K_ID = new OptionID("ldof.k", "The number of nearest neighbors of an object to be considered for computing its LDOF_SCORE.");
 
   /**
    * The baseline for LDOF values. The paper gives 0.5 for uniform
@@ -128,21 +129,22 @@ public class LDOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBas
     WritableDoubleDataStore ldofs = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP);
 
     // compute LOF_SCORE of each db object
-    if(logger.isVerbose()) {
-      logger.verbose("Computing LDOFs");
+    if(LOG.isVerbose()) {
+      LOG.verbose("Computing LDOFs");
     }
-    FiniteProgress progressLDOFs = logger.isVerbose() ? new FiniteProgress("LDOF_SCORE for objects", relation.size(), logger) : null;
+    FiniteProgress progressLDOFs = LOG.isVerbose() ? new FiniteProgress("LDOF_SCORE for objects", relation.size(), LOG) : null;
 
     Mean dxp = new Mean(), Dxp = new Mean();
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       KNNResult<D> neighbors = knnQuery.getKNNForDBID(iditer, k);
       // skip the point itself
       dxp.reset(); Dxp.reset();
-      for(DistanceResultPair<D> neighbor1 : neighbors) {
-        if(!neighbor1.sameDBID(iditer)) {
+      // TODO: optimize for double distances
+      for (DistanceDBIDResultIter<D> neighbor1 = neighbors.iter(); neighbor1.valid(); neighbor1.advance()) {
+        if(!DBIDUtil.equal(neighbor1, iditer)) {
           dxp.put(neighbor1.getDistance().doubleValue());
-          for(DistanceResultPair<D> neighbor2 : neighbors) {
-            if(!neighbor1.sameDBID(neighbor2) && !neighbor2.sameDBID(iditer)) {
+          for (DistanceDBIDResultIter<D> neighbor2 = neighbors.iter(); neighbor2.valid(); neighbor2.advance()) {
+            if(!DBIDUtil.equal(neighbor1, neighbor2) && !DBIDUtil.equal(neighbor2, iditer)) {
               Dxp.put(distFunc.distance(neighbor1, neighbor2).doubleValue());
             }
           }
@@ -157,11 +159,11 @@ public class LDOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBas
       ldofminmax.put(ldof);
 
       if(progressLDOFs != null) {
-        progressLDOFs.incrementProcessed(logger);
+        progressLDOFs.incrementProcessed(LOG);
       }
     }
     if(progressLDOFs != null) {
-      progressLDOFs.ensureCompleted(logger);
+      progressLDOFs.ensureCompleted(LOG);
     }
 
     // Build result representation.
@@ -177,7 +179,7 @@ public class LDOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBas
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -193,7 +195,8 @@ public class LDOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBas
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(1));
+      final IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(new GreaterConstraint(1));
       if(config.grab(kP)) {
         k = kP.getValue();
       }
