@@ -33,15 +33,18 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DoubleDistanceKNNList;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.index.preprocessed.knn.MaterializeKNNPreprocessor;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -76,7 +79,8 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  * 
  * @apiviz.has KNNQuery
  * 
- * @param <O> the type of DatabaseObjects handled by this Algorithm
+ * @param <O> type of objects handled by this algorithm
+ * @param <D> type of distances used
  */
 @Title("LoOP: Local Outlier Probabilities")
 @Description("Variant of the LOF algorithm normalized using statistical values.")
@@ -85,37 +89,37 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(LoOP.class);
+  private static final Logging LOG = Logging.getLogger(LoOP.class);
 
   /**
    * The distance function to determine the reachability distance between
    * database objects.
    */
-  public static final OptionID REACHABILITY_DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("loop.referencedistfunction", "Distance function to determine the density of an object.");
+  public static final OptionID REACHABILITY_DISTANCE_FUNCTION_ID = new OptionID("loop.referencedistfunction", "Distance function to determine the density of an object.");
 
   /**
    * The distance function to determine the reachability distance between
    * database objects.
    */
-  public static final OptionID COMPARISON_DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("loop.comparedistfunction", "Distance function to determine the reference set of an object.");
+  public static final OptionID COMPARISON_DISTANCE_FUNCTION_ID = new OptionID("loop.comparedistfunction", "Distance function to determine the reference set of an object.");
 
   /**
    * Parameter to specify the number of nearest neighbors of an object to be
    * considered for computing its LOOP_SCORE, must be an integer greater than 1.
    */
-  public static final OptionID KREACH_ID = OptionID.getOrCreateOptionID("loop.kref", "The number of nearest neighbors of an object to be used for the PRD value.");
+  public static final OptionID KREACH_ID = new OptionID("loop.kref", "The number of nearest neighbors of an object to be used for the PRD value.");
 
   /**
    * Parameter to specify the number of nearest neighbors of an object to be
    * considered for computing its LOOP_SCORE, must be an integer greater than 1.
    */
-  public static final OptionID KCOMP_ID = OptionID.getOrCreateOptionID("loop.kcomp", "The number of nearest neighbors of an object to be considered for computing its LOOP_SCORE.");
+  public static final OptionID KCOMP_ID = new OptionID("loop.kcomp", "The number of nearest neighbors of an object to be considered for computing its LOOP_SCORE.");
 
   /**
    * Parameter to specify the number of nearest neighbors of an object to be
    * considered for computing its LOOP_SCORE, must be an integer greater than 1.
    */
-  public static final OptionID LAMBDA_ID = OptionID.getOrCreateOptionID("loop.lambda", "The number of standard deviations to consider for density computation.");
+  public static final OptionID LAMBDA_ID = new OptionID("loop.lambda", "The number of standard deviations to consider for density computation.");
 
   /**
    * Holds the value of {@link #KREACH_ID}.
@@ -133,12 +137,12 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   double lambda;
 
   /**
-   * Preprocessor Step 1
+   * Preprocessor Step 1.
    */
   protected DistanceFunction<? super O, D> reachabilityDistanceFunction;
 
   /**
-   * Preprocessor Step 2
+   * Preprocessor Step 2.
    */
   protected DistanceFunction<? super O, D> comparisonDistanceFunction;
 
@@ -150,11 +154,11 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   /**
    * Constructor with parameters.
    * 
-   * @param kreach
-   * @param kcomp
-   * @param reachabilityDistanceFunction
-   * @param comparisonDistanceFunction
-   * @param lambda
+   * @param kreach k for reachability
+   * @param kcomp k for comparison
+   * @param reachabilityDistanceFunction distance function for reachability
+   * @param comparisonDistanceFunction distance function for comparison
+   * @param lambda Lambda parameter
    */
   public LoOP(int kreach, int kcomp, DistanceFunction<? super O, D> reachabilityDistanceFunction, DistanceFunction<? super O, D> comparisonDistanceFunction, double lambda) {
     super();
@@ -168,36 +172,35 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   /**
    * Get the kNN queries for the algorithm.
    * 
-   * @param database Database
-   * @param stepprog Progress logger
+   * @param database Database to analyze
+   * @param relation Relation to analyze
+   * @param stepprog Progress logger, may be {@code null}
    * @return result
    */
   protected Pair<KNNQuery<O, D>, KNNQuery<O, D>> getKNNQueries(Database database, Relation<O> relation, StepProgress stepprog) {
     KNNQuery<O, D> knnComp;
     KNNQuery<O, D> knnReach;
-    if(comparisonDistanceFunction == reachabilityDistanceFunction || comparisonDistanceFunction.equals(reachabilityDistanceFunction)) {
+    if (comparisonDistanceFunction == reachabilityDistanceFunction || comparisonDistanceFunction.equals(reachabilityDistanceFunction)) {
       // We need each neighborhood twice - use "HEAVY" flag.
       knnComp = QueryUtil.getKNNQuery(relation, comparisonDistanceFunction, Math.max(kreach, kcomp), DatabaseQuery.HINT_HEAVY_USE, DatabaseQuery.HINT_OPTIMIZED_ONLY, DatabaseQuery.HINT_NO_CACHE);
       // No optimized kNN query - use a preprocessor!
-      if(knnComp == null) {
-        if(stepprog != null) {
-          stepprog.beginStep(1, "Materializing neighborhoods with respect to reference neighborhood distance function.", logger);
+      if (knnComp == null) {
+        if (stepprog != null) {
+          stepprog.beginStep(1, "Materializing neighborhoods with respect to reference neighborhood distance function.", LOG);
         }
         MaterializeKNNPreprocessor<O, D> preproc = new MaterializeKNNPreprocessor<O, D>(relation, comparisonDistanceFunction, kcomp);
         database.addIndex(preproc);
         DistanceQuery<O, D> cdq = database.getDistanceQuery(relation, comparisonDistanceFunction);
         knnComp = preproc.getKNNQuery(cdq, kreach, DatabaseQuery.HINT_HEAVY_USE);
-      }
-      else {
-        if(stepprog != null) {
-          stepprog.beginStep(1, "Optimized neighborhoods provided by database.", logger);
+      } else {
+        if (stepprog != null) {
+          stepprog.beginStep(1, "Optimized neighborhoods provided by database.", LOG);
         }
       }
       knnReach = knnComp;
-    }
-    else {
-      if(stepprog != null) {
-        stepprog.beginStep(1, "Not materializing distance functions, since we request each DBID once only.", logger);
+    } else {
+      if (stepprog != null) {
+        stepprog.beginStep(1, "Not materializing distance functions, since we request each DBID once only.", LOG);
       }
       knnComp = QueryUtil.getKNNQuery(relation, comparisonDistanceFunction, kreach);
       knnReach = QueryUtil.getKNNQuery(relation, reachabilityDistanceFunction, kcomp);
@@ -215,17 +218,17 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   public OutlierResult run(Database database, Relation<O> relation) {
     final double sqrt2 = Math.sqrt(2.0);
 
-    StepProgress stepprog = logger.isVerbose() ? new StepProgress(5) : null;
+    StepProgress stepprog = LOG.isVerbose() ? new StepProgress(5) : null;
 
     Pair<KNNQuery<O, D>, KNNQuery<O, D>> pair = getKNNQueries(database, relation, stepprog);
     KNNQuery<O, D> knnComp = pair.getFirst();
     KNNQuery<O, D> knnReach = pair.getSecond();
 
     // Assert we got something
-    if(knnComp == null) {
+    if (knnComp == null) {
       throw new AbortException("No kNN queries supported by database for comparison distance function.");
     }
-    if(knnReach == null) {
+    if (knnReach == null) {
       throw new AbortException("No kNN queries supported by database for density estimation distance function.");
     }
 
@@ -233,29 +236,43 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
     WritableDoubleDataStore pdists = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP);
     Mean mean = new Mean();
     {// computing PRDs
-      if(stepprog != null) {
-        stepprog.beginStep(3, "Computing pdists", logger);
+      if (stepprog != null) {
+        stepprog.beginStep(3, "Computing pdists", LOG);
       }
-      FiniteProgress prdsProgress = logger.isVerbose() ? new FiniteProgress("pdists", relation.size(), logger) : null;
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      FiniteProgress prdsProgress = LOG.isVerbose() ? new FiniteProgress("pdists", relation.size(), LOG) : null;
+      for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         final KNNResult<D> neighbors = knnReach.getKNNForDBID(iditer, kreach);
         mean.reset();
         // use first kref neighbors as reference set
         int ks = 0;
-        for(DistanceResultPair<D> neighbor : neighbors) {
-          if(objectIsInKNN || !neighbor.sameDBID(iditer)) {
-            double d = neighbor.getDistance().doubleValue();
-            mean.put(d * d);
-            ks++;
-            if(ks >= kreach) {
-              break;
+        // TODO: optimize for double distances
+        if (neighbors instanceof DoubleDistanceKNNList) {
+          for (DoubleDistanceDBIDResultIter neighbor = ((DoubleDistanceKNNList) neighbors).iter(); neighbor.valid(); neighbor.advance()) {
+            if (objectIsInKNN || !DBIDUtil.equal(neighbor, iditer)) {
+              final double d = neighbor.doubleDistance();
+              mean.put(d * d);
+              ks++;
+              if (ks >= kreach) {
+                break;
+              }
+            }
+          }
+        } else {
+          for (DistanceDBIDResultIter<D> neighbor = neighbors.iter(); neighbor.valid(); neighbor.advance()) {
+            if (objectIsInKNN || !DBIDUtil.equal(neighbor, iditer)) {
+              double d = neighbor.getDistance().doubleValue();
+              mean.put(d * d);
+              ks++;
+              if (ks >= kreach) {
+                break;
+              }
             }
           }
         }
         double pdist = lambda * Math.sqrt(mean.getMean());
         pdists.putDouble(iditer, pdist);
-        if(prdsProgress != null) {
-          prdsProgress.incrementProcessed(logger);
+        if (prdsProgress != null) {
+          prdsProgress.incrementProcessed(LOG);
         }
       }
     }
@@ -263,63 +280,63 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
     WritableDoubleDataStore plofs = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_TEMP);
     MeanVariance mvplof = new MeanVariance();
     {// compute LOOP_SCORE of each db object
-      if(stepprog != null) {
-        stepprog.beginStep(4, "Computing PLOF", logger);
+      if (stepprog != null) {
+        stepprog.beginStep(4, "Computing PLOF", LOG);
       }
 
-      FiniteProgress progressPLOFs = logger.isVerbose() ? new FiniteProgress("PLOFs for objects", relation.size(), logger) : null;
+      FiniteProgress progressPLOFs = LOG.isVerbose() ? new FiniteProgress("PLOFs for objects", relation.size(), LOG) : null;
       MeanVariance mv = new MeanVariance();
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         final KNNResult<D> neighbors = knnComp.getKNNForDBID(iditer, kcomp);
         mv.reset();
         // use first kref neighbors as comparison set.
         int ks = 0;
-        for(DistanceResultPair<D> neighbor1 : neighbors) {
-          if(objectIsInKNN || !neighbor1.sameDBID(iditer)) {
-            mv.put(pdists.doubleValue(neighbor1));
+        for (DBIDIter neighbor = neighbors.iter(); neighbor.valid(); neighbor.advance()) {
+          if (objectIsInKNN || !DBIDUtil.equal(neighbor, iditer)) {
+            mv.put(pdists.doubleValue(neighbor));
             ks++;
-            if(ks >= kcomp) {
+            if (ks >= kcomp) {
               break;
             }
           }
         }
         double plof = Math.max(pdists.doubleValue(iditer) / mv.getMean(), 1.0);
-        if(Double.isNaN(plof) || Double.isInfinite(plof)) {
+        if (Double.isNaN(plof) || Double.isInfinite(plof)) {
           plof = 1.0;
         }
         plofs.putDouble(iditer, plof);
         mvplof.put((plof - 1.0) * (plof - 1.0));
 
-        if(progressPLOFs != null) {
-          progressPLOFs.incrementProcessed(logger);
+        if (progressPLOFs != null) {
+          progressPLOFs.incrementProcessed(LOG);
         }
       }
     }
 
     double nplof = lambda * Math.sqrt(mvplof.getMean());
-    if(logger.isDebugging()) {
-      logger.verbose("nplof normalization factor is " + nplof + " " + mvplof.getMean() + " " + mvplof.getSampleStddev());
+    if (LOG.isDebugging()) {
+      LOG.verbose("nplof normalization factor is " + nplof + " " + mvplof.getMean() + " " + mvplof.getSampleStddev());
     }
 
     // Compute final LoOP values.
     WritableDoubleDataStore loops = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
     {// compute LOOP_SCORE of each db object
-      if(stepprog != null) {
-        stepprog.beginStep(5, "Computing LoOP scores", logger);
+      if (stepprog != null) {
+        stepprog.beginStep(5, "Computing LoOP scores", LOG);
       }
 
-      FiniteProgress progressLOOPs = logger.isVerbose() ? new FiniteProgress("LoOP for objects", relation.size(), logger) : null;
-      for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
+      FiniteProgress progressLOOPs = LOG.isVerbose() ? new FiniteProgress("LoOP for objects", relation.size(), LOG) : null;
+      for (DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
         loops.putDouble(iditer, NormalDistribution.erf((plofs.doubleValue(iditer) - 1) / (nplof * sqrt2)));
 
-        if(progressLOOPs != null) {
-          progressLOOPs.incrementProcessed(logger);
+        if (progressLOOPs != null) {
+          progressLOOPs.incrementProcessed(LOG);
         }
       }
     }
 
-    if(stepprog != null) {
-      stepprog.setCompleted(logger);
+    if (stepprog != null) {
+      stepprog.setCompleted(LOG);
     }
 
     // Build result representation.
@@ -331,10 +348,9 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
   @Override
   public TypeInformation[] getInputTypeRestriction() {
     final TypeInformation type;
-    if(reachabilityDistanceFunction.equals(comparisonDistanceFunction)) {
+    if (reachabilityDistanceFunction.equals(comparisonDistanceFunction)) {
       type = reachabilityDistanceFunction.getInputTypeRestriction();
-    }
-    else {
+    } else {
       type = new CombinedTypeInformation(reachabilityDistanceFunction.getInputTypeRestriction(), comparisonDistanceFunction.getInputTypeRestriction());
     }
     return TypeUtil.array(type);
@@ -342,7 +358,7 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -369,45 +385,48 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
     double lambda = 2.0;
 
     /**
-     * Preprocessor Step 1
+     * Preprocessor Step 1.
      */
     protected DistanceFunction<O, D> reachabilityDistanceFunction = null;
 
     /**
-     * Preprocessor Step 2
+     * Preprocessor Step 2.
      */
     protected DistanceFunction<O, D> comparisonDistanceFunction = null;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter kcompP = new IntParameter(KCOMP_ID, new GreaterConstraint(1));
-      if(config.grab(kcompP)) {
-        kcomp = kcompP.getValue();
+      final IntParameter kcompP = new IntParameter(KCOMP_ID);
+      kcompP.addConstraint(new GreaterConstraint(1));
+      if (config.grab(kcompP)) {
+        kcomp = kcompP.intValue();
       }
 
       final ObjectParameter<DistanceFunction<O, D>> compDistP = new ObjectParameter<DistanceFunction<O, D>>(COMPARISON_DISTANCE_FUNCTION_ID, DistanceFunction.class, EuclideanDistanceFunction.class);
-      if(config.grab(compDistP)) {
+      if (config.grab(compDistP)) {
         comparisonDistanceFunction = compDistP.instantiateClass(config);
       }
 
-      final IntParameter kreachP = new IntParameter(KREACH_ID, new GreaterConstraint(1), true);
-      if(config.grab(kreachP)) {
-        kreach = kreachP.getValue();
-      }
-      else {
+      final IntParameter kreachP = new IntParameter(KREACH_ID);
+      kreachP.addConstraint(new GreaterConstraint(1));
+      kreachP.setOptional(true);
+      if (config.grab(kreachP)) {
+        kreach = kreachP.intValue();
+      } else {
         kreach = kcomp;
       }
 
       final ObjectParameter<DistanceFunction<O, D>> reachDistP = new ObjectParameter<DistanceFunction<O, D>>(REACHABILITY_DISTANCE_FUNCTION_ID, DistanceFunction.class, true);
-      if(config.grab(reachDistP)) {
+      if (config.grab(reachDistP)) {
         reachabilityDistanceFunction = reachDistP.instantiateClass(config);
       }
 
       // TODO: make default 1.0?
-      final DoubleParameter lambdaP = new DoubleParameter(LAMBDA_ID, new GreaterConstraint(0.0), 2.0);
-      if(config.grab(lambdaP)) {
-        lambda = lambdaP.getValue();
+      final DoubleParameter lambdaP = new DoubleParameter(LAMBDA_ID, 2.0);
+      lambdaP.addConstraint(new GreaterConstraint(0.0));
+      if (config.grab(lambdaP)) {
+        lambda = lambdaP.doubleValue();
       }
     }
 
@@ -417,4 +436,4 @@ public class LoOP<O, D extends NumberDistance<D, ?>> extends AbstractAlgorithm<O
       return new LoOP<O, D>(kreach, kcomp, realreach, comparisonDistanceFunction, lambda);
     }
   }
-}
\ No newline at end of file
+}
