@@ -37,14 +37,14 @@ import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableIntegerDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.query.range.RangeQuery;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
@@ -83,7 +83,7 @@ public class OPTICSOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanc
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(OPTICSOF.class);
+  private static final Logging LOG = Logging.getLogger(OPTICSOF.class);
 
   /**
    * Parameter to specify the threshold MinPts.
@@ -136,9 +136,10 @@ public class OPTICSOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanc
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       List<Double> core = new ArrayList<Double>();
       double lrd = 0;
-      for(DistanceResultPair<D> neighPair : nMinPts.get(iditer)) {
-        double coreDist = coreDistance.doubleValue(neighPair);
-        double dist = distQuery.distance(iditer, neighPair).doubleValue();
+      // TODO: optimize for double distances
+      for (DistanceDBIDResultIter<D> neighbor = nMinPts.get(iditer).iter(); neighbor.valid(); neighbor.advance()) {
+        double coreDist = coreDistance.doubleValue(neighbor);
+        double dist = distQuery.distance(iditer, neighbor).doubleValue();
         double rd = Math.max(coreDist, dist);
         lrd = rd + lrd;
         core.add(rd);
@@ -153,9 +154,9 @@ public class OPTICSOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanc
     WritableDoubleDataStore ofs = DataStoreUtil.makeDoubleStorage(ids, DataStoreFactory.HINT_STATIC);
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       double of = 0;
-      for(DistanceResultPair<D> pair : nMinPts.get(iditer)) {
+      for (DBIDIter neighbor = nMinPts.get(iditer).iter(); neighbor.valid(); neighbor.advance()) {
         double lrd = lrds.doubleValue(iditer);
-        double lrdN = lrds.doubleValue(pair);
+        double lrdN = lrds.doubleValue(neighbor);
         of = of + lrdN / lrd;
       }
       of = of / minPtsNeighborhoodSize.intValue(iditer);
@@ -176,7 +177,7 @@ public class OPTICSOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanc
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -192,7 +193,8 @@ public class OPTICSOF<O, D extends NumberDistance<D, ?>> extends AbstractDistanc
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter param = new IntParameter(OPTICS.MINPTS_ID, new GreaterConstraint(1));
+      final IntParameter param = new IntParameter(OPTICS.MINPTS_ID);
+      param.addConstraint(new GreaterConstraint(1));
       if(config.grab(param)) {
         minpts = param.getValue();
       }
