@@ -39,6 +39,7 @@ import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.subspace.SubspaceEuclideanDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -47,7 +48,7 @@ import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.result.outlier.BasicOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.utilities.RandomFactory;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -57,7 +58,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualCons
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.LongParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
 
 /**
@@ -85,22 +86,22 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(FeatureBagging.class);
+  private static final Logging LOG = Logging.getLogger(FeatureBagging.class);
 
   /**
-   * Number of instances to use
+   * Number of instances to use.
    */
   protected int num = 1;
 
   /**
-   * Cumulative sum or breadth first combinations
+   * Cumulative sum or breadth first combinations.
    */
   protected boolean breadth = false;
 
   /**
-   * Random number generator for subspace choice
+   * Random number generator for subspace choice.
    */
-  private Random RANDOM;
+  private RandomFactory rnd;
 
   /**
    * The parameters k for LOF.
@@ -113,18 +114,14 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
    * @param k k Parameter for LOF
    * @param num Number of subspaces to use
    * @param breadth Flag for breadth-first merging
+   * @param rnd Random generator
    */
-  public FeatureBagging(int k, int num, boolean breadth, Long seed) {
+  public FeatureBagging(int k, int num, boolean breadth, RandomFactory rnd) {
     super();
     this.k = k;
     this.num = num;
     this.breadth = breadth;
-    if(seed != null) {
-      this.RANDOM = new Random(seed);
-    }
-    else {
-      this.RANDOM = new Random();
-    }
+    this.rnd = rnd;
   }
 
   /**
@@ -133,80 +130,79 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
    * @param relation Relation to use
    * @return Outlier detection result
    */
-  public OutlierResult run(Relation<NumberVector<?, ?>> relation) {
-    final int dbdim = DatabaseUtil.dimensionality(relation);
-    final int mindim = dbdim / 2;
+  public OutlierResult run(Relation<NumberVector<?>> relation) {
+    final int dbdim = RelationUtil.dimensionality(relation);
+    final int mindim = dbdim >> 1;
     final int maxdim = dbdim - 1;
+    final Random rand = rnd.getRandom();
 
     ArrayList<OutlierResult> results = new ArrayList<OutlierResult>(num);
     {
-      FiniteProgress prog = logger.isVerbose() ? new FiniteProgress("LOF iterations", num, logger) : null;
-      for(int i = 0; i < num; i++) {
-        BitSet dimset = randomSubspace(dbdim, mindim, maxdim);
+      FiniteProgress prog = LOG.isVerbose() ? new FiniteProgress("LOF iterations", num, LOG) : null;
+      for (int i = 0; i < num; i++) {
+        BitSet dimset = randomSubspace(dbdim, mindim, maxdim, rand);
         SubspaceEuclideanDistanceFunction df = new SubspaceEuclideanDistanceFunction(dimset);
-        LOF<NumberVector<?, ?>, DoubleDistance> lof = new LOF<NumberVector<?, ?>, DoubleDistance>(k, df);
+        LOF<NumberVector<?>, DoubleDistance> lof = new LOF<NumberVector<?>, DoubleDistance>(k, df);
 
         // run LOF and collect the result
         OutlierResult result = lof.run(relation);
         results.add(result);
-        if(prog != null) {
-          prog.incrementProcessed(logger);
+        if (prog != null) {
+          prog.incrementProcessed(LOG);
         }
       }
-      if(prog != null) {
-        prog.ensureCompleted(logger);
+      if (prog != null) {
+        prog.ensureCompleted(LOG);
       }
     }
 
     WritableDoubleDataStore scores = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
     DoubleMinMax minmax = new DoubleMinMax();
-    if(breadth) {
-      FiniteProgress cprog = logger.isVerbose() ? new FiniteProgress("Combining results", relation.size(), logger) : null;
+    if (breadth) {
+      FiniteProgress cprog = LOG.isVerbose() ? new FiniteProgress("Combining results", relation.size(), LOG) : null;
       Pair<DBIDIter, Relation<Double>>[] IDVectorOntoScoreVector = Pair.newPairArray(results.size());
 
       // Mapping score-sorted DBID-Iterators onto their corresponding scores.
       // We need to initialize them now be able to iterate them "in parallel".
       {
         int i = 0;
-        for(OutlierResult r : results) {
+        for (OutlierResult r : results) {
           IDVectorOntoScoreVector[i] = new Pair<DBIDIter, Relation<Double>>(r.getOrdering().iter(relation.getDBIDs()).iter(), r.getScores());
           i++;
         }
       }
 
       // Iterating over the *lines* of the AS_t(i)-matrix.
-      for(int i = 0; i < relation.size(); i++) {
+      for (int i = 0; i < relation.size(); i++) {
         // Iterating over the elements of a line (breadth-first).
-        for(Pair<DBIDIter, Relation<Double>> pair : IDVectorOntoScoreVector) {
+        for (Pair<DBIDIter, Relation<Double>> pair : IDVectorOntoScoreVector) {
           DBIDIter iter = pair.first;
           // Always true if every algorithm returns a complete result (one score
           // for every DBID).
-          if(iter.valid()) {
+          if (iter.valid()) {
             double score = pair.second.get(iter);
-            if(Double.isNaN(scores.doubleValue(iter))) {
+            if (Double.isNaN(scores.doubleValue(iter))) {
               scores.putDouble(iter, score);
               minmax.put(score);
             }
             iter.advance();
-          }
-          else {
-            logger.warning("Incomplete result: Iterator does not contain |DB| DBIDs");
+          } else {
+            LOG.warning("Incomplete result: Iterator does not contain |DB| DBIDs");
           }
         }
         // Progress does not take the initial mapping into account.
-        if(cprog != null) {
-          cprog.incrementProcessed(logger);
+        if (cprog != null) {
+          cprog.incrementProcessed(LOG);
         }
       }
-      if(cprog != null) {
-        cprog.ensureCompleted(logger);
+      if (cprog != null) {
+        cprog.ensureCompleted(LOG);
       }
-    }
-    else {
-      FiniteProgress cprog = logger.isVerbose() ? new FiniteProgress("Combining results", relation.size(), logger) : null;
-      for(DBIDIter iter = relation.iterDBIDs(); iter.valid(); iter.advance()) {
+    } else {
+      FiniteProgress cprog = LOG.isVerbose() ? new FiniteProgress("Combining results", relation.size(), LOG) : null;
+      for (DBIDIter iter = relation.iterDBIDs(); iter.valid(); iter.advance()) {
         double sum = 0.0;
-        for(OutlierResult r : results) {
+        for (OutlierResult r : results) {
           final Double s = r.getScores().get(iter);
           if (s != null && !Double.isNaN(s)) {
             sum += s;
@@ -214,12 +210,12 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
         }
         scores.putDouble(iter, sum);
         minmax.put(sum);
-        if(cprog != null) {
-          cprog.incrementProcessed(logger);
+        if (cprog != null) {
+          cprog.incrementProcessed(LOG);
         }
       }
-      if(cprog != null) {
-        cprog.ensureCompleted(logger);
+      if (cprog != null) {
+        cprog.ensureCompleted(LOG);
       }
     }
     OutlierScoreMeta meta = new BasicOutlierScoreMeta(minmax.getMin(), minmax.getMax());
@@ -228,36 +224,34 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
   }
 
   /**
-   * Choose a random subspace
+   * Choose a random subspace.
    * 
    * @param alldim Number of total dimensions
    * @param mindim Minimum number to choose
    * @param maxdim Maximum number to choose
    * @return Subspace as bits.
    */
-  private BitSet randomSubspace(final int alldim, final int mindim, final int maxdim) {
+  private BitSet randomSubspace(final int alldim, final int mindim, final int maxdim, final Random rand) {
     BitSet dimset = new BitSet();
-    {
-      // Fill with all dimensions
-      int[] dims = new int[alldim];
-      for(int d = 0; d < alldim; d++) {
-        dims[d] = d;
-      }
-      // Target dimensionality:
-      int subdim = mindim + RANDOM.nextInt(maxdim - mindim);
-      // Shrink the subspace to the destination size
-      for(int d = 0; d < alldim - subdim; d++) {
-        int s = RANDOM.nextInt(alldim - d);
-        dimset.set(dims[s]);
-        dims[s] = dims[alldim - d - 1];
-      }
+    // Fill with all dimensions
+    int[] dims = new int[alldim];
+    for (int d = 0; d < alldim; d++) {
+      dims[d] = d;
+    }
+    // Target dimensionality:
+    int subdim = mindim + rand.nextInt(maxdim - mindim);
+    // Shrink the subspace to the destination size
+    for (int d = 0; d < alldim - subdim; d++) {
+      int s = rand.nextInt(alldim - d);
+      dimset.set(dims[s]);
+      dims[s] = dims[alldim - d - 1];
     }
     return dimset;
   }
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   @Override
@@ -279,69 +273,71 @@ public class FeatureBagging extends AbstractAlgorithm<OutlierResult> implements
      * Key: {@code -fbagging.num}
      * </p>
      */
-    public static final OptionID NUM_ID = OptionID.getOrCreateOptionID("fbagging.num", "The number of instances to use in the ensemble.");
+    public static final OptionID NUM_ID = new OptionID("fbagging.num", "The number of instances to use in the ensemble.");
 
     /**
-     * The flag for using the breadth first approach
+     * The flag for using the breadth first approach.
      * <p>
      * Key: {@code -fbagging.breadth}
      * </p>
      */
-    public static final OptionID BREADTH_ID = OptionID.getOrCreateOptionID("fbagging.breadth", "Use the breadth first combinations instead of the cumulative sum approach");
+    public static final OptionID BREADTH_ID = new OptionID("fbagging.breadth", "Use the breadth first combinations instead of the cumulative sum approach");
 
     /**
-     * The parameter to specify the random seed
+     * The parameter to specify the random seed.
      * <p>
      * Key: {@code -fbagging.seed}
      * </p>
      */
-    public static final OptionID SEED_ID = OptionID.getOrCreateOptionID("fbagging.seed", "Specify a particular random seed.");
+    public static final OptionID SEED_ID = new OptionID("fbagging.seed", "Specify a particular random seed.");
 
     /**
-     * The neighborhood size to use
+     * The neighborhood size to use.
      */
     protected int k = 2;
 
     /**
-     * Number of instances to use
+     * Number of instances to use.
      */
     protected int num = 1;
 
     /**
-     * Cumulative sum or breadth first combinations
+     * Cumulative sum or breadth first combinations.
      */
     protected boolean breadth = false;
 
     /**
-     * Random generator seed
+     * Random generator.
      */
-    protected Long seed = null;
+    protected RandomFactory rnd;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter pK = new IntParameter(LOF.K_ID, new GreaterConstraint(1));
-      if(config.grab(pK)) {
+      final IntParameter pK = new IntParameter(LOF.K_ID);
+      pK.addConstraint(new GreaterConstraint(1));
+      if (config.grab(pK)) {
         k = pK.getValue();
       }
-      IntParameter NUM_PARAM = new IntParameter(NUM_ID, new GreaterEqualConstraint(1));
-      if(config.grab(NUM_PARAM)) {
-        num = NUM_PARAM.getValue();
+      IntParameter numP = new IntParameter(NUM_ID);
+      numP.addConstraint(new GreaterEqualConstraint(1));
+      if (config.grab(numP)) {
+        num = numP.getValue();
       }
-      Flag BREADTH_FLAG = new Flag(BREADTH_ID);
-      if(config.grab(BREADTH_FLAG)) {
-        breadth = BREADTH_FLAG.getValue();
+      Flag breadthF = new Flag(BREADTH_ID);
+      if (config.grab(breadthF)) {
+        breadth = breadthF.getValue();
       }
-      LongParameter seedP = new LongParameter(SEED_ID, true);
-      if(config.grab(seedP)) {
-        seed = seedP.getValue();
+      RandomParameter rndP = new RandomParameter(SEED_ID);
+      if (config.grab(rndP)) {
+        rnd = rndP.getValue();
       }
     }
 
     @Override
     protected FeatureBagging makeInstance() {
       // Default is to re-use the same distance
-      return new FeatureBagging(k, num, breadth, seed);
+      return new FeatureBagging(k, num, breadth, rnd);
     }
   }
-}
\ No newline at end of file
+}
