@@ -37,13 +37,13 @@ import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.ProxyView;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
@@ -52,7 +52,6 @@ import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
 import de.lmu.ifi.dbs.elki.result.outlier.BasicOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -85,11 +84,11 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.Pair;
  */
 @Title("GLS-Backward Search")
 @Reference(authors = "F. Chen and C.-T. Lu and A. P. Boedihardjo", title = "GLS-SOD: A Generalized Local Statistical Approach for Spatial Outlier Detection", booktitle = "Proc. 16th ACM SIGKDD international conference on Knowledge discovery and data mining", url = "http://dx.doi.org/10.1145/1835804.1835939")
-public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm<V, D, OutlierResult> implements OutlierAlgorithm {
+public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm<V, D, OutlierResult> implements OutlierAlgorithm {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(CTLuGLSBackwardSearchAlgorithm.class);
+  private static final Logging LOG = Logging.getLogger(CTLuGLSBackwardSearchAlgorithm.class);
 
   /**
    * Parameter Alpha - significance niveau
@@ -121,7 +120,7 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
    * @param relationy Attribute relation
    * @return Algorithm result
    */
-  public OutlierResult run(Relation<V> relationx, Relation<? extends NumberVector<?, ?>> relationy) {
+  public OutlierResult run(Relation<V> relationx, Relation<? extends NumberVector<?>> relationy) {
     WritableDoubleDataStore scores = DataStoreUtil.makeDoubleStorage(relationx.getDBIDs(), DataStoreFactory.HINT_STATIC);
     DoubleMinMax mm = new DoubleMinMax(0.0, 0.0);
 
@@ -130,7 +129,7 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
       ModifiableDBIDs idview = DBIDUtil.newHashSet(relationx.getDBIDs());
       ProxyView<V> proxy = new ProxyView<V>(relationx.getDatabase(), idview, relationx);
 
-      double phialpha = NormalDistribution.standardNormalQuantile(1.0 - alpha / 2);
+      double phialpha = NormalDistribution.standardNormalQuantile(1.0 - alpha *.5);
       // Detect outliers while significant.
       while(true) {
         Pair<DBID, Double> candidate = singleIteration(proxy, relationy);
@@ -138,15 +137,15 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
           break;
         }
         scores.putDouble(candidate.first, candidate.second);
-        if (!Double.isNaN(candidate.second)) {
+        if(!Double.isNaN(candidate.second)) {
           mm.put(candidate.second);
         }
         idview.remove(candidate.first);
       }
 
       // Remaining objects are inliers
-      for (DBIDIter iter = idview.iter(); iter.valid(); iter.advance()) {
-        scores.putDouble(iter.getDBID(), 0.0);
+      for(DBIDIter iter = idview.iter(); iter.valid(); iter.advance()) {
+        scores.putDouble(iter, 0.0);
       }
     }
 
@@ -162,9 +161,9 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
    * @param relationy Attribute relation
    * @return Top outlier and associated score
    */
-  private Pair<DBID, Double> singleIteration(Relation<V> relationx, Relation<? extends NumberVector<?, ?>> relationy) {
-    final int dim = DatabaseUtil.dimensionality(relationx);
-    final int dimy = DatabaseUtil.dimensionality(relationy);
+  private Pair<DBID, Double> singleIteration(Relation<V> relationx, Relation<? extends NumberVector<?>> relationy) {
+    final int dim = RelationUtil.dimensionality(relationx);
+    final int dimy = RelationUtil.dimensionality(relationy);
     assert (dim == 2);
     KNNQuery<V, D> knnQuery = QueryUtil.getKNNQuery(relationx, getDistanceFunction(), k + 1);
 
@@ -177,47 +176,51 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
     Matrix X = new Matrix(ids.size(), 6);
     Matrix F = new Matrix(ids.size(), ids.size());
     Matrix Y = new Matrix(ids.size(), dimy);
-    for(int i = 0; i < ids.size(); i++) {
-      DBID id = ids.get(i);
 
-      // Fill the data matrix
-      {
-        V vec = relationx.get(id);
-        double la = vec.doubleValue(1);
-        double lo = vec.doubleValue(2);
-        X.set(i, 0, 1.0);
-        X.set(i, 1, la);
-        X.set(i, 2, lo);
-        X.set(i, 3, la * lo);
-        X.set(i, 4, la * la);
-        X.set(i, 5, lo * lo);
-      }
-
-      {
-        for(int d = 0; d < dimy; d++) {
-          double idy = relationy.get(id).doubleValue(d + 1);
-          Y.set(i, d, idy);
+    {
+      int i = 0;
+      for(DBIDIter id = ids.iter(); id.valid(); id.advance(), i++) {
+        // Fill the data matrix
+        {
+          V vec = relationx.get(id);
+          double la = vec.doubleValue(0);
+          double lo = vec.doubleValue(1);
+          X.set(i, 0, 1.0);
+          X.set(i, 1, la);
+          X.set(i, 2, lo);
+          X.set(i, 3, la * lo);
+          X.set(i, 4, la * la);
+          X.set(i, 5, lo * lo);
         }
-      }
 
-      // Fill the neighborhood matrix F:
-      {
-        KNNResult<D> neighbors = knnQuery.getKNNForDBID(id, k + 1);
-        ModifiableDBIDs neighborhood = DBIDUtil.newArray(neighbors.size());
-        for(DistanceResultPair<D> dpair : neighbors) {
-          if(id.sameDBID(dpair.getDBID())) {
-            continue;
+        {
+          final NumberVector<?> vecy = relationy.get(id);
+          for(int d = 0; d < dimy; d++) {
+            double idy = vecy.doubleValue(d);
+            Y.set(i, d, idy);
           }
-          neighborhood.add(dpair.getDBID());
         }
-        // Weight object itself positively.
-        F.set(i, i, 1.0);
-        final int nweight = -1 / neighborhood.size();
-        // We need to find the index positions of the neighbors, unfortunately.
-        for (DBIDIter iter = neighborhood.iter(); iter.valid(); iter.advance()) {
-          int pos = ids.binarySearch(iter.getDBID());
-          assert (pos >= 0);
-          F.set(pos, i, nweight);
+
+        // Fill the neighborhood matrix F:
+        {
+          KNNResult<D> neighbors = knnQuery.getKNNForDBID(id, k + 1);
+          ModifiableDBIDs neighborhood = DBIDUtil.newArray(neighbors.size());
+          for(DBIDIter neighbor = neighbors.iter(); neighbor.valid(); neighbor.advance()) {
+            if(DBIDUtil.equal(id, neighbor)) {
+              continue;
+            }
+            neighborhood.add(neighbor);
+          }
+          // Weight object itself positively.
+          F.set(i, i, 1.0);
+          final int nweight = -1 / neighborhood.size();
+          // We need to find the index positions of the neighbors,
+          // unfortunately.
+          for(DBIDIter iter = neighborhood.iter(); iter.valid(); iter.advance()) {
+            int pos = ids.binarySearch(iter);
+            assert (pos >= 0);
+            F.set(pos, i, nweight);
+          }
         }
       }
     }
@@ -236,13 +239,13 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
 
     DBID worstid = null;
     double worstscore = Double.NEGATIVE_INFINITY;
-    for(int i = 0; i < ids.size(); i++) {
-      DBID id = ids.get(i);
+    int i = 0;
+    for(DBIDIter id = ids.iter(); id.valid(); id.advance(), i++) {
       double err = E.getRow(i).euclideanLength();
       // double err = Math.abs(E.get(i, 0));
       if(err > worstscore) {
         worstscore = err;
-        worstid = id;
+        worstid = DBIDUtil.deref(id);
       }
     }
 
@@ -256,7 +259,7 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -269,16 +272,16 @@ public class CTLuGLSBackwardSearchAlgorithm<V extends NumberVector<?, ?>, D exte
    * @param <V> Input vector type
    * @param <D> Distance type
    */
-  public static class Parameterizer<V extends NumberVector<?, ?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, D> {
+  public static class Parameterizer<V extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, D> {
     /**
      * Holds the alpha value - significance niveau
      */
-    public static final OptionID ALPHA_ID = OptionID.getOrCreateOptionID("glsbs.alpha", "Significance niveau");
+    public static final OptionID ALPHA_ID = new OptionID("glsbs.alpha", "Significance niveau");
 
     /**
      * Parameter to specify the k nearest neighbors
      */
-    public static final OptionID K_ID = OptionID.getOrCreateOptionID("glsbs.k", "k nearest neighbors to use");
+    public static final OptionID K_ID = new OptionID("glsbs.k", "k nearest neighbors to use");
 
     /**
      * Parameter Alpha - significance niveau
