@@ -33,11 +33,11 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.MaterializedRelation;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
@@ -45,14 +45,13 @@ import de.lmu.ifi.dbs.elki.math.Mean;
 import de.lmu.ifi.dbs.elki.result.outlier.BasicOutlierScoreMeta;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierResult;
 import de.lmu.ifi.dbs.elki.result.outlier.OutlierScoreMeta;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.QuickSelect;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint.IntervalBoundary;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 
@@ -83,15 +82,15 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(TrimmedMeanApproach.class);
+  private static final Logging LOG = Logging.getLogger(TrimmedMeanApproach.class);
 
   /**
-   * the parameter p
+   * the parameter p.
    */
   private double p;
 
   /**
-   * Constructor
+   * Constructor.
    * 
    * @param p Parameter p
    * @param npredf Neighborhood factory.
@@ -102,29 +101,28 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
   }
 
   /**
-   * Run the algorithm
+   * Run the algorithm.
    * 
    * @param database Database
    * @param nrel Neighborhood relation
    * @param relation Data Relation (1 dimensional!)
    * @return Outlier detection result
    */
-  public OutlierResult run(Database database, Relation<N> nrel, Relation<? extends NumberVector<?, ?>> relation) {
-    assert (DatabaseUtil.dimensionality(relation) == 1) : "TrimmedMean can only process one-dimensional data sets.";
+  public OutlierResult run(Database database, Relation<N> nrel, Relation<? extends NumberVector<?>> relation) {
+    assert (RelationUtil.dimensionality(relation) == 1) : "TrimmedMean can only process one-dimensional data sets.";
     final NeighborSetPredicate npred = getNeighborSetPredicateFactory().instantiate(nrel);
 
     WritableDoubleDataStore errors = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_TEMP);
     WritableDoubleDataStore scores = DataStoreUtil.makeDoubleStorage(relation.getDBIDs(), DataStoreFactory.HINT_STATIC);
 
-    FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Computing trimmed means", relation.size(), logger) : null;
+    FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Computing trimmed means", relation.size(), LOG) : null;
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id  = iditer.getDBID();
-      DBIDs neighbors = npred.getNeighborDBIDs(id);
+      DBIDs neighbors = npred.getNeighborDBIDs(iditer);
       int num = 0;
       double[] values = new double[neighbors.size()];
       // calculate trimmedMean
       for(DBIDIter iter = neighbors.iter(); iter.valid(); iter.advance()) {
-        values[num] = relation.get(iter).doubleValue(1);
+        values[num] = relation.get(iter).doubleValue(0);
         num++;
       }
 
@@ -141,21 +139,21 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
         tm = mean.getMean();
       }
       else {
-        tm = relation.get(id).doubleValue(1);
+        tm = relation.get(iditer).doubleValue(0);
       }
       // Error: deviation from trimmed mean
-      errors.putDouble(id, relation.get(id).doubleValue(1) - tm);
+      errors.putDouble(iditer, relation.get(iditer).doubleValue(0) - tm);
 
       if(progress != null) {
-        progress.incrementProcessed(logger);
+        progress.incrementProcessed(LOG);
       }
     }
     if(progress != null) {
-      progress.ensureCompleted(logger);
+      progress.ensureCompleted(LOG);
     }
 
-    if(logger.isVerbose()) {
-      logger.verbose("Computing median error.");
+    if(LOG.isVerbose()) {
+      LOG.verbose("Computing median error.");
     }
     double median_dev_from_median;
     {
@@ -164,8 +162,7 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
       {
         int i = 0;
         for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-          DBID id  = iditer.getDBID();
-          ei[i] = errors.doubleValue(id);
+          ei[i] = errors.doubleValue(iditer);
           i++;
         }
       }
@@ -178,15 +175,14 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
       median_dev_from_median = QuickSelect.median(ei);
     }
 
-    if(logger.isVerbose()) {
-      logger.verbose("Normalizing scores.");
+    if(LOG.isVerbose()) {
+      LOG.verbose("Normalizing scores.");
     }
     // calculate score
     DoubleMinMax minmax = new DoubleMinMax();
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id  = iditer.getDBID();
-      double score = Math.abs(errors.doubleValue(id)) * 0.6745 / median_dev_from_median;
-      scores.putDouble(id, score);
+      double score = Math.abs(errors.doubleValue(iditer)) * 0.6745 / median_dev_from_median;
+      scores.putDouble(iditer, score);
       minmax.put(score);
     }
     //
@@ -199,17 +195,17 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   @Override
   public TypeInformation[] getInputTypeRestriction() {
     // Get one dimensional attribute for analysis.
-    return TypeUtil.array(getNeighborSetPredicateFactory().getInputTypeRestriction(), VectorFieldTypeInformation.get(NumberVector.class, 1));
+    return TypeUtil.array(getNeighborSetPredicateFactory().getInputTypeRestriction(), new VectorFieldTypeInformation<NumberVector<?>>(NumberVector.class, 1));
   }
 
   /**
-   * Parameterizer
+   * Parameterizer.
    * 
    * @author Ahmed Hettab
    * 
@@ -219,19 +215,21 @@ public class TrimmedMeanApproach<N> extends AbstractNeighborhoodOutlier<N> {
    */
   public static class Parameterizer<N> extends AbstractNeighborhoodOutlier.Parameterizer<N> {
     /**
-     * Parameter for the percentile value p
+     * Parameter for the percentile value p.
      */
-    public static final OptionID P_ID = OptionID.getOrCreateOptionID("tma.p", "the percentile parameter");
+    public static final OptionID P_ID = new OptionID("tma.p", "the percentile parameter");
 
     /**
-     * Percentile parameter p
+     * Percentile parameter p.
      */
     protected double p = 0.2;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      DoubleParameter pP = new DoubleParameter(P_ID, new IntervalConstraint(0.0, IntervalBoundary.OPEN, 0.5, IntervalBoundary.OPEN));
+      DoubleParameter pP = new DoubleParameter(P_ID);
+      pP.addConstraint(new GreaterConstraint(0.0));
+      pP.addConstraint(new LessConstraint(0.5));
       if(config.grab(pP)) {
         p = pP.getValue();
       }
