@@ -28,7 +28,6 @@ import de.lmu.ifi.dbs.elki.database.datastore.DataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreUtil;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
@@ -54,7 +53,7 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
   /**
    * The logger to use.
    */
-  static final Logging logger = Logging.getLogger(ExtendedNeighborhood.class);
+  private static final Logging LOG = Logging.getLogger(ExtendedNeighborhood.class);
 
   /**
    * Constructor.
@@ -67,7 +66,7 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   @Override
@@ -132,23 +131,22 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
       final WritableDataStore<DBIDs> store = DataStoreUtil.makeStorage(database.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC | DataStoreFactory.HINT_TEMP, DBIDs.class);
 
       // Expand multiple steps
-      FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Expanding neighborhoods", database.size(), logger) : null;
+      FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Expanding neighborhoods", database.size(), LOG) : null;
       for(DBIDIter iter = database.iterDBIDs(); iter.valid(); iter.advance()) {
-        DBID id = iter.getDBID();
-        HashSetModifiableDBIDs res = DBIDUtil.newHashSet(id);
-        DBIDs todo = id;
+        HashSetModifiableDBIDs res = DBIDUtil.newHashSet();
+        res.add(iter);
+        DBIDs todo = DBIDUtil.deref(iter);
         for(int i = 0; i < steps; i++) {
           ModifiableDBIDs ntodo = DBIDUtil.newHashSet();
           for(DBIDIter iter2 = todo.iter(); iter2.valid(); iter2.advance()) {
-            DBIDs add = innerinst.getNeighborDBIDs(iter2.getDBID());
+            DBIDs add = innerinst.getNeighborDBIDs(iter2);
             if(add != null) {
-              for(DBIDIter iter3 = add.iter(); iter.valid(); iter.advance()) {
-                DBID nid = iter3.getDBID();
-                if(res.contains(nid)) {
+              for(DBIDIter iter3 = add.iter(); iter3.valid(); iter3.advance()) {
+                if(res.contains(iter3)) {
                   continue;
                 }
-                ntodo.add(nid);
-                res.add(nid);
+                ntodo.add(iter3);
+                res.add(iter3);
               }
             }
           }
@@ -157,13 +155,13 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
           }
           todo = ntodo;
         }
-        store.put(id, res);
+        store.put(iter, res);
         if(progress != null) {
-          progress.incrementProcessed(logger);
+          progress.incrementProcessed(LOG);
         }
       }
       if(progress != null) {
-        progress.ensureCompleted(logger);
+        progress.ensureCompleted(LOG);
       }
 
       return store;
@@ -180,12 +178,12 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
       /**
        * Parameter to specify the neighborhood predicate to use.
        */
-      public static final OptionID NEIGHBORHOOD_ID = OptionID.getOrCreateOptionID("extendedneighbors.neighborhood", "The inner neighborhood predicate to use.");
+      public static final OptionID NEIGHBORHOOD_ID = new OptionID("extendedneighbors.neighborhood", "The inner neighborhood predicate to use.");
 
       /**
        * Parameter to specify the number of steps allowed
        */
-      public static final OptionID STEPS_ID = OptionID.getOrCreateOptionID("extendedneighbors.steps", "The number of steps allowed in the neighborhood graph.");
+      public static final OptionID STEPS_ID = new OptionID("extendedneighbors.steps", "The number of steps allowed in the neighborhood graph.");
 
       /**
        * The number of steps to do.
@@ -225,7 +223,8 @@ public class ExtendedNeighborhood extends AbstractPrecomputedNeighborhood {
        * @return number of steps, default 1
        */
       public static int getParameterSteps(Parameterization config) {
-        final IntParameter param = new IntParameter(STEPS_ID, new GreaterEqualConstraint(1));
+        final IntParameter param = new IntParameter(STEPS_ID);
+        param.addConstraint(new GreaterEqualConstraint(1));
         if(config.grab(param)) {
           return param.getValue();
         }
