@@ -63,12 +63,12 @@ public class ExternalNeighborhood extends AbstractPrecomputedNeighborhood {
   /**
    * Logger
    */
-  static final Logging logger = Logging.getLogger(ExternalNeighborhood.class);
+  private static final Logging LOG = Logging.getLogger(ExternalNeighborhood.class);
 
   /**
    * Parameter to specify the neighborhood file
    */
-  public static final OptionID NEIGHBORHOOD_FILE_ID = OptionID.getOrCreateOptionID("externalneighbors.file", "The file listing the neighbors.");
+  public static final OptionID NEIGHBORHOOD_FILE_ID = new OptionID("externalneighbors.file", "The file listing the neighbors.");
 
   /**
    * Constructor.
@@ -91,7 +91,7 @@ public class ExternalNeighborhood extends AbstractPrecomputedNeighborhood {
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -136,33 +136,32 @@ public class ExternalNeighborhood extends AbstractPrecomputedNeighborhood {
     private DataStore<DBIDs> loadNeighbors(Relation<?> database) {
       final WritableDataStore<DBIDs> store = DataStoreUtil.makeStorage(database.getDBIDs(), DataStoreFactory.HINT_HOT | DataStoreFactory.HINT_STATIC | DataStoreFactory.HINT_TEMP, DBIDs.class);
 
-      if(logger.isVerbose()) {
-        logger.verbose("Loading external neighborhoods.");
+      if(LOG.isVerbose()) {
+        LOG.verbose("Loading external neighborhoods.");
       }
 
-      if(logger.isDebugging()) {
-        logger.verbose("Building reverse label index...");
+      if(LOG.isDebugging()) {
+        LOG.verbose("Building reverse label index...");
       }
       // Build a map label/ExternalId -> DBID
       // (i.e. a reverse index!)
       // TODO: move this into the database layer to share?
-      Map<String, DBID> lblmap = new HashMap<String, DBID>(database.size() * 2);
+      Map<String, DBID> lblmap = new HashMap<String, DBID>(database.size() << 1);
       {
         Relation<LabelList> olq = database.getDatabase().getRelation(TypeUtil.LABELLIST);
         Relation<ExternalID> eidq = database.getDatabase().getRelation(TypeUtil.EXTERNALID);
         for(DBIDIter iditer = database.iterDBIDs(); iditer.valid(); iditer.advance()) {
-          DBID id  = iditer.getDBID();
           if(eidq != null) {
-            ExternalID eid = eidq.get(id);
+            ExternalID eid = eidq.get(iditer);
             if(eid != null) {
-              lblmap.put(eid.toString(), id);
+              lblmap.put(eid.toString(), DBIDUtil.deref(iditer));
             }
           }
           if(olq != null) {
-            LabelList label = olq.get(id);
+            LabelList label = olq.get(iditer);
             if(label != null) {
               for(String lbl : label) {
-                lblmap.put(lbl, id);
+                lblmap.put(lbl, DBIDUtil.deref(iditer));
               }
             }
           }
@@ -170,8 +169,8 @@ public class ExternalNeighborhood extends AbstractPrecomputedNeighborhood {
       }
 
       try {
-        if(logger.isDebugging()) {
-          logger.verbose("Loading neighborhood file.");
+        if(LOG.isDebugging()) {
+          LOG.verbose("Loading neighborhood file.");
         }
         InputStream in = new FileInputStream(file);
         in = FileUtil.tryGzipInput(in);
@@ -187,16 +186,16 @@ public class ExternalNeighborhood extends AbstractPrecomputedNeighborhood {
                 neighbours.add(neigh);
               }
               else {
-                if(logger.isDebugging()) {
-                  logger.debug("No object found for label " + entries[i]);
+                if(LOG.isDebugging()) {
+                  LOG.debug("No object found for label " + entries[i]);
                 }
               }
             }
             store.put(id, neighbours);
           }
           else {
-            if(logger.isDebugging()) {
-              logger.warning("No object found for label " + entries[0]);
+            if(LOG.isDebugging()) {
+              LOG.warning("No object found for label " + entries[0]);
             }
           }
         }
