@@ -57,7 +57,7 @@ public class ByLabelOutlier extends AbstractAlgorithm<OutlierResult> implements
   /**
    * Our logger.
    */
-  private static final Logging logger = Logging.getLogger(ByLabelOutlier.class);
+  private static final Logging LOG = Logging.getLogger(ByLabelOutlier.class);
 
   /**
    * The default pattern to use.
@@ -124,7 +124,7 @@ public class ByLabelOutlier extends AbstractAlgorithm<OutlierResult> implements
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -145,7 +145,7 @@ public class ByLabelOutlier extends AbstractAlgorithm<OutlierResult> implements
      * Key: {@code -outlier.pattern}
      * </p>
      */
-    public static final OptionID OUTLIER_PATTERN_ID = OptionID.getOrCreateOptionID("outlier.pattern", "Label pattern to match outliers.");
+    public static final OptionID OUTLIER_PATTERN_ID = new OptionID("outlier.pattern", "Label pattern to match outliers.");
     /**
      * Stores the "outlier" class.
      */
