@@ -38,33 +38,31 @@ import de.lmu.ifi.dbs.elki.data.type.CombinedTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.evaluation.roc.ROC;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.math.MathUtil;
-import de.lmu.ifi.dbs.elki.math.MeanVariance;
-import de.lmu.ifi.dbs.elki.math.histograms.AggregatingHistogram;
+import de.lmu.ifi.dbs.elki.math.linearalgebra.CovarianceMatrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.result.CollectionResult;
 import de.lmu.ifi.dbs.elki.result.HistogramResult;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.histogram.MeanVarianceStaticHistogram;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
-import de.lmu.ifi.dbs.elki.utilities.pairs.FCPair;
 
 /**
  * Evaluate a distance function with respect to kNN queries. For each point, the
@@ -88,22 +86,22 @@ import de.lmu.ifi.dbs.elki.utilities.pairs.FCPair;
  */
 @Title("Evaluate Ranking Quality")
 @Description("Evaluates the effectiveness of a distance function via the obtained rankings.")
-public class EvaluateRankingQuality<V extends NumberVector<V, ?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm<V, D, CollectionResult<DoubleVector>> {
+public class EvaluateRankingQuality<V extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm<V, D, CollectionResult<DoubleVector>> {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(EvaluateRankingQuality.class);
+  private static final Logging LOG = Logging.getLogger(EvaluateRankingQuality.class);
 
   /**
    * Option to configure the number of bins to use.
    */
-  public static final OptionID HISTOGRAM_BINS_ID = OptionID.getOrCreateOptionID("rankqual.bins", "Number of bins to use in the histogram");
+  public static final OptionID HISTOGRAM_BINS_ID = new OptionID("rankqual.bins", "Number of bins to use in the histogram");
 
   /**
    * Constructor.
    * 
-   * @param distanceFunction
-   * @param numbins
+   * @param distanceFunction Distance function
+   * @param numbins Number of bins
    */
   public EvaluateRankingQuality(DistanceFunction<? super V, D> distanceFunction, int numbins) {
     super(distanceFunction);
@@ -121,61 +119,60 @@ public class EvaluateRankingQuality<V extends NumberVector<V, ?>, D extends Numb
     final DistanceQuery<V, D> distQuery = database.getDistanceQuery(relation, getDistanceFunction());
     final KNNQuery<V, D> knnQuery = database.getKNNQuery(distQuery, relation.size());
 
-    if(logger.isVerbose()) {
-      logger.verbose("Preprocessing clusters...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Preprocessing clusters...");
     }
     // Cluster by labels
     Collection<Cluster<Model>> split = (new ByLabelOrAllInOneClustering()).run(database).getAllClusters();
 
     // Compute cluster averages and covariance matrix
-    HashMap<Cluster<?>, V> averages = new HashMap<Cluster<?>, V>(split.size());
+    HashMap<Cluster<?>, Vector> averages = new HashMap<Cluster<?>, Vector>(split.size());
     HashMap<Cluster<?>, Matrix> covmats = new HashMap<Cluster<?>, Matrix>(split.size());
-    for(Cluster<?> clus : split) {
-      averages.put(clus, DatabaseUtil.centroid(relation, clus.getIDs()));
-      covmats.put(clus, DatabaseUtil.covarianceMatrix(relation, clus.getIDs()));
+    for (Cluster<?> clus : split) {
+      CovarianceMatrix covmat = CovarianceMatrix.make(relation, clus.getIDs());
+      averages.put(clus, covmat.getMeanVector());
+      covmats.put(clus, covmat.destroyToNaiveMatrix());
     }
 
-    AggregatingHistogram<MeanVariance, Double> hist = AggregatingHistogram.MeanVarianceHistogram(numbins, 0.0, 1.0);
+    MeanVarianceStaticHistogram hist = new MeanVarianceStaticHistogram(numbins, 0.0, 1.0);
 
-    if(logger.isVerbose()) {
-      logger.verbose("Processing points...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Processing points...");
     }
-    FiniteProgress rocloop = logger.isVerbose() ? new FiniteProgress("Computing ROC AUC values", relation.size(), logger) : null;
+    FiniteProgress rocloop = LOG.isVerbose() ? new FiniteProgress("Computing ROC AUC values", relation.size(), LOG) : null;
 
     // sort neighbors
-    for(Cluster<?> clus : split) {
-      ArrayList<FCPair<Double, DBID>> cmem = new ArrayList<FCPair<Double, DBID>>(clus.size());
-      Vector av = averages.get(clus).getColumnVector();
+    for (Cluster<?> clus : split) {
+      ArrayList<DoubleDBIDPair> cmem = new ArrayList<DoubleDBIDPair>(clus.size());
+      Vector av = averages.get(clus);
       Matrix covm = covmats.get(clus);
 
-      for(DBIDIter iter = clus.getIDs().iter(); iter.valid(); iter.advance()) {
-        DBID i1 = iter.getDBID();
-        Double d = MathUtil.mahalanobisDistance(covm, av.minus(relation.get(i1).getColumnVector()));
-        cmem.add(new FCPair<Double, DBID>(d, i1));
+      for (DBIDIter iter = clus.getIDs().iter(); iter.valid(); iter.advance()) {
+        double d = MathUtil.mahalanobisDistance(covm, relation.get(iter).getColumnVector().minusEquals(av));
+        cmem.add(DBIDUtil.newPair(d, iter));
       }
       Collections.sort(cmem);
 
-      for(int ind = 0; ind < cmem.size(); ind++) {
-        DBID i1 = cmem.get(ind).getSecond();
-        KNNResult<D> knn = knnQuery.getKNNForDBID(i1, relation.size());
+      for (int ind = 0; ind < cmem.size(); ind++) {
+        KNNResult<D> knn = knnQuery.getKNNForDBID(cmem.get(ind), relation.size());
         double result = ROC.computeROCAUCDistanceResult(relation.size(), clus, knn);
 
-        hist.aggregate(((double) ind) / clus.size(), result);
+        hist.put(((double) ind) / clus.size(), result);
 
-        if(rocloop != null) {
-          rocloop.incrementProcessed(logger);
+        if (rocloop != null) {
+          rocloop.incrementProcessed(LOG);
         }
       }
     }
-    if(rocloop != null) {
-      rocloop.ensureCompleted(logger);
+    if (rocloop != null) {
+      rocloop.ensureCompleted(LOG);
     }
     // Collections.sort(results);
 
     // Transform Histogram into a Double Vector array.
     Collection<DoubleVector> res = new ArrayList<DoubleVector>(relation.size());
-    for(DoubleObjPair<MeanVariance> pair : hist) {
-      DoubleVector row = new DoubleVector(new double[] { pair.first, pair.getSecond().getCount(), pair.getSecond().getMean(), pair.getSecond().getSampleVariance() });
+    for (MeanVarianceStaticHistogram.Iter iter = hist.iter(); iter.valid(); iter.advance()) {
+      DoubleVector row = new DoubleVector(new double[] { iter.getCenter(), iter.getValue().getCount(), iter.getValue().getMean(), iter.getValue().getSampleVariance() });
       res.add(row);
     }
     return new HistogramResult<DoubleVector>("Ranking Quality Histogram", "ranking-histogram", res);
@@ -188,7 +185,7 @@ public class EvaluateRankingQuality<V extends NumberVector<V, ?>, D extends Numb
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -198,14 +195,18 @@ public class EvaluateRankingQuality<V extends NumberVector<V, ?>, D extends Numb
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, D> {
+  public static class Parameterizer<V extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<V, D> {
+    /**
+     * Number of bins to use.
+     */
     protected int numbins = 20;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter param = new IntParameter(HISTOGRAM_BINS_ID, new GreaterEqualConstraint(2), 20);
-      if(config.grab(param)) {
+      final IntParameter param = new IntParameter(HISTOGRAM_BINS_ID, 20);
+      param.addConstraint(new GreaterEqualConstraint(2));
+      if (config.grab(param)) {
         numbins = param.getValue();
       }
     }
@@ -215,4 +216,4 @@ public class EvaluateRankingQuality<V extends NumberVector<V, ?>, D extends Numb
       return new EvaluateRankingQuality<V, D>(distanceFunction, numbins);
     }
   }
-}
\ No newline at end of file
+}
