@@ -34,28 +34,26 @@ import de.lmu.ifi.dbs.elki.data.model.Model;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.database.Database;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.evaluation.roc.ROC;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.math.MeanVariance;
-import de.lmu.ifi.dbs.elki.math.histograms.AggregatingHistogram;
 import de.lmu.ifi.dbs.elki.result.CollectionResult;
 import de.lmu.ifi.dbs.elki.result.HistogramResult;
+import de.lmu.ifi.dbs.elki.utilities.datastructures.histogram.DoubleStaticHistogram;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
-import de.lmu.ifi.dbs.elki.utilities.pairs.DoubleObjPair;
 
 /**
  * Evaluate a distance function with respect to kNN queries. For each point, the
@@ -77,12 +75,12 @@ public class RankingQualityHistogram<O, D extends NumberDistance<D, ?>> extends
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(RankingQualityHistogram.class);
+  private static final Logging LOG = Logging.getLogger(RankingQualityHistogram.class);
 
   /**
    * Option to configure the number of bins to use.
    */
-  public static final OptionID HISTOGRAM_BINS_ID = OptionID.getOrCreateOptionID("rankqual.bins", "Number of bins to use in the histogram");
+  public static final OptionID HISTOGRAM_BINS_ID = new OptionID("rankqual.bins", "Number of bins to use in the histogram");
 
   /**
    * Number of bins to use.
@@ -100,47 +98,53 @@ public class RankingQualityHistogram<O, D extends NumberDistance<D, ?>> extends
     this.numbins = numbins;
   }
 
+  /**
+   * Process a database
+   * 
+   * @param database Database to process
+   * @param relation Relation to process
+   * @return Histogram of ranking qualities
+   */
   public HistogramResult<DoubleVector> run(Database database, Relation<O> relation) {
     final DistanceQuery<O, D> distanceQuery = database.getDistanceQuery(relation, getDistanceFunction());
     final KNNQuery<O, D> knnQuery = database.getKNNQuery(distanceQuery, relation.size());
 
-    if(logger.isVerbose()) {
-      logger.verbose("Preprocessing clusters...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Preprocessing clusters...");
     }
     // Cluster by labels
     Collection<Cluster<Model>> split = (new ByLabelOrAllInOneClustering()).run(database).getAllClusters();
 
-    AggregatingHistogram<Double, Double> hist = AggregatingHistogram.DoubleSumHistogram(numbins, 0.0, 1.0);
+    DoubleStaticHistogram hist = new DoubleStaticHistogram(numbins, 0.0, 1.0);
 
-    if(logger.isVerbose()) {
-      logger.verbose("Processing points...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Processing points...");
     }
-    FiniteProgress progress = logger.isVerbose() ? new FiniteProgress("Computing ROC AUC values", relation.size(), logger) : null;
+    FiniteProgress progress = LOG.isVerbose() ? new FiniteProgress("Computing ROC AUC values", relation.size(), LOG) : null;
 
     MeanVariance mv = new MeanVariance();
     // sort neighbors
-    for(Cluster<?> clus : split) {
-      for(DBIDIter iter = clus.getIDs().iter(); iter.valid(); iter.advance()) {
-        DBID i1 = iter.getDBID();
-        KNNResult<D> knn = knnQuery.getKNNForDBID(i1, relation.size());
+    for (Cluster<?> clus : split) {
+      for (DBIDIter iter = clus.getIDs().iter(); iter.valid(); iter.advance()) {
+        KNNResult<D> knn = knnQuery.getKNNForDBID(iter, relation.size());
         double result = ROC.computeROCAUCDistanceResult(relation.size(), clus, knn);
 
         mv.put(result);
-        hist.aggregate(result, 1. / relation.size());
+        hist.increment(result, 1. / relation.size());
 
-        if(progress != null) {
-          progress.incrementProcessed(logger);
+        if (progress != null) {
+          progress.incrementProcessed(LOG);
         }
       }
     }
-    if(progress != null) {
-      progress.ensureCompleted(logger);
+    if (progress != null) {
+      progress.ensureCompleted(LOG);
     }
 
     // Transform Histogram into a Double Vector array.
     Collection<DoubleVector> res = new ArrayList<DoubleVector>(relation.size());
-    for(DoubleObjPair<Double> pair : hist) {
-      DoubleVector row = new DoubleVector(new double[] { pair.first, pair.getSecond() });
+    for (DoubleStaticHistogram.Iter iter = hist.iter(); iter.valid(); iter.advance()) {
+      DoubleVector row = new DoubleVector(new double[] { iter.getCenter(), iter.getValue() });
       res.add(row);
     }
     HistogramResult<DoubleVector> result = new HistogramResult<DoubleVector>("Ranking Quality Histogram", "ranking-histogram", res);
@@ -152,10 +156,10 @@ public class RankingQualityHistogram<O, D extends NumberDistance<D, ?>> extends
   public TypeInformation[] getInputTypeRestriction() {
     return TypeUtil.array(getDistanceFunction().getInputTypeRestriction());
   }
-  
+
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -166,13 +170,17 @@ public class RankingQualityHistogram<O, D extends NumberDistance<D, ?>> extends
    * @apiviz.exclude
    */
   public static class Parameterizer<O, D extends NumberDistance<D, ?>> extends AbstractDistanceBasedAlgorithm.Parameterizer<O, D> {
+    /**
+     * Number of bins.
+     */
     protected int numbins = 20;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter param = new IntParameter(HISTOGRAM_BINS_ID, new GreaterEqualConstraint(2), 100);
-      if(config.grab(param)) {
+      final IntParameter param = new IntParameter(HISTOGRAM_BINS_ID, 100);
+      param.addConstraint(new GreaterEqualConstraint(2));
+      if (config.grab(param)) {
         numbins = param.getValue();
       }
     }
@@ -182,4 +190,4 @@ public class RankingQualityHistogram<O, D extends NumberDistance<D, ?>> extends
       return new RankingQualityHistogram<O, D>(distanceFunction, numbins);
     }
   }
-}
\ No newline at end of file
+}
