@@ -50,7 +50,7 @@ public class ComputeSingleColorHistogram extends AbstractApplication {
    * Key: {@code -colorhist.generator}
    * </p>
    */
-  public static OptionID COLORHIST_ID = OptionID.getOrCreateOptionID("colorhist.generator", "Class that is used to generate a color histogram.");
+  public static OptionID COLORHIST_ID = new OptionID("colorhist.generator", "Class that is used to generate a color histogram.");
 
   /**
    * Parameter that specifies the name of the input file.
@@ -58,7 +58,7 @@ public class ComputeSingleColorHistogram extends AbstractApplication {
    * Key: {@code -colorhist.in}
    * </p>
    */
-  public static final OptionID INPUT_ID = OptionID.getOrCreateOptionID("colorhist.in", "Input image file for color histogram.");
+  public static final OptionID INPUT_ID = new OptionID("colorhist.in", "Input image file for color histogram.");
 
   /**
    * Parameter that specifies the name of the mask input file.
@@ -66,7 +66,7 @@ public class ComputeSingleColorHistogram extends AbstractApplication {
    * Key: {@code -colorhist.mask}
    * </p>
    */
-  public static final OptionID MASK_ID = OptionID.getOrCreateOptionID("colorhist.mask", "Input mask image file.");
+  public static final OptionID MASK_ID = new OptionID("colorhist.mask", "Input mask image file.");
 
   /**
    * Class that will compute the actual histogram
@@ -86,13 +86,12 @@ public class ComputeSingleColorHistogram extends AbstractApplication {
   /**
    * Constructor.
    * 
-   * @param verbose Verbose flag
    * @param histogrammaker Class to compute histograms with
    * @param inputFile Input file
    * @param maskFile Mask file
    */
-  public ComputeSingleColorHistogram(boolean verbose, ComputeColorHistogram histogrammaker, File inputFile, File maskFile) {
-    super(verbose);
+  public ComputeSingleColorHistogram(ComputeColorHistogram histogrammaker, File inputFile, File maskFile) {
+    super();
     this.histogrammaker = histogrammaker;
     this.inputFile = inputFile;
     this.maskFile = maskFile;
@@ -152,7 +151,7 @@ public class ComputeSingleColorHistogram extends AbstractApplication {
 
     @Override
     protected ComputeSingleColorHistogram makeInstance() {
-      return new ComputeSingleColorHistogram(verbose, histogrammaker, inputFile, maskFile);
+      return new ComputeSingleColorHistogram(histogrammaker, inputFile, maskFile);
     }
   }
 
