@@ -61,12 +61,12 @@ public class GeneratorXMLSpec extends AbstractApplication {
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(GeneratorXMLSpec.class);
+  private static final Logging LOG = Logging.getLogger(GeneratorXMLSpec.class);
 
   /**
    * Line separator for output
    */
-  public final static String LINE_SEPARATOR = System.getProperty("line.separator");
+  public static final String LINE_SEPARATOR = System.getProperty("line.separator");
 
   /**
    * Output file.
@@ -81,12 +81,11 @@ public class GeneratorXMLSpec extends AbstractApplication {
   /**
    * Constructor.
    * 
-   * @param verbose Verbose flag
    * @param output Output file
    * @param generator GeneratorXMLDatabaseConnection
    */
-  public GeneratorXMLSpec(boolean verbose, File output, GeneratorXMLDatabaseConnection generator) {
-    super(verbose);
+  public GeneratorXMLSpec(File output, GeneratorXMLDatabaseConnection generator) {
+    super();
     this.outputFile = output;
     this.generator = generator;
   }
@@ -97,13 +96,13 @@ public class GeneratorXMLSpec extends AbstractApplication {
   @Override
   public void run() throws UnableToComplyException {
     MultipleObjectsBundle data = generator.loadData();
-    if(logger.isVerbose()) {
-      logger.verbose("Writing output ...");
+    if(LOG.isVerbose()) {
+      LOG.verbose("Writing output ...");
     }
     try {
       if(outputFile.exists()) {
-        if(logger.isVerbose()) {
-          logger.verbose("The file " + outputFile + " already exists, " + "the generator result will be appended.");
+        if(LOG.isVerbose()) {
+          LOG.verbose("The file " + outputFile + " already exists, " + "the generator result will be appended.");
         }
       }
 
@@ -119,8 +118,8 @@ public class GeneratorXMLSpec extends AbstractApplication {
     catch(IOException e) {
       throw new UnableToComplyException(e);
     }
-    if(logger.isVerbose()) {
-      logger.verbose("Done.");
+    if(LOG.isVerbose()) {
+      LOG.verbose("Done.");
     }
   }
 
@@ -242,7 +241,7 @@ public class GeneratorXMLSpec extends AbstractApplication {
 
     @Override
     protected GeneratorXMLSpec makeInstance() {
-      return new GeneratorXMLSpec(verbose, outputFile, generator);
+      return new GeneratorXMLSpec(outputFile, generator);
     }
   }
 
