@@ -29,8 +29,9 @@ import java.io.IOException;
 import de.lmu.ifi.dbs.elki.application.AbstractApplication;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.StaticArrayDatabase;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDFactory;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
@@ -60,7 +61,7 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(CacheDoubleDistanceInOnDiskMatrix.class);
+  private static final Logging LOG = Logging.getLogger(CacheDoubleDistanceInOnDiskMatrix.class);
 
   /**
    * Parameter that specifies the name of the directory to be re-parsed.
@@ -68,7 +69,7 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
    * Key: {@code -loader.diskcache}
    * </p>
    */
-  public static final OptionID CACHE_ID = OptionID.getOrCreateOptionID("loader.diskcache", "File name of the disk cache to create.");
+  public static final OptionID CACHE_ID = new OptionID("loader.diskcache", "File name of the disk cache to create.");
 
   /**
    * Parameter that specifies the name of the directory to be re-parsed.
@@ -76,7 +77,7 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
    * Key: {@code -loader.distance}
    * </p>
    */
-  public static final OptionID DISTANCE_ID = OptionID.getOrCreateOptionID("loader.distance", "Distance function to cache.");
+  public static final OptionID DISTANCE_ID = new OptionID("loader.distance", "Distance function to cache.");
 
   /**
    * Debug flag, to double-check all write operations.
@@ -101,13 +102,12 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
   /**
    * Constructor.
    * 
-   * @param verbose Verbose flag
    * @param database Database
    * @param distance Distance function
    * @param out Matrix output file
    */
-  public CacheDoubleDistanceInOnDiskMatrix(boolean verbose, Database database, DistanceFunction<O, D> distance, File out) {
-    super(verbose);
+  public CacheDoubleDistanceInOnDiskMatrix(Database database, DistanceFunction<O, D> distance, File out) {
+    super();
     this.database = database;
     this.distance = distance;
     this.out = out;
@@ -121,9 +121,9 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
 
     int matrixsize = 0;
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id  = iditer.getDBID();
-      matrixsize = Math.max(matrixsize, id.getIntegerID() + 1);
-      if(id.getIntegerID() < 0) {
+      int intid = DBIDUtil.asInteger(iditer);
+      matrixsize = Math.max(matrixsize, intid + 1);
+      if(intid < 0) {
         throw new AbortException("OnDiskMatrixCache does not allow negative DBIDs.");
       }
     }
@@ -136,23 +136,21 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
       throw new AbortException("Error creating output matrix.", e);
     }
 
-    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id1  = iditer.getDBID();
-      for(DBIDIter iditer2 = relation.iterDBIDs(); iditer2.valid(); iditer2.advance()) {
-        DBID id2  = iditer2.getDBID();
-        if(id2.getIntegerID() >= id1.getIntegerID()) {
+    for(DBIDIter id1 = relation.iterDBIDs(); id1.valid(); id1.advance()) {
+      for(DBIDIter id2 = relation.iterDBIDs(); id2.valid(); id2.advance()) {
+        if(DBIDUtil.asInteger(id2) >= DBIDUtil.asInteger(id1)) {
           double d = distanceQuery.distance(id1, id2).doubleValue();
           if(debugExtraCheckSymmetry) {
             double d2 = distanceQuery.distance(id2, id1).doubleValue();
             if(Math.abs(d - d2) > 0.0000001) {
-              logger.warning("Distance function doesn't appear to be symmetric!");
+              LOG.warning("Distance function doesn't appear to be symmetric!");
             }
           }
           try {
-            matrix.getRecordBuffer(id1.getIntegerID(), id2.getIntegerID()).putDouble(d);
+            matrix.getRecordBuffer(DBIDUtil.asInteger(id1), DBIDUtil.asInteger(id2)).putDouble(d);
           }
           catch(IOException e) {
-            throw new AbortException("Error writing distance record " + id1 + "," + id2 + " to matrix.", e);
+            throw new AbortException("Error writing distance record " + DBIDFactory.FACTORY.toString(id1) + "," + DBIDFactory.FACTORY.toString(id2) + " to matrix.", e);
           }
         }
       }
@@ -205,7 +203,7 @@ public class CacheDoubleDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>
 
     @Override
     protected CacheDoubleDistanceInOnDiskMatrix<O, D> makeInstance() {
-      return new CacheDoubleDistanceInOnDiskMatrix<O, D>(verbose, database, distance, out);
+      return new CacheDoubleDistanceInOnDiskMatrix<O, D>(database, distance, out);
     }
   }
 
