@@ -29,8 +29,8 @@ import java.io.IOException;
 import de.lmu.ifi.dbs.elki.application.AbstractApplication;
 import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.StaticArrayDatabase;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
@@ -60,7 +60,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
   /**
    * The logger for this class.
    */
-  private static final Logging logger = Logging.getLogger(CacheFloatDistanceInOnDiskMatrix.class);
+  private static final Logging LOG = Logging.getLogger(CacheFloatDistanceInOnDiskMatrix.class);
 
   /**
    * Parameter that specifies the name of the directory to be re-parsed.
@@ -68,7 +68,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
    * Key: {@code -loader.diskcache}
    * </p>
    */
-  public static final OptionID CACHE_ID = OptionID.getOrCreateOptionID("loader.diskcache", "File name of the disk cache to create.");
+  public static final OptionID CACHE_ID = new OptionID("loader.diskcache", "File name of the disk cache to create.");
 
   /**
    * Parameter that specifies the name of the directory to be re-parsed.
@@ -76,7 +76,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
    * Key: {@code -loader.distance}
    * </p>
    */
-  public static final OptionID DISTANCE_ID = OptionID.getOrCreateOptionID("loader.distance", "Distance function to cache.");
+  public static final OptionID DISTANCE_ID = new OptionID("loader.distance", "Distance function to cache.");
 
   /**
    * Debug flag, to double-check all write operations.
@@ -106,13 +106,12 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
   /**
    * Constructor.
    * 
-   * @param verbose Verbose flag
    * @param database Database
    * @param distance Distance function
    * @param out Matrix output file
    */
-  public CacheFloatDistanceInOnDiskMatrix(boolean verbose, Database database, DistanceFunction<O, D> distance, File out) {
-    super(verbose);
+  public CacheFloatDistanceInOnDiskMatrix(Database database, DistanceFunction<O, D> distance, File out) {
+    super();
     this.database = database;
     this.distance = distance;
     this.out = out;
@@ -126,9 +125,9 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
 
     int matrixsize = 0;
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id  = iditer.getDBID();
-      matrixsize = Math.max(matrixsize, id.getIntegerID() + 1);
-      if(id.getIntegerID() < 0) {
+      final int intid = DBIDUtil.asInteger(iditer);
+      matrixsize = Math.max(matrixsize, intid + 1);
+      if(intid < 0) {
         throw new AbortException("OnDiskMatrixCache does not allow negative DBIDs.");
       }
     }
@@ -141,20 +140,18 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
       throw new AbortException("Error creating output matrix.", e);
     }
 
-    for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
-      DBID id1  = iditer.getDBID();
-      for(DBIDIter iditer2 = relation.iterDBIDs(); iditer2.valid(); iditer2.advance()) {
-        DBID id2  = iditer2.getDBID();
-        if(id2.getIntegerID() >= id1.getIntegerID()) {
+    for(DBIDIter id1 = relation.iterDBIDs(); id1.valid(); id1.advance()) {
+      for(DBIDIter id2 = relation.iterDBIDs(); id2.valid(); id2.advance()) {
+        if(DBIDUtil.asInteger(id2) >= DBIDUtil.asInteger(id1)) {
           float d = distanceQuery.distance(id1, id2).floatValue();
           if(debugExtraCheckSymmetry) {
             float d2 = distanceQuery.distance(id2, id1).floatValue();
             if(Math.abs(d - d2) > 0.0000001) {
-              logger.warning("Distance function doesn't appear to be symmetric!");
+              LOG.warning("Distance function doesn't appear to be symmetric!");
             }
           }
           try {
-            matrix.getRecordBuffer(id1.getIntegerID(), id2.getIntegerID()).putFloat(d);
+            matrix.getRecordBuffer(DBIDUtil.asInteger(id1), DBIDUtil.asInteger(id2)).putFloat(d);
           }
           catch(IOException e) {
             throw new AbortException("Error writing distance record " + id1 + "," + id2 + " to matrix.", e);
@@ -210,7 +207,7 @@ public class CacheFloatDistanceInOnDiskMatrix<O, D extends NumberDistance<D, ?>>
 
     @Override
     protected CacheFloatDistanceInOnDiskMatrix<O, D> makeInstance() {
-      return new CacheFloatDistanceInOnDiskMatrix<O, D>(verbose, database, distance, out);
+      return new CacheFloatDistanceInOnDiskMatrix<O, D>(database, distance, out);
     }
   }
 
