@@ -50,7 +50,9 @@ import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
+import de.lmu.ifi.dbs.elki.application.AbstractApplication;
 import de.lmu.ifi.dbs.elki.logging.Logging;
+import de.lmu.ifi.dbs.elki.logging.LoggingConfiguration;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
 import de.lmu.ifi.dbs.elki.utilities.ELKIServiceLoader;
 import de.lmu.ifi.dbs.elki.utilities.InspectionUtil;
@@ -77,7 +79,7 @@ import de.lmu.ifi.dbs.elki.utilities.xml.HTMLUtil;
  * @apiviz.uses Parameter
  */
 public class DocumentParameters {
-  static final Logging logger = Logging.getLogger(DocumentParameters.class);
+  private static final Logging LOG = Logging.getLogger(DocumentParameters.class);
 
   private static final String HEADER_PARAMETER_FOR = "Parameter for: ";
 
@@ -103,28 +105,29 @@ public class DocumentParameters {
    * @param args Command line arguments
    */
   public static void main(String[] args) {
+    LoggingConfiguration.setVerbose(true);
     if(args.length != 2) {
-      logger.warning("I need exactly two file names to operate!");
+      LOG.warning("I need exactly two file names to operate!");
       System.exit(1);
     }
     if(!args[0].endsWith(".html")) {
-      logger.warning("First file name doesn't end with .html!");
+      LOG.warning("First file name doesn't end with .html!");
       System.exit(1);
     }
     if(!args[1].endsWith(".html")) {
-      logger.warning("Second file name doesn't end with .html!");
+      LOG.warning("Second file name doesn't end with .html!");
       System.exit(1);
     }
     File byclsname = new File(args[0]);
     File byoptname = new File(args[1]);
 
-    HashMapList<Class<?>, Parameter<?, ?>> byclass = new HashMapList<Class<?>, Parameter<?, ?>>();
-    HashMapList<OptionID, Pair<Parameter<?, ?>, Class<?>>> byopt = new HashMapList<OptionID, Pair<Parameter<?, ?>, Class<?>>>();
+    HashMapList<Class<?>, Parameter<?>> byclass = new HashMapList<Class<?>, Parameter<?>>();
+    HashMapList<OptionID, Pair<Parameter<?>, Class<?>>> byopt = new HashMapList<OptionID, Pair<Parameter<?>, Class<?>>>();
     try {
       buildParameterIndex(byclass, byopt);
     }
     catch(Exception e) {
-      logger.exception(e);
+      LOG.exception(e);
       System.exit(1);
     }
 
@@ -134,7 +137,7 @@ public class DocumentParameters {
         byclassfo = new FileOutputStream(byclsname);
       }
       catch(FileNotFoundException e) {
-        logger.exception("Can't create output stream!", e);
+        LOG.exception("Can't create output stream!", e);
         throw new RuntimeException(e);
       }
       OutputStream byclassstream = new BufferedOutputStream(byclassfo);
@@ -146,7 +149,7 @@ public class DocumentParameters {
         byclassfo.close();
       }
       catch(IOException e) {
-        logger.exception("IO Exception writing output.", e);
+        LOG.exception("IO Exception writing output.", e);
         throw new RuntimeException(e);
       }
     }
@@ -157,7 +160,7 @@ public class DocumentParameters {
         byoptfo = new FileOutputStream(byoptname);
       }
       catch(FileNotFoundException e) {
-        logger.exception("Can't create output stream!", e);
+        LOG.exception("Can't create output stream!", e);
         throw new RuntimeException(e);
       }
       OutputStream byoptstream = new BufferedOutputStream(byoptfo);
@@ -169,22 +172,25 @@ public class DocumentParameters {
         byoptfo.close();
       }
       catch(IOException e) {
-        logger.exception("IO Exception writing output.", e);
+        LOG.exception("IO Exception writing output.", e);
         throw new RuntimeException(e);
       }
     }
+    // Forcibly terminate, as some class may have screwed up.
+    System.exit(0);
   }
 
-  private static void buildParameterIndex(HashMapList<Class<?>, Parameter<?, ?>> byclass, HashMapList<OptionID, Pair<Parameter<?, ?>, Class<?>>> byopt) {
-    final ArrayList<Pair<Object, Parameter<?, ?>>> options = new ArrayList<Pair<Object, Parameter<?, ?>>>();
+  private static void buildParameterIndex(HashMapList<Class<?>, Parameter<?>> byclass, HashMapList<OptionID, Pair<Parameter<?>, Class<?>>> byopt) {
+    final ArrayList<Pair<Object, Parameter<?>>> options = new ArrayList<Pair<Object, Parameter<?>>>();
     ExecutorService es = Executors.newSingleThreadExecutor();
     for(final Class<?> cls : InspectionUtil.findAllImplementations(Parameterizable.class, false)) {
       // Doesn't have a proper name?
       if(cls.getCanonicalName() == null) {
         continue;
       }
-      // Special cases we need to skip...
-      if(cls.getCanonicalName() == "experimentalcode.elke.AlgorithmTest") {
+      // Some of the "applications" do currently not have appropriate
+      // constructors / parameterizers and may start AWT threads - skip them.
+      if(AbstractApplication.class.isAssignableFrom(cls)) {
         continue;
       }
 
@@ -204,7 +210,7 @@ public class DocumentParameters {
               ClassGenericsUtil.tryInstantiate(Object.class, cls, track);
             }
             catch(java.lang.NoSuchMethodException e) {
-              logger.warning("Could not instantiate class " + cls.getName() + " - no appropriate constructor or parameterizer found.");
+              LOG.warning("Could not instantiate class " + cls.getName() + " - no appropriate constructor or parameterizer found.");
             }
             catch(java.lang.reflect.InvocationTargetException e) {
               if(e.getCause() instanceof RuntimeException) {
@@ -225,7 +231,7 @@ public class DocumentParameters {
               throw new RuntimeException(e);
             }
           }
-          for(Pair<Object, Parameter<?, ?>> pair : track.getAllParameters()) {
+          for(Pair<Object, Parameter<?>> pair : track.getAllParameters()) {
             if(pair.first == null) {
               pair.first = cls;
             }
@@ -236,35 +242,45 @@ public class DocumentParameters {
       es.submit(instantiator);
       try {
         // Wait up to one second.
-        instantiator.get(100000L, TimeUnit.MILLISECONDS);
+        instantiator.get(1L, TimeUnit.SECONDS);
       }
       catch(TimeoutException e) {
-        de.lmu.ifi.dbs.elki.logging.LoggingUtil.warning("Timeout on instantiating " + cls.getName());
+        LOG.warning("Timeout on instantiating " + cls.getName());
         es.shutdownNow();
         throw new RuntimeException(e);
       }
       catch(java.util.concurrent.ExecutionException e) {
-        de.lmu.ifi.dbs.elki.logging.LoggingUtil.warning("Error instantiating " + cls.getName(), e.getCause());
-        /*
-         * es.shutdownNow(); if(e.getCause() instanceof RuntimeException) {
-         * throw (RuntimeException) e.getCause(); } throw new
-         * RuntimeException(e.getCause());
-         */
+        // Do full reporting only on release branch.
+        if(cls.getName().startsWith("de.lmu.ifi.dbs.elki")) {
+          LOG.warning("Error instantiating " + cls.getName(), e.getCause());
+        }
+        else {
+          LOG.warning("Error instantiating " + cls.getName());
+        }
+        // es.shutdownNow();
+        // if(e.getCause() instanceof RuntimeException) {
+        // throw (RuntimeException) e.getCause();
+        // }
+        // throw new RuntimeException(e.getCause());
         continue;
       }
       catch(Exception e) {
-        /*
-         * de.lmu.ifi.dbs.elki.logging.LoggingUtil.warning("Error instantiating "
-         * + cls.getName()); es.shutdownNow(); throw new RuntimeException(e);
-         */
+        // Do full reporting only on release branch.
+        if(cls.getName().startsWith("de.lmu.ifi.dbs.elki")) {
+          LOG.warning("Error instantiating " + cls.getName(), e.getCause());
+        }
+        else {
+          LOG.warning("Error instantiating " + cls.getName());
+        }
+        // es.shutdownNow();
+        // throw new RuntimeException(e);
         continue;
       }
     }
-
-    logger.debug("Documenting " + options.size() + " parameter instances.");
-    for(Pair<Object, Parameter<?, ?>> pp : options) {
+    LOG.debug("Documenting " + options.size() + " parameter instances.");
+    for(Pair<Object, Parameter<?>> pp : options) {
       if(pp.first == null || pp.second == null) {
-        logger.debugFiner("Null: " + pp.first + " " + pp.second);
+        LOG.debugFiner("Null: " + pp.first + " " + pp.second);
         continue;
       }
       Class<?> c;
@@ -274,14 +290,14 @@ public class DocumentParameters {
       else {
         c = pp.first.getClass();
       }
-      Parameter<?, ?> o = pp.second;
+      Parameter<?> o = pp.second;
 
       // just collect unique occurrences
       {
-        List<Parameter<?, ?>> byc = byclass.get(c);
+        List<Parameter<?>> byc = byclass.get(c);
         boolean inlist = false;
         if(byc != null) {
-          for(Parameter<?, ?> par : byc) {
+          for(Parameter<?> par : byc) {
             if(par.getOptionID() == o.getOptionID()) {
               inlist = true;
               break;
@@ -293,10 +309,10 @@ public class DocumentParameters {
         }
       }
       {
-        List<Pair<Parameter<?, ?>, Class<?>>> byo = byopt.get(o.getOptionID());
+        List<Pair<Parameter<?>, Class<?>>> byo = byopt.get(o.getOptionID());
         boolean inlist = false;
         if(byo != null) {
-          for(Pair<Parameter<?, ?>, Class<?>> pair : byo) {
+          for(Pair<Parameter<?>, Class<?>> pair : byo) {
             if(pair.second.equals(c)) {
               inlist = true;
               break;
@@ -304,12 +320,11 @@ public class DocumentParameters {
           }
         }
         if(!inlist) {
-          byopt.add(o.getOptionID(), new Pair<Parameter<?, ?>, Class<?>>(o, c));
+          byopt.add(o.getOptionID(), new Pair<Parameter<?>, Class<?>>(o, c));
         }
       }
     }
-    es.shutdownNow();
-    logger.debug("byClass: " + byclass.size() + " byOpt: " + byopt.size());
+    LOG.debug("byClass: " + byclass.size() + " byOpt: " + byopt.size());
   }
 
   protected static Constructor<?> getConstructor(final Class<?> cls) {
@@ -321,19 +336,19 @@ public class DocumentParameters {
     }
     catch(RuntimeException e) {
       // Not parameterizable, usually not even found ...
-      logger.warning("RuntimeException: ", e);
+      LOG.warning("RuntimeException: ", e);
     }
     catch(Exception e) {
       // Not parameterizable.
     }
     catch(java.lang.Error e) {
       // Not parameterizable.
-      logger.warning("Error: ", e);
+      LOG.warning("Error: ", e);
     }
     return null;
   }
 
-  private static Document makeByClassOverview(HashMapList<Class<?>, Parameter<?, ?>> byclass) {
+  private static Document makeByClassOverview(HashMapList<Class<?>, Parameter<?>> byclass) {
     DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
     DocumentBuilder builder;
     try {
@@ -415,7 +430,7 @@ public class DocumentParameters {
       // nested definition list for options
       Element classdl = htmldoc.createElement(HTMLUtil.HTML_DL_TAG);
       classdd.appendChild(classdl);
-      for(Parameter<?, ?> opt : byclass.get(cls)) {
+      for(Parameter<?> opt : byclass.get(cls)) {
         // DT definition term: option name, in TT for typewriter optics
         Element elemdt = htmldoc.createElement(HTMLUtil.HTML_DT_TAG);
         {
@@ -447,7 +462,7 @@ public class DocumentParameters {
     return htmldoc;
   }
 
-  private static Document makeByOptOverview(HashMapList<OptionID, Pair<Parameter<?, ?>, Class<?>>> byopt) {
+  private static Document makeByOptOverview(HashMapList<OptionID, Pair<Parameter<?>, Class<?>>> byopt) {
     DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
     DocumentBuilder builder;
     try {
@@ -507,7 +522,7 @@ public class DocumentParameters {
     Collections.sort(opts, new SortByOption());
 
     for(OptionID oid : opts) {
-      final Parameter<?, ?> firstopt = byopt.get(oid).get(0).getFirst();
+      final Parameter<?> firstopt = byopt.get(oid).get(0).getFirst();
       // DT = definition term
       Element optdt = htmldoc.createElement(HTMLUtil.HTML_DT_TAG);
       // Anchor for references
@@ -535,9 +550,9 @@ public class DocumentParameters {
       if(firstopt instanceof ClassParameter<?>) {
         // Find superclass heuristically
         superclass = ((ClassParameter<?>) firstopt).getRestrictionClass();
-        for (Pair<Parameter<?, ?>, Class<?>> clinst : byopt.get(oid)) {
+        for(Pair<Parameter<?>, Class<?>> clinst : byopt.get(oid)) {
           ClassParameter<?> cls = (ClassParameter<?>) clinst.getFirst();
-          if (!cls.getRestrictionClass().equals(superclass) && cls.getRestrictionClass().isAssignableFrom(superclass)) {
+          if(!cls.getRestrictionClass().equals(superclass) && cls.getRestrictionClass().isAssignableFrom(superclass)) {
             superclass = cls.getRestrictionClass();
           }
         }
@@ -558,7 +573,7 @@ public class DocumentParameters {
         optdd.appendChild(p);
       }
       optdd.appendChild(classesul);
-      for(Pair<Parameter<?, ?>, Class<?>> clinst : byopt.get(oid)) {
+      for(Pair<Parameter<?>, Class<?>> clinst : byopt.get(oid)) {
         // DT definition term: option name, in TT for typewriter optics
         Element classli = htmldoc.createElement(HTMLUtil.HTML_LI_TAG);
 
@@ -581,7 +596,7 @@ public class DocumentParameters {
             appendNoClassRestriction(htmldoc, classli);
           }
         }
-        Parameter<?, ?> param = clinst.getFirst();
+        Parameter<?> param = clinst.getFirst();
         if(param.getDefaultValue() != null) {
           if(!param.getDefaultValue().equals(firstopt.getDefaultValue())) {
             appendDefaultValueIfSet(htmldoc, param, classli);
@@ -599,7 +614,7 @@ public class DocumentParameters {
     return htmldoc;
   }
 
-  private static void appendDefaultClassLink(Document htmldoc, Parameter<?, ?> opt, Element p) {
+  private static void appendDefaultClassLink(Document htmldoc, Parameter<?> opt, Element p) {
     Element defa = htmldoc.createElement(HTMLUtil.HTML_A_TAG);
     defa.setAttribute(HTMLUtil.HTML_HREF_ATTRIBUTE, linkForClassName(((ClassParameter<?>) opt).getDefaultValue().getCanonicalName()));
     defa.setTextContent(((ClassParameter<?>) opt).getDefaultValue().getCanonicalName());
@@ -608,7 +623,7 @@ public class DocumentParameters {
 
   private static void appendClassRestriction(Document htmldoc, Class<?> restriction, Element elemdd) {
     if(restriction == null) {
-      logger.warning("No restriction class!");
+      LOG.warning("No restriction class!");
       return;
     }
     Element p = htmldoc.createElement(HTMLUtil.HTML_P_TAG);
@@ -653,8 +668,8 @@ public class DocumentParameters {
       }
       // Report when not in properties file.
       Iterator<Class<?>> clss = new ELKIServiceLoader(opt.getRestrictionClass());
-      if (!clss.hasNext()) {
-        logger.warning(opt.getRestrictionClass().getName() + " not in properties. No autocompletion available in release GUI.");
+      if(!clss.hasNext()) {
+        LOG.warning(opt.getRestrictionClass().getName() + " not in properties. No autocompletion available in release GUI.");
       }
     }
   }
@@ -666,7 +681,7 @@ public class DocumentParameters {
    * @param par Parameter
    * @param optdd HTML Element
    */
-  private static void appendDefaultValueIfSet(Document htmldoc, Parameter<?, ?> par, Element optdd) {
+  private static void appendDefaultValueIfSet(Document htmldoc, Parameter<?> par, Element optdd) {
     if(par.hasDefaultValue()) {
       Element p = htmldoc.createElement(HTMLUtil.HTML_P_TAG);
       p.appendChild(htmldoc.createTextNode(HEADER_DEFAULT_VALUE));
