@@ -58,13 +58,16 @@ import de.lmu.ifi.dbs.elki.database.Database;
 import de.lmu.ifi.dbs.elki.database.StaticArrayDatabase;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
+import de.lmu.ifi.dbs.elki.database.ids.DistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.NumberDistance;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
@@ -127,7 +130,7 @@ import de.lmu.ifi.dbs.elki.visualization.svg.SVGUtil;
  * @param <D> Distance type
  */
 @Reference(authors = "E. Achtert, T. Bernecker, H.-P. Kriegel, E. Schubert, A. Zimek", title = "ELKI in Time: ELKI 0.2 for the Performance Evaluation of Distance Measures for Time Series", booktitle = "Proceedings of the 11th International Symposium on Spatial and Temporal Databases (SSTD), Aalborg, Denmark, 2009", url = "http://dx.doi.org/10.1007/978-3-642-02982-0_35")
-public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<D, ?>> extends AbstractApplication {
+public class KNNExplorer<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractApplication {
   // TODO: replace by a wrapper creating appropriate visualization modules.
 
   /**
@@ -142,7 +145,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
    * {@link de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction}
    * </p>
    */
-  public static final OptionID DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("explorer.distancefunction", "Distance function to determine the distance between database objects.");
+  public static final OptionID DISTANCE_FUNCTION_ID = new OptionID("explorer.distancefunction", "Distance function to determine the distance between database objects.");
 
   /**
    * Holds the database connection to have the algorithm run with.
@@ -158,12 +161,11 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
   /**
    * Constructor.
    * 
-   * @param verbose Verbose flag
    * @param database Database
    * @param distanceFunction Distance function
    */
-  public KNNExplorer(boolean verbose, Database database, DistanceFunction<O, D> distanceFunction) {
-    super(verbose);
+  public KNNExplorer(Database database, DistanceFunction<O, D> distanceFunction) {
+    super();
     this.database = database;
     this.distanceFunction = distanceFunction;
   }
@@ -370,15 +372,14 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
       this.db = db;
       this.data = distanceQuery.getRelation();
       this.labelRep = DatabaseUtil.guessObjectLabelRepresentation(db);
-      this.dim = DatabaseUtil.dimensionality(distanceQuery.getRelation());
+      this.dim = RelationUtil.dimensionality(distanceQuery.getRelation());
       this.distanceQuery = distanceQuery;
       this.updateK(k);
 
       double min = Double.MAX_VALUE;
       double max = Double.MIN_VALUE;
       for(DBIDIter iditer = data.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        DBID objID  = iditer.getDBID();
-        O vec = data.get(objID);
+        O vec = data.get(iditer);
         DoubleMinMax mm = VectorUtil.getRangeDouble(vec);
         min = Math.min(min, mm.getMin());
         max = Math.max(max, mm.getMax());
@@ -411,8 +412,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
 
       DefaultListModel m = new DefaultListModel();
       for(DBIDIter iditer = data.iterDBIDs(); iditer.valid(); iditer.advance()) {
-        DBID dbid  = iditer.getDBID();
-        m.addElement(dbid);
+        m.addElement(DBIDUtil.deref(iditer));
       }
       seriesList.setModel(m);
 
@@ -447,18 +447,18 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
         }
 
         for (int i = knn.size() - 1; i >= 0; i--) {
-          DistanceResultPair<D> pair = knn.get(i);
-          Element line = plotSeries(pair.getDBID(), MAXRESOLUTION);
+          DistanceDBIDPair<D> pair = knn.get(i);
+          Element line = plotSeries(pair, MAXRESOLUTION);
           double dist = pair.getDistance().doubleValue() / maxdist;
           Color color = getColor(dist);
           String colstr = "#" + Integer.toHexString(color.getRGB()).substring(2);
-          String width = (pair.getDBID().sameDBID(idx)) ? "0.5%" : "0.2%";
+          String width = (DBIDUtil.equal(pair, idx)) ? "0.5%" : "0.2%";
           SVGUtil.setStyle(line, "stroke: " + colstr + "; stroke-width: " + width + "; fill: none");
           newe.appendChild(line);
           // put into cache
-          Double known = distancecache.get(pair.getDBID());
+          Double known = distancecache.get(DBIDUtil.deref(pair));
           if(known == null || dist < known) {
-            distancecache.put(pair.getDBID(), dist);
+            distancecache.put(DBIDUtil.deref(pair), dist);
           }
         }
       }
@@ -484,7 +484,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
      * @param resolution Maximum number of steps to plot
      * @return SVG element
      */
-    private Element plotSeries(DBID idx, int resolution) {
+    private Element plotSeries(DBIDRef idx, int resolution) {
       O series = data.get(idx);
 
       double step = 1.0;
@@ -495,7 +495,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
       SVGPath path = new SVGPath();
       for(double id = 0; id < dim; id += step) {
         int i = (int) Math.floor(id);
-        path.drawTo(StyleLibrary.SCALE * ratio * (((double) i) / (dim - 1)), StyleLibrary.SCALE * (1.0 - s.getScaled(series.doubleValue(i + 1))));
+        path.drawTo(StyleLibrary.SCALE * ratio * (((double) i) / (dim - 1)), StyleLibrary.SCALE * (1.0 - s.getScaled(series.doubleValue(i))));
       }
       Element p = path.makeElement(plot);
       return p;
@@ -548,7 +548,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<O extends NumberVector<?, ?>, D extends NumberDistance<D, ?>> extends AbstractApplication.Parameterizer {
+  public static class Parameterizer<O extends NumberVector<?>, D extends NumberDistance<D, ?>> extends AbstractApplication.Parameterizer {
     protected Database database = null;
 
     protected DistanceFunction<O, D> distanceFunction = null;
@@ -571,7 +571,7 @@ public class KNNExplorer<O extends NumberVector<?, ?>, D extends NumberDistance<
 
     @Override
     protected KNNExplorer<O, D> makeInstance() {
-      return new KNNExplorer<O, D>(verbose, database, distanceFunction);
+      return new KNNExplorer<O, D>(database, distanceFunction);
     }
   }
 
