@@ -1,7 +1,5 @@
 package de.lmu.ifi.dbs.elki.data;
 
-import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
-
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
@@ -25,54 +23,21 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
+
 /**
  * AbstractNumberVector is an abstract implementation of FeatureVector.
  * 
  * @author Arthur Zimek
- * @param <V> the concrete type of this AbstractNumberVector
- * @param <N> the type of number, this AbstractNumberVector consists of (i.e., a
- *        AbstractNumberVector {@code v} of type {@code V} and dimensionality
- *        {@code d} is an element of {@code N}<sup>{@code d}</sup>)
+ * 
+ * @param <N> the type of number stored in this vector
  */
-public abstract class AbstractNumberVector<V extends AbstractNumberVector<? extends V, N>, N extends Number> implements NumberVector<V, N> {
+public abstract class AbstractNumberVector<N extends Number> implements NumberVector<N> {
   /**
    * The String to separate attribute values in a String that represents the
    * values.
    */
-  public final static String ATTRIBUTE_SEPARATOR = " ";
-
-  /**
-   * An Object obj is equal to this AbstractNumberVector if it is an instance of
-   * the same runtime class and is of the identical dimensionality and the
-   * values of this AbstractNumberVector are equal to the values of obj in all
-   * dimensions, respectively.
-   * 
-   * @param obj another Object
-   * @return true if the specified Object is an instance of the same runtime
-   *         class and is of the identical dimensionality and the values of this
-   *         AbstractNumberVector are equal to the values of obj in all
-   *         dimensions, respectively
-   */
-  @SuppressWarnings("unchecked")
-  @Override
-  public boolean equals(Object obj) {
-    if(this.getClass().isInstance(obj)) {
-      V rv = (V) obj;
-      boolean equal = (this.getDimensionality() == rv.getDimensionality());
-      for(int i = 1; i <= getDimensionality() && equal; i++) {
-        equal &= this.getValue(i).equals(rv.getValue(i));
-      }
-      return equal;
-    }
-    else {
-      return false;
-    }
-  }
-
-  @Override
-  public int hashCode() {
-    return super.hashCode();
-  }
+  public static final String ATTRIBUTE_SEPARATOR = " ";
 
   @Override
   public double getMin(int dimension) {
@@ -104,8 +69,20 @@ public abstract class AbstractNumberVector<V extends AbstractNumberVector<? exte
     return (short) longValue(dimension);
   }
 
-  @Override
-  public V newNumberVector(double[] values) {
-    return newNumberVector(values, ArrayLikeUtil.doubleArrayAdapter());
+  /**
+   * Factory class.
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.has AbstractNumberVector
+   * 
+   * @param <V> Vector type
+   * @param <N> Number type
+   */
+  public abstract static class Factory<V extends AbstractNumberVector<N>, N extends Number> implements NumberVector.Factory<V, N> {
+    @Override
+    public V newNumberVector(double[] values) {
+      return newNumberVector(values, ArrayLikeUtil.doubleArrayAdapter());
+    }
   }
-}
\ No newline at end of file
+}
