@@ -23,7 +23,9 @@ package de.lmu.ifi.dbs.elki.data;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayAdapter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
 
 /**
  * Generic FeatureVector class that can contain any type of data (i.e. numerical
@@ -32,10 +34,9 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayAdapter;
  * 
  * @author Erich Schubert
  * 
- * @param <V> Vector class
  * @param <D> Data type
  */
-public interface FeatureVector<V extends FeatureVector<? extends V, D>, D> {
+public interface FeatureVector<D> {
   /**
    * The dimensionality of the vector space where of this FeatureVector of V is
    * an element.
@@ -63,12 +64,38 @@ public interface FeatureVector<V extends FeatureVector<? extends V, D>, D> {
   String toString();
 
   /**
-   * Returns a new FeatureVector of V for the given values.
+   * Factory API for this feature vector.
    * 
-   * @param array the values of the featureVector
-   * @param adapter adapter class
-   * @param <A> Array type
-   * @return a new FeatureVector of V for the given values
+   * @author Erich Schubert
+   *
+   * @param <V> Vector type
+   * @param <D> Data type of vector
    */
-  <A> V newFeatureVector(A array, ArrayAdapter<D, A> adapter);
-}
\ No newline at end of file
+  interface Factory<V extends FeatureVector<? extends D>, D> extends Parameterizable {
+    /**
+     * Returns a new FeatureVector of V for the given values.
+     * 
+     * @param array the values of the featureVector
+     * @param adapter adapter class
+     * @param <A> Array type
+     * @return a new FeatureVector of V for the given values
+     */
+    <A> V newFeatureVector(A array, ArrayAdapter<D, A> adapter);
+
+    /**
+     * Get the default serializer for this type.
+     * 
+     * Note, this may be {@code null} when no serializer is available.
+     * 
+     * @return Serializer
+     */
+    ByteBufferSerializer<V> getDefaultSerializer();
+    
+    /**
+     * Get the objects type restriction.
+     * 
+     * @return Restriction class
+     */
+    Class<? super V> getRestrictionClass();
+  }
+}
