@@ -26,7 +26,6 @@ package de.lmu.ifi.dbs.elki.data;
 import de.lmu.ifi.dbs.elki.data.spatial.SpatialComparable;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
 
 /**
  * Interface NumberVector defines the methods that should be implemented by any
@@ -34,13 +33,16 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.Parameterizable;
  * 
  * @author Arthur Zimek
  * 
- * @param <V> the type of NumberVector implemented by a subclass
  * @param <N> the type of the attribute values
  * 
  * @apiviz.landmark
  * @apiviz.has Vector
  */
-public interface NumberVector<V extends NumberVector<? extends V, N>, N extends Number> extends FeatureVector<V, N>, SpatialComparable, Parameterizable {
+public interface NumberVector<N extends Number> extends FeatureVector<N>, SpatialComparable {
+  @Deprecated
+  @Override
+  N getValue(int dimension);
+  
   /**
    * Returns the value in the specified dimension as double.
    * 
@@ -48,7 +50,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).doubleValue()}, but usually this is much more
    * efficient due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -61,7 +63,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).floatValue()}, but usually this is much more efficient
    * due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -74,7 +76,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).intValue()}, but usually this is much more efficient
    * due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -87,7 +89,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).longValue()}, but usually this is much more efficient
    * due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -100,7 +102,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).shortValue()}, but usually this is much more efficient
    * due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -113,7 +115,7 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
    * {@code getValue(dim).byteValue()}, but usually this is much more efficient
    * due to boxing/unboxing cost.
    * 
-   * @param dimension the desired dimension, where 1 &le; dimension &le;
+   * @param dimension the desired dimension, where 0 &le; dimension &lt;
    *        <code>this.getDimensionality()</code>
    * @return the value in the specified dimension
    */
@@ -130,20 +132,32 @@ public interface NumberVector<V extends NumberVector<? extends V, N>, N extends
   Vector getColumnVector();
 
   /**
-   * Returns a new NumberVector of N for the given values.
+   * Factory API for this feature vector.
    * 
-   * @param values the values of the NumberVector
-   * @return a new NumberVector of N for the given values
+   * @author Erich Schubert
+   * 
+   * @apiviz.has NumberVector
+   * 
+   * @param <V> Vector type
+   * @param <N> Data type of vector
    */
-  V newNumberVector(double[] values);
+  interface Factory<V extends NumberVector<? extends N>, N extends Number> extends FeatureVector.Factory<V, N> {
+    /**
+     * Returns a new NumberVector of N for the given values.
+     * 
+     * @param values the values of the NumberVector
+     * @return a new NumberVector of N for the given values
+     */
+    V newNumberVector(double[] values);
 
-  /**
-   * Instantiate from any number-array like object.
-   * 
-   * @param <A> Array type
-   * @param array Array
-   * @param adapter Adapter
-   * @return a new NumberVector of N for the given values.
-   */
-  <A> V newNumberVector(A array, NumberArrayAdapter<?, A> adapter);
-}
\ No newline at end of file
+    /**
+     * Instantiate from any number-array like object.
+     * 
+     * @param <A> Array type
+     * @param array Array
+     * @param adapter Adapter
+     * @return a new NumberVector of N for the given values.
+     */
+    <A> V newNumberVector(A array, NumberArrayAdapter<?, ? super A> adapter);
+  }
+}
