@@ -26,16 +26,15 @@ package de.lmu.ifi.dbs.elki.data;
 import java.util.BitSet;
 
 /**
- * Extended interface for sparse feature vector types
+ * Extended interface for sparse feature vector types.
  * 
  * @author Erich Schubert
  *
- * @param <V> Vector type (self-reference
  * @param <D> Data type
  */
-public interface SparseFeatureVector<V extends SparseFeatureVector<V, D>, D> extends FeatureVector<V, D> {
+public interface SparseFeatureVector<D> extends FeatureVector<D> {
   /**
-   * Bit set of non-null features
+   * Bit set of non-null features.
    * 
    * @return Bit set
    */
