@@ -23,20 +23,22 @@ package de.lmu.ifi.dbs.elki.data;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.map.hash.TIntDoubleHashMap;
+
 import java.util.BitSet;
 import java.util.Comparator;
 import java.util.Random;
 
 import de.lmu.ifi.dbs.elki.data.spatial.SpatialComparable;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.math.DoubleMinMax;
 import de.lmu.ifi.dbs.elki.math.MathUtil;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.QuickSelect;
 
 /**
@@ -49,6 +51,13 @@ import de.lmu.ifi.dbs.elki.utilities.datastructures.QuickSelect;
  * @apiviz.uses NumberVector
  */
 public final class VectorUtil {
+  /**
+   * Fake constructor. Do not instantiate, use static methods.
+   */
+  private VectorUtil() {
+    // Do not instantiate - utility class.
+  }
+
   /**
    * Return the range across all dimensions. Useful in particular for time
    * series.
@@ -56,37 +65,39 @@ public final class VectorUtil {
    * @param vec Vector to process.
    * @return [min, max]
    */
-  public static DoubleMinMax getRangeDouble(NumberVector<?, ?> vec) {
+  public static DoubleMinMax getRangeDouble(NumberVector<?> vec) {
     DoubleMinMax minmax = new DoubleMinMax();
 
     for(int i = 0; i < vec.getDimensionality(); i++) {
-      minmax.put(vec.doubleValue(i + 1));
+      minmax.put(vec.doubleValue(i));
     }
 
     return minmax;
   }
 
   /**
-   * Produce a new vector based on random numbers in [0:1] of the same type and
-   * dimensionality as the given vector.
-   * 
-   * @param template existing instance of wanted dimensionality.
+   * Produce a new vector based on random numbers in [0:1].
+   *
+   * @param factory Vector factory
+   * @param dim desired dimensionality
    * @param r Random generator
+   * @param <V> vector type
    * @return new instance
    */
-  public static <V extends NumberVector<V, ?>> V randomVector(V template, Random r) {
-    return template.newNumberVector(MathUtil.randomDoubleArray(template.getDimensionality(), r));
+  public static <V extends NumberVector<?>> V randomVector(NumberVector.Factory<V, ?> factory, int dim, Random r) {
+    return factory.newNumberVector(MathUtil.randomDoubleArray(dim, r));
   }
 
   /**
-   * Produce a new vector based on random numbers in [0:1] of the same type and
-   * dimensionality as the given vector.
+   * Produce a new vector based on random numbers in [0:1].
    * 
-   * @param template existing instance of wanted dimensionality.
+   * @param factory Vector factory
+   * @param dim desired dimensionality
+   * @param <V> vector type
    * @return new instance
    */
-  public static <V extends NumberVector<V, ?>> V randomVector(V template) {
-    return randomVector(template, new Random());
+  public static <V extends NumberVector<?>> V randomVector(NumberVector.Factory<V, ?> factory, int dim) {
+    return randomVector(factory, dim, new Random());
   }
 
   /**
@@ -96,7 +107,7 @@ public final class VectorUtil {
    * @param v2 Second vector
    * @return angle
    */
-  public static double angleSparse(SparseNumberVector<?, ?> v1, SparseNumberVector<?, ?> v2) {
+  public static double angleSparse(SparseNumberVector<?> v1, SparseNumberVector<?> v2) {
     BitSet b1 = v1.getNotNullMask();
     BitSet b2 = v2.getNotNullMask();
     BitSet both = (BitSet) b1.clone();
@@ -134,7 +145,7 @@ public final class VectorUtil {
    * @param o Origin
    * @return Angle
    */
-  public static double angle(NumberVector<?, ?> v1, NumberVector<?, ?> v2, Vector o) {
+  public static double angle(NumberVector<?> v1, NumberVector<?> v2, Vector o) {
     // Essentially, we want to compute this:
     // v1' = v1 - o, v2' = v2 - o
     // v1'.transposeTimes(v2') / (v1'.euclideanLength()*v2'.euclideanLength());
@@ -143,8 +154,8 @@ public final class VectorUtil {
     final int dim = v1.getDimensionality();
     double s = 0, e1 = 0, e2 = 0;
     for(int k = 0; k < dim; k++) {
-      final double r1 = v1.doubleValue(k + 1) - oe[k];
-      final double r2 = v2.doubleValue(k + 1) - oe[k];
+      final double r1 = v1.doubleValue(k) - oe[k];
+      final double r2 = v2.doubleValue(k) - oe[k];
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
@@ -160,7 +171,7 @@ public final class VectorUtil {
    * @param o Origin
    * @return Angle
    */
-  public static double angle(NumberVector<?, ?> v1, NumberVector<?, ?> v2, NumberVector<?, ?> o) {
+  public static double angle(NumberVector<?> v1, NumberVector<?> v2, NumberVector<?> o) {
     // Essentially, we want to compute this:
     // v1' = v1 - o, v2' = v2 - o
     // v1'.transposeTimes(v2') / (v1'.euclideanLength()*v2'.euclideanLength());
@@ -168,8 +179,8 @@ public final class VectorUtil {
     final int dim = v1.getDimensionality();
     double s = 0, e1 = 0, e2 = 0;
     for(int k = 0; k < dim; k++) {
-      final double r1 = v1.doubleValue(k + 1) - o.doubleValue(k + 1);
-      final double r2 = v2.doubleValue(k + 1) - o.doubleValue(k + 1);
+      final double r1 = v1.doubleValue(k) - o.doubleValue(k);
+      final double r2 = v2.doubleValue(k) - o.doubleValue(k);
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
@@ -186,9 +197,9 @@ public final class VectorUtil {
    * @param v2 second vector
    * @return Angle
    */
-  public static double cosAngle(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
-    if(v1 instanceof SparseNumberVector<?, ?> && v2 instanceof SparseNumberVector<?, ?>) {
-      return angleSparse((SparseNumberVector<?, ?>) v1, (SparseNumberVector<?, ?>) v2);
+  public static double cosAngle(NumberVector<?> v1, NumberVector<?> v2) {
+    if(v1 instanceof SparseNumberVector<?> && v2 instanceof SparseNumberVector<?>) {
+      return angleSparse((SparseNumberVector<?>) v1, (SparseNumberVector<?>) v2);
     }
     // Essentially, we want to compute this:
     // v1.transposeTimes(v2) / (v1.euclideanLength() * v2.euclideanLength());
@@ -198,18 +209,18 @@ public final class VectorUtil {
     final int dim = Math.min(d1, d2);
     double s = 0, e1 = 0, e2 = 0;
     for(int k = 0; k < dim; k++) {
-      final double r1 = v1.doubleValue(k + 1);
-      final double r2 = v2.doubleValue(k + 1);
+      final double r1 = v1.doubleValue(k);
+      final double r2 = v2.doubleValue(k);
       s += r1 * r2;
       e1 += r1 * r1;
       e2 += r2 * r2;
     }
     for(int k = dim; k < d1; k++) {
-      final double r1 = v1.doubleValue(k + 1);
+      final double r1 = v1.doubleValue(k);
       e1 += r1 * r1;
     }
     for(int k = dim; k < d2; k++) {
-      final double r2 = v2.doubleValue(k + 1);
+      final double r2 = v2.doubleValue(k);
       e2 += r2 * r2;
     }
     return Math.min(Math.sqrt((s / e1) * (s / e2)), 1);
@@ -227,8 +238,8 @@ public final class VectorUtil {
    * @return Angle
    */
   public static double minCosAngle(SpatialComparable v1, SpatialComparable v2) {
-    if(v1 instanceof NumberVector<?, ?> && v2 instanceof NumberVector<?, ?>) {
-      return cosAngle((NumberVector<?, ?>) v1, (NumberVector<?, ?>) v2);
+    if(v1 instanceof NumberVector<?> && v2 instanceof NumberVector<?>) {
+      return cosAngle((NumberVector<?>) v1, (NumberVector<?>) v2);
     }
     // Essentially, we want to compute this:
     // absmax(v1.transposeTimes(v2))/(min(v1.euclideanLength())*min(v2.euclideanLength()));
@@ -236,8 +247,8 @@ public final class VectorUtil {
     final int dim = v1.getDimensionality();
     double s1 = 0, s2 = 0, e1 = 0, e2 = 0;
     for(int k = 0; k < dim; k++) {
-      final double min1 = v1.getMin(k + 1), max1 = v1.getMax(k + 1);
-      final double min2 = v2.getMin(k + 1), max2 = v2.getMax(k + 1);
+      final double min1 = v1.getMin(k), max1 = v1.getMax(k);
+      final double min2 = v2.getMin(k), max2 = v2.getMax(k);
       final double p1 = min1 * min2, p2 = min1 * max2;
       final double p3 = max1 * min2, p4 = max1 * max2;
       s1 += Math.max(Math.max(p1, p2), Math.max(p3, p4));
@@ -268,10 +279,10 @@ public final class VectorUtil {
    * @return the scalar product (inner product) of this and the given
    *         DoubleVector
    */
-  public static double scalarProduct(NumberVector<?, ?> d1, NumberVector<?, ?> d2) {
+  public static double scalarProduct(NumberVector<?> d1, NumberVector<?> d2) {
     final int dim = d1.getDimensionality();
     double result = 0.0;
-    for(int i = 1; i <= dim; i++) {
+    for(int i = 0; i < dim; i++) {
       result += d1.doubleValue(i) * d2.doubleValue(i);
     }
     return result;
@@ -284,46 +295,48 @@ public final class VectorUtil {
    * @param sample Sample set
    * @return Medoid vector
    */
-  public static Vector computeMedoid(Relation<? extends NumberVector<?, ?>> relation, DBIDs sample) {
-    final int dim = DatabaseUtil.dimensionality(relation);
+  public static Vector computeMedoid(Relation<? extends NumberVector<?>> relation, DBIDs sample) {
+    final int dim = RelationUtil.dimensionality(relation);
     ArrayModifiableDBIDs mids = DBIDUtil.newArray(sample);
     SortDBIDsBySingleDimension s = new SortDBIDsBySingleDimension(relation);
     Vector medoid = new Vector(dim);
     for (int d = 0; d < dim; d++) {
-      s.setDimension(d + 1);
-      medoid.set(d, relation.get(QuickSelect.median(mids, s)).doubleValue(d + 1));
+      s.setDimension(d);
+      medoid.set(d, relation.get(QuickSelect.median(mids, s)).doubleValue(d));
     }
     return medoid;
   }
 
   /**
-   * Compare number vectors by a single dimension
+   * Compare number vectors by a single dimension.
    * 
    * @author Erich Schubert
+   *
+   * @apiviz.exclude
    */
-  public static class SortDBIDsBySingleDimension implements Comparator<DBID> {
+  public static class SortDBIDsBySingleDimension implements Comparator<DBIDRef> {
     /**
-     * Dimension to sort with
+     * Dimension to sort with.
      */
-    public int d;
+    private int d;
 
     /**
      * The relation to sort.
      */
-    private Relation<? extends NumberVector<?, ?>> data;
+    private Relation<? extends NumberVector<?>> data;
 
     /**
      * Constructor.
      * 
      * @param data Vector data source
      */
-    public SortDBIDsBySingleDimension(Relation<? extends NumberVector<?, ?>> data) {
+    public SortDBIDsBySingleDimension(Relation<? extends NumberVector<?>> data) {
       super();
       this.data = data;
     };
 
     /**
-     * Get the dimension to sort by
+     * Get the dimension to sort by.
      * 
      * @return Dimension to sort with
      */
@@ -332,30 +345,32 @@ public final class VectorUtil {
     }
 
     /**
-     * Set the dimension to sort by
+     * Set the dimension to sort by.
      * 
-     * @param d Dimension to sort with
+     * @param d2 Dimension to sort with
      */
-    public void setDimension(int d) {
-      this.d = d;
+    public void setDimension(int d2) {
+      this.d = d2;
     }
 
     @Override
-    public int compare(DBID id1, DBID id2) {
+    public int compare(DBIDRef id1, DBIDRef id2) {
       return Double.compare(data.get(id1).doubleValue(d), data.get(id2).doubleValue(d));
     }
   }
 
   /**
-   * Compare number vectors by a single dimension
+   * Compare number vectors by a single dimension.
    * 
    * @author Erich Schubert
+   *
+   * @apiviz.exclude
    */
-  public static class SortVectorsBySingleDimension implements Comparator<NumberVector<?, ?>> {
+  public static class SortVectorsBySingleDimension implements Comparator<NumberVector<?>> {
     /**
-     * Dimension to sort with
+     * Dimension to sort with.
      */
-    public int d;
+    private int d;
 
     /**
      * Constructor.
@@ -365,7 +380,7 @@ public final class VectorUtil {
     };
 
     /**
-     * Get the dimension to sort by
+     * Get the dimension to sort by.
      * 
      * @return Dimension to sort with
      */
@@ -374,17 +389,50 @@ public final class VectorUtil {
     }
 
     /**
-     * Set the dimension to sort by
+     * Set the dimension to sort by.
      * 
-     * @param d Dimension to sort with
+     * @param d2 Dimension to sort with
      */
-    public void setDimension(int d) {
-      this.d = d;
+    public void setDimension(int d2) {
+      this.d = d2;
     }
 
     @Override
-    public int compare(NumberVector<?, ?> o1, NumberVector<?, ?> o2) {
+    public int compare(NumberVector<?> o1, NumberVector<?> o2) {
       return Double.compare(o1.doubleValue(d), o2.doubleValue(d));
     }
   }
-}
\ No newline at end of file
+
+  /**
+   * Provides a new NumberVector as a projection on the specified attributes.
+   * 
+   * @param v a NumberVector to project
+   * @param selectedAttributes the attributes selected for projection
+   * @param factory Vector factory
+   * @param <V> Vector type
+   * @return a new NumberVector as a projection on the specified attributes
+   */
+  public static <V extends NumberVector<?>> V project(V v, BitSet selectedAttributes, NumberVector.Factory<V, ?> factory) {
+    if (factory instanceof SparseNumberVector.Factory) {
+      final SparseNumberVector.Factory<?, ?> sfactory = (SparseNumberVector.Factory<?, ?>) factory;
+      TIntDoubleHashMap values = new TIntDoubleHashMap(selectedAttributes.cardinality(), 1);
+      for (int d = selectedAttributes.nextSetBit(0); d >= 0; d = selectedAttributes.nextSetBit(d + 1)) {
+        if (v.doubleValue(d) != 0.0) {
+          values.put(d, v.doubleValue(d));
+        }
+      }
+      // We can't avoid this cast, because Java doesn't know that V is a SparseNumberVector:
+      @SuppressWarnings("unchecked")
+      V projectedVector = (V) sfactory.newNumberVector(values, selectedAttributes.cardinality());
+      return projectedVector;
+    } else {
+      double[] newAttributes = new double[selectedAttributes.cardinality()];
+      int i = 0;
+      for (int d = selectedAttributes.nextSetBit(0); d >= 0; d = selectedAttributes.nextSetBit(d + 1)) {
+        newAttributes[i] = v.doubleValue(d);
+        i++;
+      }
+      return factory.newNumberVector(newAttributes);
+    }
+  }
+}
