@@ -41,7 +41,7 @@ public class ComputeNaiveHSBColorHistogram extends ComputeHSBColorHistogram {
    * Key: {@code -hsbhist.bpp}
    * </p>
    */
-  public static final OptionID BINSPERPLANE_ID = OptionID.getOrCreateOptionID("hsbhist.bpp", "Bins per plane for HSV/HSB histogram. This will result in bpp ** 3 bins.");
+  public static final OptionID BINSPERPLANE_ID = new OptionID("hsbhist.bpp", "Bins per plane for HSV/HSB histogram. This will result in bpp ** 3 bins.");
 
   /**
    * Constructor.
