@@ -37,7 +37,7 @@ public final class ClusterModel extends BaseModel {
    * Static cluster model that can be shared for all clusters (since the object
    * doesn't include meta information.
    */
-  public final static ClusterModel CLUSTER = new ClusterModel();
+  public static final ClusterModel CLUSTER = new ClusterModel();
 
   @Override
   public String toString() {
