@@ -32,7 +32,7 @@ import java.util.ListIterator;
 import de.lmu.ifi.dbs.elki.data.type.NoSupportedDataTypeException;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreListener;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
@@ -72,7 +72,7 @@ public abstract class AbstractDatabase extends AbstractHierarchicalResult implem
    * Key: {@code -db.index}
    * </p>
    */
-  public static final OptionID INDEX_ID = OptionID.getOrCreateOptionID("db.index", "Database indexes to add.");
+  public static final OptionID INDEX_ID = new OptionID("db.index", "Database indexes to add.");
 
   /**
    * The event manager, collects events and fires them on demand.
@@ -85,12 +85,12 @@ public abstract class AbstractDatabase extends AbstractHierarchicalResult implem
   protected final List<Relation<?>> relations = new java.util.Vector<Relation<?>>();
 
   /**
-   * Indexes
+   * Indexes.
    */
   protected final List<Index> indexes = new java.util.Vector<Index>();
 
   /**
-   * Index factories
+   * Index factories.
    */
   protected final Collection<IndexFactory<?, ?>> indexFactories = new java.util.Vector<IndexFactory<?, ?>>();
 
@@ -120,7 +120,7 @@ public abstract class AbstractDatabase extends AbstractHierarchicalResult implem
   }
 
   @Override
-  public SingleObjectBundle getBundle(DBID id) {
+  public SingleObjectBundle getBundle(DBIDRef id) {
     assert (id != null);
     // TODO: ensure that the ID actually exists in the database?
     try {
@@ -302,5 +302,5 @@ public abstract class AbstractDatabase extends AbstractHierarchicalResult implem
     return "database";
   }
 
-  abstract protected Logging getLogger();
+  protected abstract Logging getLogger();
 }
\ No newline at end of file
