@@ -29,6 +29,7 @@ import java.util.Map;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 
 /**
  * A class to answer representation queries using a map. Basically, it is just a
@@ -40,9 +41,8 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
  */
 public class MapStore<T> implements WritableDataStore<T> {
   /**
-   * Storage Map
+   * Storage Map.
    */
-  // TODO: use trove maps?
   private Map<DBID, T> data;
 
   /**
@@ -65,15 +65,15 @@ public class MapStore<T> implements WritableDataStore<T> {
 
   @Override
   public T get(DBIDRef id) {
-    return data.get(id.getDBID());
+    return data.get(DBIDUtil.deref(id));
   }
 
   @Override
   public T put(DBIDRef id, T value) {
     if(value == null) {
-      return data.remove(id.getDBID());
+      return data.remove(DBIDUtil.deref(id));
     }
-    return data.put(id.getDBID(), value);
+    return data.put(DBIDUtil.deref(id), value);
   }
 
   @Override
