@@ -24,13 +24,15 @@ package de.lmu.ifi.dbs.elki.database.datastore.memory;
  */
 
 import de.lmu.ifi.dbs.elki.database.datastore.DataStoreFactory;
-import de.lmu.ifi.dbs.elki.database.datastore.RangeIDMap;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableDBIDDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDataStore;
+import de.lmu.ifi.dbs.elki.database.datastore.WritableDoubleDistanceDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableIntegerDataStore;
 import de.lmu.ifi.dbs.elki.database.datastore.WritableRecordStore;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRange;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 
 /**
  * Simple factory class that will store all data in memory using object arrays
@@ -51,6 +53,9 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
   @SuppressWarnings("unchecked")
   @Override
   public <T> WritableDataStore<T> makeStorage(DBIDs ids, int hints, Class<? super T> dataclass) {
+    if (DoubleDistance.class.equals(dataclass)) {
+      return (WritableDataStore<T>) makeDoubleDistanceStorage(ids, hints);
+    }
     if (Double.class.equals(dataclass)) {
       return (WritableDataStore<T>) makeDoubleStorage(ids, hints);
     }
@@ -60,18 +65,46 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
       Object[] data = new Object[range.size()];
-      return new ArrayStore<T>(data, new RangeIDMap(range));
+      return new ArrayStore<T>(data, range);
     }
     else {
       return new MapIntegerDBIDStore<T>(ids.size());
     }
   }
 
+  @Override
+  public WritableDBIDDataStore makeDBIDStorage(DBIDs ids, int hints) {
+    if(ids instanceof DBIDRange) {
+      DBIDRange range = (DBIDRange) ids;
+      return new ArrayDBIDStore(range.size(), range);
+    }
+    else {
+      return new MapIntegerDBIDDBIDStore(ids.size());
+    }
+  }
+
+  /**
+   * Make a data storage for double distances.
+   * 
+   * @param ids IDs to store for
+   * @param hints Storage hints
+   * @return double distance storage
+   */
+  public WritableDoubleDistanceDataStore makeDoubleDistanceStorage(DBIDs ids, int hints) {
+    if(ids instanceof DBIDRange) {
+      DBIDRange range = (DBIDRange) ids;
+      return new ArrayDoubleDistanceStore(range.size(), range);
+    }
+    else {
+      return new MapIntegerDBIDDoubleDistanceStore(ids.size());
+    }
+  }
+
   @Override
   public WritableDoubleDataStore makeDoubleStorage(DBIDs ids, int hints) {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
-      return new ArrayDoubleStore(range.size(), new RangeIDMap(range));
+      return new ArrayDoubleStore(range.size(), range);
     }
     else {
       return new MapIntegerDBIDDoubleStore(ids.size());
@@ -82,7 +115,7 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
   public WritableDoubleDataStore makeDoubleStorage(DBIDs ids, int hints, double def) {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
-      return new ArrayDoubleStore(range.size(), new RangeIDMap(range), def);
+      return new ArrayDoubleStore(range.size(), range, def);
     }
     else {
       return new MapIntegerDBIDDoubleStore(ids.size(), def);
@@ -93,7 +126,7 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
   public WritableIntegerDataStore makeIntegerStorage(DBIDs ids, int hints) {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
-      return new ArrayIntegerStore(range.size(), new RangeIDMap(range));
+      return new ArrayIntegerStore(range.size(), range);
     }
     else {
       return new MapIntegerDBIDIntegerStore(ids.size());
@@ -104,7 +137,7 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
   public WritableIntegerDataStore makeIntegerStorage(DBIDs ids, int hints, int def) {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
-      return new ArrayIntegerStore(range.size(), new RangeIDMap(range), def);
+      return new ArrayIntegerStore(range.size(), range, def);
     }
     else {
       return new MapIntegerDBIDIntegerStore(ids.size(), def);
@@ -116,7 +149,7 @@ public class MemoryDataStoreFactory implements DataStoreFactory {
     if(ids instanceof DBIDRange) {
       DBIDRange range = (DBIDRange) ids;
       Object[][] data = new Object[range.size()][dataclasses.length];
-      return new ArrayRecordStore(data, new RangeIDMap(range));
+      return new ArrayRecordStore(data, range);
     }
     else {
       return new MapIntegerDBIDRecordStore(ids.size(), dataclasses.length);
