@@ -24,6 +24,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  */
 
 import de.lmu.ifi.dbs.elki.database.ids.integer.TrivialDBIDFactory;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.persistent.FixedSizeByteBufferSerializer;
 
@@ -47,29 +48,49 @@ public interface DBIDFactory {
   /**
    * Static DBID factory to use.
    */
-  public static DBIDFactory FACTORY = new TrivialDBIDFactory();
-  
+  final static DBIDFactory FACTORY = new TrivialDBIDFactory();
+
   /**
-   * Import an integer ID
+   * Make a new DBID variable.
+   * 
+   * @param val Initial value.
+   * @return Variable
+   */
+  DBIDVar newVar(DBIDRef val);
+
+  /**
+   * Import and integer as DBID.
+   * 
+   * Note: this may not be possible for some factories!
    * 
    * @param id Integer ID to import
    * @return DBID
    */
-  public DBID importInteger(int id);
+  DBID importInteger(int id);
 
   /**
-   * Generate a single DBID
+   * Assign an integer value to a DBID variable.
+   * 
+   * Note: this may not be possible for some factories!
+   * 
+   * @param var Variable
+   * @param val Integer value
+   */
+  void assignVar(DBIDVar var, int val);
+
+  /**
+   * Generate a single DBID.
    * 
    * @return A single DBID
    */
-  public DBID generateSingleDBID();
+  DBID generateSingleDBID();
 
   /**
    * Return a single DBID for reuse.
    * 
    * @param id DBID to deallocate
    */
-  public void deallocateSingleDBID(DBID id);
+  void deallocateSingleDBID(DBIDRef id);
 
   /**
    * Generate a static DBID range.
@@ -77,14 +98,14 @@ public interface DBIDFactory {
    * @param size Requested size
    * @return DBID range
    */
-  public DBIDRange generateStaticDBIDRange(int size);
+  DBIDRange generateStaticDBIDRange(int size);
 
   /**
    * Deallocate a static DBID range.
    * 
    * @param range Range to deallocate
    */
-  public void deallocateDBIDRange(DBIDRange range);
+  void deallocateDBIDRange(DBIDRange range);
 
   /**
    * Make a DBID pair from two existing DBIDs.
@@ -94,72 +115,133 @@ public interface DBIDFactory {
    * 
    * @return new pair.
    */
-  public DBIDPair makePair(DBIDRef id1, DBIDRef id2);
-  
+  DBIDPair newPair(DBIDRef id1, DBIDRef id2);
+
+  /**
+   * Make a double-DBID pair.
+   * 
+   * @param val Double value
+   * @param id DBID
+   * @return New pair
+   */
+  DoubleDBIDPair newPair(double val, DBIDRef id);
+
+  /**
+   * Make a new distance-DBID pair.
+   * 
+   * @param val Distance value
+   * @param id Object ID
+   * @param <D> Distance type
+   * @return New pair
+   */
+  <D extends Distance<D>> DistanceDBIDPair<D> newDistancePair(D val, DBIDRef id);
+
+  /**
+   * Make a new distance-DBID pair.
+   * 
+   * @param val Distance value
+   * @param id Object ID
+   * @return New pair
+   */
+  DoubleDistanceDBIDPair newDistancePair(double val, DBIDRef id);
+
   /**
    * Make a new (modifiable) array of DBIDs.
    * 
    * @return New array
    */
-  public ArrayModifiableDBIDs newArray();
-  
+  ArrayModifiableDBIDs newArray();
+
   /**
    * Make a new (modifiable) hash set of DBIDs.
    * 
    * @return New hash set
    */
-  public HashSetModifiableDBIDs newHashSet();
-  
+  HashSetModifiableDBIDs newHashSet();
+
   /**
    * Make a new (modifiable) array of DBIDs.
    * 
    * @param size Size hint
    * @return New array
    */
-  public ArrayModifiableDBIDs newArray(int size);
-  
+  ArrayModifiableDBIDs newArray(int size);
+
   /**
    * Make a new (modifiable) hash set of DBIDs.
    * 
    * @param size Size hint
    * @return New hash set
    */
-  public HashSetModifiableDBIDs newHashSet(int size);
-  
+  HashSetModifiableDBIDs newHashSet(int size);
+
   /**
    * Make a new (modifiable) array of DBIDs.
    * 
    * @param existing existing DBIDs to use
    * @return New array
    */
-  public ArrayModifiableDBIDs newArray(DBIDs existing);
-  
+  ArrayModifiableDBIDs newArray(DBIDs existing);
+
   /**
    * Make a new (modifiable) hash set of DBIDs.
    * 
    * @param existing existing DBIDs to use
    * @return New hash set
    */
-  public HashSetModifiableDBIDs newHashSet(DBIDs existing);
-  
+  HashSetModifiableDBIDs newHashSet(DBIDs existing);
+
   /**
-   * Get a serializer for DBIDs
-   * 
-   * @return DBID serializer 
-   */
-  public ByteBufferSerializer<DBID> getDBIDSerializer();
-  
-  /**
-   * Get a serializer for DBIDs with static size
+   * Get a serializer for DBIDs.
    * 
    * @return DBID serializer
    */
-  public FixedSizeByteBufferSerializer<DBID> getDBIDSerializerStatic();
-  
+  ByteBufferSerializer<DBID> getDBIDSerializer();
+
   /**
-   * Get type restriction
+   * Get a serializer for DBIDs with static size.
+   * 
+   * @return DBID serializer
+   */
+  FixedSizeByteBufferSerializer<DBID> getDBIDSerializerStatic();
+
+  /**
+   * Get type restriction.
    * 
    * @return type restriction for DBIDs
    */
-  public Class<? extends DBID> getTypeRestriction();
-}
\ No newline at end of file
+  Class<? extends DBID> getTypeRestriction();
+
+  /**
+   * Compare two DBIDs, for sorting.
+   * 
+   * @param a First
+   * @param b Second
+   * @return Comparison result
+   */
+  int compare(DBIDRef a, DBIDRef b);
+
+  /**
+   * Compare two DBIDs, for equality testing.
+   * 
+   * @param a First
+   * @param b Second
+   * @return Comparison result
+   */
+  boolean equal(DBIDRef a, DBIDRef b);
+
+  /**
+   * Print a DBID as string.
+   * 
+   * @param id DBID reference
+   * @return Formatted ID
+   */
+  String toString(DBIDRef id);
+
+  /**
+   * Get the invalid DBID value, usable as "undefined" placeholder.
+   * 
+   * @return Invalid value
+   */
+  DBIDRef invalid();
+}
