@@ -1,4 +1,5 @@
 package de.lmu.ifi.dbs.elki.database.ids;
+
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
@@ -22,17 +23,21 @@ package de.lmu.ifi.dbs.elki.database.ids;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import de.lmu.ifi.dbs.elki.utilities.iterator.MIter;
+
 /**
  * Modifiable DBID iterator.
  * 
  * @author Erich Schubert
  */
-public interface DBIDMIter extends DBIDIter {
+public interface DBIDMIter extends DBIDIter, MIter {
   /**
    * Remove the object the iterator currently points to.
    * 
-   * Subsequent calls to {@link #getDBID} may return a different element.
-   * Call {@link #advance()} to advance the iterator to the next element for further processing.
+   * Note: Subsequent calls to {@link DBIDUtil#deref} may return a different
+   * element. Call {@link #advance()} to advance the iterator to the next
+   * element for further processing.
    */
+  @Override
   void remove();
 }
