@@ -1,5 +1,7 @@
 package de.lmu.ifi.dbs.elki.database.ids;
 
+import de.lmu.ifi.dbs.elki.database.datastore.DataStoreIDMap;
+
 /*
  This file is part of ELKI:
  Environment for Developing KDD-Applications Supported by Index-Structures
@@ -28,7 +30,7 @@ package de.lmu.ifi.dbs.elki.database.ids;
  * 
  * @author Erich Schubert
  */
-public interface DBIDRange extends ArrayStaticDBIDs {
+public interface DBIDRange extends ArrayStaticDBIDs, DataStoreIDMap {
   /**
    * Get offset in the array for a particular DBID.
    * 
@@ -38,5 +40,5 @@ public interface DBIDRange extends ArrayStaticDBIDs {
    * @param dbid ID to compute index for
    * @return index
    */
-  public int getOffset(DBIDRef dbid);
+  int getOffset(DBIDRef dbid);
 }
\ No newline at end of file
