@@ -24,10 +24,10 @@ package de.lmu.ifi.dbs.elki.database.ids.generic;
  */
 
 import java.util.BitSet;
-import java.util.Iterator;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDFactory;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
@@ -41,12 +41,12 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
  */
 public class MaskedDBIDs implements DBIDs {
   /**
-   * Data storage
+   * Data storage.
    */
   protected ArrayDBIDs data;
 
   /**
-   * The bitmask used for masking
+   * The bitmask used for masking.
    */
   protected BitSet bits;
 
@@ -69,16 +69,6 @@ public class MaskedDBIDs implements DBIDs {
     this.inverse = inverse;
   }
 
-  @Override
-  public Iterator<DBID> iterator() {
-    if(inverse) {
-      return new InvItr();
-    }
-    else {
-      return new Itr();
-    }
-  }
-
   @Override
   public DBIDIter iter() {
     if(inverse) {
@@ -102,8 +92,8 @@ public class MaskedDBIDs implements DBIDs {
   @Override
   public boolean contains(DBIDRef o) {
     // TODO: optimize.
-    for(DBID id : this) {
-      if(id.sameDBID(o)) {
+    for(DBIDIter iter = iter(); iter.valid(); iter.advance()) {
+      if(DBIDFactory.FACTORY.equal(iter, o)) {
         return true;
       }
     }
@@ -116,45 +106,7 @@ public class MaskedDBIDs implements DBIDs {
   }
 
   /**
-   * Iterator over set bits
-   * 
-   * @author Erich Schubert
-   * 
-   * @apiviz.exclude
-   */
-  protected class Itr implements Iterator<DBID> {
-    /**
-     * Next position.
-     */
-    private int pos;
-
-    /**
-     * Constructor
-     */
-    protected Itr() {
-      this.pos = bits.nextSetBit(0);
-    }
-
-    @Override
-    public boolean hasNext() {
-      return (pos >= 0) && (pos < data.size());
-    }
-
-    @Override
-    public DBID next() {
-      DBID cur = data.get(pos);
-      pos = bits.nextSetBit(pos + 1);
-      return cur;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  /**
-   * Iterator over set bits
+   * Iterator over set bits.
    * 
    * @author Erich Schubert
    * 
@@ -162,15 +114,21 @@ public class MaskedDBIDs implements DBIDs {
    */
   protected class DBIDItr implements DBIDIter {
     /**
-     * Next position.
+     * Current position.
      */
     private int pos;
 
     /**
-     * Constructor
+     * Array iterator, for referencing.
+     */
+    private DBIDArrayIter iter;
+
+    /**
+     * Constructor.
      */
     protected DBIDItr() {
       this.pos = bits.nextSetBit(0);
+      this.iter = data.iter();
     }
 
     @Override
@@ -184,68 +142,14 @@ public class MaskedDBIDs implements DBIDs {
     }
 
     @Override
-    public int getIntegerID() {
-      return data.get(pos).getIntegerID();
-    }
-
-    @Override
-    public DBID getDBID() {
-      return data.get(pos);
-    }
-
-    @Override
-    public boolean sameDBID(DBIDRef other) {
-      return getIntegerID() == other.getIntegerID();
-    }
-
-    @Override
-    public int compareDBID(DBIDRef o) {
-      final int thisVal = getIntegerID();
-      final int anotherVal = o.getIntegerID();
-      return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
+    public int internalGetIndex() {
+      iter.seek(pos);
+      return iter.internalGetIndex();
     }
   }
 
   /**
-   * Iterator over unset elements.
-   * 
-   * @author Erich Schubert
-   * 
-   * @apiviz.exclude
-   */
-  protected class InvItr implements Iterator<DBID> {
-    /**
-     * Next unset position.
-     */
-    private int pos;
-
-    /**
-     * Constructor
-     */
-    protected InvItr() {
-      this.pos = bits.nextClearBit(0);
-    }
-
-    @Override
-    public boolean hasNext() {
-      return (pos >= 0) && (pos < data.size());
-    }
-
-    @Override
-    public DBID next() {
-      DBID cur = data.get(pos);
-      pos = bits.nextClearBit(pos + 1);
-      return cur;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  /**
-   * Iterator over set bits
+   * Iterator over set bits.
    * 
    * @author Erich Schubert
    * 
@@ -258,10 +162,16 @@ public class MaskedDBIDs implements DBIDs {
     private int pos;
 
     /**
-     * Constructor
+     * Array iterator, for referencing.
+     */
+    private DBIDArrayIter iter;
+
+    /**
+     * Constructor.
      */
     protected InvDBIDItr() {
       this.pos = bits.nextClearBit(0);
+      this.iter = data.iter();
     }
 
     @Override
@@ -275,25 +185,9 @@ public class MaskedDBIDs implements DBIDs {
     }
 
     @Override
-    public int getIntegerID() {
-      return data.get(pos).getIntegerID();
-    }
-
-    @Override
-    public DBID getDBID() {
-      return data.get(pos);
-    }
-
-    @Override
-    public boolean sameDBID(DBIDRef other) {
-      return getIntegerID() == other.getIntegerID();
-    }
-
-    @Override
-    public int compareDBID(DBIDRef o) {
-      final int thisVal = getIntegerID();
-      final int anotherVal = o.getIntegerID();
-      return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
+    public int internalGetIndex() {
+      iter.seek(pos);
+      return iter.internalGetIndex();
     }
   }
 }
\ No newline at end of file
