@@ -23,27 +23,25 @@ package de.lmu.ifi.dbs.elki.database.ids.generic;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.Iterator;
-
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDMIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.StaticDBIDs;
-import de.lmu.ifi.dbs.elki.utilities.iterator.UnmodifiableIterator;
 
 /**
  * Unmodifiable wrapper for DBIDs.
  * 
  * @author Erich Schubert
  * 
- * @apiviz.uses de.lmu.ifi.dbs.elki.database.ids.DBIDs
+ * @apiviz.uses DBIDs
+ * @apiviz.has UnmodifiableDBIDIter
  */
 public class UnmodifiableDBIDs implements StaticDBIDs {
   /**
    * The DBIDs we wrap.
    */
-  final private DBIDs inner;
+  private final DBIDs inner;
 
   /**
    * Constructor.
@@ -65,15 +63,13 @@ public class UnmodifiableDBIDs implements StaticDBIDs {
     return inner.isEmpty();
   }
 
-  @SuppressWarnings("deprecation")
-  @Override
-  public Iterator<DBID> iterator() {
-    return new UnmodifiableIterator<DBID>(inner.iterator());
-  }
-  
   @Override
   public DBIDIter iter() {
-    return inner.iter();
+    DBIDIter it = inner.iter();
+    if(it instanceof DBIDMIter) {
+      return new UnmodifiableDBIDIter(it);
+    }
+    return it;
   }
 
   @Override
@@ -81,11 +77,45 @@ public class UnmodifiableDBIDs implements StaticDBIDs {
     return inner.size();
   }
 
-  /**
-   * Returns a string representation of the inner DBID collection.
-   */
   @Override
   public String toString() {
     return inner.toString();
   }
-}
+
+  /**
+   * Make an existing DBIDMIter unmodifiable.
+   * 
+   * @author Erich Schubert
+   */
+  class UnmodifiableDBIDIter implements DBIDIter {
+    /**
+     * Wrapped iterator.
+     */
+    private DBIDIter it;
+
+    /**
+     * Constructor.
+     * 
+     * @param it inner iterator
+     */
+    public UnmodifiableDBIDIter(DBIDIter it) {
+      super();
+      this.it = it;
+    }
+
+    @Override
+    public boolean valid() {
+      return it.valid();
+    }
+
+    @Override
+    public void advance() {
+      it.advance();
+    }
+
+    @Override
+    public int internalGetIndex() {
+      return it.internalGetIndex();
+    }
+  }
+}
\ No newline at end of file
