@@ -24,11 +24,11 @@ package de.lmu.ifi.dbs.elki.database.ids.integer;
  */
 
 import java.nio.ByteBuffer;
-import java.util.Iterator;
 
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDArrayIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDVar;
 import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
 import de.lmu.ifi.dbs.elki.persistent.ByteArrayUtil;
 import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
@@ -51,11 +51,11 @@ import de.lmu.ifi.dbs.elki.persistent.FixedSizeByteBufferSerializer;
  * @apiviz.composedOf DynamicSerializer
  * @apiviz.composedOf StaticSerializer
  */
-final class IntegerDBID implements DBID {
+final class IntegerDBID implements DBID, IntegerDBIDRef {
   /**
    * The actual object ID.
    */
-  final protected int id;
+  protected final int id;
 
   /**
    * Constructor from integer id.
@@ -74,12 +74,7 @@ final class IntegerDBID implements DBID {
    */
   protected IntegerDBID(Integer id) {
     super();
-    this.id = id;
-  }
-
-  @Override
-  public DBID getDBID() {
-    return this;
+    this.id = id.intValue();
   }
 
   /**
@@ -88,7 +83,7 @@ final class IntegerDBID implements DBID {
    * @return integer id
    */
   @Override
-  public int getIntegerID() {
+  public int internalGetIndex() {
     return this.id;
   }
 
@@ -103,8 +98,12 @@ final class IntegerDBID implements DBID {
   }
 
   @Override
+  @Deprecated
   public boolean equals(Object obj) {
-    if(!(obj instanceof IntegerDBID)) {
+    if (this == obj) {
+      return true;
+    }
+    if (!(obj instanceof IntegerDBID)) {
       if (obj instanceof DBIDRef) {
         LoggingUtil.warning("Programming error: DBID.equals(DBIDRef) is not well-defined. use sameDBID!", new Throwable());
       }
@@ -113,47 +112,37 @@ final class IntegerDBID implements DBID {
     IntegerDBID other = (IntegerDBID) obj;
     return this.id == other.id;
   }
-  
-    @Override
-  public boolean sameDBID(DBIDRef other) {
-    return this.id == other.getIntegerID();
-  }
 
   @Override
   public int compareTo(DBIDRef o) {
-    int thisVal = this.id;
-    int anotherVal = o.getIntegerID();
-    return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
+    final int anotherVal = o.internalGetIndex();
+    return (this.id < anotherVal ? -1 : (this.id == anotherVal ? 0 : 1));
   }
 
   @Override
-  public int compareDBID(DBIDRef o) {
-    int thisVal = this.id;
-    int anotherVal = o.getIntegerID();
-    return (thisVal < anotherVal ? -1 : (thisVal == anotherVal ? 0 : 1));
-  }
-
-  @Override
-  public DBIDIter iter() {
+  public DBIDArrayIter iter() {
     return new DBIDItr();
   }
 
   @Override
   public DBID get(int i) {
-    if(i != 0) {
+    if (i != 0) {
       throw new ArrayIndexOutOfBoundsException();
     }
     return this;
   }
 
   @Override
-  public Iterator<DBID> iterator() {
-    return new Itr();
+  public void assign(int index, DBIDVar var) {
+    if (index != 0) {
+      throw new ArrayIndexOutOfBoundsException();
+    }
+    var.set(this);
   }
 
   @Override
   public boolean contains(DBIDRef o) {
-    return o.getIntegerID() == id;
+    return o.internalGetIndex() == id;
   }
 
   @Override
@@ -163,7 +152,8 @@ final class IntegerDBID implements DBID {
 
   @Override
   public int binarySearch(DBIDRef key) {
-    return (id == key.getIntegerID()) ? 0 : -1;
+    final int other = key.internalGetIndex();
+    return (other == id) ? 0 : (other < id) ? -1 : -2;
   }
 
   /**
@@ -173,61 +163,51 @@ final class IntegerDBID implements DBID {
    * 
    * @apiviz.exclude
    */
-  protected class Itr implements Iterator<DBID> {
+  protected class DBIDItr implements DBIDArrayIter, IntegerDBIDRef {
     /**
-     * Whether we've already returned our object.
+     * Iterator position: We use an integer so we can support retract().
      */
-    boolean first = true;
-
-    @Override
-    public boolean hasNext() {
-      return first == true;
-    }
-
-    @Override
-    public DBID next() {
-      assert (first);
-      first = false;
-      return IntegerDBID.this;
-    }
-
-    @Override
-    public void remove() {
-      throw new UnsupportedOperationException();
-    }
-  }
-
-  /**
-   * Pseudo iterator for DBIDs interface.
-   * 
-   * @author Erich Schubert
-   * 
-   * @apiviz.exclude
-   */
-  protected class DBIDItr implements DBIDIter {
-    /**
-     * Whether we've already returned our object.
-     */
-    boolean first = true;
+    int pos = 0;
 
     @Override
     public void advance() {
-      first = false;
+      pos++;
     }
 
     @Override
-    public int getIntegerID() {
-      return id;
+    public void advance(int count) {
+      pos += count;
     }
 
     @Override
-    public DBID getDBID() {
-      return IntegerDBID.this;
+    public void retract() {
+      pos--;
+    }
+
+    @Override
+    public void seek(int off) {
+      pos = off;
+    }
+
+    @Override
+    public int getOffset() {
+      return pos;
+    }
+
+    @Override
+    public int internalGetIndex() {
+      return IntegerDBID.this.id;
     }
 
     @Override
     public boolean valid() {
-      return first;
+      return (pos == 0);
+    }
+
+    @Override
+    public int hashCode() {
+      // Override, because we also are overriding equals.
+      return super.hashCode();
     }
 
     @Override
@@ -239,14 +219,8 @@ final class IntegerDBID implements DBID {
     }
 
     @Override
-    public boolean sameDBID(DBIDRef other) {
-      return id == other.getIntegerID();
-    }
-
-    @Override
-    public int compareDBID(DBIDRef o) {
-      int anotherVal = o.getIntegerID();
-      return (id < anotherVal ? -1 : (id == anotherVal ? 0 : 1));
+    public String toString() {
+      return Integer.toString(internalGetIndex());
     }
   }
 
@@ -321,10 +295,10 @@ final class IntegerDBID implements DBID {
   /**
    * The public instance to use for dynamic serialization.
    */
-  public final static DynamicSerializer dynamicSerializer = new DynamicSerializer();
+  public static final ByteBufferSerializer<DBID> DYNAMIC_SERIALIZER = new DynamicSerializer();
 
   /**
    * The public instance to use for static serialization.
    */
-  public final static StaticSerializer staticSerializer = new StaticSerializer();
-}
\ No newline at end of file
+  public static final FixedSizeByteBufferSerializer<DBID> STATIC_SERIALIZER = new StaticSerializer();
+}
