@@ -29,6 +29,7 @@ import java.util.BitSet;
 import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDFactory;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRange;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 
 /**
@@ -45,19 +46,20 @@ public class ReusingDBIDFactory extends SimpleDBIDFactory {
   /**
    * Logging for error messages.
    */
-  private static final Logging logger = Logging.getLogger(ReusingDBIDFactory.class);
-  
+  private static final Logging LOG = Logging.getLogger(ReusingDBIDFactory.class);
+
   /**
    * Bit set to keep track of dynamic DBIDs
    */
   BitSet dynamicUsed = new BitSet();
-  
+
   /**
    * Keep track of the lowest unused dynamic DBID
    */
   int dynamicStart = 0;
 
-  // TODO: add an offset, to save keeping long bit sets of 1s for heavy dynamic use?
+  // TODO: add an offset, to save keeping long bit sets of 1s for heavy dynamic
+  // use?
 
   /**
    * Returned range allocations
@@ -79,12 +81,13 @@ public class ReusingDBIDFactory extends SimpleDBIDFactory {
   }
 
   @Override
-  public synchronized void deallocateSingleDBID(DBID id) {
-    if (id.getIntegerID() >= 0) {
-      logger.warning("Single DBID returned is from a range allocation!");
+  public synchronized void deallocateSingleDBID(DBIDRef id) {
+    final int intid = id.internalGetIndex();
+    if (intid >= 0) {
+      LOG.warning("Single DBID returned is from a range allocation!");
       return;
     }
-    int pos = - id.getIntegerID() - 1;
+    final int pos = -intid - 1;
     dynamicUsed.clear(pos);
     dynamicStart = Math.min(dynamicStart, pos);
   }
@@ -113,6 +116,6 @@ public class ReusingDBIDFactory extends SimpleDBIDFactory {
   @Override
   public synchronized void deallocateDBIDRange(DBIDRange range) {
     // TODO: catch an eventual cast exception?
-    returnedAllocations.add((IntegerDBIDRange)range);
+    returnedAllocations.add((IntegerDBIDRange) range);
   }
-}
\ No newline at end of file
+}
