@@ -31,15 +31,21 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDFactory;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRange;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDVar;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
+import de.lmu.ifi.dbs.elki.database.ids.DistanceDBIDPair;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDBIDPair;
+import de.lmu.ifi.dbs.elki.database.ids.DoubleDistanceDBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.HashSetModifiableDBIDs;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.persistent.FixedSizeByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 
 /**
- * Trivial DBID management, that never reuses IDs and just gives them out in sequence.
- * Statically allocated DBID ranges are given positive values,
+ * Trivial DBID management, that never reuses IDs and just gives them out in
+ * sequence. Statically allocated DBID ranges are given positive values,
  * Dynamically allocated DBIDs are given negative values.
  * 
  * @author Erich Schubert
@@ -52,21 +58,26 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
  * @apiviz.uses TroveArrayModifiableDBIDs oneway - - «create»
  * @apiviz.uses TroveHashSetModifiableDBIDs oneway - - «create»
  */
-public class TrivialDBIDFactory implements DBIDFactory {
+final public class TrivialDBIDFactory implements DBIDFactory {
   /**
-   * Keep track of the smallest dynamic DBID offset not used
+   * Keep track of the smallest dynamic DBID offset not used.
    */
   AtomicInteger next = new AtomicInteger(1);
-  
+
   /**
-   * Constructor
+   * Invalid ID.
+   */
+  DBID invalid = new IntegerDBID(Integer.MIN_VALUE);
+
+  /**
+   * Constructor.
    */
   public TrivialDBIDFactory() {
     super();
   }
 
   @Override
-  public DBID generateSingleDBID() {
+  final public DBID generateSingleDBID() {
     final int id = next.getAndIncrement();
     if (id == Integer.MAX_VALUE) {
       throw new AbortException("DBID allocation error - too many objects allocated!");
@@ -76,12 +87,12 @@ public class TrivialDBIDFactory implements DBIDFactory {
   }
 
   @Override
-  public void deallocateSingleDBID(DBID id) {
-    // ignore.
+  final public void deallocateSingleDBID(DBIDRef id) {
+    // ignore for now
   }
 
   @Override
-  public DBIDRange generateStaticDBIDRange(int size) {
+  final public DBIDRange generateStaticDBIDRange(int size) {
     final int start = next.getAndAdd(size);
     if (start > next.get()) {
       throw new AbortException("DBID range allocation error - too many objects allocated!");
@@ -100,6 +111,37 @@ public class TrivialDBIDFactory implements DBIDFactory {
     return new IntegerDBID(id);
   }
 
+  @Override
+  public void assignVar(DBIDVar var, int val) {
+    if (var instanceof IntegerDBIDVar) {
+      ((IntegerDBIDVar)var).internalSetIndex(val);
+    } else {
+      var.set(new IntegerDBID(val));
+    }
+  }
+
+  @Override
+  public int compare(DBIDRef a, DBIDRef b) {
+    final int inta = a.internalGetIndex();
+    final int intb = b.internalGetIndex();
+    return (inta < intb ? -1 : (inta == intb ? 0 : 1));
+  }
+
+  @Override
+  public boolean equal(DBIDRef a, DBIDRef b) {
+    return a.internalGetIndex() == b.internalGetIndex();
+  }
+
+  @Override
+  public String toString(DBIDRef id) {
+    return Integer.toString(id.internalGetIndex());
+  }
+
+  @Override
+  public DBIDVar newVar(DBIDRef val) {
+    return new IntegerDBIDVar(val);
+  }
+
   @Override
   public ArrayModifiableDBIDs newArray() {
     return new TroveArrayModifiableDBIDs();
@@ -131,22 +173,46 @@ public class TrivialDBIDFactory implements DBIDFactory {
   }
 
   @Override
-  public DBIDPair makePair(DBIDRef first, DBIDRef second) {
-    return new IntegerDBIDPair(first.getIntegerID(), second.getIntegerID());
+  public DBIDPair newPair(DBIDRef first, DBIDRef second) {
+    return new IntegerDBIDPair(first.internalGetIndex(), second.internalGetIndex());
+  }
+
+  @Override
+  public DoubleDBIDPair newPair(double val, DBIDRef id) {
+    return new IntegerDoubleDBIDPair(val, id.internalGetIndex());
+  }
+
+  @SuppressWarnings("unchecked")
+  @Override
+  public <D extends Distance<D>> DistanceDBIDPair<D> newDistancePair(D val, DBIDRef id) {
+    if (val instanceof DoubleDistance) {
+      return (DistanceDBIDPair<D>) new DoubleDistanceIntegerDBIDPair(((DoubleDistance) val).doubleValue(), id.internalGetIndex());
+    }
+    return new DistanceIntegerDBIDPair<D>(val, id.internalGetIndex());
+  }
+
+  @Override
+  public DoubleDistanceDBIDPair newDistancePair(double val, DBIDRef id) {
+    return new DoubleDistanceIntegerDBIDPair(val, id.internalGetIndex());
   }
 
   @Override
   public ByteBufferSerializer<DBID> getDBIDSerializer() {
-    return IntegerDBID.dynamicSerializer;
+    return IntegerDBID.DYNAMIC_SERIALIZER;
   }
 
   @Override
   public FixedSizeByteBufferSerializer<DBID> getDBIDSerializerStatic() {
-    return IntegerDBID.staticSerializer;
+    return IntegerDBID.STATIC_SERIALIZER;
   }
 
   @Override
   public Class<? extends DBID> getTypeRestriction() {
     return IntegerDBID.class;
   }
-}
\ No newline at end of file
+
+  @Override
+  public DBIDRef invalid() {
+    return invalid;
+  }
+}
