@@ -24,14 +24,12 @@ package de.lmu.ifi.dbs.elki.database.query.knn;
  */
 
 import java.util.List;
-import java.util.Map;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNHeap;
 
 /**
  * The interface of an actual instance.
@@ -61,17 +59,7 @@ public interface KNNQuery<O, D extends Distance<D>> extends DatabaseQuery {
    * @param k Number of neighbors requested
    * @return neighbors
    */
-  public List<KNNResult<D>> getKNNForBulkDBIDs(ArrayDBIDs ids, int k);
-
-  /**
-   * Bulk query method configured by a map.
-   * 
-   * Warning: this API is not optimal, and might be removed soon (in fact, it is
-   * used in a single place)
-   * 
-   * @param heaps Map of heaps to fill.
-   */
-  public void getKNNForBulkHeaps(Map<DBID, KNNHeap<D>> heaps);
+  public List<? extends KNNResult<D>> getKNNForBulkDBIDs(ArrayDBIDs ids, int k);
 
   /**
    * Get the k nearest neighbors for a particular id.
