@@ -25,20 +25,17 @@ package de.lmu.ifi.dbs.elki.database.query.knn;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.query.LinearScanQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNHeap;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNHeap;
 
 /**
  * Instance of this query for a particular database.
@@ -67,11 +64,10 @@ public class LinearScanKNNQuery<O, D extends Distance<D>> extends AbstractDistan
   private void linearScanBatchKNN(ArrayDBIDs ids, List<KNNHeap<D>> heaps) {
     // The distance is computed on database IDs
     for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-      DBID candidateID = iter.getDBID();
       int index = 0;
-      for (DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
+      for(DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
         KNNHeap<D> heap = heaps.get(index);
-        heap.add(distanceQuery.distance(iter2, candidateID), candidateID);
+        heap.add(distanceQuery.distance(iter2, iter), iter);
         index++;
       }
     }
@@ -79,16 +75,29 @@ public class LinearScanKNNQuery<O, D extends Distance<D>> extends AbstractDistan
 
   @Override
   public KNNResult<D> getKNNForDBID(DBIDRef id, int k) {
-    KNNHeap<D> heap = new KNNHeap<D>(k);
+    KNNHeap<D> heap = KNNUtil.newHeap(distanceQuery.getDistanceFactory(), k);
+    D max = distanceQuery.getDistanceFactory().infiniteDistance();
     if(PrimitiveDistanceQuery.class.isInstance(distanceQuery)) {
       O obj = relation.get(id);
       for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-        heap.add(distanceQuery.distance(obj, relation.get(iter)), iter);
+        final D dist = distanceQuery.distance(obj, relation.get(iter));
+        if(max.compareTo(dist) > 0) {
+          heap.add(dist, iter);
+          if(heap.size() >= k) {
+            max = heap.getKNNDistance();
+          }
+        }
       }
     }
     else {
       for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-        heap.add(distanceQuery.distance(id, iter), iter);
+        final D dist = distanceQuery.distance(id, iter);
+        if(max.compareTo(dist) > 0) {
+          heap.add(dist, iter);
+          if(heap.size() >= k) {
+            max = heap.getKNNDistance();
+          }
+        }
       }
     }
     return heap.toKNNList();
@@ -99,7 +108,7 @@ public class LinearScanKNNQuery<O, D extends Distance<D>> extends AbstractDistan
     final int size = ids.size();
     final List<KNNHeap<D>> heaps = new ArrayList<KNNHeap<D>>(size);
     for(int i = 0; i < size; i++) {
-      heaps.add(new KNNHeap<D>(k));
+      heaps.add(KNNUtil.newHeap(distanceQuery.getDistanceFactory(), k));
     }
     linearScanBatchKNN(ids, heaps);
     // Serialize heaps
@@ -110,21 +119,9 @@ public class LinearScanKNNQuery<O, D extends Distance<D>> extends AbstractDistan
     return result;
   }
 
-  @Override
-  public void getKNNForBulkHeaps(Map<DBID, KNNHeap<D>> heaps) {
-    final int size = heaps.size();
-    ArrayModifiableDBIDs ids = DBIDUtil.newArray(size);
-    List<KNNHeap<D>> kheaps = new ArrayList<KNNHeap<D>>(size);
-    for(Entry<DBID, KNNHeap<D>> ent : heaps.entrySet()) {
-      ids.add(ent.getKey());
-      kheaps.add(ent.getValue());
-    }
-    linearScanBatchKNN(ids, kheaps);
-  }
-
   @Override
   public KNNResult<D> getKNNForObject(O obj, int k) {
-    KNNHeap<D> heap = new KNNHeap<D>(k);
+    KNNHeap<D> heap = KNNUtil.newHeap(distanceQuery.getDistanceFactory(), k);
     for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
       heap.add(distanceQuery.distance(obj, iter), iter);
     }
