@@ -25,16 +25,15 @@ package de.lmu.ifi.dbs.elki.database.query.knn;
 
 import java.util.ArrayList;
 import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.query.distance.PrimitiveDistanceQuery;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNHeap;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNHeap;
 
 /**
  * Instance of this query for a particular database.
@@ -66,12 +65,10 @@ public class LinearScanPrimitiveDistanceKNNQuery<O, D extends Distance<D>> exten
     final int size = objs.size();
     // Linear scan style KNN.
     for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
-      DBID candidateID = iter.getDBID();
-      O candidate = relation.get(candidateID);
+      O candidate = relation.get(iter);
       for(int index = 0; index < size; index++) {
         O object = objs.get(index);
-        KNNHeap<D> heap = heaps.get(index);
-        heap.add(distanceQuery.distance(object, candidate), candidateID);
+        heaps.get(index).add(distanceQuery.distance(object, candidate), iter);
       }
     }
   }
@@ -87,7 +84,7 @@ public class LinearScanPrimitiveDistanceKNNQuery<O, D extends Distance<D>> exten
     final List<KNNHeap<D>> heaps = new ArrayList<KNNHeap<D>>(size);
     List<O> objs = new ArrayList<O>(size);
     for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
-      heaps.add(new KNNHeap<D>(k));
+      heaps.add(KNNUtil.newHeap(distanceQuery.getDistanceFactory(), k));
       objs.add(relation.get(iter));
     }
     linearScanBatchKNN(objs, heaps);
@@ -98,15 +95,4 @@ public class LinearScanPrimitiveDistanceKNNQuery<O, D extends Distance<D>> exten
     }
     return result;
   }
-
-  @Override
-  public void getKNNForBulkHeaps(Map<DBID, KNNHeap<D>> heaps) {
-    List<O> objs = new ArrayList<O>(heaps.size());
-    List<KNNHeap<D>> kheaps = new ArrayList<KNNHeap<D>>(heaps.size());
-    for(Entry<DBID, KNNHeap<D>> ent : heaps.entrySet()) {
-      objs.add(relation.get(ent.getKey()));
-      kheaps.add(ent.getValue());
-    }
-    linearScanBatchKNN(objs, kheaps);
-  }
 }
\ No newline at end of file
