@@ -23,15 +23,12 @@ package de.lmu.ifi.dbs.elki.database.query.range;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.Collections;
-
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
-import de.lmu.ifi.dbs.elki.database.query.DistanceDBIDResult;
-import de.lmu.ifi.dbs.elki.database.query.GenericDistanceDBIDList;
-import de.lmu.ifi.dbs.elki.database.query.GenericDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.LinearScanQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.GenericDistanceDBIDList;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
@@ -61,10 +58,10 @@ public class LinearScanRangeQuery<O, D extends Distance<D>> extends AbstractDist
     for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
       D currentDistance = distanceQuery.distance(id, iter);
       if(currentDistance.compareTo(range) <= 0) {
-        result.add(new GenericDistanceResultPair<D>(currentDistance, iter.getDBID()));
+        result.add(currentDistance, iter);
       }
     }
-    Collections.sort(result);
+    result.sort();
     return result;
   }
 
@@ -74,10 +71,10 @@ public class LinearScanRangeQuery<O, D extends Distance<D>> extends AbstractDist
     for(DBIDIter iter = relation.getDBIDs().iter(); iter.valid(); iter.advance()) {
       D currentDistance = distanceQuery.distance(obj, iter);
       if(currentDistance.compareTo(range) <= 0) {
-        result.add(new GenericDistanceResultPair<D>(currentDistance, iter.getDBID()));
+        result.add(currentDistance, iter);
       }
     }
-    Collections.sort(result);
+    result.sort();
     return result;
   }
 }
\ No newline at end of file
