@@ -23,12 +23,10 @@ package de.lmu.ifi.dbs.elki.database.query.rknn;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.List;
-
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.query.AbstractDataBasedQuery;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
@@ -53,5 +51,5 @@ public abstract class AbstractRKNNQuery<O, D extends Distance<D>> extends Abstra
   }
 
   @Override
-  abstract public List<DistanceResultPair<D>> getRKNNForDBID(DBIDRef id, int k);
+  abstract public DistanceDBIDResult<D> getRKNNForDBID(DBIDRef id, int k);
 }
\ No newline at end of file
