@@ -24,20 +24,19 @@ package de.lmu.ifi.dbs.elki.database.query.rknn;
  */
 
 import java.util.ArrayList;
-import java.util.Collections;
 import java.util.List;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
-import de.lmu.ifi.dbs.elki.database.query.GenericDistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.LinearScanQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.DistanceDBIDResultIter;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.GenericDistanceDBIDList;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 
 /**
@@ -71,8 +70,8 @@ public class LinearScanRKNNQuery<O, D extends Distance<D>> extends AbstractRKNNQ
   }
 
   @Override
-  public List<DistanceResultPair<D>> getRKNNForObject(O obj, int k) {
-    ArrayList<DistanceResultPair<D>> rNNlist = new ArrayList<DistanceResultPair<D>>();
+  public DistanceDBIDResult<D> getRKNNForObject(O obj, int k) {
+    GenericDistanceDBIDList<D> rNNlist = new GenericDistanceDBIDList<D>();
 
     ArrayDBIDs allIDs = DBIDUtil.ensureArray(relation.getDBIDs());
     List<? extends KNNResult<D>> kNNLists = knnQuery.getKNNForBulkDBIDs(allIDs, k);
@@ -83,17 +82,17 @@ public class LinearScanRKNNQuery<O, D extends Distance<D>> extends AbstractRKNNQ
       int last = Math.min(k - 1, knn.size() - 1);
       D dist = distanceQuery.distance(obj, iter);
       if(last < k - 1 || dist.compareTo(knn.get(last).getDistance()) < 1) {
-        rNNlist.add(new GenericDistanceResultPair<D>(dist, iter.getDBID()));
+        rNNlist.add(dist, iter);
       }
       i++;
     }
-    Collections.sort(rNNlist);
+    rNNlist.sort();
     return rNNlist;
   }
 
   @Override
-  public List<DistanceResultPair<D>> getRKNNForDBID(DBIDRef id, int k) {
-    ArrayList<DistanceResultPair<D>> rNNList = new ArrayList<DistanceResultPair<D>>();
+  public DistanceDBIDResult<D> getRKNNForDBID(DBIDRef id, int k) {
+    GenericDistanceDBIDList<D> rNNList = new GenericDistanceDBIDList<D>();
     
     ArrayDBIDs allIDs = DBIDUtil.ensureArray(relation.getDBIDs());
     List<? extends KNNResult<D>> kNNList = knnQuery.getKNNForBulkDBIDs(allIDs, k);
@@ -101,22 +100,22 @@ public class LinearScanRKNNQuery<O, D extends Distance<D>> extends AbstractRKNNQ
     int i = 0;
     for (DBIDIter iter = allIDs.iter(); iter.valid(); iter.advance()) {
       KNNResult<D> knn = kNNList.get(i);
-      for(DistanceResultPair<D> n : knn) {
-        if(n.sameDBID(id)) {
-          rNNList.add(new GenericDistanceResultPair<D>(n.getDistance(), iter.getDBID()));
+      for(DistanceDBIDResultIter<D> n = knn.iter(); n.valid(); n.advance()) {
+        if(DBIDUtil.equal(n, id)) {
+          rNNList.add(n.getDistance(), iter);
         }
       }
       i++;
     }
-    Collections.sort(rNNList);
+    rNNList.sort();
     return rNNList;
   }
 
   @Override
-  public List<List<DistanceResultPair<D>>> getRKNNForBulkDBIDs(ArrayDBIDs ids, int k) {
-    List<List<DistanceResultPair<D>>> rNNList = new ArrayList<List<DistanceResultPair<D>>>(ids.size());
+  public List<GenericDistanceDBIDList<D>> getRKNNForBulkDBIDs(ArrayDBIDs ids, int k) {
+    List<GenericDistanceDBIDList<D>> rNNList = new ArrayList<GenericDistanceDBIDList<D>>(ids.size());
     for(int i = 0; i < ids.size(); i++) {
-      rNNList.add(new ArrayList<DistanceResultPair<D>>());
+      rNNList.add(new GenericDistanceDBIDList<D>());
     }
 
     ArrayDBIDs allIDs = DBIDUtil.ensureArray(relation.getDBIDs());
@@ -124,14 +123,13 @@ public class LinearScanRKNNQuery<O, D extends Distance<D>> extends AbstractRKNNQ
 
     int i = 0;
     for (DBIDIter iter = allIDs.iter(); iter.valid(); iter.advance()) {
-      DBID qid = iter.getDBID();
       KNNResult<D> knn = kNNList.get(i);
-      for(DistanceResultPair<D> n : knn) {
+      for(DistanceDBIDResultIter<D> n = knn.iter(); n.valid(); n.advance()) {
         int j = 0;
         for (DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
-          if(n.getDBID().sameDBID(iter2)) {
-            List<DistanceResultPair<D>> rNN = rNNList.get(j);
-            rNN.add(new GenericDistanceResultPair<D>(n.getDistance(), qid));
+          if(DBIDUtil.equal(n, iter2)) {
+            GenericDistanceDBIDList<D> rNN = rNNList.get(j);
+            rNN.add(n.getDistance(), iter);
           }
           j++;
         }
@@ -139,8 +137,7 @@ public class LinearScanRKNNQuery<O, D extends Distance<D>> extends AbstractRKNNQ
       i++;
     }
     for(int j = 0; j < ids.size(); j++) {
-      List<DistanceResultPair<D>> rNN = rNNList.get(j);
-      Collections.sort(rNN);
+      rNNList.get(j).sort();
     }
     return rNNList;
   }
