@@ -70,7 +70,7 @@ public class DBIDView extends AbstractHierarchicalResult implements Relation<DBI
   @Override
   public DBID get(DBIDRef id) {
     assert (ids.contains(id));
-    return id.getDBID();
+    return DBIDUtil.deref(id);
   }
 
   @Override
@@ -81,8 +81,7 @@ public class DBIDView extends AbstractHierarchicalResult implements Relation<DBI
   @Override
   public void delete(DBIDRef id) {
     if(database instanceof UpdatableDatabase) {
-      // TODO: skip getDBID()
-      ((UpdatableDatabase) database).delete(id.getDBID());
+      ((UpdatableDatabase) database).delete(id);
     }
     else {
       throw new UnsupportedOperationException("Deletions are not supported.");
