@@ -23,10 +23,11 @@ package de.lmu.ifi.dbs.elki.datasource;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import gnu.trove.impl.Constants;
+import gnu.trove.map.hash.TObjectIntHashMap;
+
 import java.util.ArrayList;
-import java.util.HashMap;
 import java.util.List;
-import java.util.Map;
 
 import de.lmu.ifi.dbs.elki.data.ExternalID;
 import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
@@ -51,7 +52,7 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
   /**
    * Logger
    */
-  private static final Logging logger = Logging.getLogger(ExternalIDJoinDatabaseConnection.class);
+  private static final Logging LOG = Logging.getLogger(ExternalIDJoinDatabaseConnection.class);
 
   /**
    * The filters to invoke
@@ -72,60 +73,60 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
   @Override
   public MultipleObjectsBundle loadData() {
     List<MultipleObjectsBundle> bundles = new ArrayList<MultipleObjectsBundle>(sources.size());
-    for(DatabaseConnection dbc : sources) {
+    for (DatabaseConnection dbc : sources) {
       bundles.add(dbc.loadData());
     }
 
     MultipleObjectsBundle first = bundles.get(0);
-    Map<ExternalID, Integer> labelmap = new HashMap<ExternalID, Integer>(first.dataLength());
+    TObjectIntHashMap<ExternalID> labelmap = new TObjectIntHashMap<ExternalID>(first.dataLength(), Constants.DEFAULT_LOAD_FACTOR, -1);
     // Process first bundle
     {
       // Identify a label column
       final int lblcol;
       {
         int lblc = -1;
-        for(int i = 0; i < first.metaLength(); i++) {
-          if(TypeUtil.EXTERNALID.isAssignableFromType(first.meta(i))) {
+        for (int i = 0; i < first.metaLength(); i++) {
+          if (TypeUtil.EXTERNALID.isAssignableFromType(first.meta(i))) {
             lblc = i;
             break;
           }
         }
         lblcol = lblc; // make static
       }
-      if(lblcol == -1) {
+      if (lblcol == -1) {
         throw new AbortException("No external ID column found in primary source.");
       }
-      for(int i = 0; i < first.dataLength(); i++) {
+      for (int i = 0; i < first.dataLength(); i++) {
         ExternalID data = (ExternalID) first.data(i, lblcol);
-        if(data == null) {
-          logger.debug("Object without ID encountered.");
+        if (data == null) {
+          LOG.debug("Object without ID encountered.");
           continue;
         }
-        Integer old = labelmap.put(data, i);
-        if(old != null) {
-          logger.debug("Duplicate id encountered: " + data + " in rows " + old + " and " + i);
+        int old = labelmap.put(data, i);
+        if (old != -1) {
+          LOG.debug("Duplicate id encountered: " + data + " in rows " + old + " and " + i);
         }
       }
     }
     // Process additional columns
-    for(int c = 1; c < sources.size(); c++) {
+    for (int c = 1; c < sources.size(); c++) {
       MultipleObjectsBundle cur = bundles.get(c);
       final int lblcol;
       {
         int lblc = -1;
-        for(int i = 0; i < cur.metaLength(); i++) {
-          if(TypeUtil.EXTERNALID.isAssignableFromType(cur.meta(i))) {
+        for (int i = 0; i < cur.metaLength(); i++) {
+          if (TypeUtil.EXTERNALID.isAssignableFromType(cur.meta(i))) {
             lblc = i;
             break;
           }
         }
         lblcol = lblc; // make static
       }
-      if(lblcol == -1) {
-        StringBuffer buf = new StringBuffer();
-        for(int i = 0; i < cur.metaLength(); i++) {
-          if(buf.length() > 0) {
-            buf.append(",");
+      if (lblcol == -1) {
+        StringBuilder buf = new StringBuilder();
+        for (int i = 0; i < cur.metaLength(); i++) {
+          if (buf.length() > 0) {
+            buf.append(',');
           }
           buf.append(cur.meta(i));
         }
@@ -133,33 +134,33 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
       }
       // Destination columns
       List<ArrayList<Object>> dcol = new ArrayList<ArrayList<Object>>(cur.metaLength());
-      for(int i = 0; i < cur.metaLength(); i++) {
+      for (int i = 0; i < cur.metaLength(); i++) {
         // Skip the label columns
-        if(i == lblcol) {
+        if (i == lblcol) {
           dcol.add(null);
           continue;
         }
         ArrayList<Object> newcol = new ArrayList<Object>(first.dataLength());
         // Pre-fill with nulls.
-        for(int j = 0; j < first.dataLength(); j++) {
+        for (int j = 0; j < first.dataLength(); j++) {
           newcol.add(null);
         }
         first.appendColumn(cur.meta(i), newcol);
         dcol.add(newcol);
       }
-      for(int i = 0; i < cur.dataLength(); i++) {
+      for (int i = 0; i < cur.dataLength(); i++) {
         ExternalID data = (ExternalID) cur.data(i, lblcol);
-        if(data == null) {
-          logger.warning("Object without label encountered.");
+        if (data == null) {
+          LOG.warning("Object without label encountered.");
           continue;
         }
-        Integer row = labelmap.get(data);
-        if(row == null) {
-          logger.debug("ID not found for join: " + data + " in row " + i);
+        int row = labelmap.get(data);
+        if (row == -1) {
+          LOG.debug("ID not found for join: " + data + " in row " + i);
           continue;
         }
-        for(int d = 0; d < cur.metaLength(); d++) {
-          if(d == lblcol) {
+        for (int d = 0; d < cur.metaLength(); d++) {
+          if (d == lblcol) {
             continue;
           }
           List<Object> col = dcol.get(d);
@@ -168,22 +169,21 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
         }
       }
     }
-    for(int i = 0; i < first.dataLength(); i++) {
-      for(int d = 0; d < first.metaLength(); d++) {
-        if(first.data(i, d) == null) {
-          StringBuffer buf = new StringBuffer();
-          for(int d2 = 0; d2 < first.metaLength(); d2++) {
-            if(buf.length() > 0) {
+    for (int i = 0; i < first.dataLength(); i++) {
+      for (int d = 0; d < first.metaLength(); d++) {
+        if (first.data(i, d) == null) {
+          StringBuilder buf = new StringBuilder();
+          for (int d2 = 0; d2 < first.metaLength(); d2++) {
+            if (buf.length() > 0) {
               buf.append(", ");
             }
-            if(first.data(i, d2) == null) {
+            if (first.data(i, d2) == null) {
               buf.append("null");
-            }
-            else {
+            } else {
               buf.append(first.data(i, d2));
             }
           }
-          logger.warning("null value in joined data, row " + i + " column " + d + FormatUtil.NEWLINE + "[" + buf.toString() + "]");
+          LOG.warning("null value in joined data, row " + i + " column " + d + FormatUtil.NEWLINE + "[" + buf.toString() + "]");
           break;
         }
       }
@@ -194,7 +194,7 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -208,7 +208,7 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
     /**
      * The static option ID
      */
-    public static final OptionID SOURCES_ID = OptionID.getOrCreateOptionID("join.sources", "The data sources to join.");
+    public static final OptionID SOURCES_ID = new OptionID("join.sources", "The data sources to join.");
 
     /**
      * The data souces to use.
@@ -220,7 +220,7 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
       super.makeOptions(config);
       super.configFilters(config);
       final ObjectListParameter<DatabaseConnection> sourcesParam = new ObjectListParameter<DatabaseConnection>(SOURCES_ID, DatabaseConnection.class);
-      if(config.grab(sourcesParam)) {
+      if (config.grab(sourcesParam)) {
         sources = sourcesParam.instantiateClasses(config);
       }
     }
@@ -230,4 +230,4 @@ public class ExternalIDJoinDatabaseConnection extends AbstractDatabaseConnection
       return new ExternalIDJoinDatabaseConnection(filters, sources);
     }
   }
-}
\ No newline at end of file
+}
