@@ -23,6 +23,7 @@ package de.lmu.ifi.dbs.elki.datasource;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import java.io.BufferedInputStream;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
@@ -50,7 +51,7 @@ public class FileBasedDatabaseConnection extends InputStreamDatabaseConnection {
    * Key: {@code -dbc.in}
    * </p>
    */
-  public static final OptionID INPUT_ID = OptionID.getOrCreateOptionID("dbc.in", "The name of the input file to be parsed.");
+  public static final OptionID INPUT_ID = new OptionID("dbc.in", "The name of the input file to be parsed.");
 
   /**
    * Constructor.
@@ -80,8 +81,7 @@ public class FileBasedDatabaseConnection extends InputStreamDatabaseConnection {
       final FileParameter inputParam = new FileParameter(INPUT_ID, FileParameter.FileType.INPUT_FILE);
       if(config.grab(inputParam)) {
         try {
-          inputStream = new FileInputStream(inputParam.getValue());
-          inputStream = FileUtil.tryGzipInput(inputStream);
+          inputStream = new BufferedInputStream(FileUtil.tryGzipInput(new FileInputStream(inputParam.getValue())));
         }
         catch(IOException e) {
           config.reportError(new WrongParameterValueException(inputParam, inputParam.getValue().getPath(), e));
