@@ -24,10 +24,8 @@ package de.lmu.ifi.dbs.elki.datasource;
  */
 
 import java.io.File;
-import java.io.FileInputStream;
 import java.io.FileNotFoundException;
 import java.io.IOException;
-import java.io.InputStream;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.List;
@@ -54,6 +52,7 @@ import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.DistributionWithRandom;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.GammaDistribution;
+import de.lmu.ifi.dbs.elki.math.statistics.distribution.HaltonUniformDistribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.NormalDistribution;
 import de.lmu.ifi.dbs.elki.math.statistics.distribution.UniformDistribution;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
@@ -63,7 +62,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.FileParameter;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.RandomParameter;
 import de.lmu.ifi.dbs.elki.utilities.xml.XMLNodeIterator;
 
 /**
@@ -77,10 +76,90 @@ import de.lmu.ifi.dbs.elki.utilities.xml.XMLNodeIterator;
  * @apiviz.composedOf GeneratorMain
  */
 public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
+  /** Dataset tag */
+  public static final String TAG_DATASET = "dataset";
+
+  /** Cluster tag */
+  public static final String TAG_CLUSTER = "cluster";
+
+  /** Uniform distribution */
+  public static final String TAG_UNIFORM = "uniform";
+
+  /** Normal distribution */
+  public static final String TAG_NORMAL = "normal";
+
+  /** Gamma distribution */
+  public static final String TAG_GAMMA = "gamma";
+
+  /**
+   * Halton pseudo uniform distribution.
+   */
+  public static final String TAG_HALTON = "halton";
+
+  /** Rotation */
+  public static final String TAG_ROTATE = "rotate";
+
+  /** Translation */
+  public static final String TAG_TRANSLATE = "translate";
+
+  /** Clipping */
+  public static final String TAG_CLIP = "clip";
+
+  /** Static cluster */
+  public static final String TAG_STATIC = "static";
+
+  /** Point in static cluster */
+  public static final String TAG_POINT = "point";
+
+  /** Attribute to control model testing */
+  public static final String ATTR_TEST = "test-model";
+
+  /** Random seed */
+  public static final String ATTR_SEED = "random-seed";
+
+  /** Density correction factor */
+  public static final String ATTR_DENSITY = "density-correction";
+
+  /** Cluster nane */
+  public static final String ATTR_NAME = "name";
+
+  /** Cluster size */
+  public static final String ATTR_SIZE = "size";
+
+  /** Minimum value */
+  public static final String ATTR_MIN = "min";
+
+  /** Maximum value */
+  public static final String ATTR_MAX = "max";
+
+  /** Mean */
+  public static final String ATTR_MEAN = "mean";
+
+  /** Standard deviation */
+  public static final String ATTR_STDDEV = "stddev";
+
+  /** Gamma k */
+  public static final String ATTR_K = "k";
+
+  /** Gamma theta */
+  public static final String ATTR_THETA = "theta";
+
+  /** Vector */
+  public static final String ATTR_VECTOR = "vector";
+
+  /** First axis for rotation plane */
+  public static final String ATTR_AXIS1 = "axis1";
+
+  /** Second axis for rotation plane */
+  public static final String ATTR_AXIS2 = "axis2";
+
+  /** Rotation angle */
+  public static final String ATTR_ANGLE = "angle";
+
   /**
    * Logger
    */
-  private static Logging logger = Logging.getLogger(GeneratorXMLDatabaseConnection.class);
+  private static final Logging LOG = Logging.getLogger(GeneratorXMLDatabaseConnection.class);
 
   /**
    * A pattern defining whitespace.
@@ -90,22 +169,22 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   /**
    * Parameter to give the configuration file
    */
-  public static final OptionID CONFIGFILE_ID = OptionID.getOrCreateOptionID("bymodel.spec", "The generator specification file.");
+  public static final OptionID CONFIGFILE_ID = new OptionID("bymodel.spec", "The generator specification file.");
 
   /**
    * Parameter to give the configuration file
    */
-  public static final OptionID RANDOMSEED_ID = OptionID.getOrCreateOptionID("bymodel.randomseed", "The random generator seed.");
+  public static final OptionID RANDOMSEED_ID = new OptionID("bymodel.randomseed", "The random generator seed.");
 
   /**
    * Parameter to give the configuration file
    */
-  public static final OptionID SIZE_SCALE_ID = OptionID.getOrCreateOptionID("bymodel.sizescale", "Factor for scaling the specified cluster sizes.");
+  public static final OptionID SIZE_SCALE_ID = new OptionID("bymodel.sizescale", "Factor for scaling the specified cluster sizes.");
 
   /**
    * File name of the generators XML Schema file.
    */
-  private static final String GENERATOR_SCHEMA_FILE = GeneratorXMLSpec.class.getPackage().getName().replace('.', '/') + '/' + "GeneratorByModel.xsd";
+  public static final String GENERATOR_SCHEMA_FILE = GeneratorXMLSpec.class.getPackage().getName().replace('.', '/') + '/' + "GeneratorByModel.xsd";
 
   /**
    * The configuration file.
@@ -139,33 +218,31 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     this.specfile = specfile;
     this.sizescale = sizescale;
     this.clusterRandom = clusterRandom;
-    if(this.clusterRandom == null) {
+    if (this.clusterRandom == null) {
       this.clusterRandom = new Random();
     }
   }
 
   @Override
   public MultipleObjectsBundle loadData() {
-    if(logger.isVerbose()) {
-      logger.verbose("Loading specification ...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Loading specification ...");
     }
     GeneratorMain gen;
     try {
       gen = loadXMLSpecification();
-    }
-    catch(UnableToComplyException e) {
+    } catch (UnableToComplyException e) {
       throw new AbortException("Cannot load XML specification", e);
     }
-    if(logger.isVerbose()) {
-      logger.verbose("Generating clusters ...");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Generating clusters ...");
     }
-    if(testAgainstModel != null) {
-      gen.setTestAgainstModel(testAgainstModel);
+    if (testAgainstModel != null) {
+      gen.setTestAgainstModel(testAgainstModel.booleanValue());
     }
     try {
       return gen.generate();
-    }
-    catch(UnableToComplyException e) {
+    } catch (UnableToComplyException e) {
       throw new AbortException("Data generation failed. ", e);
     }
   }
@@ -179,44 +256,36 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    */
   private GeneratorMain loadXMLSpecification() throws UnableToComplyException {
     try {
-      InputStream in = new FileInputStream(specfile);
       DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
       dbf.setFeature("http://apache.org/xml/features/nonvalidating/load-external-dtd", false);
       URL url = ClassLoader.getSystemResource(GENERATOR_SCHEMA_FILE);
-      if(url != null) {
+      if (url != null) {
         try {
           Schema schema = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI).newSchema(url);
           dbf.setSchema(schema);
           dbf.setIgnoringElementContentWhitespace(true);
+        } catch (Exception e) {
+          LOG.warning("Could not set up XML Schema validation for speciciation file.", e);
         }
-        catch(Exception e) {
-          logger.warning("Could not set up XML Schema validation for speciciation file.");
-        }
+      } else {
+        LOG.warning("Could not set up XML Schema validation for speciciation file.");
       }
-      else {
-        logger.warning("Could not set up XML Schema validation for speciciation file.");
-      }
-      Document doc = dbf.newDocumentBuilder().parse(in);
+      Document doc = dbf.newDocumentBuilder().parse(specfile);
       Node root = doc.getDocumentElement();
-      if(root.getNodeName() == "dataset") {
+      if (TAG_DATASET.equals(root.getNodeName())) {
         GeneratorMain gen = new GeneratorMain();
         processElementDataset(gen, root);
         return gen;
-      }
-      else {
+      } else {
         throw new UnableToComplyException("Experiment specification has incorrect document element: " + root.getNodeName());
       }
-    }
-    catch(FileNotFoundException e) {
+    } catch (FileNotFoundException e) {
       throw new UnableToComplyException("Can't open specification file.", e);
-    }
-    catch(SAXException e) {
+    } catch (SAXException e) {
       throw new UnableToComplyException("Error parsing specification file.", e);
-    }
-    catch(IOException e) {
+    } catch (IOException e) {
       throw new UnableToComplyException("IO Exception loading specification file.", e);
-    }
-    catch(ParserConfigurationException e) {
+    } catch (ParserConfigurationException e) {
       throw new UnableToComplyException("Parser Configuration Error", e);
     }
   }
@@ -230,26 +299,24 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    */
   private void processElementDataset(GeneratorMain gen, Node cur) throws UnableToComplyException {
     // *** get parameters
-    String seedstr = ((Element) cur).getAttribute("random-seed");
-    if(seedstr != null && seedstr != "") {
-      clusterRandom = new Random((int) (Integer.valueOf(seedstr) * sizescale));
+    String seedstr = ((Element) cur).getAttribute(ATTR_SEED);
+    if (seedstr != null && seedstr.length() > 0) {
+      clusterRandom = new Random((int) (Integer.parseInt(seedstr) * sizescale));
     }
-    String testmod = ((Element) cur).getAttribute("test-model");
-    if(testmod != null && testmod != "") {
-      testAgainstModel = (Integer.valueOf(testmod) != 0);
+    String testmod = ((Element) cur).getAttribute(ATTR_TEST);
+    if (testmod != null && testmod.length() > 0) {
+      testAgainstModel = Boolean.valueOf(Integer.parseInt(testmod) != 0);
     }
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeName() == "cluster") {
+      if (TAG_CLUSTER.equals(child.getNodeName())) {
         processElementCluster(gen, child);
-      }
-      else if(child.getNodeName() == "static") {
+      } else if (TAG_STATIC.equals(child.getNodeName())) {
         processElementStatic(gen, child);
-      }
-      else if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      } else if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -265,22 +332,22 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     int size = -1;
     double overweight = 1.0;
 
-    String sizestr = ((Element) cur).getAttribute("size");
-    if(sizestr != null && sizestr != "") {
-      size = (int) (Integer.valueOf(sizestr) * sizescale);
+    String sizestr = ((Element) cur).getAttribute(ATTR_SIZE);
+    if (sizestr != null && sizestr.length() > 0) {
+      size = (int) (Integer.parseInt(sizestr) * sizescale);
     }
 
-    String name = ((Element) cur).getAttribute("name");
+    String name = ((Element) cur).getAttribute(ATTR_NAME);
 
-    String dcostr = ((Element) cur).getAttribute("density-correction");
-    if(dcostr != null && dcostr != "") {
+    String dcostr = ((Element) cur).getAttribute(ATTR_DENSITY);
+    if (dcostr != null && dcostr.length() > 0) {
       overweight = Double.parseDouble(dcostr);
     }
 
-    if(size < 0) {
+    if (size < 0) {
       throw new UnableToComplyException("No valid cluster size given in specification file.");
     }
-    if(name == null || name == "") {
+    if (name == null || name.length() == 0) {
       throw new UnableToComplyException("No cluster name given in specification file.");
     }
 
@@ -290,28 +357,24 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeName() == "uniform") {
+      if (TAG_UNIFORM.equals(child.getNodeName())) {
         processElementUniform(cluster, child);
-      }
-      else if(child.getNodeName() == "normal") {
+      } else if (TAG_NORMAL.equals(child.getNodeName())) {
         processElementNormal(cluster, child);
-      }
-      else if(child.getNodeName() == "gamma") {
+      } else if (TAG_GAMMA.equals(child.getNodeName())) {
         processElementGamma(cluster, child);
-      }
-      else if(child.getNodeName() == "rotate") {
+      } else if (TAG_HALTON.equals(child.getNodeName())) {
+        processElementHalton(cluster, child);
+      } else if (TAG_ROTATE.equals(child.getNodeName())) {
         processElementRotate(cluster, child);
-      }
-      else if(child.getNodeName() == "translate") {
+      } else if (TAG_TRANSLATE.equals(child.getNodeName())) {
         processElementTranslate(cluster, child);
-      }
-      else if(child.getNodeName() == "clip") {
+      } else if (TAG_CLIP.equals(child.getNodeName())) {
         processElementClipping(cluster, child);
-      }
-      else if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      } else if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
 
@@ -329,12 +392,12 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     double min = 0.0;
     double max = 1.0;
 
-    String minstr = ((Element) cur).getAttribute("min");
-    if(minstr != null && minstr != "") {
+    String minstr = ((Element) cur).getAttribute(ATTR_MIN);
+    if (minstr != null && minstr.length() > 0) {
       min = Double.parseDouble(minstr);
     }
-    String maxstr = ((Element) cur).getAttribute("max");
-    if(maxstr != null && maxstr != "") {
+    String maxstr = ((Element) cur).getAttribute(ATTR_MAX);
+    if (maxstr != null && maxstr.length() > 0) {
       max = Double.parseDouble(maxstr);
     }
 
@@ -345,10 +408,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -363,12 +426,12 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   private void processElementNormal(GeneratorSingleCluster cluster, Node cur) throws UnableToComplyException {
     double mean = 0.0;
     double stddev = 1.0;
-    String meanstr = ((Element) cur).getAttribute("mean");
-    if(meanstr != null && meanstr != "") {
+    String meanstr = ((Element) cur).getAttribute(ATTR_MEAN);
+    if (meanstr != null && meanstr.length() > 0) {
       mean = Double.parseDouble(meanstr);
     }
-    String stddevstr = ((Element) cur).getAttribute("stddev");
-    if(stddevstr != null && stddevstr != "") {
+    String stddevstr = ((Element) cur).getAttribute(ATTR_STDDEV);
+    if (stddevstr != null && stddevstr.length() > 0) {
       stddev = Double.parseDouble(stddevstr);
     }
 
@@ -379,10 +442,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -397,12 +460,12 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   private void processElementGamma(GeneratorSingleCluster cluster, Node cur) throws UnableToComplyException {
     double k = 1.0;
     double theta = 1.0;
-    String kstr = ((Element) cur).getAttribute("k");
-    if(kstr != null && kstr != "") {
+    String kstr = ((Element) cur).getAttribute(ATTR_K);
+    if (kstr != null && kstr.length() > 0) {
       k = Double.parseDouble(kstr);
     }
-    String thetastr = ((Element) cur).getAttribute("theta");
-    if(thetastr != null && thetastr != "") {
+    String thetastr = ((Element) cur).getAttribute(ATTR_THETA);
+    if (thetastr != null && thetastr.length() > 0) {
       theta = Double.parseDouble(thetastr);
     }
 
@@ -413,10 +476,45 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
+      }
+    }
+  }
+
+  /**
+   * Process a 'halton' Element in the XML stream.
+   * 
+   * @param cluster
+   * @param cur Current document nod
+   * @throws UnableToComplyException
+   */
+  private void processElementHalton(GeneratorSingleCluster cluster, Node cur) throws UnableToComplyException {
+    double min = 0.0;
+    double max = 1.0;
+
+    String minstr = ((Element) cur).getAttribute(ATTR_MIN);
+    if (minstr != null && minstr.length() > 0) {
+      min = Double.parseDouble(minstr);
+    }
+    String maxstr = ((Element) cur).getAttribute(ATTR_MAX);
+    if (maxstr != null && maxstr.length() > 0) {
+      max = Double.parseDouble(maxstr);
+    }
+
+    // *** new uniform generator
+    Random random = cluster.getNewRandomGenerator();
+    DistributionWithRandom generator = new HaltonUniformDistribution(min, max, random);
+    cluster.addGenerator(generator);
+
+    // TODO: check for unknown attributes.
+    XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
+    while (iter.hasNext()) {
+      Node child = iter.next();
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -433,25 +531,25 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     int axis2 = 0;
     double angle = 0.0;
 
-    String a1str = ((Element) cur).getAttribute("axis1");
-    if(a1str != null && a1str != "") {
-      axis1 = Integer.valueOf(a1str);
+    String a1str = ((Element) cur).getAttribute(ATTR_AXIS1);
+    if (a1str != null && a1str.length() > 0) {
+      axis1 = Integer.parseInt(a1str);
     }
-    String a2str = ((Element) cur).getAttribute("axis2");
-    if(a2str != null && a2str != "") {
-      axis2 = Integer.valueOf(a2str);
+    String a2str = ((Element) cur).getAttribute(ATTR_AXIS2);
+    if (a2str != null && a2str.length() > 0) {
+      axis2 = Integer.parseInt(a2str);
     }
-    String anstr = ((Element) cur).getAttribute("angle");
-    if(anstr != null && anstr != "") {
+    String anstr = ((Element) cur).getAttribute(ATTR_ANGLE);
+    if (anstr != null && anstr.length() > 0) {
       angle = Double.parseDouble(anstr);
     }
-    if(axis1 <= 0 || axis1 > cluster.getDim()) {
+    if (axis1 <= 0 || axis1 > cluster.getDim()) {
       throw new UnableToComplyException("Invalid axis1 number given in specification file.");
     }
-    if(axis1 <= 0 || axis1 > cluster.getDim()) {
+    if (axis1 <= 0 || axis1 > cluster.getDim()) {
       throw new UnableToComplyException("Invalid axis2 number given in specification file.");
     }
-    if(axis1 == axis2) {
+    if (axis1 == axis2) {
       throw new UnableToComplyException("Invalid axis numbers given in specification file.");
     }
 
@@ -460,10 +558,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -477,11 +575,11 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    */
   private void processElementTranslate(GeneratorSingleCluster cluster, Node cur) throws UnableToComplyException {
     Vector offset = null;
-    String vstr = ((Element) cur).getAttribute("vector");
-    if(vstr != null && vstr != "") {
+    String vstr = ((Element) cur).getAttribute(ATTR_VECTOR);
+    if (vstr != null && vstr.length() > 0) {
       offset = parseVector(vstr);
     }
-    if(offset == null) {
+    if (offset == null) {
       throw new UnableToComplyException("No translation vector given.");
     }
 
@@ -490,10 +588,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -509,15 +607,15 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
     Vector cmin = null;
     Vector cmax = null;
 
-    String minstr = ((Element) cur).getAttribute("min");
-    if(minstr != null && minstr != "") {
+    String minstr = ((Element) cur).getAttribute(ATTR_MIN);
+    if (minstr != null && minstr.length() > 0) {
       cmin = parseVector(minstr);
     }
-    String maxstr = ((Element) cur).getAttribute("max");
-    if(maxstr != null && maxstr != "") {
+    String maxstr = ((Element) cur).getAttribute(ATTR_MAX);
+    if (maxstr != null && maxstr.length() > 0) {
       cmax = parseVector(maxstr);
     }
-    if(cmin == null || cmax == null) {
+    if (cmin == null || cmax == null) {
       throw new UnableToComplyException("No or incomplete clipping vectors given.");
     }
 
@@ -526,10 +624,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -542,29 +640,28 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    * @throws UnableToComplyException
    */
   private void processElementStatic(GeneratorMain gen, Node cur) throws UnableToComplyException {
-    String name = ((Element) cur).getAttribute("name");
-    if(name == null) {
+    String name = ((Element) cur).getAttribute(ATTR_NAME);
+    if (name == null) {
       throw new UnableToComplyException("No cluster name given in specification file.");
     }
 
     ArrayList<Vector> points = new ArrayList<Vector>();
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeName() == "point") {
+      if (TAG_POINT.equals(child.getNodeName())) {
         processElementPoint(points, child);
-      }
-      else if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      } else if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
     // *** add new cluster object
     GeneratorStatic cluster = new GeneratorStatic(name, points);
 
     gen.addCluster(cluster);
-    if(logger.isVerbose()) {
-      logger.verbose("Loaded cluster " + cluster.name + " from specification.");
+    if (LOG.isVerbose()) {
+      LOG.verbose("Loaded cluster " + cluster.name + " from specification.");
     }
   }
 
@@ -577,11 +674,11 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
    */
   private void processElementPoint(List<Vector> points, Node cur) throws UnableToComplyException {
     Vector point = null;
-    String vstr = ((Element) cur).getAttribute("vector");
-    if(vstr != null && vstr != "") {
+    String vstr = ((Element) cur).getAttribute(ATTR_VECTOR);
+    if (vstr != null && vstr.length() > 0) {
       point = parseVector(vstr);
     }
-    if(point == null) {
+    if (point == null) {
       throw new UnableToComplyException("No translation vector given.");
     }
 
@@ -590,10 +687,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
 
     // TODO: check for unknown attributes.
     XMLNodeIterator iter = new XMLNodeIterator(cur.getFirstChild());
-    while(iter.hasNext()) {
+    while (iter.hasNext()) {
       Node child = iter.next();
-      if(child.getNodeType() == Node.ELEMENT_NODE) {
-        logger.warning("Unknown element in XML specification file: " + child.getNodeName());
+      if (child.getNodeType() == Node.ELEMENT_NODE) {
+        LOG.warning("Unknown element in XML specification file: " + child.getNodeName());
       }
     }
   }
@@ -610,11 +707,10 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
   private Vector parseVector(String s) throws UnableToComplyException {
     String[] entries = WHITESPACE_PATTERN.split(s);
     double[] d = new double[entries.length];
-    for(int i = 0; i < entries.length; i++) {
+    for (int i = 0; i < entries.length; i++) {
       try {
         d[i] = Double.parseDouble(entries[i]);
-      }
-      catch(NumberFormatException e) {
+      } catch (NumberFormatException e) {
         throw new UnableToComplyException("Could not parse vector.");
       }
     }
@@ -649,18 +745,19 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       super.makeOptions(config);
       // Specification file
       final FileParameter cfgparam = new FileParameter(CONFIGFILE_ID, FileParameter.FileType.INPUT_FILE);
-      if(config.grab(cfgparam)) {
+      if (config.grab(cfgparam)) {
         specfile = cfgparam.getValue();
       }
       // Cluster size scaling
-      final DoubleParameter scalepar = new DoubleParameter(SIZE_SCALE_ID, 1.0);
-      if(config.grab(scalepar)) {
-        sizescale = scalepar.getValue();
+      final DoubleParameter scalepar = new DoubleParameter(SIZE_SCALE_ID, Double.valueOf(1.0));
+      if (config.grab(scalepar)) {
+        sizescale = scalepar.getValue().doubleValue();
       }
       // Random generator
-      final IntParameter seedpar = new IntParameter(RANDOMSEED_ID, true);
-      if(config.grab(seedpar)) {
-        clusterRandom = new Random(seedpar.getValue());
+      final RandomParameter rndP = new RandomParameter(RANDOMSEED_ID);
+      if (config.grab(rndP)) {
+        // TODO: use RandomFactory in cluster
+        clusterRandom = rndP.getValue().getRandom();
       }
     }
 
@@ -669,4 +766,4 @@ public class GeneratorXMLDatabaseConnection implements DatabaseConnection {
       return new GeneratorXMLDatabaseConnection(specfile, sizescale, clusterRandom);
     }
   }
-}
\ No newline at end of file
+}
