@@ -30,7 +30,6 @@ import de.lmu.ifi.dbs.elki.data.ClassLabel;
 import de.lmu.ifi.dbs.elki.data.LabelList;
 import de.lmu.ifi.dbs.elki.data.SimpleClassLabel;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
-import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
@@ -56,7 +55,7 @@ public class ClassLabelFilter implements ObjectFilter {
    * Key: {@code -dbc.classLabelIndex}
    * </p>
    */
-  public static final OptionID CLASS_LABEL_INDEX_ID = OptionID.getOrCreateOptionID("dbc.classLabelIndex", "The index of the label to be used as class label.");
+  public static final OptionID CLASS_LABEL_INDEX_ID = new OptionID("dbc.classLabelIndex", "The index of the label to be used as class label.");
 
   /**
    * Parameter to specify the class of occurring class labels.
@@ -64,7 +63,7 @@ public class ClassLabelFilter implements ObjectFilter {
    * Key: {@code -dbc.classLabelClass}
    * </p>
    */
-  public static final OptionID CLASS_LABEL_CLASS_ID = OptionID.getOrCreateOptionID("dbc.classLabelClass", "Class label class to use.");
+  public static final OptionID CLASS_LABEL_CLASS_ID = new OptionID("dbc.classLabelClass", "Class label class to use.");
 
   /**
    * The index of the label to be used as class label, null if no class label is
@@ -95,10 +94,10 @@ public class ClassLabelFilter implements ObjectFilter {
     // Find a labellist column
     boolean done = false;
     boolean keeplabelcol = false;
-    for(int i = 0; i < objects.metaLength(); i++) {
+    for (int i = 0; i < objects.metaLength(); i++) {
       SimpleTypeInformation<?> meta = objects.meta(i);
       // Skip non-labellist columns - or if we already had a labellist
-      if(done || meta.getRestrictionClass() != LabelList.class) {
+      if (done || !LabelList.class.equals(meta.getRestrictionClass())) {
         bundle.appendColumn(meta, objects.getColumn(i));
         continue;
       }
@@ -109,29 +108,27 @@ public class ClassLabelFilter implements ObjectFilter {
       List<LabelList> lblcol = new ArrayList<LabelList>(objects.dataLength());
 
       // Split the column
-      for(Object obj : objects.getColumn(i)) {
-        if(obj != null) {
+      for (Object obj : objects.getColumn(i)) {
+        if (obj != null) {
           LabelList ll = (LabelList) obj;
           try {
             ClassLabel lbl = classLabelFactory.makeFromString(ll.remove(classLabelIndex));
             clscol.add(lbl);
-          }
-          catch(Exception e) {
-            throw new AbortException("Cannot initialize class labels: "+e.getMessage(), e);
+          } catch (Exception e) {
+            throw new AbortException("Cannot initialize class labels: " + e.getMessage(), e);
           }
           lblcol.add(ll);
-          if(ll.size() > 0) {
+          if (ll.size() > 0) {
             keeplabelcol = true;
           }
-        }
-        else {
+        } else {
           clscol.add(null);
           lblcol.add(null);
         }
       }
-      bundle.appendColumn(TypeUtil.CLASSLABEL, clscol);
+      bundle.appendColumn(classLabelFactory.getTypeInformation(), clscol);
       // Only add the label column when it's not empty.
-      if(keeplabelcol) {
+      if (keeplabelcol) {
         bundle.appendColumn(meta, lblcol);
       }
     }
@@ -150,7 +147,7 @@ public class ClassLabelFilter implements ObjectFilter {
      * The index of the label to be used as class label, null if no class label
      * is specified.
      */
-    protected Integer classLabelIndex;
+    protected int classLabelIndex;
 
     /**
      * The class label factory to use.
@@ -161,13 +158,14 @@ public class ClassLabelFilter implements ObjectFilter {
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       // parameter class label index
-      final IntParameter classLabelIndexParam = new IntParameter(CLASS_LABEL_INDEX_ID, new GreaterEqualConstraint(0));
+      final IntParameter classLabelIndexParam = new IntParameter(CLASS_LABEL_INDEX_ID);
+      classLabelIndexParam.addConstraint(new GreaterEqualConstraint(0));
       final ObjectParameter<ClassLabel.Factory<?>> classlabelClassParam = new ObjectParameter<ClassLabel.Factory<?>>(CLASS_LABEL_CLASS_ID, ClassLabel.Factory.class, SimpleClassLabel.Factory.class);
 
       config.grab(classLabelIndexParam);
       config.grab(classlabelClassParam);
-      if(classLabelIndexParam.isDefined() && classlabelClassParam.isDefined()) {
-        classLabelIndex = classLabelIndexParam.getValue();
+      if (classLabelIndexParam.isDefined() && classlabelClassParam.isDefined()) {
+        classLabelIndex = classLabelIndexParam.intValue();
         classLabelFactory = classlabelClassParam.instantiateClass(config);
       }
     }
@@ -177,4 +175,4 @@ public class ClassLabelFilter implements ObjectFilter {
       return new ClassLabelFilter(classLabelIndex, classLabelFactory);
     }
   }
-}
\ No newline at end of file
+}
