@@ -27,6 +27,7 @@ import java.util.ArrayList;
 import java.util.List;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
+import de.lmu.ifi.dbs.elki.data.NumberVector.Factory;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.TypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
@@ -34,7 +35,8 @@ import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListGreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.ListEachConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
 
@@ -44,8 +46,10 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntListParameter;
  * @author Erich Schubert
  * 
  * @apiviz.uses NumberVector
+ * 
+ * @param <V> Vector type
  */
-public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements ObjectFilter {
+public class SplitNumberVectorFilter<V extends NumberVector<?>> implements ObjectFilter {
   /**
    * Selected dimensions.
    */
@@ -63,46 +67,47 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
 
   @Override
   public MultipleObjectsBundle filter(MultipleObjectsBundle objects) {
-    if(objects.dataLength() == 0) {
+    if (objects.dataLength() == 0) {
       return objects;
     }
     MultipleObjectsBundle bundle = new MultipleObjectsBundle();
 
-    for(int r = 0; r < objects.metaLength(); r++) {
+    for (int r = 0; r < objects.metaLength(); r++) {
       @SuppressWarnings("unchecked")
       SimpleTypeInformation<Object> type = (SimpleTypeInformation<Object>) objects.meta(r);
       @SuppressWarnings("unchecked")
       final List<Object> column = (List<Object>) objects.getColumn(r);
-      if(!getInputTypeRestriction().isAssignableFromType(type)) {
+      if (!getInputTypeRestriction().isAssignableFromType(type)) {
         bundle.appendColumn(type, column);
         continue;
       }
       // Should be a vector type after above test.
       @SuppressWarnings("unchecked")
       final VectorFieldTypeInformation<V> vtype = VectorFieldTypeInformation.class.cast(type);
+      Factory<V, ?> factory = FilterUtil.guessFactory(vtype);
 
       // Get the replacement type informations
-      VectorFieldTypeInformation<V> type1 = new VectorFieldTypeInformation<V>(type.getRestrictionClass(), type.getSerializer(), dims.length, dims.length);
-      VectorFieldTypeInformation<V> type2 = new VectorFieldTypeInformation<V>(type.getRestrictionClass(), type.getSerializer(), vtype.dimensionality() - dims.length, vtype.dimensionality() - dims.length);
+      VectorFieldTypeInformation<V> type1 = new VectorFieldTypeInformation<V>(factory, dims.length);
+      VectorFieldTypeInformation<V> type2 = new VectorFieldTypeInformation<V>(factory, vtype.getDimensionality() - dims.length);
       final List<V> col1 = new ArrayList<V>(column.size());
       final List<V> col2 = new ArrayList<V>(column.size());
       bundle.appendColumn(type1, col1);
       bundle.appendColumn(type2, col2);
 
       // Build other dimensions array.
-      int[] odims = new int[vtype.dimensionality() - dims.length];
+      int[] odims = new int[vtype.getDimensionality() - dims.length];
       {
         int i = 0;
-        for(int d = 1; d <= vtype.dimensionality(); d++) {
+        for (int d = 0; d < vtype.getDimensionality(); d++) {
           boolean found = false;
-          for(int j = 0; j < dims.length; j++) {
-            if(dims[j] == d) {
+          for (int j = 0; j < dims.length; j++) {
+            if (dims[j] == d) {
               found = true;
               break;
             }
           }
-          if(!found) {
-            if(i >= odims.length) {
+          if (!found) {
+            if (i >= odims.length) {
               throw new AbortException("Dimensionalities not proper!");
             }
             odims[i] = d;
@@ -110,20 +115,20 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
           }
         }
       }
-      // Normalization scan
-      for(int i = 0; i < objects.dataLength(); i++) {
+      // Splitting scan.
+      for (int i = 0; i < objects.dataLength(); i++) {
         @SuppressWarnings("unchecked")
         final V obj = (V) column.get(i);
         double[] part1 = new double[dims.length];
         double[] part2 = new double[obj.getDimensionality() - dims.length];
-        for(int d = 0; d < dims.length; d++) {
+        for (int d = 0; d < dims.length; d++) {
           part1[d] = obj.doubleValue(dims[d]);
         }
-        for(int d = 0; d < odims.length; d++) {
+        for (int d = 0; d < odims.length; d++) {
           part2[d] = obj.doubleValue(odims[d]);
         }
-        col1.add(obj.newNumberVector(part1));
-        col2.add(obj.newNumberVector(part2));
+        col1.add(factory.newNumberVector(part1));
+        col2.add(factory.newNumberVector(part2));
       }
     }
     return bundle;
@@ -137,10 +142,10 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
   private TypeInformation getInputTypeRestriction() {
     // Find maximum dimension requested
     int m = dims[0];
-    for(int i = 1; i < dims.length; i++) {
+    for (int i = 1; i < dims.length; i++) {
       m = Math.max(dims[i], m);
     }
-    return new VectorFieldTypeInformation<NumberVector<?, ?>>(NumberVector.class, m, Integer.MAX_VALUE);
+    return new VectorFieldTypeInformation<NumberVector<?>>(NumberVector.class, m, Integer.MAX_VALUE);
   }
 
   /**
@@ -150,11 +155,11 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParameterizer {
     /**
      * The parameter listing the split dimensions.
      */
-    public static final OptionID SELECTED_ATTRIBUTES_ID = OptionID.getOrCreateOptionID("split.dims", "Dimensions to split into the first relation.");
+    public static final OptionID SELECTED_ATTRIBUTES_ID = new OptionID("split.dims", "Dimensions to split into the first relation.");
 
     /**
      * Dimensions to use.
@@ -164,12 +169,13 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      IntListParameter selectedAttributesP = new IntListParameter(SELECTED_ATTRIBUTES_ID, new ListGreaterEqualConstraint<Integer>(1));
-      if(config.grab(selectedAttributesP)) {
+      IntListParameter selectedAttributesP = new IntListParameter(SELECTED_ATTRIBUTES_ID);
+      selectedAttributesP.addConstraint(new ListEachConstraint<Integer>(new GreaterEqualConstraint(0)));
+      if (config.grab(selectedAttributesP)) {
         List<Integer> dimensionList = selectedAttributesP.getValue();
         dims = new int[dimensionList.size()];
-        for(int i = 0; i < dimensionList.size(); i++) {
-          dims[i] = dimensionList.get(i);
+        for (int i = 0; i < dimensionList.size(); i++) {
+          dims[i] = dimensionList.get(i).intValue();
         }
       }
     }
@@ -179,4 +185,4 @@ public class SplitNumberVectorFilter<V extends NumberVector<V, ?>> implements Ob
       return new SplitNumberVectorFilter<V>(dims);
     }
   }
-}
\ No newline at end of file
+}
