@@ -65,13 +65,13 @@ public abstract class AbstractParser {
    * OptionID for the column separator parameter (defaults to whitespace as in
    * {@link #DEFAULT_SEPARATOR}.
    */
-  public static final OptionID COLUMN_SEPARATOR_ID = OptionID.getOrCreateOptionID("parser.colsep", "Column separator pattern. The default assumes whitespace separated data.");
+  public static final OptionID COLUMN_SEPARATOR_ID = new OptionID("parser.colsep", "Column separator pattern. The default assumes whitespace separated data.");
 
   /**
    * OptionID for the quote character parameter (defaults to a double quotation
    * mark as in {@link #QUOTE_CHAR}.
    */
-  public static final OptionID QUOTE_ID = OptionID.getOrCreateOptionID("parser.quote", "Quotation character. The default is to use a double quote.");
+  public static final OptionID QUOTE_ID = new OptionID("parser.quote", "Quotation character. The default is to use a double quote.");
 
   /**
    * Stores the column separator pattern
@@ -196,7 +196,7 @@ public abstract class AbstractParser {
    * 
    * @apiviz.exclude
    */
-  public static abstract class Parameterizer extends AbstractParameterizer {
+  public abstract static class Parameterizer extends AbstractParameterizer {
     /**
      * Stores the column separator pattern
      */
@@ -214,7 +214,8 @@ public abstract class AbstractParser {
       if(config.grab(colParam)) {
         colSep = colParam.getValue();
       }
-      StringParameter quoteParam = new StringParameter(QUOTE_ID, new StringLengthConstraint(1, 1), ""+QUOTE_CHAR);
+      StringParameter quoteParam = new StringParameter(QUOTE_ID, String.valueOf(QUOTE_CHAR));
+      quoteParam.addConstraint(new StringLengthConstraint(1, 1));
       if(config.grab(quoteParam)) {
         quoteChar = quoteParam.getValue().charAt(0);
       }
