@@ -66,22 +66,22 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.PatternParameter;
  */
 public class ArffParser implements Parser {
   /**
-   * Logger
+   * Logger.
    */
-  private static final Logging logger = Logging.getLogger(ArffParser.class);
+  private static final Logging LOG = Logging.getLogger(ArffParser.class);
 
   /**
-   * Arff file marker
+   * Arff file marker.
    */
   public static final Pattern ARFF_HEADER_RELATION = Pattern.compile("@relation\\s+(.*)", Pattern.CASE_INSENSITIVE);
 
   /**
-   * Arff attribute declaration marker
+   * Arff attribute declaration marker.
    */
   public static final Pattern ARFF_HEADER_ATTRIBUTE = Pattern.compile("@attribute\\s+([^ ]+|['\"].*?['\"])\\s+(numeric|real|integer|string|double|date(\\s.*)|\\{.*\\})\\s*", Pattern.CASE_INSENSITIVE);
 
   /**
-   * Arff data marker
+   * Arff data marker.
    */
   public static final Pattern ARFF_HEADER_DATA = Pattern.compile("@data\\s*", Pattern.CASE_INSENSITIVE);
 
@@ -101,7 +101,7 @@ public class ArffParser implements Parser {
   public static final String DEFAULT_ARFF_MAGIC_CLASS = "(Class|Class-?Label)";
 
   /**
-   * Pattern for numeric columns
+   * Pattern for numeric columns.
    */
   public static final Pattern ARFF_NUMERIC = Pattern.compile("(numeric|real|integer|double)", Pattern.CASE_INSENSITIVE);
 
@@ -111,12 +111,12 @@ public class ArffParser implements Parser {
   public static final Pattern EMPTY = Pattern.compile("^\\s*$");
 
   /**
-   * Pattern to recognize external ids
+   * Pattern to recognize external ids.
    */
   Pattern magic_eid;
 
   /**
-   * Pattern to recognize class label columns
+   * Pattern to recognize class label columns.
    */
   Pattern magic_class;
 
@@ -225,7 +225,7 @@ public class ArffParser implements Parser {
         }
         nextToken(tokenizer);
         if(tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
-          map.put(dim, tokenizer.nval);
+          map.put(dim, Double.valueOf(tokenizer.nval));
         }
         else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
           map.put(dim, tokenizer.sval);
@@ -246,7 +246,7 @@ public class ArffParser implements Parser {
         }
       }
       assert (s >= 0);
-      if(elkitypes[out] == TypeUtil.NUMBER_VECTOR_FIELD) {
+      if(TypeUtil.NUMBER_VECTOR_FIELD.equals(elkitypes[out])) {
         TIntFloatHashMap f = new TIntFloatHashMap(dimsize[out]);
         for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
           iter.advance();
@@ -257,12 +257,12 @@ public class ArffParser implements Parser {
           if(i >= s + dimsize[out]) {
             break;
           }
-          double v = (Double) iter.value();
+          double v = ((Double) iter.value()).doubleValue();
           f.put(i - s + 1, (float) v);
         }
         data[out] = new SparseFloatVector(f, dimsize[out]);
       }
-      else if(elkitypes[out] == TypeUtil.LABELLIST) {
+      else if(TypeUtil.LABELLIST.equals(elkitypes[out])) {
         // Build a label list out of successive labels
         LabelList ll = new LabelList(1);
         for(TIntObjectIterator<Object> iter = map.iterator(); iter.hasNext();) {
@@ -276,13 +276,13 @@ public class ArffParser implements Parser {
           }
           String v = (String) iter.value();
           if(ll.size() < i - s) {
-            logger.warning("Sparse consecutive labels are currently not correctly supported.");
+            LOG.warning("Sparse consecutive labels are currently not correctly supported.");
           }
           ll.add(v);
         }
         data[out] = ll;
       }
-      else if(elkitypes[out] == TypeUtil.EXTERNALID) {
+      else if(TypeUtil.EXTERNALID.equals(elkitypes[out])) {
         String val = (String) map.get(s);
         if(val != null) {
           data[out] = new ExternalID(val);
@@ -291,7 +291,7 @@ public class ArffParser implements Parser {
           throw new AbortException("External ID column not set in sparse instance." + tokenizer.toString());
         }
       }
-      else if(elkitypes[out] == TypeUtil.CLASSLABEL) {
+      else if(TypeUtil.CLASSLABEL.equals(elkitypes[out])) {
         String val = (String) map.get(s);
         if(val != null) {
           // TODO: support other class label types.
@@ -312,7 +312,7 @@ public class ArffParser implements Parser {
   private Object[] loadDenseInstance(StreamTokenizer tokenizer, int[] dimsize, TypeInformation[] etyp, int outdim) throws IOException {
     Object[] data = new Object[outdim];
     for(int out = 0; out < outdim; out++) {
-      if(etyp[out] == TypeUtil.NUMBER_VECTOR_FIELD) {
+      if(TypeUtil.NUMBER_VECTOR_FIELD.equals(etyp[out])) {
         // For multi-column vectors, read successive columns
         double[] cur = new double[dimsize[out]];
         for(int k = 0; k < dimsize[out]; k++) {
@@ -334,7 +334,7 @@ public class ArffParser implements Parser {
         }
         data[out] = new DoubleVector(cur);
       }
-      else if(etyp[out] == TypeUtil.LABELLIST) {
+      else if(TypeUtil.LABELLIST.equals(etyp[out])) {
         // Build a label list out of successive labels
         LabelList ll = new LabelList(dimsize[out]);
         for(int k = 0; k < dimsize[out]; k++) {
@@ -346,14 +346,14 @@ public class ArffParser implements Parser {
         }
         data[out] = ll;
       }
-      else if(etyp[out] == TypeUtil.EXTERNALID) {
+      else if(TypeUtil.EXTERNALID.equals(etyp[out])) {
         if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
           throw new AbortException("Expected word token, got: " + tokenizer.toString());
         }
         data[out] = new ExternalID(tokenizer.sval);
         nextToken(tokenizer);
       }
-      else if(etyp[out] == TypeUtil.CLASSLABEL) {
+      else if(TypeUtil.CLASSLABEL.equals(etyp[out])) {
         if(tokenizer.ttype != StreamTokenizer.TT_WORD) {
           throw new AbortException("Expected word token, got: " + tokenizer.toString());
         }
@@ -414,32 +414,32 @@ public class ArffParser implements Parser {
           break;
         }
       }
-      if(etyp[out] == TypeUtil.NUMBER_VECTOR_FIELD) {
+      if(TypeUtil.NUMBER_VECTOR_FIELD.equals(etyp[out])) {
         String[] labels = new String[dimsize[out]];
         // Collect labels:
         for(int i = 0; i < dimsize[out]; i++) {
           labels[i] = names.get(out + i);
         }
         if(!sparse) {
-          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.class, DoubleVector.STATIC, dimsize[out], labels, new DoubleVector(new double[dimsize[out]]));
+          VectorFieldTypeInformation<DoubleVector> type = new VectorFieldTypeInformation<DoubleVector>(DoubleVector.FACTORY, dimsize[out], labels);
           bundle.appendColumn(type, new ArrayList<DoubleVector>());
         }
         else {
-          VectorFieldTypeInformation<SparseFloatVector> type = new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.class, dimsize[out], labels, new SparseFloatVector(SparseFloatVector.EMPTYMAP, dimsize[out]));
+          VectorFieldTypeInformation<SparseFloatVector> type = new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.FACTORY, dimsize[out], labels);
           bundle.appendColumn(type, new ArrayList<SparseFloatVector>());
         }
       }
-      else if(etyp[out] == TypeUtil.LABELLIST) {
-        String label = names.get(out);
+      else if(TypeUtil.LABELLIST.equals(etyp[out])) {
+        StringBuilder label = new StringBuilder(names.get(out));
         for(int i = 1; i < dimsize[out]; i++) {
-          label = label + " " + names.get(out + i);
+          label.append(' ').append(names.get(out + i));
         }
-        bundle.appendColumn(new SimpleTypeInformation<LabelList>(LabelList.class, label), new ArrayList<LabelList>());
+        bundle.appendColumn(new SimpleTypeInformation<LabelList>(LabelList.class, label.toString()), new ArrayList<LabelList>());
       }
-      else if(etyp[out] == TypeUtil.EXTERNALID) {
+      else if(TypeUtil.EXTERNALID.equals(etyp[out])) {
         bundle.appendColumn(new SimpleTypeInformation<ExternalID>(ExternalID.class, names.get(out)), new ArrayList<ExternalID>());
       }
-      else if(etyp[out] == TypeUtil.CLASSLABEL) {
+      else if(TypeUtil.CLASSLABEL.equals(etyp[out])) {
         bundle.appendColumn(new SimpleTypeInformation<ClassLabel>(ClassLabel.class, names.get(out)), new ArrayList<ClassLabel>());
       }
       else {
@@ -553,7 +553,7 @@ public class ArffParser implements Parser {
       }
       else if(ARFF_NUMERIC.matcher(types.get(i)).matches()) {
         // Create a number vector field
-        if(next > 0 && etyp[next - 1] == TypeUtil.NUMBER_VECTOR_FIELD) {
+        if(next > 0 && TypeUtil.NUMBER_VECTOR_FIELD.equals(etyp[next - 1])) {
           targ[i] = next - 1;
           dims[next - 1]++;
           continue;
@@ -568,7 +568,7 @@ public class ArffParser implements Parser {
       }
       else {
         // Use LabelList
-        if(next > 0 && etyp[next - 1] == TypeUtil.LABELLIST) {
+        if(next > 0 && TypeUtil.LABELLIST.equals(etyp[next - 1])) {
           targ[i] = next - 1;
           dims[next - 1]++;
           continue;
@@ -598,21 +598,21 @@ public class ArffParser implements Parser {
     else if((tokenizer.ttype == StreamTokenizer.TT_WORD) && (tokenizer.sval.equals("?"))) {
       tokenizer.ttype = '?';
     }
-    if(logger.isDebugging()) {
+    if(LOG.isDebugging()) {
       if(tokenizer.ttype == StreamTokenizer.TT_NUMBER) {
-        logger.debug("token: " + tokenizer.nval);
+        LOG.debug("token: " + tokenizer.nval);
       }
       else if(tokenizer.ttype == StreamTokenizer.TT_WORD) {
-        logger.debug("token: " + tokenizer.sval);
+        LOG.debug("token: " + tokenizer.sval);
       }
       else if(tokenizer.ttype == StreamTokenizer.TT_EOF) {
-        logger.debug("token: EOF");
+        LOG.debug("token: EOF");
       }
       else if(tokenizer.ttype == StreamTokenizer.TT_EOL) {
-        logger.debug("token: EOL");
+        LOG.debug("token: EOL");
       }
       else {
-        logger.debug("token type: " + tokenizer.ttype);
+        LOG.debug("token type: " + tokenizer.ttype);
       }
     }
   }
@@ -628,12 +628,12 @@ public class ArffParser implements Parser {
     /**
      * Pattern for recognizing external ID attributes.
      */
-    public static final OptionID MAGIC_EID_ID = OptionID.getOrCreateOptionID("arff.externalid", "Pattern to recognize external ID attributes.");
+    public static final OptionID MAGIC_EID_ID = new OptionID("arff.externalid", "Pattern to recognize external ID attributes.");
 
     /**
      * Pattern for recognizing class label attributes.
      */
-    public static final OptionID MAGIC_CLASS_ID = OptionID.getOrCreateOptionID("arff.classlabel", "Pattern to recognize class label attributes.");
+    public static final OptionID MAGIC_CLASS_ID = new OptionID("arff.classlabel", "Pattern to recognize class label attributes.");
 
     /**
      * Pattern to recognize external ids
