@@ -59,7 +59,7 @@ public class FloatVectorLabelParser extends NumberVectorLabelParser<FloatVector>
   /**
    * Class logger
    */
-  private static final Logging logger = Logging.getLogger(FloatVectorLabelParser.class);
+  private static final Logging LOG = Logging.getLogger(FloatVectorLabelParser.class);
 
   /**
    * Constructor.
@@ -69,12 +69,12 @@ public class FloatVectorLabelParser extends NumberVectorLabelParser<FloatVector>
    * @param labelIndices
    */
   public FloatVectorLabelParser(Pattern colSep, char quoteChar, BitSet labelIndices) {
-    super(colSep, quoteChar, labelIndices, FloatVector.STATIC);
+    super(colSep, quoteChar, labelIndices, FloatVector.FACTORY);
   }
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
