@@ -42,7 +42,6 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.datasource.bundle.BundleMeta;
 import de.lmu.ifi.dbs.elki.logging.Logging;
-import de.lmu.ifi.dbs.elki.persistent.ByteBufferSerializer;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.ArrayLikeUtil;
 import de.lmu.ifi.dbs.elki.utilities.datastructures.arraylike.NumberArrayAdapter;
 import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
@@ -72,11 +71,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * 
  * @param <V> the type of NumberVector used
  */
-public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends AbstractStreamingParser {
+public class NumberVectorLabelParser<V extends NumberVector<?>> extends AbstractStreamingParser {
   /**
    * Logging class.
    */
-  private static final Logging logger = Logging.getLogger(NumberVectorLabelParser.class);
+  private static final Logging LOG = Logging.getLogger(NumberVectorLabelParser.class);
 
   /**
    * A comma separated list of the indices of labels (may be numeric), counting
@@ -86,7 +85,7 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
    * Key: {@code -parser.labelIndices}
    * </p>
    */
-  public static final OptionID LABEL_INDICES_ID = OptionID.getOrCreateOptionID("parser.labelIndices", "A comma separated list of the indices of labels (may be numeric), counting whitespace separated entries in a line starting with 0. The corresponding entries will be treated as a label.");
+  public static final OptionID LABEL_INDICES_ID = new OptionID("parser.labelIndices", "A comma separated list of the indices of labels (may be numeric), counting whitespace separated entries in a line starting with 0. The corresponding entries will be treated as a label.");
 
   /**
    * Parameter to specify the type of vectors to produce.
@@ -95,7 +94,7 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
    * Default: DoubleVector
    * </p>
    */
-  public static final OptionID VECTOR_TYPE_ID = OptionID.getOrCreateOptionID("parser.vector-type", "The type of vectors to create for numerical attributes.");
+  public static final OptionID VECTOR_TYPE_ID = new OptionID("parser.vector-type", "The type of vectors to create for numerical attributes.");
 
   /**
    * Constant used for unknown dimensionality (e.g. empty files)
@@ -113,73 +112,73 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
   protected BitSet labelIndices;
 
   /**
-   * Vector factory class
+   * Vector factory class.
    */
-  protected V factory;
+  protected NumberVector.Factory<V, ?> factory;
 
   /**
-   * Buffer reader
+   * Buffer reader.
    */
   private BufferedReader reader;
 
   /**
-   * Current line number
+   * Current line number.
    */
   protected int lineNumber;
 
   /**
-   * Dimensionality reported
+   * Dimensionality reported.
    */
   protected int dimensionality;
 
   /**
-   * Metadata
+   * Metadata.
    */
   protected BundleMeta meta = null;
 
   /**
-   * Column names
+   * Column names.
    */
   protected List<String> columnnames = null;
 
   /**
-   * Bitset to indicate which columns are numeric
+   * Bitset to indicate which columns are not numeric.
    */
   protected BitSet labelcolumns = null;
 
   /**
-   * Current vector
+   * Current vector.
    */
   protected V curvec = null;
 
   /**
-   * Current labels
+   * Current labels.
    */
   protected LabelList curlbl = null;
 
   /**
-   * Event to report next
+   * Event to report next.
    */
   Event nextevent = null;
 
   /**
-   * Constructor with defaults
+   * Constructor with defaults.
    * 
    * @param factory Vector factory
    */
-  public NumberVectorLabelParser(V factory) {
+  public NumberVectorLabelParser(NumberVector.Factory<V, ?> factory) {
     this(Pattern.compile(DEFAULT_SEPARATOR), QUOTE_CHAR, null, factory);
   }
 
   /**
-   * Constructor
+   * Constructor.
    * 
-   * @param colSep
-   * @param quoteChar
-   * @param labelIndices
+   * @param colSep Column separator
+   * @param quoteChar Quote character
+   * @param labelIndices Column indexes that are numeric.
    * @param factory Vector factory
    */
-  public NumberVectorLabelParser(Pattern colSep, char quoteChar, BitSet labelIndices, V factory) {
+  public NumberVectorLabelParser(Pattern colSep, char quoteChar, BitSet labelIndices, NumberVector.Factory<V, ?> factory) {
     super(colSep, quoteChar);
     this.labelIndices = labelIndices;
     this.factory = factory;
@@ -192,6 +191,9 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
     dimensionality = DIMENSIONALITY_UNKNOWN;
     columnnames = null;
     labelcolumns = new BitSet();
+    if (labelIndices != null) {
+      labelcolumns.or(labelIndices);
+    }
   }
 
   @Override
@@ -201,27 +203,26 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
 
   @Override
   public Event nextEvent() {
-    if(nextevent != null) {
+    if (nextevent != null) {
       Event ret = nextevent;
       nextevent = null;
       return ret;
     }
     try {
-      for(String line; (line = reader.readLine()) != null; lineNumber++) {
-        if(!line.startsWith(COMMENT) && line.length() > 0) {
+      for (String line; (line = reader.readLine()) != null; lineNumber++) {
+        if (!line.startsWith(COMMENT) && line.length() > 0) {
           parseLineInternal(line);
           // Maybe a header column?
-          if(curvec == null) {
+          if (curvec == null) {
             continue;
           }
-          if(dimensionality == DIMENSIONALITY_UNKNOWN) {
+          if (dimensionality == DIMENSIONALITY_UNKNOWN) {
             dimensionality = curvec.getDimensionality();
             buildMeta();
             nextevent = Event.NEXT_OBJECT;
             return Event.META_CHANGED;
-          }
-          else if(dimensionality > 0) {
-            if(dimensionality != curvec.getDimensionality()) {
+          } else if (dimensionality > 0) {
+            if (dimensionality != curvec.getDimensionality()) {
               dimensionality = DIMENSIONALITY_VARIABLE;
               buildMeta();
               nextevent = Event.NEXT_OBJECT;
@@ -238,8 +239,7 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
       reader.close();
       reader = null;
       return Event.END_OF_STREAM;
-    }
-    catch(IOException e) {
+    } catch (IOException e) {
       throw new IllegalArgumentException("Error while parsing line " + lineNumber + ".");
     }
   }
@@ -248,12 +248,11 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
    * Update the meta element.
    */
   protected void buildMeta() {
-    if(labelcolumns.cardinality() > 0) {
+    if (labelcolumns.cardinality() > 0 || (labelIndices != null && labelIndices.cardinality() > 0)) {
       meta = new BundleMeta(2);
       meta.add(getTypeInformation(dimensionality));
       meta.add(TypeUtil.LABELLIST);
-    }
-    else {
+    } else {
       meta = new BundleMeta(1);
       meta.add(getTypeInformation(dimensionality));
     }
@@ -261,10 +260,10 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
 
   @Override
   public Object data(int rnum) {
-    if(rnum == 0) {
+    if (rnum == 0) {
       return curvec;
     }
-    if(rnum == 1) {
+    if (rnum == 1) {
       return curlbl;
     }
     throw new ArrayIndexOutOfBoundsException();
@@ -284,28 +283,32 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
     LabelList labels = null;
 
     Iterator<String> itr = entries.iterator();
-    for(int i = 0; itr.hasNext(); i++) {
+    for (int i = 0; itr.hasNext(); i++) {
       String ent = itr.next();
-      if(!labelIndices.get(i)) {
+      if (labelIndices == null || !labelIndices.get(i)) {
         try {
           double attribute = Double.parseDouble(ent);
           attributes.add(attribute);
           continue;
-        }
-        catch(NumberFormatException e) {
+        } catch (NumberFormatException e) {
           // Ignore attempt, add to labels below.
           labelcolumns.set(i);
         }
       }
-      if(labels == null) {
+      // Else: labels.
+      if (labels == null) {
         labels = new LabelList(1);
       }
-      labels.add(ent);
+      // Make a new string, to not keep the whole file in memory!
+      labels.add(new String(ent));
     }
     // Maybe a label row?
-    if(lineNumber == 1 && attributes.size() == 0) {
+    if (lineNumber == 1 && attributes.size() == 0) {
       columnnames = labels;
       labelcolumns.clear();
+      if (labelIndices != null) {
+        labelcolumns.or(labelIndices);
+      }
       curvec = null;
       curlbl = null;
       return;
@@ -316,11 +319,11 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
   }
 
   /**
-   * <p>
    * Creates a database object of type V.
-   * </p>
    * 
    * @param attributes the attributes of the vector to create.
+   * @param adapter Array adapter
+   * @param <A> attribute type
    * @return a RalVector of type V containing the given attribute values
    */
   protected <A> V createDBObject(A attributes, NumberArrayAdapter<?, A> adapter) {
@@ -334,47 +337,31 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
    * @return Prototype object
    */
   SimpleTypeInformation<V> getTypeInformation(int dimensionality) {
-    @SuppressWarnings("unchecked")
-    Class<V> cls = (Class<V>) factory.getClass();
-    if(dimensionality > 0) {
+    if (dimensionality > 0) {
       String[] colnames = null;
-      if(columnnames != null) {
-        if(columnnames.size() - labelcolumns.cardinality() == dimensionality) {
+      if (columnnames != null) {
+        if (columnnames.size() - labelcolumns.cardinality() == dimensionality) {
           colnames = new String[dimensionality];
-          for(int i = 0, j = 0; i < columnnames.size(); i++) {
-            if(labelcolumns.get(i) == false) {
+          for (int i = 0, j = 0; i < columnnames.size(); i++) {
+            if (!labelcolumns.get(i)) {
               colnames[j] = columnnames.get(i);
               j++;
             }
           }
         }
       }
-      V f = factory.newNumberVector(new double[dimensionality]);
-      if(f instanceof ByteBufferSerializer) {
-        // TODO: Remove, once we have serializers for all types
-        @SuppressWarnings("unchecked")
-        final ByteBufferSerializer<V> ser = (ByteBufferSerializer<V>) f;
-        return new VectorFieldTypeInformation<V>(cls, ser, dimensionality, colnames, f);
-      }
-      return new VectorFieldTypeInformation<V>(cls, dimensionality, colnames, f);
+      return new VectorFieldTypeInformation<V>(factory, dimensionality, colnames);
     }
     // Variable dimensionality - return non-vector field type
-    if(dimensionality == DIMENSIONALITY_VARIABLE) {
-      V f = factory.newNumberVector(new double[0]);
-      if(f instanceof ByteBufferSerializer) {
-        // TODO: Remove, once we have serializers for all types
-        @SuppressWarnings("unchecked")
-        final ByteBufferSerializer<V> ser = (ByteBufferSerializer<V>) f;
-        return new SimpleTypeInformation<V>(cls, ser);
-      }
-      return new SimpleTypeInformation<V>(cls);
+    if (dimensionality == DIMENSIONALITY_VARIABLE) {
+      return new SimpleTypeInformation<V>(factory.getRestrictionClass(), factory.getDefaultSerializer());
     }
     throw new AbortException("No vectors were read from the input file - cannot determine vector data type.");
   }
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -384,16 +371,16 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<V, ?>> extends AbstractParser.Parameterizer {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractParser.Parameterizer {
     /**
      * Keeps the indices of the attributes to be treated as a string label.
      */
     protected BitSet labelIndices = null;
 
     /**
-     * Factory
+     * Factory object.
      */
-    protected V factory;
+    protected NumberVector.Factory<V, ?> factory;
 
     @Override
     protected void makeOptions(Parameterization config) {
@@ -402,21 +389,31 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
       getFactory(config);
     }
 
+    /**
+     * Get the object factory.
+     * 
+     * @param config Parameterization
+     */
     protected void getFactory(Parameterization config) {
-      ObjectParameter<V> factoryP = new ObjectParameter<V>(VECTOR_TYPE_ID, NumberVector.class, DoubleVector.class);
-      if(config.grab(factoryP)) {
+      ObjectParameter<NumberVector.Factory<V, ?>> factoryP = new ObjectParameter<NumberVector.Factory<V, ?>>(VECTOR_TYPE_ID, NumberVector.Factory.class, DoubleVector.Factory.class);
+      if (config.grab(factoryP)) {
         factory = factoryP.instantiateClass(config);
       }
     }
 
+    /**
+     * Get the label indices.
+     * 
+     * @param config Parameterization
+     */
     protected void getLabelIndices(Parameterization config) {
       IntListParameter labelIndicesP = new IntListParameter(LABEL_INDICES_ID, true);
 
-      labelIndices = new BitSet();
-      if(config.grab(labelIndicesP)) {
+      if (config.grab(labelIndicesP)) {
+        labelIndices = new BitSet();
         List<Integer> labelcols = labelIndicesP.getValue();
-        for(Integer idx : labelcols) {
-          labelIndices.set(idx);
+        for (Integer idx : labelcols) {
+          labelIndices.set(idx.intValue());
         }
       }
     }
@@ -426,4 +423,4 @@ public class NumberVectorLabelParser<V extends NumberVector<V, ?>> extends Abstr
       return new NumberVectorLabelParser<V>(colSep, quoteChar, labelIndices, factory);
     }
   }
-}
\ No newline at end of file
+}
