@@ -41,7 +41,10 @@ import de.lmu.ifi.dbs.elki.data.type.TypeUtil;
 import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.StringLengthConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.PatternParameter;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.StringParameter;
 
 /**
  * Parser to load polygon data (2D and 3D only) from a simple format. One record
@@ -59,7 +62,7 @@ public class SimplePolygonParser extends AbstractParser implements Parser {
   /**
    * Class logger
    */
-  private static final Logging logger = Logging.getLogger(SimplePolygonParser.class);
+  private static final Logging LOG = Logging.getLogger(SimplePolygonParser.class);
 
   /**
    * Pattern to catch coordinates
@@ -132,7 +135,7 @@ public class SimplePolygonParser extends AbstractParser implements Parser {
 
     ExternalID eid = null;
     LabelList labels = null;
-    List<Polygon> polys = new java.util.Vector<Polygon>(1);
+    List<Polygon> polys = new ArrayList<Polygon>(1);
 
     List<Vector> coords = new ArrayList<Vector>();
     while(iter.hasNext()) {
@@ -152,7 +155,7 @@ public class SimplePolygonParser extends AbstractParser implements Parser {
           continue;
         }
         catch(NumberFormatException e) {
-          logger.warning("Looked like a coordinate pair but didn't parse: " + cur);
+          LOG.warning("Looked like a coordinate pair but didn't parse: " + cur);
         }
       }
       // Polygon separator.
@@ -184,7 +187,7 @@ public class SimplePolygonParser extends AbstractParser implements Parser {
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -197,7 +200,15 @@ public class SimplePolygonParser extends AbstractParser implements Parser {
   public static class Parameterizer extends AbstractParser.Parameterizer {
     @Override
     protected void makeOptions(Parameterization config) {
-      super.makeOptions(config);
+      PatternParameter colParam = new PatternParameter(COLUMN_SEPARATOR_ID, "\\s+");
+      if(config.grab(colParam)) {
+        colSep = colParam.getValue();
+      }
+      StringParameter quoteParam = new StringParameter(QUOTE_ID, String.valueOf(QUOTE_CHAR));
+      quoteParam.addConstraint(new StringLengthConstraint(1, 1));
+      if(config.grab(quoteParam)) {
+        quoteChar = quoteParam.getValue().charAt(0);
+      }
     }
 
     @Override
