@@ -40,7 +40,6 @@ import de.lmu.ifi.dbs.elki.datasource.bundle.MultipleObjectsBundle;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 
 /**
  * Provides a parser for parsing one sparse BitVector per line, where the
@@ -60,7 +59,7 @@ public class SparseBitVectorLabelParser extends AbstractParser implements Parser
   /**
    * Class logger
    */
-  private static final Logging logger = Logging.getLogger(SparseBitVectorLabelParser.class);
+  private static final Logging LOG = Logging.getLogger(SparseBitVectorLabelParser.class);
 
   /**
    * Constructor.
@@ -90,7 +89,7 @@ public class SparseBitVectorLabelParser extends AbstractParser implements Parser
 
           for(String entry : entries) {
             try {
-              Integer index = Integer.valueOf(entry);
+              int index = Integer.parseInt(entry);
               bitSet.set(index);
               dimensionality = Math.max(dimensionality, index);
             }
@@ -122,12 +121,12 @@ public class SparseBitVectorLabelParser extends AbstractParser implements Parser
   }
 
   protected VectorFieldTypeInformation<BitVector> getTypeInformation(int dimensionality) {
-    return new VectorFieldTypeInformation<BitVector>(BitVector.class, dimensionality, new BitVector(new BitSet(), dimensionality));
+    return new VectorFieldTypeInformation<BitVector>(BitVector.FACTORY, dimensionality);
   }
   
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -138,11 +137,6 @@ public class SparseBitVectorLabelParser extends AbstractParser implements Parser
    * @apiviz.exclude
    */
   public static class Parameterizer extends AbstractParser.Parameterizer {
-    @Override
-    protected void makeOptions(Parameterization config) {
-      super.makeOptions(config);
-    }
-
     @Override
     protected SparseBitVectorLabelParser makeInstance() {
       return new SparseBitVectorLabelParser(colSep, quoteChar);
