@@ -23,8 +23,8 @@ package de.lmu.ifi.dbs.elki.datasource.parser;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import gnu.trove.iterator.TIntFloatIterator;
-import gnu.trove.map.hash.TIntFloatHashMap;
+import gnu.trove.iterator.TIntDoubleIterator;
+import gnu.trove.map.hash.TIntDoubleHashMap;
 
 import java.util.BitSet;
 import java.util.HashMap;
@@ -33,6 +33,7 @@ import java.util.regex.Pattern;
 
 import de.lmu.ifi.dbs.elki.data.LabelList;
 import de.lmu.ifi.dbs.elki.data.SparseFloatVector;
+import de.lmu.ifi.dbs.elki.data.SparseNumberVector;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
 import de.lmu.ifi.dbs.elki.logging.Logging;
@@ -42,6 +43,7 @@ import de.lmu.ifi.dbs.elki.utilities.exceptions.AbortException;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
 
 /**
  * A parser to load term frequency data, which essentially are sparse vectors
@@ -53,27 +55,32 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.Flag;
  */
 @Title("Term frequency parser")
 @Description("Parse a file containing term frequencies. The expected format is 'label term1 <freq> term2 <freq> ...'. Terms must not contain the separator character!")
-public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVector> {
+public class TermFrequencyParser<V extends SparseNumberVector<?>> extends NumberVectorLabelParser<V> {
   /**
-   * Class logger
+   * Class logger.
    */
-  private static final Logging logger = Logging.getLogger(TermFrequencyParser.class);
+  private static final Logging LOG = Logging.getLogger(TermFrequencyParser.class);
 
   /**
-   * Maximum dimension used
+   * Maximum dimension used.
    */
   int maxdim;
 
   /**
-   * Map
+   * Map.
    */
   HashMap<String, Integer> keymap;
 
   /**
-   * Normalize
+   * Normalize.
    */
   boolean normalize;
 
+  /**
+   * Same as {@link #factory}, but subtype.
+   */
+  private SparseNumberVector.Factory<V, ?> sparsefactory;
+
   /**
    * Constructor.
    * 
@@ -82,11 +89,12 @@ public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVect
    * @param quoteChar
    * @param labelIndices
    */
-  public TermFrequencyParser(boolean normalize, Pattern colSep, char quoteChar, BitSet labelIndices) {
-    super(colSep, quoteChar, labelIndices, SparseFloatVector.STATIC);
+  public TermFrequencyParser(boolean normalize, Pattern colSep, char quoteChar, BitSet labelIndices, SparseNumberVector.Factory<V, ?> factory) {
+    super(colSep, quoteChar, labelIndices, factory);
     this.normalize = normalize;
     this.maxdim = 0;
     this.keymap = new HashMap<String, Integer>();
+    this.sparsefactory = factory;
   }
 
   @Override
@@ -94,30 +102,28 @@ public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVect
     List<String> entries = tokenize(line);
 
     double len = 0;
-    TIntFloatHashMap values = new TIntFloatHashMap();
+    TIntDoubleHashMap values = new TIntDoubleHashMap();
     LabelList labels = null;
 
     String curterm = null;
-    for(int i = 0; i < entries.size(); i++) {
-      if(curterm == null) {
+    for (int i = 0; i < entries.size(); i++) {
+      if (curterm == null) {
         curterm = entries.get(i);
-      }
-      else {
+      } else {
         try {
-          float attribute = Float.valueOf(entries.get(i));
+          double attribute = Double.parseDouble(entries.get(i));
           Integer curdim = keymap.get(curterm);
-          if(curdim == null) {
-            curdim = maxdim + 1;
+          if (curdim == null) {
+            curdim = Integer.valueOf(maxdim + 1);
             keymap.put(curterm, curdim);
             maxdim += 1;
           }
           values.put(curdim, attribute);
           len += attribute;
           curterm = null;
-        }
-        catch(NumberFormatException e) {
-          if(curterm != null) {
-            if(labels == null) {
+        } catch (NumberFormatException e) {
+          if (curterm != null) {
+            if (labels == null) {
               labels = new LabelList(1);
             }
             labels.add(curterm);
@@ -126,39 +132,39 @@ public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVect
         }
       }
     }
-    if(curterm != null) {
-      if(labels == null) {
+    if (curterm != null) {
+      if (labels == null) {
         labels = new LabelList(1);
       }
       labels.add(curterm);
     }
-    if(normalize) {
-      if(Math.abs(len - 1.0) > 1E-10 && len > 1E-10) {
-        for(TIntFloatIterator iter = values.iterator(); iter.hasNext();) {
+    if (normalize) {
+      if (Math.abs(len - 1.0) > 1E-10 && len > 1E-10) {
+        for (TIntDoubleIterator iter = values.iterator(); iter.hasNext();) {
           iter.advance();
-          iter.setValue((float) (iter.value() / len));
+          iter.setValue(iter.value() / len);
         }
       }
     }
 
-    curvec = new SparseFloatVector(values, maxdim);
+    curvec = sparsefactory.newNumberVector(values, maxdim);
     curlbl = labels;
   }
 
   @Override
-  protected SimpleTypeInformation<SparseFloatVector> getTypeInformation(int dimensionality) {
-    if(dimensionality > 0) {
-      return new VectorFieldTypeInformation<SparseFloatVector>(SparseFloatVector.class, dimensionality, new SparseFloatVector(SparseFloatVector.EMPTYMAP, dimensionality));
+  protected SimpleTypeInformation<V> getTypeInformation(int dimensionality) {
+    if (dimensionality > 0) {
+      return new VectorFieldTypeInformation<V>(factory, dimensionality);
     }
-    if(dimensionality == DIMENSIONALITY_VARIABLE) {
-      return new SimpleTypeInformation<SparseFloatVector>(SparseFloatVector.class);
+    if (dimensionality == DIMENSIONALITY_VARIABLE) {
+      return new SimpleTypeInformation<V>(factory.getRestrictionClass(), factory.getDefaultSerializer());
     }
     throw new AbortException("No vectors were read from the input file - cannot determine vector data type.");
   }
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
@@ -168,14 +174,14 @@ public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVect
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer extends NumberVectorLabelParser.Parameterizer<SparseFloatVector> {
+  public static class Parameterizer<V extends SparseNumberVector<?>> extends NumberVectorLabelParser.Parameterizer<V> {
     /**
-     * Option ID for normalization
+     * Option ID for normalization.
      */
-    public static final OptionID NORMALIZE_FLAG = OptionID.getOrCreateOptionID("tf.normalize", "Normalize vectors to manhattan length 1 (convert term counts to term frequencies)");
+    public static final OptionID NORMALIZE_FLAG = new OptionID("tf.normalize", "Normalize vectors to manhattan length 1 (convert term counts to term frequencies)");
 
     /**
-     * Normalization flag
+     * Normalization flag.
      */
     boolean normalize = false;
 
@@ -183,14 +189,22 @@ public class TermFrequencyParser extends NumberVectorLabelParser<SparseFloatVect
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
       Flag normF = new Flag(NORMALIZE_FLAG);
-      if(config.grab(normF)) {
-        normalize = normF.getValue();
+      if (config.grab(normF)) {
+        normalize = normF.getValue().booleanValue();
       }
     }
 
     @Override
-    protected TermFrequencyParser makeInstance() {
-      return new TermFrequencyParser(normalize, colSep, quoteChar, labelIndices);
+    protected void getFactory(Parameterization config) {
+      ObjectParameter<SparseNumberVector.Factory<V, ?>> factoryP = new ObjectParameter<SparseNumberVector.Factory<V, ?>>(VECTOR_TYPE_ID, SparseNumberVector.Factory.class, SparseFloatVector.Factory.class);
+      if (config.grab(factoryP)) {
+        factory = factoryP.instantiateClass(config);
+      }
+    }
+
+    @Override
+    protected TermFrequencyParser<V> makeInstance() {
+      return new TermFrequencyParser<V>(normalize, colSep, quoteChar, labelIndices, (SparseNumberVector.Factory<V, ?>) factory);
     }
   }
-}
\ No newline at end of file
+}
