@@ -32,7 +32,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
  * 
  * @author Arthur Zimek
  */
-public class EuclideanDistanceFunction extends LPNormDistanceFunction implements SpatialPrimitiveDoubleDistanceFunction<NumberVector<?, ?>> {
+public class EuclideanDistanceFunction extends LPNormDistanceFunction implements SpatialPrimitiveDoubleDistanceFunction<NumberVector<?>> {
   /**
    * Static instance. Use this!
    */
@@ -49,20 +49,14 @@ public class EuclideanDistanceFunction extends LPNormDistanceFunction implements
     super(2.0);
   }
 
-  /**
-   * Provides the Euclidean distance between the given two vectors.
-   * 
-   * @return the Euclidean distance between the given two vectors as raw double
-   *         value
-   */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     final int dim1 = v1.getDimensionality();
     if(dim1 != v2.getDimensionality()) {
       throw new IllegalArgumentException("Different dimensionality of FeatureVectors" + "\n  first argument: " + v1.toString() + "\n  second argument: " + v2.toString() + "\n" + v1.getDimensionality() + "!=" + v2.getDimensionality());
     }
     double sqrDist = 0;
-    for(int i = 1; i <= dim1; i++) {
+    for(int i = 0; i < dim1; i++) {
       final double delta = v1.doubleValue(i) - v2.doubleValue(i);
       sqrDist += delta * delta;
     }
@@ -70,24 +64,24 @@ public class EuclideanDistanceFunction extends LPNormDistanceFunction implements
   }
 
   @Override
-  public double doubleNorm(NumberVector<?, ?> v) {
+  public double doubleNorm(NumberVector<?> v) {
     final int dim = v.getDimensionality();
     double sqrDist = 0;
-    for(int i = 1; i <= dim; i++) {
+    for(int i = 0; i < dim; i++) {
       final double delta = v.doubleValue(i);
       sqrDist += delta * delta;
     }
     return Math.sqrt(sqrDist);
   }
 
-  protected double doubleMinDistObject(SpatialComparable mbr, NumberVector<?, ?> v) {
+  protected double doubleMinDistObject(SpatialComparable mbr, NumberVector<?> v) {
     final int dim = mbr.getDimensionality();
     if(dim != v.getDimensionality()) {
       throw new IllegalArgumentException("Different dimensionality of objects\n  " + "first argument: " + mbr.toString() + "\n  " + "second argument: " + v.toString() + "\n" + dim + "!=" + v.getDimensionality());
     }
 
     double sqrDist = 0;
-    for(int d = 1; d <= dim; d++) {
+    for(int d = 0; d < dim; d++) {
       double value = v.doubleValue(d);
       double r;
       if(value < mbr.getMin(d)) {
@@ -111,14 +105,14 @@ public class EuclideanDistanceFunction extends LPNormDistanceFunction implements
     // Some optimizations for simpler cases.
     if(mbr1 instanceof NumberVector) {
       if(mbr2 instanceof NumberVector) {
-        return doubleDistance((NumberVector<?, ?>) mbr1, (NumberVector<?, ?>) mbr2);
+        return doubleDistance((NumberVector<?>) mbr1, (NumberVector<?>) mbr2);
       }
       else {
-        return doubleMinDistObject(mbr2, (NumberVector<?, ?>) mbr1);
+        return doubleMinDistObject(mbr2, (NumberVector<?>) mbr1);
       }
     }
     else if(mbr2 instanceof NumberVector) {
-      return doubleMinDistObject(mbr1, (NumberVector<?, ?>) mbr2);
+      return doubleMinDistObject(mbr1, (NumberVector<?>) mbr2);
     }
     final int dim1 = mbr1.getDimensionality();
     if(dim1 != mbr2.getDimensionality()) {
@@ -126,7 +120,7 @@ public class EuclideanDistanceFunction extends LPNormDistanceFunction implements
     }
 
     double sqrDist = 0;
-    for(int d = 1; d <= dim1; d++) {
+    for(int d = 0; d < dim1; d++) {
       final double m1, m2;
       if(mbr1.getMax(d) < mbr2.getMin(d)) {
         m1 = mbr2.getMin(d);
