@@ -39,7 +39,7 @@ import de.lmu.ifi.dbs.elki.index.preprocessed.localpca.FilteredLocalPCAIndex;
  * @param <O> Database object type
  * @param <D> Distance type
  */
-public interface FilteredLocalPCABasedDistanceFunction<O extends NumberVector<?, ?>, P extends FilteredLocalPCAIndex<? super O>, D extends Distance<D>> extends IndexBasedDistanceFunction<O, D> {
+public interface FilteredLocalPCABasedDistanceFunction<O extends NumberVector<?>, P extends FilteredLocalPCAIndex<? super O>, D extends Distance<D>> extends IndexBasedDistanceFunction<O, D> {
   /**
    * Instantiate with a database to get the actual distance query.
    * 
@@ -58,7 +58,7 @@ public interface FilteredLocalPCABasedDistanceFunction<O extends NumberVector<?,
    * @param <I> Index type
    * @param <D> Distance type
    */
-  public static interface Instance<T extends NumberVector<?, ?>, I extends Index, D extends Distance<D>> extends IndexBasedDistanceFunction.Instance<T, I, D> {
+  public static interface Instance<T extends NumberVector<?>, I extends Index, D extends Distance<D>> extends IndexBasedDistanceFunction.Instance<T, I, D> {
     // No additional restrictions
   }
 }
\ No newline at end of file
