@@ -41,11 +41,11 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
  * 
  * @apiviz.landmark
  */
-public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm implements SpatialPrimitiveDoubleDistanceFunction<NumberVector<?, ?>> {
+public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm implements SpatialPrimitiveDoubleDistanceFunction<NumberVector<?>> {
   /**
    * OptionID for the "p" parameter
    */
-  public static final OptionID P_ID = OptionID.getOrCreateOptionID("lpnorm.p", "the degree of the L-P-Norm (positive number)");
+  public static final OptionID P_ID = new OptionID("lpnorm.p", "the degree of the L-P-Norm (positive number)");
 
   /**
    * Keeps the currently set p.
@@ -72,14 +72,14 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
    *         the currently set p
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     final int dim1 = v1.getDimensionality();
     if(dim1 != v2.getDimensionality()) {
       throw new IllegalArgumentException("Different dimensionality of FeatureVectors\n  first argument: " + v1.toString() + "\n  second argument: " + v2.toString());
     }
 
     double sqrDist = 0;
-    for(int i = 1; i <= dim1; i++) {
+    for(int i = 0; i < dim1; i++) {
       final double delta = Math.abs(v1.doubleValue(i) - v2.doubleValue(i));
       sqrDist += Math.pow(delta, p);
     }
@@ -87,10 +87,10 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
   }
 
   @Override
-  public double doubleNorm(NumberVector<?, ?> v) {
+  public double doubleNorm(NumberVector<?> v) {
     final int dim = v.getDimensionality();
     double sqrDist = 0;
-    for(int i = 1; i <= dim; i++) {
+    for(int i = 0; i < dim; i++) {
       final double delta = v.doubleValue(i);
       sqrDist += Math.pow(delta, p);
     }
@@ -111,7 +111,7 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
     // Optimization for the simplest case
     if(mbr1 instanceof NumberVector) {
       if(mbr2 instanceof NumberVector) {
-        return doubleDistance((NumberVector<?, ?>) mbr1, (NumberVector<?, ?>) mbr2);
+        return doubleDistance((NumberVector<?>) mbr1, (NumberVector<?>) mbr2);
       }
     }
     // TODO: optimize for more simpler cases: obj vs. rect?
@@ -121,7 +121,7 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
     }
 
     double sumDist = 0;
-    for(int d = 1; d <= dim1; d++) {
+    for(int d = 0; d < dim1; d++) {
       final double m1, m2;
       if(mbr1.getMax(d) < mbr2.getMin(d)) {
         m1 = mbr2.getMin(d);
@@ -167,7 +167,7 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
   }
 
   @Override
-  public <T extends NumberVector<?, ?>> SpatialPrimitiveDistanceQuery<T, DoubleDistance> instantiate(Relation<T> relation) {
+  public <T extends NumberVector<?>> SpatialPrimitiveDistanceQuery<T, DoubleDistance> instantiate(Relation<T> relation) {
     return new SpatialPrimitiveDistanceQuery<T, DoubleDistance>(relation, this);
   }
 
@@ -187,7 +187,8 @@ public class LPNormDistanceFunction extends AbstractVectorDoubleDistanceNorm imp
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final DoubleParameter paramP = new DoubleParameter(P_ID, new GreaterConstraint(0));
+      final DoubleParameter paramP = new DoubleParameter(P_ID);
+      paramP.addConstraint(new GreaterConstraint(0));
       if(config.grab(paramP)) {
         p = paramP.getValue();
       }
