@@ -47,7 +47,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
  * @param <V> the type of NumberVector to compute the distances in between
  */
 // FIXME: implements SpatialPrimitiveDistanceFunction<V, DoubleDistance>
-public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> extends AbstractIndexBasedDistanceFunction<V, FilteredLocalPCAIndex<V>, DoubleDistance> implements FilteredLocalPCABasedDistanceFunction<V, FilteredLocalPCAIndex<V>, DoubleDistance> {
+public class LocallyWeightedDistanceFunction<V extends NumberVector<?>> extends AbstractIndexBasedDistanceFunction<V, FilteredLocalPCAIndex<V>, DoubleDistance> implements FilteredLocalPCABasedDistanceFunction<V, FilteredLocalPCAIndex<V>, DoubleDistance> {
   /**
    * Constructor
    * 
@@ -100,7 +100,7 @@ public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> exten
    * 
    * @author Erich Schubert
    */
-  public static class Instance<V extends NumberVector<?, ?>> extends AbstractIndexBasedDistanceFunction.Instance<V, LocalProjectionIndex<V, ?>, DoubleDistance, LocallyWeightedDistanceFunction<? super V>> implements FilteredLocalPCABasedDistanceFunction.Instance<V, LocalProjectionIndex<V, ?>, DoubleDistance> {
+  public static class Instance<V extends NumberVector<?>> extends AbstractIndexBasedDistanceFunction.Instance<V, LocalProjectionIndex<V, ?>, DoubleDistance, LocallyWeightedDistanceFunction<? super V>> implements FilteredLocalPCABasedDistanceFunction.Instance<V, LocalProjectionIndex<V, ?>, DoubleDistance> {
     /**
      * Constructor.
      * 
@@ -166,16 +166,16 @@ public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> exten
       }
 
       double[] r = new double[v.getDimensionality()];
-      for(int d = 1; d <= v.getDimensionality(); d++) {
+      for(int d = 0; d < v.getDimensionality(); d++) { 
         double value = v.doubleValue(d);
         if(value < mbr.getMin(d)) {
-          r[d - 1] = mbr.getMin(d);
+          r[d] = mbr.getMin(d);
         }
         else if(value > mbr.getMax(d)) {
-          r[d - 1] = mbr.getMax(d);
+          r[d] = mbr.getMax(d);
         }
         else {
-          r[d - 1] = value;
+          r[d] = value;
         }
       }
 
@@ -200,7 +200,7 @@ public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> exten
       }
 
       double sqrDist = 0;
-      for(int d = 1; d <= mbr1.getDimensionality(); d++) {
+      for(int d = 0; d < mbr1.getDimensionality(); d++) {
         double m1, m2;
         if(mbr1.getMax(d) < mbr2.getMin(d)) {
           m1 = mbr2.getMin(d);
@@ -228,11 +228,10 @@ public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> exten
       }
 
       double sqrDist = 0;
-      for(int d = 1; d <= mbr1.getDimensionality(); d++) {
-        double c1 = (mbr1.getMin(d) + mbr1.getMax(d)) / 2;
-        double c2 = (mbr2.getMin(d) + mbr2.getMax(d)) / 2;
-
-        double manhattanI = c1 - c2;
+      for(int d = 0; d < mbr1.getDimensionality(); d++) {
+        final double c1 = .5 * (mbr1.getMin(d) + mbr1.getMax(d));
+        final double c2 = .5 * (mbr2.getMin(d) + mbr2.getMax(d));
+        final double manhattanI = c1 - c2;
         sqrDist += manhattanI * manhattanI;
       }
       return new DoubleDistance(Math.sqrt(sqrDist));
@@ -246,7 +245,7 @@ public class LocallyWeightedDistanceFunction<V extends NumberVector<?, ?>> exten
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<?, ?>> extends AbstractIndexBasedDistanceFunction.Parameterizer<LocalProjectionIndex.Factory<V, FilteredLocalPCAIndex<V>>> {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractIndexBasedDistanceFunction.Parameterizer<LocalProjectionIndex.Factory<V, FilteredLocalPCAIndex<V>>> {
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
