@@ -30,9 +30,9 @@ import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.AbstractDatabaseDistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.DistanceUtil;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
@@ -71,17 +71,17 @@ public class MinKDistance<O, D extends Distance<D>> extends AbstractDatabaseDist
   /**
    * OptionID for the base distance used to compute reachability
    */
-  public static final OptionID DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("reachdist.basedistance", "Base distance function to use.");
+  public static final OptionID DISTANCE_FUNCTION_ID = new OptionID("reachdist.basedistance", "Base distance function to use.");
 
   /**
    * OptionID for the KNN query class to use (preprocessor, approximation, ...)
    */
-  public static final OptionID KNNQUERY_ID = OptionID.getOrCreateOptionID("reachdist.knnquery", "kNN query to use");
+  public static final OptionID KNNQUERY_ID = new OptionID("reachdist.knnquery", "kNN query to use");
 
   /**
    * OptionID for the "k" parameter.
    */
-  public static final OptionID K_ID = OptionID.getOrCreateOptionID("reachdist.k", "The number of nearest neighbors of an object to be considered for computing its reachability distance.");
+  public static final OptionID K_ID = new OptionID("reachdist.k", "The number of nearest neighbors of an object to be considered for computing its reachability distance.");
 
   /**
    * The distance function to determine the exact distance.
@@ -124,6 +124,7 @@ public class MinKDistance<O, D extends Distance<D>> extends AbstractDatabaseDist
    * @author Erich Schubert
    * 
    * @apiviz.uses KNNQuery
+   * @apiviz.exclude
    */
   public class Instance<T extends O> extends AbstractDatabaseDistanceQuery<T, D> {
     /**
@@ -236,7 +237,8 @@ public class MinKDistance<O, D extends Distance<D>> extends AbstractDatabaseDist
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(1));
+      final IntParameter kP = new IntParameter(K_ID);
+      kP.addConstraint(new GreaterConstraint(1));
       if(config.grab(kP)) {
         k = kP.getValue();
       }
@@ -252,4 +254,4 @@ public class MinKDistance<O, D extends Distance<D>> extends AbstractDatabaseDist
       return new MinKDistance<O, D>(parentDistance, k + (objectIsInKNN ? 0 : 1));
     }
   }
-}
\ No newline at end of file
+}
