@@ -52,7 +52,7 @@ public class WeightedLPNormDistanceFunction extends LPNormDistanceFunction {
   }
 
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     final int dim = weights.length;
     if(dim != v1.getDimensionality()) {
       throw new IllegalArgumentException("Dimensionality of FeatureVector doesn't match weights!");
@@ -63,7 +63,7 @@ public class WeightedLPNormDistanceFunction extends LPNormDistanceFunction {
 
     final double p = getP();
     double sqrDist = 0;
-    for(int i = 1; i <= dim; i++) {
+    for(int i = 0; i < dim; i++) {
       final double delta = Math.abs(v1.doubleValue(i) - v2.doubleValue(i));
       sqrDist += Math.pow(delta, p) * weights[i - 1];
     }
@@ -75,7 +75,7 @@ public class WeightedLPNormDistanceFunction extends LPNormDistanceFunction {
     // Optimization for the simplest case
     if(mbr1 instanceof NumberVector) {
       if(mbr2 instanceof NumberVector) {
-        return doubleDistance((NumberVector<?, ?>) mbr1, (NumberVector<?, ?>) mbr2);
+        return doubleDistance((NumberVector<?>) mbr1, (NumberVector<?>) mbr2);
       }
     }
     // TODO: optimize for more simpler cases: obj vs. rect?
@@ -86,7 +86,7 @@ public class WeightedLPNormDistanceFunction extends LPNormDistanceFunction {
 
     final double p = getP();
     double sumDist = 0;
-    for(int d = 1; d <= dim1; d++) {
+    for(int d = 0; d < dim1; d++) {
       final double m1, m2;
       if(mbr1.getMax(d) < mbr2.getMin(d)) {
         m1 = mbr2.getMin(d);
