@@ -32,7 +32,6 @@ import de.lmu.ifi.dbs.elki.distance.distancevalue.PCACorrelationDistance;
 import de.lmu.ifi.dbs.elki.index.IndexFactory;
 import de.lmu.ifi.dbs.elki.index.preprocessed.localpca.FilteredLocalPCAIndex;
 import de.lmu.ifi.dbs.elki.index.preprocessed.localpca.KNNQueryFilteredPCAIndex;
-import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Matrix;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.Vector;
 import de.lmu.ifi.dbs.elki.math.linearalgebra.pca.PCAFilteredResult;
@@ -48,12 +47,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
  * 
  * @apiviz.has Instance
  */
-public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDistanceFunction<NumberVector<?, ?>, FilteredLocalPCAIndex<NumberVector<?, ?>>, PCACorrelationDistance> implements FilteredLocalPCABasedDistanceFunction<NumberVector<?, ?>, FilteredLocalPCAIndex<NumberVector<?, ?>>, PCACorrelationDistance> {
-  /**
-   * Logger for debug.
-   */
-  static Logging logger = Logging.getLogger(PCABasedCorrelationDistanceFunction.class);
-
+public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDistanceFunction<NumberVector<?>, FilteredLocalPCAIndex<NumberVector<?>>, PCACorrelationDistance> implements FilteredLocalPCABasedDistanceFunction<NumberVector<?>, FilteredLocalPCAIndex<NumberVector<?>>, PCACorrelationDistance> {
   /**
    * Parameter to specify the threshold of a distance between a vector q and a
    * given space that indicates that q adds a new dimension to the space, must
@@ -65,7 +59,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
    * Key: {@code -pcabasedcorrelationdf.delta}
    * </p>
    */
-  public static final OptionID DELTA_ID = OptionID.getOrCreateOptionID("pcabasedcorrelationdf.delta", "Threshold of a distance between a vector q and a given space that indicates that " + "q adds a new dimension to the space.");
+  public static final OptionID DELTA_ID = new OptionID("pcabasedcorrelationdf.delta", "Threshold of a distance between a vector q and a given space that indicates that " + "q adds a new dimension to the space.");
 
   /**
    * Holds the value of {@link #DELTA_ID}.
@@ -78,7 +72,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
    * @param indexFactory index factory
    * @param delta Delta parameter
    */
-  public PCABasedCorrelationDistanceFunction(IndexFactory<NumberVector<?, ?>, FilteredLocalPCAIndex<NumberVector<?, ?>>> indexFactory, double delta) {
+  public PCABasedCorrelationDistanceFunction(IndexFactory<NumberVector<?>, FilteredLocalPCAIndex<NumberVector<?>>> indexFactory, double delta) {
     super(indexFactory);
     this.delta = delta;
   }
@@ -89,20 +83,20 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
   }
 
   @Override
-  public <T extends NumberVector<?, ?>> Instance<T> instantiate(Relation<T> database) {
+  public <T extends NumberVector<?>> Instance<T> instantiate(Relation<T> database) {
     // We can't really avoid these warnings, due to a limitation in Java
     // Generics (AFAICT)
     @SuppressWarnings("unchecked")
-    FilteredLocalPCAIndex<T> indexinst = (FilteredLocalPCAIndex<T>) indexFactory.instantiate((Relation<NumberVector<?, ?>>) database);
+    FilteredLocalPCAIndex<T> indexinst = (FilteredLocalPCAIndex<T>) indexFactory.instantiate((Relation<NumberVector<?>>) database);
     return new Instance<T>(database, indexinst, delta, this);
   }
 
   @Override
   public boolean equals(Object obj) {
-    if(obj == null) {
+    if (obj == null) {
       return false;
     }
-    if(!this.getClass().equals(obj.getClass())) {
+    if (!this.getClass().equals(obj.getClass())) {
       return false;
     }
     PCABasedCorrelationDistanceFunction other = (PCABasedCorrelationDistanceFunction) obj;
@@ -114,7 +108,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
    * 
    * @author Erich Schubert
    */
-  public static class Instance<V extends NumberVector<?, ?>> extends AbstractIndexBasedDistanceFunction.Instance<V, FilteredLocalPCAIndex<V>, PCACorrelationDistance, PCABasedCorrelationDistanceFunction> implements FilteredLocalPCABasedDistanceFunction.Instance<V, FilteredLocalPCAIndex<V>, PCACorrelationDistance> {
+  public static class Instance<V extends NumberVector<?>> extends AbstractIndexBasedDistanceFunction.Instance<V, FilteredLocalPCAIndex<V>, PCACorrelationDistance, PCABasedCorrelationDistanceFunction> implements FilteredLocalPCABasedDistanceFunction.Instance<V, FilteredLocalPCAIndex<V>, PCACorrelationDistance> {
     /**
      * Delta value
      */
@@ -172,7 +166,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
 
       // for all strong eigenvectors of rv2
       Matrix m1_czech = pca1.dissimilarityMatrix();
-      for(int i = 0; i < v2_strong.getColumnDimensionality(); i++) {
+      for (int i = 0; i < v2_strong.getColumnDimensionality(); i++) {
         Vector v2_i = v2_strong.getCol(i);
         // check, if distance of v2_i to the space of rv1 > delta
         // (i.e., if v2_i spans up a new dimension)
@@ -180,7 +174,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
 
         // if so, insert v2_i into v1 and adjust v1
         // and compute m1_czech new, increase lambda1
-        if(lambda1 < dimensionality && dist > delta) {
+        if (lambda1 < dimensionality && dist > delta) {
           adjust(v1, e1_czech, v2_i, lambda1++);
           m1_czech = v1.times(e1_czech).timesTranspose(v1);
         }
@@ -188,7 +182,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
 
       // for all strong eigenvectors of rv1
       Matrix m2_czech = pca2.dissimilarityMatrix();
-      for(int i = 0; i < v1_strong.getColumnDimensionality(); i++) {
+      for (int i = 0; i < v1_strong.getColumnDimensionality(); i++) {
         Vector v1_i = v1_strong.getCol(i);
         // check, if distance of v1_i to the space of rv2 > delta
         // (i.e., if v1_i spans up a new dimension)
@@ -196,7 +190,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
 
         // if so, insert v1_i into v2 and adjust v2
         // and compute m2_czech new , increase lambda2
-        if(lambda2 < dimensionality && dist > delta) {
+        if (lambda2 < dimensionality && dist > delta) {
           adjust(v2, e2_czech, v1_i, lambda2++);
           m2_czech = v2.times(e2_czech).timesTranspose(v2);
         }
@@ -237,7 +231,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
       // normalize v
       Vector v_i = vector.copy();
       Vector sum = new Vector(dim);
-      for(int k = 0; k < corrDim; k++) {
+      for (int k = 0; k < corrDim; k++) {
         Vector v_k = v.getCol(k);
         sum.plusTimesEquals(v_k, v_i.transposeTimes(v_k));
       }
@@ -254,12 +248,12 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
      * @return the Euclidean distance between the given two vectors
      */
     private double euclideanDistance(V dv1, V dv2) {
-      if(dv1.getDimensionality() != dv2.getDimensionality()) {
+      if (dv1.getDimensionality() != dv2.getDimensionality()) {
         throw new IllegalArgumentException("Different dimensionality of FeatureVectors\n  first argument: " + dv1.toString() + "\n  second argument: " + dv2.toString());
       }
 
       double sqrDist = 0;
-      for(int i = 1; i <= dv1.getDimensionality(); i++) {
+      for (int i = 0; i < dv1.getDimensionality(); i++) {
         double manhattanI = dv1.doubleValue(i) - dv2.doubleValue(i);
         sqrDist += manhattanI * manhattanI;
       }
@@ -274,7 +268,7 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer extends AbstractIndexBasedDistanceFunction.Parameterizer<FilteredLocalPCAIndex.Factory<NumberVector<?, ?>, FilteredLocalPCAIndex<NumberVector<?, ?>>>> {
+  public static class Parameterizer extends AbstractIndexBasedDistanceFunction.Parameterizer<FilteredLocalPCAIndex.Factory<NumberVector<?>, FilteredLocalPCAIndex<NumberVector<?>>>> {
     protected double delta = 0.0;
 
     @Override
@@ -282,9 +276,10 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
       super.makeOptions(config);
       configIndexFactory(config, FilteredLocalPCAIndex.Factory.class, KNNQueryFilteredPCAIndex.Factory.class);
 
-      final DoubleParameter param = new DoubleParameter(DELTA_ID, new GreaterEqualConstraint(0), 0.25);
-      if(config.grab(param)) {
-        delta = param.getValue();
+      final DoubleParameter param = new DoubleParameter(DELTA_ID, 0.25);
+      param.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(param)) {
+        delta = param.doubleValue();
       }
     }
 
@@ -293,4 +288,4 @@ public class PCABasedCorrelationDistanceFunction extends AbstractIndexBasedDista
       return new PCABasedCorrelationDistanceFunction(factory, delta);
     }
   }
-}
\ No newline at end of file
+}
