@@ -78,7 +78,7 @@ public class WeightedSquaredPearsonCorrelationDistanceFunction extends AbstractV
    *         vectors v1 and v2
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     final double pcc = MathUtil.weightedPearsonCorrelationCoefficient(v1, v2, weights);
     return 1 - pcc * pcc;
   }
