@@ -23,6 +23,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction.external;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -32,8 +33,6 @@ import java.util.Map;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDPair;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
-import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.ModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.AbstractDBIDDistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
 import de.lmu.ifi.dbs.elki.utilities.FileUtil;
@@ -52,6 +51,8 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * Provides a DistanceFunction that is based on double distances given by a
  * distance matrix of an external file.
  * 
+ * See {@link NumberDistanceParser} for the default input format.
+ * 
  * @author Elke Achtert
  */
 @Title("File based double distance for database objects.")
@@ -63,7 +64,7 @@ public class FileBasedDoubleDistanceFunction extends AbstractDBIDDistanceFunctio
    * Key: {@code -distance.matrix}
    * </p>
    */
-  public static final OptionID MATRIX_ID = OptionID.getOrCreateOptionID("distance.matrix", "The name of the file containing the distance matrix.");
+  public static final OptionID MATRIX_ID = new OptionID("distance.matrix", "The name of the file containing the distance matrix.");
 
   /**
    * Optional parameter to specify the parsers to provide a database, must
@@ -73,7 +74,7 @@ public class FileBasedDoubleDistanceFunction extends AbstractDBIDDistanceFunctio
    * Key: {@code -distance.parser}
    * </p>
    */
-  public static final OptionID PARSER_ID = OptionID.getOrCreateOptionID("distance.parser", "Parser used to load the distance matrix.");
+  public static final OptionID PARSER_ID = new OptionID("distance.parser", "Parser used to load the distance matrix.");
 
   /**
    * The distance cache
@@ -115,33 +116,23 @@ public class FileBasedDoubleDistanceFunction extends AbstractDBIDDistanceFunctio
       return getDistanceFactory().undefinedDistance();
     }
     // the smaller id is the first key
-    if(id1.getIntegerID() > id2.getIntegerID()) {
+    if(DBIDUtil.compare(id1, id2) > 0) {
       return distance(id2, id1);
     }
 
-    return cache.get(DBIDUtil.newPair(id1, id2));
+    DoubleDistance ret = cache.get(DBIDUtil.newPair(id1, id2));
+    if (ret == null && DBIDUtil.equal(id1, id2)) {
+      return DoubleDistance.ZERO_DISTANCE;
+    }
+    return ret;
   }
 
   private void loadCache(DistanceParser<DoubleDistance> parser, File matrixfile) throws IOException {
-    InputStream in = FileUtil.tryGzipInput(new FileInputStream(matrixfile));
+    InputStream in = new BufferedInputStream(FileUtil.tryGzipInput(new FileInputStream(matrixfile)));
     DistanceParsingResult<DoubleDistance> res = parser.parse(in);
     cache = res.getDistanceCache();
   }
 
-  /**
-   * Return a collection of all IDs in the cache.
-   * 
-   * @return Collection of all IDs in the cache.
-   */
-  public DBIDs getIDs() {
-    ModifiableDBIDs ids = DBIDUtil.newHashSet();
-    for(DBIDPair pair : cache.keySet()) {
-      ids.add(pair.getFirst());
-      ids.add(pair.getSecond());
-    }
-    return ids;
-  }
-
   @Override
   public DoubleDistance getDistanceFactory() {
     return DoubleDistance.FACTORY;
