@@ -23,6 +23,7 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction.external;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+import java.io.BufferedInputStream;
 import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
@@ -50,6 +51,8 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * Provides a DistanceFunction that is based on float distances given by a
  * distance matrix of an external file.
  * 
+ * See {@link NumberDistanceParser} for the default input format.
+ * 
  * @author Elke Achtert
  */
 @Title("File based float distance for database objects.")
@@ -61,7 +64,7 @@ public class FileBasedFloatDistanceFunction extends AbstractDBIDDistanceFunction
    * Key: {@code -distance.matrix}
    * </p>
    */
-  public static final OptionID MATRIX_ID = OptionID.getOrCreateOptionID("distance.matrix", "The name of the file containing the distance matrix.");
+  public static final OptionID MATRIX_ID = new OptionID("distance.matrix", "The name of the file containing the distance matrix.");
 
   /**
    * Optional parameter to specify the parsers to provide a database, must
@@ -71,7 +74,7 @@ public class FileBasedFloatDistanceFunction extends AbstractDBIDDistanceFunction
    * Key: {@code -distance.parser}
    * </p>
    */
-  public static final OptionID PARSER_ID = OptionID.getOrCreateOptionID("distance.parser", "Parser used to load the distance matrix.");
+  public static final OptionID PARSER_ID = new OptionID("distance.parser", "Parser used to load the distance matrix.");
 
   /**
    * The distance cache
@@ -113,15 +116,18 @@ public class FileBasedFloatDistanceFunction extends AbstractDBIDDistanceFunction
       return getDistanceFactory().undefinedDistance();
     }
     // the smaller id is the first key
-    if(id1.getIntegerID() > id2.getIntegerID()) {
+    if(DBIDUtil.compare(id1, id2) > 0) {
       return distance(id2, id1);
     }
-
-    return cache.get(DBIDUtil.newPair(id1, id2));
+    FloatDistance ret = cache.get(DBIDUtil.newPair(id1, id2));
+    if (ret == null && DBIDUtil.equal(id1, id2)) {
+      return FloatDistance.ZERO_DISTANCE;
+    }
+    return ret;
   }
 
   private void loadCache(DistanceParser<FloatDistance> parser, File matrixfile) throws IOException {
-    InputStream in = FileUtil.tryGzipInput(new FileInputStream(matrixfile));
+    InputStream in = new BufferedInputStream(FileUtil.tryGzipInput(new FileInputStream(matrixfile)));
     DistanceParsingResult<FloatDistance> res = parser.parse(in);
     cache = res.getDistanceCache();
   }
