@@ -24,10 +24,15 @@ package de.lmu.ifi.dbs.elki.distance.distancefunction.geo;
  */
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
+import de.lmu.ifi.dbs.elki.data.spatial.SpatialComparable;
 import de.lmu.ifi.dbs.elki.data.type.SimpleTypeInformation;
 import de.lmu.ifi.dbs.elki.data.type.VectorFieldTypeInformation;
+import de.lmu.ifi.dbs.elki.database.query.distance.SpatialPrimitiveDistanceQuery;
+import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.AbstractVectorDoubleDistanceFunction;
-import de.lmu.ifi.dbs.elki.math.MathUtil;
+import de.lmu.ifi.dbs.elki.distance.distancefunction.SpatialPrimitiveDoubleDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distancevalue.DoubleDistance;
+import de.lmu.ifi.dbs.elki.math.GeoUtil;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
@@ -40,14 +45,14 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.IntParameter;
  * 
  * @author Erich Schubert
  */
-public class DimensionSelectingLatLngDistanceFunction extends AbstractVectorDoubleDistanceFunction {
+public class DimensionSelectingLatLngDistanceFunction extends AbstractVectorDoubleDistanceFunction implements SpatialPrimitiveDoubleDistanceFunction<NumberVector<?>> {
   /**
-   * Latitude dimension
+   * Latitude dimension.
    */
   final int dimlat;
 
   /**
-   * Longitude dimension
+   * Longitude dimension.
    */
   final int dimlng;
 
@@ -64,13 +69,42 @@ public class DimensionSelectingLatLngDistanceFunction extends AbstractVectorDoub
   }
 
   @Override
-  public double doubleDistance(NumberVector<?, ?> o1, NumberVector<?, ?> o2) {
-    return MathUtil.latlngDistance(o1.doubleValue(dimlat), o1.doubleValue(dimlng), o2.doubleValue(dimlat), o2.doubleValue(dimlng));
+  public double doubleDistance(NumberVector<?> o1, NumberVector<?> o2) {
+    return GeoUtil.haversineFormulaDeg(o1.doubleValue(dimlat), o1.doubleValue(dimlng), o2.doubleValue(dimlat), o2.doubleValue(dimlng));
   }
 
   @Override
-  public SimpleTypeInformation<? super NumberVector<?, ?>> getInputTypeRestriction() {
-    return new VectorFieldTypeInformation<NumberVector<?, ?>>(NumberVector.class, Math.max(dimlat, dimlng), Integer.MAX_VALUE);
+  public double doubleMinDist(SpatialComparable mbr1, SpatialComparable mbr2) {
+    if (mbr1 instanceof NumberVector) {
+      if (mbr2 instanceof NumberVector) {
+        return doubleDistance((NumberVector<?>) mbr1, (NumberVector<?>) mbr2);
+      } else {
+        NumberVector<?> o1 = (NumberVector<?>) mbr1;
+        return GeoUtil.latlngMinDistDeg(o1.doubleValue(dimlat), o1.doubleValue(dimlng), mbr2.getMin(dimlat), mbr2.getMin(dimlng), mbr2.getMax(dimlat), mbr2.getMax(dimlng));
+      }
+    } else {
+      if (mbr2 instanceof NumberVector) {
+        NumberVector<?> o2 = (NumberVector<?>) mbr2;
+        return GeoUtil.latlngMinDistDeg(o2.doubleValue(dimlat), o2.doubleValue(dimlng), mbr1.getMin(dimlat), mbr1.getMin(dimlng), mbr1.getMax(dimlat), mbr1.getMax(dimlng));
+      } else {
+        throw new UnsupportedOperationException("MBR to MBR mindist is not yet implemented.");
+      }
+    }
+  }
+
+  @Override
+  public DoubleDistance minDist(SpatialComparable mbr1, SpatialComparable mbr2) {
+    return new DoubleDistance(doubleMinDist(mbr1, mbr2));
+  }
+
+  @Override
+  public SimpleTypeInformation<? super NumberVector<?>> getInputTypeRestriction() {
+    return new VectorFieldTypeInformation<NumberVector<?>>(NumberVector.class, Math.max(dimlat, dimlng), Integer.MAX_VALUE);
+  }
+
+  @Override
+  public <T extends NumberVector<?>> SpatialPrimitiveDistanceQuery<T, DoubleDistance> instantiate(Relation<T> relation) {
+    return new SpatialPrimitiveDistanceQuery<T, DoubleDistance>(relation, this);
   }
 
   /**
@@ -84,32 +118,34 @@ public class DimensionSelectingLatLngDistanceFunction extends AbstractVectorDoub
     /**
      * Latitude dimension parameter.
      */
-    public static final OptionID LATDIM_ID = OptionID.getOrCreateOptionID("distance.latitudedim", "The dimension containing the latitude.");
+    public static final OptionID LATDIM_ID = new OptionID("distance.latitudedim", "The dimension containing the latitude.");
 
     /**
      * Longitude dimension parameter.
      */
-    public static final OptionID LNGDIM_ID = OptionID.getOrCreateOptionID("distance.longitudedim", "The dimension containing the longitude.");
+    public static final OptionID LNGDIM_ID = new OptionID("distance.longitudedim", "The dimension containing the longitude.");
 
     /**
-     * Latitude dimension
+     * Latitude dimension.
      */
     int dimlat;
 
     /**
-     * Longitude dimension
+     * Longitude dimension.
      */
     int dimlng;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final IntParameter dimlatP = new IntParameter(LATDIM_ID, new GreaterEqualConstraint(1));
-      if(config.grab(dimlatP)) {
+      final IntParameter dimlatP = new IntParameter(LATDIM_ID);
+      dimlatP.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(dimlatP)) {
         dimlat = dimlatP.getValue();
       }
-      final IntParameter dimlngP = new IntParameter(LNGDIM_ID, new GreaterEqualConstraint(1));
-      if(config.grab(dimlngP)) {
+      final IntParameter dimlngP = new IntParameter(LNGDIM_ID);
+      dimlngP.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(dimlngP)) {
         dimlng = dimlngP.getValue();
       }
       config.checkConstraint(new NoDuplicateValueGlobalConstraint(dimlatP, dimlngP));
@@ -120,4 +156,4 @@ public class DimensionSelectingLatLngDistanceFunction extends AbstractVectorDoub
       return new DimensionSelectingLatLngDistanceFunction(dimlat, dimlng);
     }
   }
-}
\ No newline at end of file
+}
