@@ -27,11 +27,11 @@ import java.util.BitSet;
 
 import de.lmu.ifi.dbs.elki.data.NumberVector;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
+import de.lmu.ifi.dbs.elki.database.relation.RelationUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.PreferenceVectorBasedCorrelationDistance;
 import de.lmu.ifi.dbs.elki.index.preprocessed.preference.HiSCPreferenceVectorIndex;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.utilities.ClassGenericsUtil;
-import de.lmu.ifi.dbs.elki.utilities.DatabaseUtil;
 import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 
@@ -44,17 +44,17 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameteriz
  * 
  * @param <V> the type of NumberVector to compute the distances in between
  */
-public class HiSCDistanceFunction<V extends NumberVector<?, ?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction<V, HiSCPreferenceVectorIndex<V>> {
+public class HiSCDistanceFunction<V extends NumberVector<?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction<V, HiSCPreferenceVectorIndex<V>> {
   /**
    * Logger for debug.
    */
-  static Logging logger = Logging.getLogger(DiSHDistanceFunction.class);
+  private static final Logging LOG = Logging.getLogger(HiSCDistanceFunction.class);
 
   /**
    * Constructor.
    * 
-   * @param indexFactory
-   * @param epsilon
+   * @param indexFactory HiSC index factory
+   * @param epsilon Epsilon value
    */
   public HiSCDistanceFunction(HiSCPreferenceVectorIndex.Factory<V> indexFactory, double epsilon) {
     super(indexFactory, epsilon);
@@ -76,7 +76,7 @@ public class HiSCDistanceFunction<V extends NumberVector<?, ?>> extends Abstract
    * 
    * @param <V> the type of NumberVector to compute the distances in between
    */
-  public static class Instance<V extends NumberVector<?, ?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction.Instance<V, HiSCPreferenceVectorIndex<V>> {
+  public static class Instance<V extends NumberVector<?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction.Instance<V, HiSCPreferenceVectorIndex<V>> {
     /**
      * Constructor.
      * 
@@ -114,16 +114,16 @@ public class HiSCDistanceFunction<V extends NumberVector<?, ?>> extends Abstract
 
       if(Math.max(dist1, dist2) > epsilon) {
         subspaceDim++;
-        if(logger.isDebugging()) {
+        if(LOG.isDebugging()) {
           //Representation<String> rep = rep.getObjectLabelQuery();
-          StringBuffer msg = new StringBuffer();
-          msg.append("\ndist1 " + dist1);
-          msg.append("\ndist2 " + dist2);
-          // msg.append("\nv1 " + rep.get(v1.getID()));
-          // msg.append("\nv2 " + rep.get(v2.getID()));
-          msg.append("\nsubspaceDim " + subspaceDim);
-          msg.append("\ncommon pv " + FormatUtil.format(dim, commonPreferenceVector));
-          logger.debugFine(msg.toString());
+          StringBuilder msg = new StringBuilder();
+          msg.append("\ndist1 ").append(dist1);
+          msg.append("\ndist2 ").append(dist2);
+          // msg.append("\nv1 ").append(rep.get(v1.getID()));
+          // msg.append("\nv2 ").append(rep.get(v2.getID()));
+          msg.append("\nsubspaceDim ").append(subspaceDim);
+          msg.append("\ncommon pv ").append(FormatUtil.format(dim, commonPreferenceVector));
+          LOG.debugFine(msg.toString());
         }
       }
 
@@ -131,7 +131,7 @@ public class HiSCDistanceFunction<V extends NumberVector<?, ?>> extends Abstract
       BitSet inverseCommonPreferenceVector = (BitSet) commonPreferenceVector.clone();
       inverseCommonPreferenceVector.flip(0, dim);
 
-      return new PreferenceVectorBasedCorrelationDistance(DatabaseUtil.dimensionality(relation), subspaceDim, weightedDistance(v1, v2, inverseCommonPreferenceVector), commonPreferenceVector);
+      return new PreferenceVectorBasedCorrelationDistance(RelationUtil.dimensionality(relation), subspaceDim, weightedDistance(v1, v2, inverseCommonPreferenceVector), commonPreferenceVector);
     }
   }
 
@@ -142,7 +142,7 @@ public class HiSCDistanceFunction<V extends NumberVector<?, ?>> extends Abstract
    * 
    * @apiviz.exclude
    */
-  public static class Parameterizer<V extends NumberVector<?, ?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction.Parameterizer<HiSCPreferenceVectorIndex.Factory<V>> {
+  public static class Parameterizer<V extends NumberVector<?>> extends AbstractPreferenceVectorBasedCorrelationDistanceFunction.Parameterizer<HiSCPreferenceVectorIndex.Factory<V>> {
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
