@@ -52,7 +52,7 @@ public class DTWDistanceFunction extends AbstractEditDistanceFunction {
    *         an instance of {@link DoubleDistance DoubleDistance}.
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     // Current and previous columns of the matrix
     double[] curr = new double[v2.getDimensionality()];
     double[] prev = new double[v2.getDimensionality()];
@@ -82,8 +82,8 @@ public class DTWDistanceFunction extends AbstractEditDistanceFunction {
 
       for(int j = l; j <= r; j++) {
         if(Math.abs(i - j) <= band) {
-          double val1 = v1.doubleValue(i + 1);
-          double val2 = v2.doubleValue(j + 1);
+          double val1 = v1.doubleValue(i);
+          double val2 = v2.doubleValue(j);
           double diff = (val1 - val2);
           // Formally: diff = Math.sqrt(diff * diff);
 
