@@ -43,7 +43,7 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
   /**
    * DELTA parameter
    */
-  public static final OptionID DELTA_ID = OptionID.getOrCreateOptionID("edr.delta", "the delta parameter (similarity threshold) for EDR (positive number)");
+  public static final OptionID DELTA_ID = new OptionID("edr.delta", "the delta parameter (similarity threshold) for EDR (positive number)");
 
   /**
    * Keeps the currently set delta.
@@ -69,7 +69,7 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
    *         vectors as an instance of {@link DoubleDistance DoubleDistance}.
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     // Current and previous columns of the matrix
     double[] curr = new double[v2.getDimensionality()];
     double[] prev = new double[v2.getDimensionality()];
@@ -82,7 +82,7 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
     // features2.length + ", band: " + band);
     final double deltaValue = delta;
 
-    for(int i = 0; i < v1.getDimensionality(); i++) {
+    for (int i = 0; i < v1.getDimensionality(); i++) {
       // Swap current and prev arrays. We'll just overwrite the new curr.
       {
         double[] temp = prev;
@@ -90,19 +90,19 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
         curr = temp;
       }
       int l = i - (band + 1);
-      if(l < 0) {
+      if (l < 0) {
         l = 0;
       }
       int r = i + (band + 1);
-      if(r > (v2.getDimensionality() - 1)) {
+      if (r > (v2.getDimensionality() - 1)) {
         r = (v2.getDimensionality() - 1);
       }
 
-      for(int j = l; j <= r; j++) {
-        if(Math.abs(i - j) <= band) {
+      for (int j = l; j <= r; j++) {
+        if (Math.abs(i - j) <= band) {
           // compute squared distance
-          double val1 = v1.doubleValue(i + 1);
-          double val2 = v2.doubleValue(j + 1);
+          double val1 = v1.doubleValue(i);
+          double val2 = v2.doubleValue(j);
           double diff = (val1 - val2);
           final double d = Math.sqrt(diff * diff);
 
@@ -110,27 +110,23 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
 
           final double subcost = (d <= deltaValue) ? 0 : 1;
 
-          if((i + j) != 0) {
-            if((i == 0) || ((j != 0) && (((prev[j - 1] + subcost) > (curr[j - 1] + 1)) && ((curr[j - 1] + 1) < (prev[j] + 1))))) {
+          if ((i + j) != 0) {
+            if ((i == 0) || ((j != 0) && (((prev[j - 1] + subcost) > (curr[j - 1] + 1)) && ((curr[j - 1] + 1) < (prev[j] + 1))))) {
               // del
               cost = curr[j - 1] + 1;
-            }
-            else if((j == 0) || ((i != 0) && (((prev[j - 1] + subcost) > (prev[j] + 1)) && ((prev[j] + 1) < (curr[j - 1] + 1))))) {
+            } else if ((j == 0) || ((i != 0) && (((prev[j - 1] + subcost) > (prev[j] + 1)) && ((prev[j] + 1) < (curr[j - 1] + 1))))) {
               // ins
               cost = prev[j] + 1;
-            }
-            else {
+            } else {
               // match
               cost = prev[j - 1] + subcost;
             }
-          }
-          else {
+          } else {
             cost = 0;
           }
 
           curr[j] = cost;
-        }
-        else {
+        } else {
           curr[j] = Double.POSITIVE_INFINITY; // outside band
         }
       }
@@ -141,7 +137,7 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
 
   @Override
   public boolean equals(Object obj) {
-    if(!super.equals(obj)) {
+    if (!super.equals(obj)) {
       return false;
     }
     return this.delta == ((EDRDistanceFunction) obj).delta;
@@ -160,9 +156,10 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final DoubleParameter deltaP = new DoubleParameter(DELTA_ID, new GreaterEqualConstraint(0), 1.0);
-      if(config.grab(deltaP)) {
-        delta = deltaP.getValue();
+      final DoubleParameter deltaP = new DoubleParameter(DELTA_ID, 1.0);
+      deltaP.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(deltaP)) {
+        delta = deltaP.doubleValue();
       }
     }
 
@@ -181,4 +178,4 @@ public class EDRDistanceFunction extends AbstractEditDistanceFunction {
       return new EDRDistanceFunction(bandSize, delta);
     }
   }
-}
\ No newline at end of file
+}
