@@ -43,7 +43,7 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
   /**
    * G parameter
    */
-  public static final OptionID G_ID = OptionID.getOrCreateOptionID("erp.g", "the g parameter ERP (positive number)");
+  public static final OptionID G_ID = new OptionID("erp.g", "the g parameter ERP (positive number)");
 
   /**
    * Keeps the currently set g.
@@ -69,7 +69,7 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
    *         vectors as an instance of {@link DoubleDistance DoubleDistance}.
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     // Current and previous columns of the matrix
     double[] curr = new double[v2.getDimensionality()];
     double[] prev = new double[v2.getDimensionality()];
@@ -78,7 +78,7 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
     // bandsize is the maximum allowed distance to the diagonal
     final int band = (int) Math.ceil(v2.getDimensionality() * bandSize);
 
-    for(int i = 0; i < v1.getDimensionality(); i++) {
+    for (int i = 0; i < v1.getDimensionality(); i++) {
       // Swap current and prev arrays. We'll just overwrite the new curr.
       {
         double[] temp = prev;
@@ -86,29 +86,29 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
         curr = temp;
       }
       int l = i - (band + 1);
-      if(l < 0) {
+      if (l < 0) {
         l = 0;
       }
       int r = i + (band + 1);
-      if(r > (v2.getDimensionality() - 1)) {
+      if (r > (v2.getDimensionality() - 1)) {
         r = (v2.getDimensionality() - 1);
       }
 
-      for(int j = l; j <= r; j++) {
-        if(Math.abs(i - j) <= band) {
+      for (int j = l; j <= r; j++) {
+        if (Math.abs(i - j) <= band) {
           // compute squared distance of feature vectors
-          double val1 = v1.doubleValue(i + 1);
+          double val1 = v1.doubleValue(i);
           double val2 = g;
           double diff = (val1 - val2);
           final double d1 = Math.sqrt(diff * diff);
 
           val1 = g;
-          val2 = v2.doubleValue(j + 1);
+          val2 = v2.doubleValue(j);
           diff = (val1 - val2);
           final double d2 = Math.sqrt(diff * diff);
 
-          val1 = v1.doubleValue(i + 1);
-          val2 = v2.doubleValue(j + 1);
+          val1 = v1.doubleValue(i);
+          val2 = v2.doubleValue(j);
           diff = (val1 - val2);
           final double d12 = Math.sqrt(diff * diff);
 
@@ -118,28 +118,24 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
 
           final double cost;
 
-          if((i + j) != 0) {
-            if((i == 0) || ((j != 0) && (((prev[j - 1] + dist12) > (curr[j - 1] + dist2)) && ((curr[j - 1] + dist2) < (prev[j] + dist1))))) {
+          if ((i + j) != 0) {
+            if ((i == 0) || ((j != 0) && (((prev[j - 1] + dist12) > (curr[j - 1] + dist2)) && ((curr[j - 1] + dist2) < (prev[j] + dist1))))) {
               // del
               cost = curr[j - 1] + dist2;
-            }
-            else if((j == 0) || ((i != 0) && (((prev[j - 1] + dist12) > (prev[j] + dist1)) && ((prev[j] + dist1) < (curr[j - 1] + dist2))))) {
+            } else if ((j == 0) || ((i != 0) && (((prev[j - 1] + dist12) > (prev[j] + dist1)) && ((prev[j] + dist1) < (curr[j - 1] + dist2))))) {
               // ins
               cost = prev[j] + dist1;
-            }
-            else {
+            } else {
               // match
               cost = prev[j - 1] + dist12;
             }
-          }
-          else {
+          } else {
             cost = 0;
           }
 
           curr[j] = cost;
           // steps[i][j] = step;
-        }
-        else {
+        } else {
           curr[j] = Double.POSITIVE_INFINITY; // outside band
         }
       }
@@ -150,12 +146,12 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
 
   @Override
   public boolean equals(Object obj) {
-    if(!super.equals(obj)) {
+    if (!super.equals(obj)) {
       return false;
     }
     return this.g == ((ERPDistanceFunction) obj).g;
   }
-  
+
   /**
    * Parameterization class.
    * 
@@ -169,9 +165,10 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final DoubleParameter gP = new DoubleParameter(G_ID, new GreaterEqualConstraint(0), 0.0);
-      if(config.grab(gP)) {
-        g = gP.getValue();
+      final DoubleParameter gP = new DoubleParameter(G_ID, 0.0);
+      gP.addConstraint(new GreaterEqualConstraint(0));
+      if (config.grab(gP)) {
+        g = gP.doubleValue();
       }
     }
 
@@ -180,4 +177,4 @@ public class ERPDistanceFunction extends AbstractEditDistanceFunction {
       return new ERPDistanceFunction(bandSize, g);
     }
   }
-}
\ No newline at end of file
+}
