@@ -34,8 +34,8 @@ import de.lmu.ifi.dbs.elki.utilities.documentation.Reference;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.AbstractParameterizer;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.OptionID;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint;
-import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.IntervalConstraint.IntervalBoundary;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.GreaterEqualConstraint;
+import de.lmu.ifi.dbs.elki.utilities.optionhandling.constraints.LessEqualConstraint;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameterization.Parameterization;
 import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.DoubleParameter;
 
@@ -80,12 +80,12 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
   /**
    * PDELTA parameter
    */
-  public static final OptionID PDELTA_ID = OptionID.getOrCreateOptionID("lcss.pDelta", "the allowed deviation in x direction for LCSS alignment (positive double value, 0 <= pDelta <= 1)");
+  public static final OptionID PDELTA_ID = new OptionID("lcss.pDelta", "the allowed deviation in x direction for LCSS alignment (positive double value, 0 <= pDelta <= 1)");
 
   /**
    * PEPSILON parameter
    */
-  public static final OptionID PEPSILON_ID = OptionID.getOrCreateOptionID("lcss.pEpsilon", "the allowed deviation in y directionfor LCSS alignment (positive double value, 0 <= pEpsilon <= 1)");
+  public static final OptionID PEPSILON_ID = new OptionID("lcss.pEpsilon", "the allowed deviation in y directionfor LCSS alignment (positive double value, 0 <= pEpsilon <= 1)");
 
   /**
    * Keeps the currently set pDelta.
@@ -117,7 +117,7 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
    *         vectors as an instance of {@link DoubleDistance DoubleDistance}.
    */
   @Override
-  public double doubleDistance(NumberVector<?, ?> v1, NumberVector<?, ?> v2) {
+  public double doubleDistance(NumberVector<?> v1, NumberVector<?> v2) {
     final int delta = (int) Math.ceil(v2.getDimensionality() * pDelta);
 
     DoubleMinMax extrema1 = VectorUtil.getRangeDouble(v1);
@@ -130,45 +130,42 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
     double[] a, b;
 
     // put shorter vector first
-    if(v1.getDimensionality() < v2.getDimensionality()) {
+    if (v1.getDimensionality() < v2.getDimensionality()) {
       m = v1.getDimensionality();
       n = v2.getDimensionality();
       a = new double[m];
       b = new double[n];
 
-      for(int i = 0; i < v1.getDimensionality(); i++) {
-        a[i] = v1.doubleValue(i + 1);
+      for (int i = 0; i < v1.getDimensionality(); i++) {
+        a[i] = v1.doubleValue(i);
       }
-      for(int j = 0; j < v2.getDimensionality(); j++) {
-        b[j] = v2.doubleValue(j + 1);
+      for (int j = 0; j < v2.getDimensionality(); j++) {
+        b[j] = v2.doubleValue(j);
       }
-    }
-    else {
+    } else {
       m = v2.getDimensionality();
       n = v1.getDimensionality();
       a = new double[m];
       b = new double[n];
 
-      for(int i = 0; i < v2.getDimensionality(); i++) {
-        a[i] = v2.doubleValue(i + 1);
+      for (int i = 0; i < v2.getDimensionality(); i++) {
+        a[i] = v2.doubleValue(i);
       }
-      for(int j = 0; j < v1.getDimensionality(); j++) {
-        b[j] = v1.doubleValue(j + 1);
+      for (int j = 0; j < v1.getDimensionality(); j++) {
+        b[j] = v1.doubleValue(j);
       }
     }
 
     double[] curr = new double[n + 1];
 
-    for(int i = 0; i < m; i++) {
+    for (int i = 0; i < m; i++) {
       double[] next = new double[n + 1];
-      for(int j = Math.max(0, i - delta); j <= Math.min(n - 1, i + delta); j++) {
-        if((b[j] + epsilon) >= a[i] & (b[j] - epsilon) <= a[i]) { // match
+      for (int j = Math.max(0, i - delta); j <= Math.min(n - 1, i + delta); j++) {
+        if ((b[j] + epsilon) >= a[i] && (b[j] - epsilon) <= a[i]) { // match
           next[j + 1] = curr[j] + 1;
-        }
-        else if(curr[j + 1] > next[j]) { // ins
+        } else if (curr[j + 1] > next[j]) { // ins
           next[j + 1] = curr[j + 1];
-        }
-        else { // del
+        } else { // del
           next[j + 1] = next[j];
         }
       }
@@ -177,8 +174,8 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
 
     // search for maximum in the last line
     double maxEntry = -1;
-    for(int i = 1; i < n + 1; i++) {
-      if(curr[i] > maxEntry) {
+    for (int i = 1; i < n + 1; i++) {
+      if (curr[i] > maxEntry) {
         maxEntry = curr[i];
       }
     }
@@ -188,13 +185,16 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
 
   // TODO: relax this to VectorTypeInformation!
   @Override
-  public VectorFieldTypeInformation<? super NumberVector<?, ?>> getInputTypeRestriction() {
+  public VectorFieldTypeInformation<? super NumberVector<?>> getInputTypeRestriction() {
     return TypeUtil.NUMBER_VECTOR_FIELD;
   }
 
   @Override
   public boolean equals(Object obj) {
-    if(obj == null) {
+    if (obj == this) {
+      return true;
+    }
+    if (obj == null) {
       return false;
     }
     if (!this.getClass().equals(obj.getClass())) {
@@ -202,7 +202,7 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
     }
     return (this.pDelta == ((LCSSDistanceFunction) obj).pDelta) && (this.pEpsilon == ((LCSSDistanceFunction) obj).pEpsilon);
   }
-  
+
   /**
    * Parameterization class.
    * 
@@ -211,21 +211,25 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
    * @apiviz.exclude
    */
   public static class Parameterizer extends AbstractParameterizer {
-    protected Double pDelta = 0.0;
+    protected double pDelta = 0.0;
 
-    protected Double pEpsilon = 0.0;
+    protected double pEpsilon = 0.0;
 
     @Override
     protected void makeOptions(Parameterization config) {
       super.makeOptions(config);
-      final DoubleParameter pDeltaP = new DoubleParameter(PDELTA_ID, new IntervalConstraint(0, IntervalBoundary.CLOSE, 1, IntervalBoundary.CLOSE), 0.1);
-      if(config.grab(pDeltaP)) {
-        pDelta = pDeltaP.getValue();
+      final DoubleParameter pDeltaP = new DoubleParameter(PDELTA_ID, 0.1);
+      pDeltaP.addConstraint(new GreaterEqualConstraint(0));
+      pDeltaP.addConstraint(new LessEqualConstraint(1));
+      if (config.grab(pDeltaP)) {
+        pDelta = pDeltaP.doubleValue();
       }
 
-      final DoubleParameter pEpsilonP = new DoubleParameter(PEPSILON_ID, new IntervalConstraint(0, IntervalBoundary.CLOSE, 1, IntervalBoundary.CLOSE), 0.05);
-      if(config.grab(pEpsilonP)) {
-        pEpsilon = pEpsilonP.getValue();
+      final DoubleParameter pEpsilonP = new DoubleParameter(PEPSILON_ID, 0.05);
+      pEpsilonP.addConstraint(new GreaterEqualConstraint(0));
+      pEpsilonP.addConstraint(new LessEqualConstraint(1));
+      if (config.grab(pEpsilonP)) {
+        pEpsilon = pEpsilonP.doubleValue();
       }
     }
 
@@ -234,4 +238,4 @@ public class LCSSDistanceFunction extends AbstractVectorDoubleDistanceFunction {
       return new LCSSDistanceFunction(pDelta, pEpsilon);
     }
   }
-}
\ No newline at end of file
+}
