@@ -28,16 +28,33 @@ import java.io.ObjectInput;
 import java.io.ObjectOutput;
 import java.util.regex.Pattern;
 
+import de.lmu.ifi.dbs.elki.utilities.FormatUtil;
+
 /**
  * Provides a Distance for a double-valued distance.
  * 
  * @author Elke Achtert
  */
 public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
+  /**
+   * Zero distance constant
+   */
+  public static final DoubleDistance ZERO_DISTANCE = new DoubleDistance(0.0);
+
+  /**
+   * Infinite distance constant
+   */
+  public static final DoubleDistance INFINITE_DISTANCE = new DoubleDistance(Double.POSITIVE_INFINITY);
+
+  /**
+   * Undefined distance constant
+   */
+  public static final DoubleDistance UNDEFINED_DISTANCE = new DoubleDistance(Double.NaN);
+
   /**
    * The static factory instance
    */
-  public final static DoubleDistance FACTORY = new DoubleDistance();
+  public static final DoubleDistance FACTORY = UNDEFINED_DISTANCE;
 
   /**
    * The actual value.
@@ -118,7 +135,7 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
    */
   @Override
   public void readExternal(ObjectInput in) throws IOException {
-    setValue(in.readDouble());
+    this.value = in.readDouble();
   }
 
   /**
@@ -132,16 +149,6 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
     return 8;
   }
 
-  @Override
-  public Double getValue() {
-    return this.value;
-  }
-
-  @Override
-  void setValue(Double value) {
-    this.value = value;
-  }
-
   @Override
   public double doubleValue() {
     return value;
@@ -157,25 +164,13 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
     return Double.compare(this.value, other.value);
   }
 
-  @Override
-  public boolean equals(Object o) {
-    if(this == o) {
-      return true;
-    }
-    if(o == null || getClass() != o.getClass()) {
-      return false;
-    }
-    double delta = Math.abs(value - ((DoubleDistance) o).value);
-    return delta < Double.MIN_NORMAL;
-  }
-
   /**
    * An infinite DoubleDistance is based on {@link Double#POSITIVE_INFINITY
    * Double.POSITIVE_INFINITY}.
    */
   @Override
   public DoubleDistance infiniteDistance() {
-    return new DoubleDistance(Double.POSITIVE_INFINITY);
+    return INFINITE_DISTANCE;
   }
 
   /**
@@ -183,7 +178,7 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
    */
   @Override
   public DoubleDistance nullDistance() {
-    return new DoubleDistance(0.0);
+    return ZERO_DISTANCE;
   }
 
   /**
@@ -191,7 +186,7 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
    */
   @Override
   public DoubleDistance undefinedDistance() {
-    return new DoubleDistance(Double.NaN);
+    return UNDEFINED_DISTANCE;
   }
 
   /**
@@ -199,13 +194,12 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
    */
   @Override
   public DoubleDistance parseString(String val) throws IllegalArgumentException {
-    if(val.equals(INFINITY_PATTERN)) {
+    if (val.equals(INFINITY_PATTERN)) {
       return infiniteDistance();
     }
-    if(testInputPattern(val)) {
+    if (testInputPattern(val)) {
       return new DoubleDistance(Double.parseDouble(val));
-    }
-    else {
+    } else {
       throw new IllegalArgumentException("Given pattern \"" + val + "\" does not match required pattern \"" + requiredInputPattern() + "\"");
     }
   }
@@ -217,7 +211,7 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
 
   @Override
   public boolean isNullDistance() {
-    return (value == 0.0);
+    return (value <= 0.0);
   }
 
   @Override
@@ -230,9 +224,32 @@ public class DoubleDistance extends NumberDistance<DoubleDistance, Double> {
     return DOUBLE_PATTERN;
   }
 
+  @Override
+  public String toString() {
+    return FormatUtil.NF8.format(value);
+  }
+
   @Override
   public int hashCode() {
-    long bits = Double.doubleToLongBits(value);
+    final long bits = Double.doubleToLongBits(value);
     return (int) (bits ^ (bits >>> 32));
   }
-}
\ No newline at end of file
+
+  @Override
+  public boolean equals(Object obj) {
+    if (this == obj) {
+      return true;
+    }
+    if (!super.equals(obj)) {
+      return false;
+    }
+    if (getClass() != obj.getClass()) {
+      return false;
+    }
+    DoubleDistance other = (DoubleDistance) obj;
+    if (Double.doubleToLongBits(value) != Double.doubleToLongBits(other.value)) {
+      return false;
+    }
+    return true;
+  }
+}
