@@ -145,7 +145,7 @@ public class ClassListParameterConfigurator extends AbstractSingleParameterConfi
   }
 
   @Override
-  public void addParameter(Object owner, Parameter<?, ?> param, TrackParameters track) {
+  public void addParameter(Object owner, Parameter<?> param, TrackParameters track) {
     child.addParameter(owner, param, track);
   }
 
