@@ -99,7 +99,7 @@ public class ClassParameterConfigurator extends AbstractSingleParameterConfigura
   }
 
   @Override
-  public void addParameter(Object owner, Parameter<?, ?> param, TrackParameters track) {
+  public void addParameter(Object owner, Parameter<?> param, TrackParameters track) {
     child.addParameter(owner, param, track);
   }
 
@@ -129,7 +129,7 @@ public class ClassParameterConfigurator extends AbstractSingleParameterConfigura
     if(val.startsWith(DynamicParameters.STRING_USE_DEFAULT)) {
       return null;
     }
-    if(val == DynamicParameters.STRING_OPTIONAL) {
+    if(DynamicParameters.STRING_OPTIONAL.equals(val)) {
       return null;
     }
     return val;
