@@ -40,55 +40,55 @@ import de.lmu.ifi.dbs.elki.logging.LoggingUtil;
  * @apiviz.landmark
  */
 public class StockIcon {
-  public final static String DIALOG_ERROR = "dialog-error";
+  public static final String DIALOG_ERROR = "dialog-error";
 
-  public final static String DIALOG_INFORMATION = "dialog-information";
+  public static final String DIALOG_INFORMATION = "dialog-information";
 
-  public final static String DIALOG_WARNING = "dialog-warning";
+  public static final String DIALOG_WARNING = "dialog-warning";
 
-  public final static String DOCUMENT_OPEN = "document-open";
+  public static final String DOCUMENT_OPEN = "document-open";
 
-  public final static String DOCUMENT_PROPERTIES = "document-properties";
+  public static final String DOCUMENT_PROPERTIES = "document-properties";
 
-  public final static String DOCUMENT_SAVE = "document-save";
+  public static final String DOCUMENT_SAVE = "document-save";
 
-  public final static String EDIT_CLEAR = "edit-clear";
+  public static final String EDIT_CLEAR = "edit-clear";
 
-  public final static String EDIT_REDO = "edit-redo";
+  public static final String EDIT_REDO = "edit-redo";
 
-  public final static String EDIT_UNDO = "edit-undo";
+  public static final String EDIT_UNDO = "edit-undo";
 
-  public final static String EMBLEM_IMPORTANT = "emblem-important";
+  public static final String EMBLEM_IMPORTANT = "emblem-important";
 
-  public final static String GO_BOTTOM = "go-bottom";
+  public static final String GO_BOTTOM = "go-bottom";
 
-  public final static String GO_DOWN = "go-down";
+  public static final String GO_DOWN = "go-down";
 
-  public final static String GO_FIRST = "go-first";
+  public static final String GO_FIRST = "go-first";
 
-  public final static String GO_HOME = "go-home";
+  public static final String GO_HOME = "go-home";
 
-  public final static String GO_JUMP = "go-jump";
+  public static final String GO_JUMP = "go-jump";
 
-  public final static String GO_LAST = "go-last";
+  public static final String GO_LAST = "go-last";
 
-  public final static String GO_NEXT = "go-next";
+  public static final String GO_NEXT = "go-next";
 
-  public final static String GO_PREVIOUS = "go-previous";
+  public static final String GO_PREVIOUS = "go-previous";
 
-  public final static String GO_TOP = "go-top";
+  public static final String GO_TOP = "go-top";
 
-  public final static String GO_UP = "go-up";
+  public static final String GO_UP = "go-up";
 
-  public final static String HELP_BROWSER = "help-browser";
+  public static final String HELP_BROWSER = "help-browser";
 
-  public final static String LIST_ADD = "list-add";
+  public static final String LIST_ADD = "list-add";
 
-  public final static String LIST_REMOVE = "list-remove";
+  public static final String LIST_REMOVE = "list-remove";
 
-  public final static String PROCESS_STOP = "process-stop";
+  public static final String PROCESS_STOP = "process-stop";
 
-  private final static Map<String, SoftReference<Icon>> iconcache = new HashMap<String, SoftReference<Icon>>();
+  private static final Map<String, SoftReference<Icon>> iconcache = new HashMap<String, SoftReference<Icon>>();
 
   /**
    * Get a particular stock icon.
