@@ -62,34 +62,58 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
   private static final long serialVersionUID = 1L;
 
   /**
-   * Status to signal the step has been configured properly.
+   * Status code enumeration
+   * 
+   * @author Erich Schubert
+   * 
+   * @apiviz.exclude
    */
-  public static final String STATUS_UNCONFIGURED = "unconfigured";
+  public enum Status {
+    /** Status to signal the step has been configured properly. */
+    STATUS_UNCONFIGURED {
+      @Override
+      public String toString() {
+        return "unconfigured";
+      }
+    },
 
-  /**
-   * Status to signal the step has been configured properly.
-   */
-  public static final String STATUS_CONFIGURED = "configured";
+    /** Status to signal the step has been configured properly. */
+    STATUS_CONFIGURED {
+      @Override
+      public String toString() {
+        return "configured";
+      }
+    },
 
-  /**
-   * Status to signal the step is ready to run
-   */
-  public static final String STATUS_READY = "ready to run";
+    /** Status to signal the step is ready to run */
+    STATUS_READY {
+      @Override
+      public String toString() {
+        return "ready to run";
+      }
+    },
 
-  /**
-   * Status to signal the step has been run completely.
-   */
-  public static final String STATUS_COMPLETE = "complete";
+    /** Status to signal the step has been run completely. */
+    STATUS_COMPLETE {
+      @Override
+      public String toString() {
+        return "complete";
+      }
+    },
 
-  /**
-   * Status to signal the step has failed somehow
-   */
-  public static final String STATUS_FAILED = "failed";
+    /** Status to signal the step has failed somehow */
+    STATUS_FAILED {
+      @Override
+      public String toString() {
+        return "failed";
+      }
+    }
+  }
 
   /**
    * ELKI logger for the GUI
    */
-  protected static final Logging logger = Logging.getLogger(ParameterTabPanel.class);
+  private static final Logging LOG = Logging.getLogger(ParameterTabPanel.class);
 
   /**
    * The parameter table
@@ -154,7 +178,9 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
 
     {
       // Create parameter table
-      parameterTable = new ConfiguratorPanel(); // new ParameterTable(parameterModel, parameters);
+      parameterTable = new ConfiguratorPanel(); // new
+                                                // ParameterTable(parameterModel,
+                                                // parameters);
       parameterTable.addChangeListener(this);
       // Create the scroll pane and add the table to it.
       JScrollPane scrollPane = new JScrollPane(parameterTable);
@@ -179,7 +205,7 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
     ListParameterization config = new ListParameterization();
     parameterTable.appendParameters(config);
     setParameters(config);
-    if(config.getErrors().size() > 0) {
+    if (config.getErrors().size() > 0) {
       reportErrors(config);
     }
     config.clearErrors();
@@ -198,12 +224,12 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
     // update parameter table
     {
       parameterTable.setEnabled(false);
-      
+
       parameterTable.clear();
-      for (Pair<Object, Parameter<?,?>> pair : track.getAllParameters()) {
+      for (Pair<Object, Parameter<?>> pair : track.getAllParameters()) {
         parameterTable.addParameter(pair.first, pair.getSecond(), track);
       }
-      //parameters.updateFromTrackParameters(track);
+      // parameters.updateFromTrackParameters(track);
 
       parameterTable.revalidate();
       parameterTable.setEnabled(true);
@@ -213,7 +239,7 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
     updateStatus();
     observers.notifyObservers(this);
   }
-  
+
   /**
    * Collect parameters
    * 
@@ -229,17 +255,17 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
    * @param config Parameterization
    */
   protected void reportErrors(Parameterization config) {
-    StringBuffer buf = new StringBuffer();
-    for(ParameterException e : config.getErrors()) {
-      if(e instanceof UnspecifiedParameterException) {
+    StringBuilder buf = new StringBuilder();
+    for (ParameterException e : config.getErrors()) {
+      if (e instanceof UnspecifiedParameterException) {
         continue;
       }
-      buf.append(e.getMessage() + FormatUtil.NEWLINE);
+      buf.append(e.getMessage()).append(FormatUtil.NEWLINE);
     }
-    if(buf.length() > 0) {
-      logger.warning("Configuration errors:" + FormatUtil.NEWLINE + FormatUtil.NEWLINE + buf.toString());
+    if (buf.length() > 0) {
+      LOG.warning("Configuration errors:" + FormatUtil.NEWLINE + FormatUtil.NEWLINE + buf.toString());
     }
-    //config.clearErrors();
+    // config.clearErrors();
   }
 
   /**
@@ -252,18 +278,17 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
     try {
       configureStep(config);
       if (config.hasUnusedParameters()) {
-        //List<Pair<OptionID, Object>> remainingParameters = config.getRemainingParameters();
-        logger.warning("Unused parameters: "+"FIXME");
+        // List<Pair<OptionID, Object>> remainingParameters =
+        // config.getRemainingParameters();
+        LOG.warning("Unused parameters: " + "FIXME");
       }
-      if(config.getErrors().size() > 0) {
+      if (config.getErrors().size() > 0) {
         reportErrors(config);
-      }
-      else {
+      } else {
         executeStep();
       }
-    }
-    catch(Exception e) {
-      logger.exception(e);
+    } catch (Exception e) {
+      LOG.exception(e);
     }
     updateStatus();
     observers.notifyObservers(this);
@@ -281,7 +306,7 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
    * 
    * @return current status
    */
-  protected abstract String getStatus();
+  protected abstract Status getStatus();
 
   /**
    * Execute the configured step.
@@ -294,8 +319,8 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
    * @return can-run status
    */
   public boolean canRun() {
-    String status = getStatus();
-    if(status == STATUS_READY || status == STATUS_COMPLETE) {
+    Status status = getStatus();
+    if (Status.STATUS_READY.equals(status) || Status.STATUS_COMPLETE.equals(status)) {
       return true;
     }
     return false;
@@ -307,8 +332,8 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
    * @return completeness status
    */
   public boolean isComplete() {
-    String status = getStatus();
-    if(status == STATUS_COMPLETE) {
+    Status status = getStatus();
+    if (Status.STATUS_COMPLETE.equals(status)) {
       return true;
     }
     return false;
@@ -318,7 +343,7 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
    * Invoked to update the UI when the status could have changed.
    */
   protected void updateStatus() {
-    statusText.setText(getStatus());
+    statusText.setText(getStatus().toString());
     runButton.setEnabled(canRun());
   }
 
@@ -329,12 +354,12 @@ public abstract class ParameterTabPanel extends JPanel implements ChangeListener
   public void removeObserver(Observer<? super ParameterTabPanel> o) {
     observers.remove(o);
   }
-  
+
   @Override
   public void stateChanged(ChangeEvent e) {
     if (e.getSource() == this.parameterTable) {
-      //logger.warning("stateChanged!");
+      // logger.warning("stateChanged!");
       updateParameterTable();
     }
   }
-}
\ No newline at end of file
+}
