@@ -87,7 +87,7 @@ public class DynamicParameters {
    * @apiviz.exclude
    */
   public class Node {
-    protected Parameter<?, ?> param;
+    protected Parameter<?> param;
 
     protected String value;
 
@@ -103,7 +103,7 @@ public class DynamicParameters {
      * @param flags Flags
      * @param depth Depth (for tree representation)
      */
-    public Node(Parameter<?, ?> param, String value, BitSet flags, int depth) {
+    public Node(Parameter<?> param, String value, BitSet flags, int depth) {
       super();
       this.param = param;
       this.value = value;
@@ -141,54 +141,50 @@ public class DynamicParameters {
    */
   public synchronized void updateFromTrackParameters(TrackParameters track) {
     parameters.clear();
-    for(Pair<Object, Parameter<?, ?>> p : track.getAllParameters()) {
-      Parameter<?, ?> option = p.getSecond();
+    for (Pair<Object, Parameter<?>> p : track.getAllParameters()) {
+      Parameter<?> option = p.getSecond();
       String value = null;
-      if(option.isDefined()) {
-        if(option.tookDefaultValue()) {
+      if (option.isDefined()) {
+        if (option.tookDefaultValue()) {
           value = DynamicParameters.STRING_USE_DEFAULT + option.getDefaultValueAsString();
-        }
-        else {
+        } else {
           value = option.getValueAsString();
         }
       }
-      if(value == null) {
-        if(option instanceof Flag) {
+      if (value == null) {
+        if (option instanceof Flag) {
           value = Flag.NOT_SET;
-        }
-        else {
+        } else {
           value = "";
         }
       }
       BitSet bits = new BitSet();
-      if(option.isOptional()) {
+      if (option.isOptional()) {
         bits.set(BIT_OPTIONAL);
       }
-      if(option.hasDefaultValue() && option.tookDefaultValue()) {
+      if (option.hasDefaultValue() && option.tookDefaultValue()) {
         bits.set(BIT_DEFAULT_VALUE);
       }
-      if(value == "") {
-        if(!bits.get(BIT_DEFAULT_VALUE) && !bits.get(BIT_OPTIONAL)) {
+      if (value.length() <= 0) {
+        if (!bits.get(BIT_DEFAULT_VALUE) && !bits.get(BIT_OPTIONAL)) {
           bits.set(BIT_INCOMPLETE);
         }
-      }
-      if(value != "") {
+      } else {
         try {
-          if(!option.tookDefaultValue() && !option.isValid(value)) {
+          if (!option.tookDefaultValue() && !option.isValid(value)) {
             bits.set(BIT_INVALID);
           }
-        }
-        catch(ParameterException e) {
+        } catch (ParameterException e) {
           bits.set(BIT_INVALID);
         }
       }
       int depth = 0;
       {
         Object pos = track.getParent(option);
-        while(pos != null) {
+        while (pos != null) {
           pos = track.getParent(pos);
           depth += 1;
-          if(depth > 10) {
+          if (depth > 10) {
             break;
           }
         }
@@ -206,7 +202,7 @@ public class DynamicParameters {
    * @param bits Bits
    * @param depth Depth
    */
-  public synchronized void addParameter(Parameter<?, ?> option, String value, BitSet bits, int depth) {
+  public synchronized void addParameter(Parameter<?> option, String value, BitSet bits, int depth) {
     Node t = new Node(option, value, bits, depth);
     parameters.add(t);
   }
@@ -218,22 +214,20 @@ public class DynamicParameters {
    */
   public synchronized ArrayList<String> serializeParameters() {
     ArrayList<String> p = new ArrayList<String>(2 * parameters.size());
-    for(Node t : parameters) {
-      if(t.param != null) {
-        if(t.param instanceof RemainingOptions) {
-          for(String str : t.value.split(" ")) {
-            if(str.length() > 0) {
+    for (Node t : parameters) {
+      if (t.param != null) {
+        if (t.param instanceof RemainingOptions) {
+          for (String str : t.value.split(" ")) {
+            if (str.length() > 0) {
               p.add(str);
             }
           }
-        }
-        else if(t.param instanceof Flag) {
-          if(t.value == Flag.SET) {
+        } else if (t.param instanceof Flag) {
+          if (Flag.SET.equals(t.value)) {
             p.add(SerializedParameterization.OPTION_PREFIX + t.param.getOptionID().getName());
           }
-        }
-        else if(t.value != null && t.value.length() > 0) {
-          if(!t.value.startsWith(STRING_USE_DEFAULT) && t.value != STRING_OPTIONAL) {
+        } else if (t.value != null && t.value.length() > 0) {
+          if (!t.value.startsWith(STRING_USE_DEFAULT) && !STRING_OPTIONAL.equals(t.value)) {
             p.add(SerializedParameterization.OPTION_PREFIX + t.param.getOptionID().getName());
             p.add(t.value);
           }
@@ -256,7 +250,7 @@ public class DynamicParameters {
   /**
    * OptionID for unrecognized options.
    */
-  protected static OptionID REMAINING_OPTIONS_ID = OptionID.getOrCreateOptionID("UNUSED", "Unrecognized options.");
+  protected static OptionID REMAINING_OPTIONS_ID = new OptionID("UNUSED", "Unrecognized options.");
 
   /**
    * Dummy option class that represents unhandled options
@@ -272,4 +266,4 @@ public class DynamicParameters {
       super.setOptional(true);
     }
   }
-}
\ No newline at end of file
+}
