@@ -147,11 +147,11 @@ public class ParameterTable extends JTable {
         return;
       }
       if(value instanceof DynamicParameters.Node) {
-        Parameter<?, ?> o = ((DynamicParameters.Node) value).param;
+        Parameter<?> o = ((DynamicParameters.Node) value).param;
         // Simulate a tree using indentation - there is no JTreeTable AFAICT
-        StringBuffer buf = new StringBuffer();
+        StringBuilder buf = new StringBuilder();
         for(int i = 1; i < ((DynamicParameters.Node) value).depth; i++) {
-          buf.append(" ");
+          buf.append(' ');
         }
         buf.append(o.getOptionID().getName());
         setText(buf.toString());
@@ -230,7 +230,7 @@ public class ParameterTable extends JTable {
         comboBox.setSelectedIndex(0);
       }
       if(row < parameters.size()) {
-        Parameter<?, ?> option = parameters.getNode(row).param;
+        Parameter<?> option = parameters.getNode(row).param;
         // We can do dropdown choices for class parameters
         if(option instanceof ClassParameter<?>) {
           ClassParameter<?> cp = (ClassParameter<?>) option;
@@ -302,11 +302,6 @@ public class ParameterTable extends JTable {
      */
     final JButton button = new JButton("...");
 
-    /**
-     * The actual file chooser
-     */
-    final JFileChooser fc = new JFileChooser();
-
     /**
      * Constructor.
      */
@@ -322,6 +317,11 @@ public class ParameterTable extends JTable {
      */
     @Override
     public void actionPerformed(ActionEvent e) {
+      final JFileChooser fc = new JFileChooser(new File("."));
+      final String curr = textfield.getText();
+      if (curr != null && curr.length() > 0) {
+        fc.setSelectedFile(new File(curr));
+      }
       int returnVal = fc.showOpenDialog(button);
 
       if(returnVal == JFileChooser.APPROVE_OPTION) {
@@ -347,7 +347,7 @@ public class ParameterTable extends JTable {
     @Override
     public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
       if(row < parameters.size()) {
-        Parameter<?, ?> option = parameters.getNode(row).param;
+        Parameter<?> option = parameters.getNode(row).param;
         if(option instanceof FileParameter) {
           FileParameter fp = (FileParameter) option;
           File f = null;
@@ -357,11 +357,9 @@ public class ParameterTable extends JTable {
           if(f != null) {
             String fn = f.getPath();
             textfield.setText(fn);
-            fc.setSelectedFile(f);
           }
           else {
             textfield.setText("");
-            fc.setSelectedFile(null);
           }
         }
       }
@@ -519,7 +517,7 @@ public class ParameterTable extends JTable {
     public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column) {
       combo.removeAllItems();
       if(row < parameters.size()) {
-        Parameter<?, ?> option = parameters.getNode(row).param;
+        Parameter<?> option = parameters.getNode(row).param;
         // We can do dropdown choices for class parameters
         if(option instanceof ClassListParameter<?>) {
           ClassListParameter<?> cp = (ClassListParameter<?>) option;
@@ -620,7 +618,7 @@ public class ParameterTable extends JTable {
         }
       }
       if(row < parameters.size()) {
-        Parameter<?, ?> option = parameters.getNode(row).param;
+        Parameter<?> option = parameters.getNode(row).param;
         if(option instanceof Flag) {
           activeEditor = dropdownEditor;
           return dropdownEditor.getTableCellEditorComponent(table, value, isSelected, row, column);
