@@ -31,11 +31,11 @@ import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
 import de.lmu.ifi.dbs.elki.database.query.distance.DistanceQuery;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.query.knn.PreprocessorKNNQuery;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.EuclideanDistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
 import de.lmu.ifi.dbs.elki.index.IndexFactory;
 import de.lmu.ifi.dbs.elki.index.KNNIndex;
@@ -54,6 +54,7 @@ import de.lmu.ifi.dbs.elki.utilities.optionhandling.parameters.ObjectParameter;
  * 
  * @param <O> Object type
  * @param <D> Distance type
+ * @param <T> Result type
  */
 public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D>, T extends KNNResult<D>> extends AbstractPreprocessorIndex<O, T> implements KNNIndex<O> {
   /**
@@ -155,8 +156,8 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
 
   @SuppressWarnings("unchecked")
   @Override
-  public <S extends Distance<S>> KNNQuery<O, S> getKNNQuery(DistanceQuery<O, S> distanceQuery, Object... hints) {
-    if(!this.distanceFunction.equals(distanceQuery.getDistanceFunction())) {
+  public <S extends Distance<S>> KNNQuery<O, S> getKNNQuery(DistanceQuery<O, S> distQ, Object... hints) {
+    if(!this.distanceFunction.equals(distQ.getDistanceFunction())) {
       return null;
     }
     // k max supported?
@@ -185,7 +186,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
    * @param <O> The object type
    * @param <D> The distance type
    */
-  public static abstract class Factory<O, D extends Distance<D>, T extends KNNResult<D>> implements IndexFactory<O, KNNIndex<O>> {
+  public abstract static class Factory<O, D extends Distance<D>, T extends KNNResult<D>> implements IndexFactory<O, KNNIndex<O>> {
     /**
      * Parameter to specify the number of nearest neighbors of an object to be
      * materialized. must be an integer greater than 1.
@@ -193,7 +194,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * Key: {@code -materialize.k}
      * </p>
      */
-    public static final OptionID K_ID = OptionID.getOrCreateOptionID("materialize.k", "The number of nearest neighbors of an object to be materialized.");
+    public static final OptionID K_ID = new OptionID("materialize.k", "The number of nearest neighbors of an object to be materialized.");
 
     /**
      * Parameter to indicate the distance function to be used to ascertain the
@@ -206,7 +207,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * Key: {@code materialize.distance}
      * </p>
      */
-    public static final OptionID DISTANCE_FUNCTION_ID = OptionID.getOrCreateOptionID("materialize.distance", "the distance function to materialize the nearest neighbors");
+    public static final OptionID DISTANCE_FUNCTION_ID = new OptionID("materialize.distance", "the distance function to materialize the nearest neighbors");
 
     /**
      * Holds the value of {@link #K_ID}.
@@ -231,7 +232,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
     }
 
     @Override
-    abstract public AbstractMaterializeKNNPreprocessor<O, D, T> instantiate(Relation<O> relation);
+    public abstract AbstractMaterializeKNNPreprocessor<O, D, T> instantiate(Relation<O> relation);
 
     /**
      * Get the distance function.
@@ -264,7 +265,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
      * 
      * @apiviz.exclude
      */
-    public static abstract class Parameterizer<O, D extends Distance<D>> extends AbstractParameterizer {
+    public abstract static class Parameterizer<O, D extends Distance<D>> extends AbstractParameterizer {
       /**
        * Holds the value of {@link #K_ID}.
        */
@@ -279,7 +280,8 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
       protected void makeOptions(Parameterization config) {
         super.makeOptions(config);
         // number of neighbors
-        final IntParameter kP = new IntParameter(K_ID, new GreaterConstraint(1));
+        final IntParameter kP = new IntParameter(K_ID);
+        kP.addConstraint(new GreaterConstraint(1));
         if(config.grab(kP)) {
           k = kP.getValue();
         }
@@ -292,7 +294,7 @@ public abstract class AbstractMaterializeKNNPreprocessor<O, D extends Distance<D
       }
 
       @Override
-      abstract protected Factory<O, D, ?> makeInstance();
+      protected abstract Factory<O, D, ?> makeInstance();
     }
   }
 }
\ No newline at end of file
