@@ -23,31 +23,28 @@ package de.lmu.ifi.dbs.elki.index.preprocessed.knn;
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-import java.util.Collection;
 import java.util.List;
 
 import javax.swing.event.EventListenerList;
 
 import de.lmu.ifi.dbs.elki.database.ids.ArrayDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.ArrayModifiableDBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.DBID;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDIter;
+import de.lmu.ifi.dbs.elki.database.ids.DBIDRef;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDUtil;
 import de.lmu.ifi.dbs.elki.database.ids.DBIDs;
-import de.lmu.ifi.dbs.elki.database.ids.HashSetModifiableDBIDs;
 import de.lmu.ifi.dbs.elki.database.ids.SetDBIDs;
 import de.lmu.ifi.dbs.elki.database.query.DatabaseQuery;
-import de.lmu.ifi.dbs.elki.database.query.DistanceResultPair;
 import de.lmu.ifi.dbs.elki.database.query.knn.KNNQuery;
-import de.lmu.ifi.dbs.elki.database.query.knn.KNNResult;
 import de.lmu.ifi.dbs.elki.database.relation.Relation;
 import de.lmu.ifi.dbs.elki.distance.distancefunction.DistanceFunction;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNHeap;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNResult;
+import de.lmu.ifi.dbs.elki.distance.distanceresultlist.KNNUtil;
 import de.lmu.ifi.dbs.elki.distance.distancevalue.Distance;
-import de.lmu.ifi.dbs.elki.index.preprocessed.knn.KNNChangeEvent.Type;
 import de.lmu.ifi.dbs.elki.logging.Logging;
 import de.lmu.ifi.dbs.elki.logging.progress.FiniteProgress;
 import de.lmu.ifi.dbs.elki.logging.progress.StepProgress;
-import de.lmu.ifi.dbs.elki.utilities.datastructures.heap.KNNHeap;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Description;
 import de.lmu.ifi.dbs.elki.utilities.documentation.Title;
 
@@ -72,7 +69,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
   /**
    * Logger to use.
    */
-  private static final Logging logger = Logging.getLogger(MaterializeKNNPreprocessor.class);
+  private static final Logging LOG = Logging.getLogger(MaterializeKNNPreprocessor.class);
 
   /**
    * Flag to use bulk operations.
@@ -114,12 +111,12 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     FiniteProgress progress = getLogger().isVerbose() ? new FiniteProgress("Materializing k nearest neighbors (k=" + k + ")", ids.size(), getLogger()) : null;
 
     // Try bulk
-    List<KNNResult<D>> kNNList = null;
+    List<? extends KNNResult<D>> kNNList = null;
     if(usebulk) {
       kNNList = knnQuery.getKNNForBulkDBIDs(ids, k);
       if(kNNList != null) {
-        for(int i = 0; i < ids.size(); i++) {
-          DBID id = ids.get(i);
+        int i = 0;
+        for(DBIDIter id = ids.iter(); id.valid(); id.advance(), i++) {
           storage.put(id, kNNList.get(i));
           if(progress != null) {
             progress.incrementProcessed(getLogger());
@@ -128,7 +125,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
       }
     }
     else {
-      for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+      for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
         KNNResult<D> knn = knnQuery.getKNNForDBID(iter, k);
         storage.put(iter, knn);
         if(progress != null) {
@@ -143,22 +140,23 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
   }
 
   @Override
-  public final void insert(DBID id) {
-    objectsInserted(id);
+  public final void insert(DBIDRef id) {
+    objectsInserted(DBIDUtil.deref(id));
   }
 
   @Override
   public void insertAll(DBIDs ids) {
     if(storage == null && ids.size() > 0) {
       preprocess();
-    } else {
+    }
+    else {
       objectsInserted(ids);
     }
   }
 
   @Override
-  public boolean delete(DBID id) {
-    objectsRemoved(id);
+  public boolean delete(DBIDRef id) {
+    objectsRemoved(DBIDUtil.deref(id));
     return true;
   }
 
@@ -181,10 +179,12 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     if(stepprog != null) {
       stepprog.beginStep(1, "New insertions ocurred, materialize their new kNNs.", getLogger());
     }
-    List<KNNResult<D>> kNNList = knnQuery.getKNNForBulkDBIDs(aids, k);
-    for(int i = 0; i < aids.size(); i++) {
-      DBID id = aids.get(i);
-      storage.put(id, kNNList.get(i));
+    // Bulk-query kNNs
+    List<? extends KNNResult<D>> kNNList = knnQuery.getKNNForBulkDBIDs(aids, k);
+    // Store in storage
+    DBIDIter iter = aids.iter();
+    for(int i = 0; i < aids.size(); i++, iter.advance()) {
+      storage.put(iter, kNNList.get(i));
     }
 
     // update the affected kNNs
@@ -215,17 +215,16 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
   private ArrayDBIDs updateKNNsAfterInsertion(DBIDs ids) {
     ArrayModifiableDBIDs rkNN_ids = DBIDUtil.newArray();
     DBIDs oldids = DBIDUtil.difference(relation.getDBIDs(), ids);
-    for (DBIDIter iter = oldids.iter(); iter.valid(); iter.advance()) {
+    for(DBIDIter iter = oldids.iter(); iter.valid(); iter.advance()) {
       KNNResult<D> kNNs = storage.get(iter);
       D knnDist = kNNs.get(kNNs.size() - 1).getDistance();
       // look for new kNNs
       KNNHeap<D> heap = null;
-      for (DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
+      for(DBIDIter iter2 = ids.iter(); iter2.valid(); iter2.advance()) {
         D dist = distanceQuery.distance(iter, iter2);
         if(dist.compareTo(knnDist) <= 0) {
           if(heap == null) {
-            heap = new KNNHeap<D>(k);
-            heap.addAll(kNNs);
+            heap = KNNUtil.newHeap(kNNs);
           }
           heap.add(dist, iter2);
         }
@@ -251,8 +250,8 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     ArrayModifiableDBIDs rkNN_ids = DBIDUtil.newArray();
     for(DBIDIter iditer = relation.iterDBIDs(); iditer.valid(); iditer.advance()) {
       KNNResult<D> kNNs = storage.get(iditer);
-      for(DistanceResultPair<D> kNN : kNNs) {
-        if(idsSet.contains(kNN)) {
+      for(DBIDIter it = kNNs.iter(); it.valid(); it.advance()) {
+        if(idsSet.contains(it)) {
           rkNN_ids.add(iditer);
           break;
         }
@@ -260,10 +259,10 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     }
 
     // update the kNNs of the RkNNs
-    List<KNNResult<D>> kNNList = knnQuery.getKNNForBulkDBIDs(rkNN_ids, k);
-    for(int i = 0; i < rkNN_ids.size(); i++) {
-      DBID id = rkNN_ids.get(i);
-      storage.put(id, kNNList.get(i));
+    List<? extends KNNResult<D>> kNNList = knnQuery.getKNNForBulkDBIDs(rkNN_ids, k);
+    DBIDIter iter = rkNN_ids.iter();
+    for(int i = 0; i < rkNN_ids.size(); i++, iter.advance()) {
+      storage.put(iter, kNNList.get(i));
     }
 
     return rkNN_ids;
@@ -282,7 +281,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     if(stepprog != null) {
       stepprog.beginStep(1, "New deletions ocurred, remove their materialized kNNs.", getLogger());
     }
-    for (DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
+    for(DBIDIter iter = ids.iter(); iter.valid(); iter.advance()) {
       storage.delete(iter);
     }
 
@@ -313,7 +312,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
    * @see KNNListener
    */
   protected void fireKNNsInserted(DBIDs insertions, DBIDs updates) {
-    KNNChangeEvent e = new KNNChangeEvent(this, Type.INSERT, insertions, updates);
+    KNNChangeEvent e = new KNNChangeEvent(this, KNNChangeEvent.Type.INSERT, insertions, updates);
     Object[] listeners = listenerList.getListenerList();
     for(int i = listeners.length - 2; i >= 0; i -= 2) {
       if(listeners[i] == KNNListener.class) {
@@ -331,7 +330,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
    * @see KNNListener
    */
   protected void fireKNNsRemoved(DBIDs removals, DBIDs updates) {
-    KNNChangeEvent e = new KNNChangeEvent(this, Type.DELETE, removals, updates);
+    KNNChangeEvent e = new KNNChangeEvent(this, KNNChangeEvent.Type.DELETE, removals, updates);
     Object[] listeners = listenerList.getListenerList();
     for(int i = listeners.length - 2; i >= 0; i -= 2) {
       if(listeners[i] == KNNListener.class) {
@@ -340,27 +339,6 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
     }
   }
 
-  /**
-   * Extracts and removes the DBIDs in the given collections.
-   * 
-   * @param extraxt a list of lists of DistanceResultPair to extract
-   * @param remove the ids to remove
-   * @return the DBIDs in the given collection
-   */
-  protected ArrayDBIDs extractAndRemoveIDs(List<? extends Collection<DistanceResultPair<D>>> extraxt, ArrayDBIDs remove) {
-    HashSetModifiableDBIDs ids = DBIDUtil.newHashSet();
-    for(Collection<DistanceResultPair<D>> drps : extraxt) {
-      for(DistanceResultPair<D> drp : drps) {
-        ids.add(drp);
-      }
-    }
-    for (DBIDIter iter = remove.iter(); iter.valid(); iter.advance()) {
-      ids.remove(iter);
-    }
-    // Convert back to array
-    return DBIDUtil.newArray(ids);
-  }
-
   /**
    * Adds a {@link KNNListener} which will be invoked when the kNNs of objects
    * are changing.
@@ -397,7 +375,7 @@ public class MaterializeKNNPreprocessor<O, D extends Distance<D>> extends Abstra
 
   @Override
   protected Logging getLogger() {
-    return logger;
+    return LOG;
   }
 
   /**
